/* --------------------------------------------------------------------------
 *
 * class BSplineBasis
 *   implementation
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <b-splines/BSplineBasis>

// ----------------------------------------------- constructors and reinitialize
// -----------------------------------------------------------------------------

BSplineBasis::BSplineBasis( const fpVector& knots,
                                  int       bsOrder,
                                  int       leftEndPoint,
                                  int       rightEndPoint,
                                  bool      rightEndLeftContinuous,
                                  bool      dropLastBasisFn )
 : my_bsOrder( max( 1, bsOrder ) ),
   my_rightEndLeftContinuous( rightEndLeftContinuous ),
   my_dropLastBasisFn( dropLastBasisFn )
 {
  initKnots( knots, leftEndPoint, rightEndPoint );
 }

BSplineBasis::BSplineBasis( const BSplineBasis& another )
 : my_knots( another.my_knots ),
   my_bsOrder( another.my_bsOrder ),
   my_rightEndLeftContinuous( another.my_rightEndLeftContinuous ),
   my_dropLastBasisFn( another.my_dropLastBasisFn )
 {
 }

BSplineBasis::BSplineBasis( )
 : my_knots( fpVector( 2 ) ),
   my_bsOrder( 1 ),
   my_rightEndLeftContinuous( true ),
   my_dropLastBasisFn( false )
 {
  my_knots[0] = 0.0;
  my_knots[1] = 1.0;
 }

void
BSplineBasis::reinitialize( const fpVector& knots,
                                  int       bsOrder,
                                  int       leftEndPoint,
                                  int       rightEndPoint,
                                  bool      rightEndLeftContinuous,
                                  bool      dropLastBasisFn )
 {
  my_bsOrder                = max( 1, bsOrder );
  my_rightEndLeftContinuous = rightEndLeftContinuous;
  my_dropLastBasisFn        = dropLastBasisFn;

  initKnots( knots, leftEndPoint, rightEndPoint );
 }
