/* --------------------------------------------------------------------------  
 *
 * class ArrayIndexMap
 *   implementation
 *
 * --------------------------------------------------------------------------  
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <basic/ArrayIndexMap>

ArrayIndexMap::ArrayIndexMap( )
 : my_sizes( ),
   my_indexOrder( ),
   my_lowerBounds( ),
   my_upperBounds( ),
   my_volume( )
 {
  reinitialize( uintVector( ) );
 }

ArrayIndexMap::ArrayIndexMap( const uintVector&   sizes,
                              const uintVector&   indexOrder,
                              const LocationType& lowerBounds )
 : my_sizes( ),
   my_indexOrder( ),
   my_lowerBounds( ),
   my_upperBounds( ),
   my_volume( )
 {
  reinitialize( sizes, indexOrder, lowerBounds );
 }

void
ArrayIndexMap::reinitialize( const uintVector&   sizes,
                             const uintVector&   indexOrder,
                             const LocationType& lowerBounds )
 {
  size_t dimensions = ( sizes.size( ) > 0 ) ? sizes.size( ) : 1;

  my_sizes.resize( dimensions );
  my_indexOrder.resize( dimensions );
  my_lowerBounds.resize( dimensions, 0 );
  my_volume.resize( dimensions + 1 );

  if ( dimensions > 1 )
   {
    for ( size_t d = 0; d < dimensions; d++ )
     my_sizes[d] = ( sizes[d] > 0 ) ? sizes[d] : 1;

    for ( size_t d = 0; d < min( dimensions, lowerBounds.size( ) ); d++ )
     my_lowerBounds[d] = lowerBounds[d];

    bool allRepresented = false, noneRepeated = false;

    if ( indexOrder.size( ) >= dimensions )
     {
      allRepresented = true;
      noneRepeated   = true;

      bool found, repeated;

      for ( size_t d = 0; d < dimensions; d++ )
       {
        found    = false;
        repeated = false;

        for ( size_t d2 = 0; d2 < dimensions; d2++ )
         {
          repeated = found && ( indexOrder[d2] == d );
          found    = found || ( indexOrder[d2] == d );
         }

        allRepresented = allRepresented && found;
        noneRepeated   = noneRepeated   && !repeated;
       }
     }

    if ( allRepresented && noneRepeated )
     for ( size_t d = 0; d < dimensions; d++ ) my_indexOrder[d] = indexOrder[d];
    else
     for ( size_t d = 0; d < dimensions; d++ ) my_indexOrder[d] = d;

    my_volume[my_indexOrder[0]] = 1;
    for ( size_t d = 1; d < dimensions; d++ )
     my_volume[my_indexOrder[d]] =   my_volume[my_indexOrder[d - 1]]
                                   * my_sizes[my_indexOrder[d - 1]];

    my_volume[dimensions] =   my_volume[my_indexOrder[dimensions - 1]]
                            * my_sizes[my_indexOrder[dimensions - 1]];
                                //
                                //      volume of subspace spanned by directions
                                // my_indexOrder[0] through my_indexOrder[d - 1]
   }
  else
   {
    my_sizes[0]           = 1;
    my_indexOrder[0]      = 0;
    my_lowerBounds[0]     = 0;
    my_volume[0]          = 1;
    my_volume[dimensions] = 1;
   }

  my_upperBounds.resize( dimensions );
  for ( size_t d = 0; d < dimensions; d++ )
   my_upperBounds[d] = my_lowerBounds[d] + my_sizes[d] - 1;
 }

ArrayIndexMap::ArrayIndexMap( const ArrayIndexMap& another )
 : my_sizes( another.my_sizes ),
   my_indexOrder( another.my_indexOrder ),
   my_lowerBounds( another.my_lowerBounds ),
   my_upperBounds( another.my_upperBounds ),
   my_volume( another.my_volume )
 {
 }

ArrayIndexMap&
ArrayIndexMap::operator=( const ArrayIndexMap& rhs )
 {
  if ( this == &rhs ) return *this;

  my_sizes.resize( rhs.my_sizes.size( ) );
  my_indexOrder.resize( rhs.my_indexOrder.size( ) );
  my_lowerBounds.resize( rhs.my_lowerBounds.size( ) );
  my_upperBounds.resize( rhs.my_upperBounds.size( ) );
  my_volume.resize( rhs.my_volume.size( ) );

  my_sizes       = rhs.my_sizes;
  my_indexOrder  = rhs.my_indexOrder;
  my_lowerBounds = rhs.my_lowerBounds;
  my_upperBounds = rhs.my_upperBounds;
  my_volume      = rhs.my_volume;

  return *this;
 }
