/* --------------------------------------------------------------------------
 *
 * class ArrayNavigator
 *   implementation
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <basic/ArrayNavigator>

ArrayNavigator::ArrayNavigator( )
 : ArrayIndexMap( ),
   my_rangeLower( indexLower( ) ),
   my_rangeUpper( indexLower( ) ),
   my_location( my_rangeLower ),
   my_offset( 0 ),
   my_anyMove( true )
 {
  my_rangeUpper += sizes( );
  my_rangeUpper -= 1;
 }

ArrayNavigator::ArrayNavigator( const uintVector&   sizes,
                                const uintVector&   indexOrder,
                                const LocationType& indexLowerBounds )
 : ArrayIndexMap( sizes, indexOrder, indexLowerBounds ),
   my_rangeLower( indexLower( ) ),
   my_rangeUpper( indexLower( ) ),
   my_location( my_rangeLower ),
   my_offset( 0 ),
   my_anyMove( true )
 {
  my_rangeUpper += ArrayIndexMap::sizes( );
  my_rangeUpper -= 1;
 }

void
ArrayNavigator::reinitialize( const uintVector&   sizes,
                              const uintVector&   indexOrder,
                              const LocationType& indexLowerBounds )
 {
  ArrayIndexMap::reinitialize( sizes, indexOrder, indexLowerBounds );

  my_rangeLower.resize( numDims( ) );
  my_rangeUpper.resize( numDims( ) );
  my_location.resize( numDims( ) );

  my_rangeLower  = indexLower( );
  my_rangeUpper  = indexLower( );
  my_rangeUpper += ArrayIndexMap::sizes( );
  my_rangeUpper -= 1;
  my_location    = my_rangeLower;
  my_offset      = 0;
  my_anyMove     = true;
 }

ArrayNavigator::ArrayNavigator( const ArrayNavigator& another )
 : ArrayIndexMap( another ),
   my_rangeLower( another.my_rangeLower ),
   my_rangeUpper( another.my_rangeUpper ),
   my_location( another.my_location ),
   my_offset( another.my_offset ),
   my_anyMove( another.my_anyMove )
 {
 }

ArrayNavigator&
ArrayNavigator::operator=( const ArrayNavigator& rhs )
 {
  if ( this == &rhs ) return *this;

  ArrayIndexMap::operator=( rhs );

  my_rangeLower.resize( rhs.my_rangeLower.size( ) );
  my_rangeUpper.resize( rhs.my_rangeUpper.size( ) );
  my_location.resize( rhs.my_location.size( ) );

  my_rangeLower = rhs.my_rangeLower;
  my_rangeUpper = rhs.my_rangeUpper;
  my_location   = rhs.my_location;
  my_offset     = rhs.my_offset;
  my_anyMove    = rhs.my_anyMove;

  return *this;
 }

void
ArrayNavigator::setRange( size_t d, size_t lower, size_t upper )
 {
  if ( d < numDims( ) )
   {
    lower = max( min( lower, indexUpper( )[d] ), indexLower( )[d] );
    upper = max( lower, min( upper, indexUpper( )[d] ) );

    my_rangeLower[d] = lower;
    my_rangeUpper[d] = upper;

    if (    ( my_location[d] < my_rangeLower[d] )
         || ( my_location[d] > my_rangeUpper[d] ) )
     {
      my_location[d] = min( max( my_location[d], lower ), upper );
      my_anyMove     = true;
     }
   }
 }
