/* --------------------------------------------------------------------------
 *
 * class SineRampC1
 *   sinusoidal symmetric up ramp: C^1 at 0 and 1
 *
 * required headers
 *   see include header file SineRampC1
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __SineRampC1_H__
#define __SineRampC1_H__

cpp_using_namespace( BasicTypes );

class SineRampC1
 {
  public:
                                     //                  sinusoidal up ramp: C^1
                                     // u( t ) = 1/2 + 1/2 sin( pi ( x - 1/2 ) )

   static
   inline
   string
   name( );

   static
   inline
   string
   description( );

   inline
   fpType
   operator()( fpType t ) const;

   inline
   fpType
   Dt( fpType t, int dOrder = 1 ) const;

   inline
   fpType
   integral( fpType upperLimit = 1.0 ) const;
                                                 //
                                                 // lower limit taken to be zero

  private:
   static const fpType our_signs[];
 };

string
SineRampC1::name( )
 {
  return "SineRampC1";
 }

string
SineRampC1::description( )
 {
  return "0.5 * ( 1 + sin( pi * ( t - 0.5 ) ) )";
 }

fpType
SineRampC1::operator()( fpType t ) const
 {
  if ( t <= 0.0 ) return 0.0;
  if ( t >= 1.0 ) return 1.0;

  return 0.5 * ( 1.0 + sin( M_PI * ( t - 0.5 ) ) );
 }

fpType
SineRampC1::Dt( fpType t, int dOrder ) const
 {
  if ( dOrder <= 0 ) return (*this)( t );

  if ( ( t <= 0.0 ) || ( t >= 1.0 ) ) return 0.0;

  return
   (
       0.5
     * our_signs[( dOrder % 4 )]
     * pow( M_PI, dOrder )
     * (
           ( dOrder % 2 )
         ? cos( M_PI * ( t - 0.5 ) )
         : sin( M_PI * ( t - 0.5 ) )
       )
   );
 }

fpType
SineRampC1::integral( fpType upperLimit ) const
 {
  if ( upperLimit <= 0.0 )
   return 0.0;
  else if ( upperLimit >= 1.0 )
   return 0.5; // ramp anitsymmetric about ( 1/2, 1/2 )
  else
   return 0.5 * ( upperLimit - cos( M_PI * ( upperLimit - 0.5 ) ) / M_PI );
 }

#endif /* __SineRampC1_H__ */
