/* --------------------------------------------------------------------------
 *
 * class SplineRampC1
 *   spline-based symmetric up ramp: C^1 at 0 and 1
 *
 * required headers
 *   see include header file SplineRampC1
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __SplineRampC1_H__
#define __SplineRampC1_H__

cpp_using_namespace( BasicTypes );

class SplineRampC1
 {
  public:

                                              //       spline-based up ramp: C^1
                                              //
                                              //         cubic spline with knots
                                              // { 0, 0, 0, 0, 1/2, 1, 1, 1, 1 }
                                              //       and b-spline coefficients
                                              //            { 0, 0, 1/2, 1, 1 };
                                              //
                                              //         The final expression is
                                              //          u( t ) = 3 t^2 - 2 t^3

   static
   inline
   string
   name( );

   static
   inline
   string
   description( );

   inline
   fpType
   operator()( fpType t ) const;

   inline
   fpType
   Dt( fpType t, int dOrder = 1 ) const;

   inline
   fpType
   integral( fpType upperLimit = 1.0 ) const;
                                                 //
                                                 // lower limit taken to be zero
 };

string
SplineRampC1::name( )
 {
  return "SplineRampC1";
 }

string
SplineRampC1::description( )
 {
  return "3 t^2 - 2 t^3";
 }

fpType
SplineRampC1::operator()( fpType t ) const
 {
  if ( t <= 0.0 )
   return 0.0;
  else if ( t >= 1.0 )
   return 1.0;
  else
   return t * t * ( 3.0 - 2.0 * t );
 }

fpType
SplineRampC1::Dt( fpType t, int dOrder ) const
 {
  if ( dOrder <= 0 )
   return (*this)( t );
  else if ( ( dOrder > 3 ) || ( t <= 0.0 ) || ( t >= 1.0 ) )
   return 0.0;
  else if ( dOrder == 1 )
   return 6.0 * t * ( 1.0 - t );
  else if ( dOrder == 2 )
   return 6.0 * ( 1.0 - 2.0 * t );
  else /* if ( dOrder == 3 ) */
   return -12.0;
 }

fpType
SplineRampC1::integral( fpType upperLimit ) const
 {
  if ( upperLimit <= 0.0 )
   return 0.0;
  else if ( upperLimit >= 1.0 )
   return 0.5; // ramp anitsymmetric about ( 1/2, 1/2 )
  else
   return upperLimit * upperLimit * upperLimit * ( 1.0 - upperLimit * 0.5 );
 }

#endif /* __SplineRampC1_H__ */
