/* --------------------------------------------------------------------------
 *
 * template class triple
 *   3-equivalent of the standard class pair
 *
 * required headers
 *   see include header file triple
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __Triple_H__
#define __Triple_H__

template< class T1, class T2, class T3 >
struct triple
 {
  typedef T1 first_type;
  typedef T2 second_type;
  typedef T3 third_type;

  T1 first;
  T2 second;
  T3 third;

  triple( )
   : first( T1( ) ),
     second( T2( ) ),
     third( T3( ) )
   {
   }

  triple( const T1& a, const T2& b, const T3& c )
   : first( a ),
     second( b ),
     third( c )
   {
   }

  triple( const triple< T1, T2, T3 >& t )
   : first( t.first ),
     second( t.second ),
     third( t.third )
   {
   }

  triple< T1, T2, T3 >&
  operator=( const triple< T1, T2, T3 >& t )
   {
    first  = t.first;
    second = t.second;
    third  = t.third;

    return *this;
   }
 };

template< class T1, class T2, class T3 >
inline
bool
operator==( const triple< T1, T2, T3 >& t1, const triple< T1, T2, T3 >& t2 )
 {
  return
      ( t1.first  == t2.first  )
   && ( t1.second == t2.second )
   && ( t1.third  == t2.third  );
 }

template< class T1, class T2, class T3 >
inline
bool
operator<( const triple< T1, T2, T3 >& t1, const triple< T1, T2, T3 >& t2 )
 {
  return
      ( t1.first < t2.first )
   || ( !( t2.first  < t1.first  ) && ( t1.second < t2.second ) )
   || ( !( t2.second < t1.second ) && ( t1.third  < t2.third  ) );
 }

template< class T1, class T2, class T3 >
inline
triple< T1, T2, T3 >
make_triple( const T1& a, const T2& b, const T3& c )
{
 return triple< T1, T2, T3 >( a, b, c );
}

#endif /* __Triple_H__ */
