/* ----------------------------------------------------------------------------
 *
 * namespace ARutils
 *
 * required headers
 *   see include header file ARutils
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __ARutils_H__
#define __ARutils_H__

cpp_using_namespace( BasicTypes );

cpp_namespace_begin( ARutils )
 cpp_namespace_decl
 inline
 void
 getARvec( int numAR, const fpType* const arParams, fpVector& v );

 cpp_namespace_decl
 inline
 void
 getARvec( int numAR,  const fpType* const arParams, const fpVector& v, fpVector& arv );

 cpp_namespace_decl
 inline
 fpVector
 ARvec( int numAR,  const fpType* const arParams, const fpVector& v );
cpp_namespace_end

void
ARutils::getARvec( int numAR, const fpType* const arParams, fpVector& v )
 {
  for ( int i = v.size( ) - 1; i >= numAR; i-- )
   for ( int j = 0; j < numAR; j++ )
    v[i] -= arParams[j] * v[i-j-1];

  for ( int j = 0; j < numAR; j++ ) v[j] = 0.0;
 }

void
ARutils::getARvec( int numAR, const fpType* const arParams, const fpVector& v, fpVector& arv )
 {
  arv.resize( v.size( ), 0.0 );
  for ( int i = numAR; i < arv.size( ); i++ )
   {
    arv[i] = v[i];
    for ( int j = 0; j < numAR; j++ )
     arv[i] -= arParams[j] * v[i-j-1];
   }
 }

fpVector
ARutils::ARvec( int numAR, const fpType* const arParams, const fpVector& v )
 {
  fpVector arv( v );
  getARvec( numAR, arParams, arv );
  return arv;
 }

#endif /* __ARutils_H_ */
