/* --------------------------------------------------------------------------
 *
 * template class Bell, forward declaration
 *   bell functions representing BOLD response
 *
 * required headers
 *   see include header file Bell
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __Bell_FW_H__
#define __Bell_FW_H__

template< class Ramp      = SplineRampC2,
          class Modulator = UnitModulator,
          bool  Dip       = false >
class Bell : public BasicBell
 {
  public:
   // ------------------------------------------------------- construct/destruct
   // --------------------------------------------------------------------------

   Bell( );
   ~Bell( );

   // ------------------------------------------------------------------ support
   // --------------------------------------------------------------------------

   inline
   fpType
   lowerSupport(       fpType  stimulusBegin,
                       fpType  stimulusLength,
                 const fpType* shapeParams ) const;
                              //
                              //                                   stimulusBegin

   inline
   fpType
   upperSupport(       fpType  stimulusBegin,
                       fpType  stimulusLength,
                 const fpType* shapeParams ) const;
                              //
                              // stimulusBegin + stimulusLength + shapeParams[1]

   // --------------------------------------------------------------- bell value
   // --------------------------------------------------------------------------

   inline
   fpType
   operator()(       fpType  time,
                     fpType  stimulusBegin,
                     fpType  stimulusLength,
               const fpType* shapeParams ) const;

   // -------------------------------------------------- derivatives w.r.t. time
   // --------------------------------------------------------------------------

   inline
   fpType
   D1t(       fpType  time,
              fpType  stimulusBegin,
              fpType  stimulusLength,
        const fpType* shapeParams ) const;
                //
                //  first derivative w. r. t. time: likely to be the common case

   inline
   fpType
   Dt(       fpType  time,
             fpType  stimulusBegin,
             fpType  stimulusLength,
       const fpType* shapeParams,
             size_t  dOrder = 1 ) const;
                //
                //                          any general derivative w. r. t. time

   // -------------------------------------- derivatives w.r.t. shape parameters
   // --------------------------------------------------------------------------

   inline
   fpType
   gradient(       fpType  time,
                   fpType  stimulusBegin,
                   fpType  stimulusLength,
             const fpType* shapeParams,
                   size_t  paramIndex ) const;

   inline
   fpType
   hessian(       fpType  time,
                  fpType  stimulusBegin,
                  fpType  stimulusLength,
            const fpType* shapeParams,
                  size_t  paramIndex1,
                  size_t  paramIndex2 ) const;
 };

#endif /* __Bell_FW_H__ */
