/* --------------------------------------------------------------------------
 *
 * template class Bell, partial specialization
 *   bell function with non-unit modulator, without dip
 *
 * required headers
 *   see include header file Bell
 *
 * notes
 *   INCOMPLETE IMPLEMENTATION: UNUSABLE
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __Bell_MN_H__
#define __Bell_MN_H__

template< class Ramp, class Modulator >
class Bell< Ramp, Modulator, false > : public BasicBell
 {
  public:
   // ------------------------------------------------------- construct/destruct
   // --------------------------------------------------------------------------

   Bell( );

   ~Bell( );

   // ------------------------------------------------------------------ support
   // --------------------------------------------------------------------------

   inline
   fpType
   lowerSupport(       fpType  stimulusBegin,
                       fpType  stimulusLength,
                 const fpType* shapeParams ) const;
                              //
                              //                                   stimulusBegin

   inline
   fpType
   upperSupport(       fpType  stimulusBegin,
                       fpType  stimulusLength,
                 const fpType* shapeParams ) const;
                              //
                              // stimulusBegin + stimulusLength + shapeParams[1]

   // --------------------------------------------------------------- bell value
   // --------------------------------------------------------------------------

   inline
   fpType
   operator()(       fpType  time,
                     fpType  stimulusBegin,
                     fpType  stimulusLength,
               const fpType* shapeParams ) const;

   // -------------------------------------------------- derivatives w.r.t. time
   // --------------------------------------------------------------------------

   inline
   fpType
   D1t(       fpType  time,
              fpType  stimulusBegin,
              fpType  stimulusLength,
        const fpType* shapeParams ) const;
                //
                //  first derivative w. r. t. time: likely to be the common case

   inline
   fpType
   Dt(       fpType  time,
             fpType  stimulusBegin,
             fpType  stimulusLength,
       const fpType* shapeParams,
             size_t  dOrder = 1 ) const;
                //
                //             any general derivative w. r. t. time; dOrder >= 0

   // -------------------------------------- derivatives w.r.t. shape parameters
   // --------------------------------------------------------------------------

   inline
   fpType
   gradient(       fpType  time,
                   fpType  stimulusBegin,
                   fpType  stimulusLength,
             const fpType* shapeParams,
                   size_t  paramIndex ) const;

   inline
   fpType
   hessian(       fpType  time,
                  fpType  stimulusBegin,
                  fpType  stimulusLength,
            const fpType* shapeParams,
                  size_t  paramIndex1,
                  size_t  paramIndex2 ) const;

  private:
   vector<string> my_paramNames;
   Ramp           my_ramp;
   Modulator      my_modulator;
 };

// ---------------------------------------------------------- construct/destruct
// -----------------------------------------------------------------------------

template< class Ramp, class Modulator >
Bell< Ramp, Modulator, false >::Bell( )
 : BasicBell( 4, "mn4",
                   string( "4-parameter bell; no dip; ramp: " ) + Ramp::name( )
                 + string( ", modulator: " ) + Modulator::name( ) )
 {
  BasicBell::setParamInfo( 0, "attack",           "attack duration",  "lower", 0.0 );
  BasicBell::setParamInfo( 1, "attack_sharpness", "attack sharpness", "none"       );
  BasicBell::setParamInfo( 2, "decay",            "decay duration",   "lower", 0.0 );
  BasicBell::setParamInfo( 3, "decay_sharpness",  "decay sharpness",  "none"       );
 }

template< class Ramp, class Modulator >
Bell< Ramp, Modulator, false >::~Bell( )
 {
 }

// --------------------------------------------------------------------- support
// -----------------------------------------------------------------------------

template< class Ramp, class Modulator >
fpType
Bell< Ramp, Modulator, false >::lowerSupport
 (       fpType  stimulusBegin,
         fpType  stimulusLength,
   const fpType* shapeParams ) const
 {
  return stimulusBegin;
 }

template< class Ramp, class Modulator >
fpType
Bell< Ramp, Modulator, false >::upperSupport
 (       fpType  stimulusBegin,
         fpType  stimulusLength,
   const fpType* shapeParams ) const
 {
  return ( stimulusBegin + stimulusLength + shapeParams[2] );
 }

// ------------------------------------------------------------------ bell value
// -----------------------------------------------------------------------------

template< class Ramp, class Modulator >
fpType
Bell< Ramp, Modulator, false >::operator()
 (       fpType  time,
         fpType  stimulusBegin,
         fpType  stimulusLength,
   const fpType* shapeParams ) const
 {
  cerr << "Bell<Ramp,Modulator,false>::operator()(...) not implemented: return 0.0" << endl;
  return 0.0;

/*
  return
     my_ramp( ( time - stimulusBegin ) / shapeParams[0] )
   * (
         1.0
       - my_ramp( ( time - stimulusBegin - stimulusLength ) / shapeParams[1] )
     );
*/
 }

// ----------------------------------------------------- derivatives w.r.t. time
// -----------------------------------------------------------------------------

template< class Ramp, class Modulator >
fpType
Bell< Ramp, Modulator, false >::D1t
 (       fpType  time,
         fpType  stimulusBegin,
         fpType  stimulusLength,
   const fpType* shapeParams ) const
 {
  cerr << "Bell<Ramp,Modulator,false>::D1t(...) not implemented: return 0.0" << endl;
  return 0.0;

/*
  fpType t0 = ( time - stimulusBegin ) / shapeParams[0];
  fpType t1 = ( time - stimulusBegin - stimulusLength ) / shapeParams[1];

  return
   (
       my_ramp.Dt( t0, 1 ) * ( 1.0 - my_ramp( t1 ) ) / shapeParams[0]
     - my_ramp( t0 ) * my_ramp.Dt( t1, 1 ) / shapeParams[1]
   );
*/
 }

template< class Ramp, class Modulator >
fpType
Bell< Ramp, Modulator, false >::Dt
 (       fpType  time,
         fpType  stimulusBegin,
         fpType  stimulusLength,
   const fpType* shapeParams,
         size_t  dOrder ) const
 {
  cerr << "Bell<Ramp,Modulator,false>::Dt(...) not implemented: return 0.0" << endl;
  return 0.0;

/*
  if ( dOrder <= 0 )
   return (*this)( time, stimulusBegin, stimulusLength, shapeParams );
  else if ( dOrder == 1 )
   return (*this).D1t( time, stimulusBegin, stimulusLength, shapeParams );
  else
   {
    fpType t0 = ( time - stimulusBegin ) / shapeParams[0];
    fpType t1 = ( time - stimulusBegin - stimulusLength ) / shapeParams[1];
  
    fpType dt = my_ramp.Dt( t0, dOrder );
  
    fpType f0  = 1.0;
    fpType f1  = 1.0 / pow( shapeParams[1], dOrder );
    fpType nCr = 1.0;
  
    for ( size_t i = 0; i <= dOrder; i++ )
     {
      dt -=   nCr
            * f0 * my_ramp.Dt( t0, i )
            * f1 * my_ramp.Dt( t1, dOrder - i );
  
      f0  /= shapeParams[0];
      f1  *= shapeParams[1];
      nCr *= (fpType) ( dOrder - i ) / (fpType) ( i + 1 );
     }
  
    return dt;
   }
*/
 }

// ----------------------------------------- derivatives w.r.t. shape parameters
// -----------------------------------------------------------------------------

template< class Ramp, class Modulator >
fpType
Bell< Ramp, Modulator, false >::gradient
 (       fpType  time,
         fpType  stimulusBegin,
         fpType  stimulusLength,
   const fpType* shapeParams,
         size_t  paramIndex ) const
 {
  cerr << "Bell<Ramp,Modulator,false>::gradient(...) not implemented: return 0.0" << endl;
  return 0.0;

/*
  if ( paramIndex == 0 )
   {
    fpType t0 = ( time - stimulusBegin ) / shapeParams[0];
    fpType t1 = ( time - stimulusBegin - stimulusLength ) / shapeParams[1];

    return
     - my_ramp.Dt( t0, 1 )
     * ( 1.0 - my_ramp( t1 ) ) * ( t0 / shapeParams[0] );
   }
  else if ( paramIndex == 1 )
   {
    fpType t0 = ( time - stimulusBegin ) / shapeParams[0];
    fpType t1 = ( time - stimulusBegin - stimulusLength ) / shapeParams[1];

    return
     my_ramp( t0 ) * my_ramp.Dt( t1, 1 ) * ( t1 / shapeParams[1] );
   }
  else
   {
    return 0.0;
   }
*/
 }

template< class Ramp, class Modulator >
fpType
Bell< Ramp, Modulator, false >::hessian
 (       fpType  time,
         fpType  stimulusBegin,
         fpType  stimulusLength,
   const fpType* shapeParams,
         size_t  paramIndex1,
         size_t  paramIndex2 ) const
 {
  cerr << "Bell<Ramp,Modulator,false>::hessian(...) not implemented: return 0.0" << endl;
  return 0.0;

/*
  if (    ( paramIndex1 < 0 ) || ( paramIndex1 > 1 )
       || ( paramIndex2 < 0 ) || ( paramIndex2 > 1 ) ) return 0.0;

  size_t pSum = paramIndex1 + paramIndex2;

  fpType t0 = ( time - stimulusBegin ) / shapeParams[0];
  fpType t1 = ( time - stimulusBegin - stimulusLength ) / shapeParams[1];

  if ( pSum == 0 )
   {
    fpType t00 = t0 / shapeParams[0];

    return
       t00 * ( 1.0 - my_ramp( t1 ) )
     * (   2.0 * my_ramp.Dt( t0, 1 ) / shapeParams[0]
         + t00 * my_ramp.Dt( t0, 2 ) );
   }
  else if ( pSum == 1 )
   {
    return
     - t0 * my_ramp.Dt( t0, 1 ) / shapeParams[0]
     * t1 * my_ramp.Dt( t1, 1 ) / shapeParams[1];
   }
  else if ( pSum == 2 )
   {
    fpType t11 = t1 / shapeParams[1];

    return
     - t11 * my_ramp( t0 )
     * (   2.0 * my_ramp.Dt( t1, 1 ) / shapeParams[1]
         + t11 * my_ramp.Dt( t1, 2 ) );
   }
  else
   return 0.0;
*/
 }

#endif /* __Bell_MN_H__ */
