/* ----------------------------------------------------------------------------
 *
 * template class LLnormal, forward declaration
 *   template for log normal/ARMA likelihood with/without activation
 *
 * required headers
 *   see include header file LLnormal
 *
 * notes
 *   currently supported AR/MA orders: 0 and 1 for AR, 0 for MA
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __LLN_FW_H__
#define __LLN_FW_H__

cpp_using_namespace( BasicTypes );

template< bool activation = true, size_t numAR = 0, size_t numMA = 0 >
class LLnormal : public BasicObjFn
 {
  public:
   // ---------------------------- construct/reinitialize/incremental initialize
   // --------------------------------------------------------------------------

   LLnormal( );

   LLnormal( const LLnormal& another );

   LLnormal&
   operator=( const LLnormal& rhs );

   LLnormal( const fpVector*                 signal,
             const dbProfile*                db,
             const acProfile*                ac,
             const ParameterManager<fpType>& pm,
                   int                       paramBufSize,
                   fpType                    sign = 1.0,
             const string&                   name = "likelihood function" );

   void
   setSignal( const fpVector* signal );

   void
   setDBprofile( const dbProfile* db );

   void
   setACprofile( const acProfile* ac );

   void
   setParamOffset( const ParameterManager<fpType>& pm );

   void
   reinitialize( const fpVector*                 signal,
                 const dbProfile*                db,
                 const acProfile*                ac,
                 const ParameterManager<fpType>& pm,
                       int                       paramBufSize,
                       fpType                    sign = 1.0,
                 const string&                   name = "likelihood function" );

   // --------------------------------------------------------------- attributes
   // --------------------------------------------------------------------------

   inline
   fpType
   additiveConst( ) const;
                                //
                                // additive constants arising from normalization
                                //   that are not included in value computation.
                                //
                                // sign (as set by setSign( ) or constructor) is
                                //             not included in additiveConst( ).

   inline
   bool
   hasActivation( ) const { return activation; }

   // inline
   // size_t
   // numAR( ) const { return numAR; }

   // inline
   // size_t
   // numMA( ) const { return numMA; }

   // -------------------------------------------------- likelihood computations
   // ----------------- without initialization of output arguments ll, Dll, DDll
   // --------------------------------------------------------------------------

            // the implementations of gradient and hessian makes the assumptions
            //         (1) drift+baseline is linear in drift+baseline parameters
            //         (and use functions in dbProfile based on this assumption)
            //             (2) activation is linear in responsiveness parameters

   inline
   void
   value_ni( const fpType* const paramBuf, fpType* ll ) const;

   inline
   void
   gradient_ni( const fpType* const paramBuf, fpType* Dll ) const;

   inline
   void
   hessian_ni( const fpType* const parameters, fpMatrix& DDlp ) const;
 };

#endif /* __LLN_FW_H__ */
