/* ----------------------------------------------------------------------------
 *
 * class LLnormal<false,0,0>
 *   template specialization: log normal/ARMA(0,0) likelihood/no activation
 *
 * required headers
 *   see include header file LLnormal
 *
 * notes
 *   0. follows convention for out-of-range parameter offsets; see BasicObjFn.h
 *   1. acProfile needed only for the sake of a common interface.
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __LLn_n_0_0_H__
#define __LLn_n_0_0_H__

cpp_using_namespace( BasicTypes );

class LLnormal<false,0,0> : public BasicObjFn
 {
  public:
   // --------------------- construct/reinitialize/incremental initialize/assign
   // --------------------------------------------------------------------------

   LLnormal( );

   LLnormal( const LLnormal& another );

   LLnormal&
   operator=( const LLnormal& rhs );

   LLnormal( const fpVector*                 signal,
             const dbProfile*                db,
             const acProfile*                ac,
             const ParameterManager<fpType>& pm,
                   int                       paramBufSize,
                   fpType                    sign = 1.0,
             const string&                   name = "LL normal/no activation/arma(0,0)" );

   void
   setSignal( const fpVector* signal );

   void
   setDBprofile( const dbProfile* db );

   void
   setACprofile( const acProfile* ac );

   void
   setParamOffset( const ParameterManager<fpType>& pm );

   void
   reinitialize( const fpVector*                 signal,
                 const dbProfile*                db,
                 const acProfile*                ac,
                 const ParameterManager<fpType>& pm,
                       int                       paramBufSize,
                       fpType                    sign = 1.0,
                 const string&                   name = "LL normal/no activation/arma(0,0)" );

   // --------------------------------------------------------------- attributes
   // --------------------------------------------------------------------------

   inline
   fpType
   additiveConst( ) const;
                                //
                                // additive constants arising from normalization
                                //   that are not included in value computation.
                                //
                                // sign (as set by setSign( ) or constructor) is
                                //             not included in additiveConst( ).

   // -------------------------------------------------- likelihood computations
   // ----------------- without initialization of output arguments ll, Dll, DDll
   // --------------------------------------------------------------------------

                             //              the implementations of gradient and
                             //  hessian make the assumption that drift+baseline
                             //        is linear in parameters and use functions
                             // in dbProfile which are based on this assumption.

   inline
   void
   value_ni( const fpType* const paramBuf, fpType* ll ) const;

   inline
   void
   gradient_ni( const fpType* const paramBuf, fpType* Dll ) const;

         // the parameters outside the parameter buffer, i. e., with offset < 0,
         // offset >= paramBufSize( ), are treated as constants in the objective
         // function for minimization purposes.  Consequently, the corresponding
         //                   gradients are not calculated for these parameters.
         //
         // the same convention can be used in the hessian calculation, but that
         // not  been  done  in  this  implementation, assuming that most of the
         // minimization methods  of  practical utility are gradient-based,  and
         //                               do not require the hessian explicitly.

   inline
   void
   hessian_ni( const fpType* const parameters, fpMatrix& DDll ) const;

  private:
   const fpVector*  my_signal;
   const dbProfile* my_db;
         int        my_dbOffset, my_noiseOffset;
         int        my_dbLength, my_noiseLength;
 };

// ------------------------ construct/reinitialize/incremental initialize/assign
// -----------------------------------------------------------------------------

LLnormal<false,0,0>::LLnormal( )
 : BasicObjFn( ),
   my_signal( 0 ),
   my_db( 0 ),
   my_dbOffset( 0 ),
   my_noiseOffset( 0 ),
   my_dbLength( 0 ),
   my_noiseLength( 0 )
 {
  BasicObjFn::setName( "LL normal/no activation/arma(0,0)" );
 }

LLnormal<false,0,0>::LLnormal( const LLnormal& another )
 : BasicObjFn( another.paramBufSize( ), another.sign( ), another.name( ) ),
   my_signal( another.my_signal ),
   my_db( another.my_db ),
   my_dbOffset( another.my_dbOffset ),
   my_noiseOffset( another.my_noiseOffset ),
   my_dbLength( another.my_dbLength ),
   my_noiseLength( another.my_noiseLength )
 {
 }

LLnormal<false,0,0>&
LLnormal<false,0,0>::operator=( const LLnormal<false,0,0>& rhs )
 {
  if ( this == &rhs ) return *this;

  BasicObjFn::setParamBufSize( rhs.paramBufSize( ) );
  BasicObjFn::setSign( rhs.sign( ) );
  BasicObjFn::setName( rhs.name( ) );

  my_signal = rhs.my_signal;

  my_db = rhs.my_db;

  my_dbOffset    = rhs.my_dbOffset;
  my_noiseOffset = rhs.my_noiseOffset;

  my_dbLength    = rhs.my_dbLength;
  my_noiseLength = rhs.my_noiseLength;

  return *this;
 }

LLnormal<false,0,0>::LLnormal( const fpVector*                 signal,
                               const dbProfile*                db,
                               const acProfile*                ac,
                               const ParameterManager<fpType>& pm,
                                     int                       paramBufSize,
                                     fpType                    sign,
                               const string&                   name )
 : BasicObjFn( paramBufSize, sign, name ),
   my_signal( signal ),
   my_db( db ),
   my_dbOffset( pm.blockOffset( ParamBlkNames::drift ) ),
   my_noiseOffset( pm.blockOffset( ParamBlkNames::noise ) ),
   my_dbLength( pm.blockLength( ParamBlkNames::drift ) ),
   my_noiseLength( pm.blockLength( ParamBlkNames::noise ) )
 {
 }

void
LLnormal<false,0,0>::setSignal( const fpVector* signal )
 {
  my_signal = signal;
 }

void
LLnormal<false,0,0>::setDBprofile( const dbProfile* db )
 {
  my_db = db;
 }

void
LLnormal<false,0,0>::setACprofile( const acProfile* ac )
 {
 }

void
LLnormal<false,0,0>::setParamOffset( const ParameterManager<fpType>& pm )
 {
  my_dbOffset    = pm.blockOffset( ParamBlkNames::drift );
  my_noiseOffset = pm.blockOffset( ParamBlkNames::noise );

  my_dbLength    = pm.blockLength( ParamBlkNames::drift );
  my_noiseLength = pm.blockLength( ParamBlkNames::noise );
 }

void
LLnormal<false,0,0>::reinitialize( const fpVector*                 signal,
                                   const dbProfile*                db,
                                   const acProfile*                ac,
                                   const ParameterManager<fpType>& pm,
                                         int                       paramBufSize,
                                         fpType                    sign,
                                   const string&                   name )
 {
  BasicObjFn::reinitialize( paramBufSize, sign, name );

  setSignal( signal );
  setDBprofile( db );
  setACprofile( ac );
  setParamOffset( pm );
 }

// ------------------------------------------------------------------ attributes
// -----------------------------------------------------------------------------

fpType
LLnormal<false,0,0>::additiveConst( ) const
 {
  // cerr << "LLnormal<false,0,0>::additiveConst( ) signalSize " << my_signal->size( ) << " log(2pi) " << log( 2.0 * M_PI ) << " const " << - 0.5 * my_signal->size( ) * log( 2.0 * M_PI ) << endl;
  return - 0.5 * my_signal->size( ) * log( 2.0 * M_PI );
 }

// ----------------------------------------------------- likelihood computations
// -------------------- without initialization of output arguments ll, Dll, DDll
// -----------------------------------------------------------------------------

void
LLnormal<false,0,0>::value_ni( const fpType* const paramBuf, fpType* ll ) const
 {
  const fpType* dbParams = paramBuf + my_dbOffset;
  fpType noiseParam = paramBuf[my_noiseOffset];
  int signalSize = my_signal->size( );

  fpVector residual( (*my_signal) );
  residual -= my_db->profile( dbParams );

  // cerr << "LLnormal<false,0,0>::value_ni( ) noiseParam " << noiseParam
  //      << " signalSize " << signalSize
  //      << " normSqr " << BlasDrivers::dot( residual, residual )
  //      << " *ll before update " << (*ll)
  //      << " ll before update " << ll;

  (*ll) += 0.5 * ( signalSize * log( noiseParam ) - noiseParam * BlasDrivers::dot( residual, residual ) );

  // cerr << " *ll after update " << (*ll)
  //      << " difference "
  //      << 0.5 * ( signalSize * log( noiseParam ) - noiseParam * BlasDrivers::dot( residual, residual ) )
  //      << " ll after update " << ll
  //      << endl;
 }

void
LLnormal<false,0,0>::gradient_ni( const fpType* const paramBuf,
                                        fpType*       Dll ) const
 {
  const fpType* dbParams = paramBuf + my_dbOffset;

  fpType noiseParam = paramBuf[my_noiseOffset];

  int signalSize = my_signal->size( );

  fpVector residual( (*my_signal) );
  residual -= my_db->profile( dbParams );

                                                  //                  D( noise )
                                                  //
  if ( ( my_noiseOffset >= 0 ) && ( my_noiseOffset < paramBufSize( ) ) )
   Dll[my_noiseOffset] += 0.5 * ( signalSize / noiseParam - BlasDrivers::dot( residual, residual ) );

                                                  //         D( drift+baseline )
                                                  //
  int ofsMax = min( my_dbOffset + my_dbLength, paramBufSize( ) );
  for ( int iOffset = max( 0, my_dbOffset ),
            i       = iOffset - my_dbOffset;
            iOffset < ofsMax; iOffset++, i++ )
   Dll[iOffset] += noiseParam * my_db->vDotDp( i, residual );
 }

void
LLnormal<false,0,0>::hessian_ni( const fpType* const paramBuf,
                                       fpMatrix&     DDll ) const
 {
  const fpType* dbParams = paramBuf + my_dbOffset;

  fpType noiseParam = paramBuf[my_noiseOffset];

  int signalSize = my_signal->size( );

  fpVector residual( (*my_signal) );
  residual -= my_db->profile( dbParams );

  int ofsMinDB = max( 0, my_dbOffset );
  int ofsMaxDB = min( my_dbOffset + my_dbLength, paramBufSize( ) );

  bool doNoise = ( my_noiseOffset >= 0 ) && ( my_noiseOffset < paramBufSize( ));

  fpType ddValue;

                                        //                    D2( noise, noise )
                                        //
  if ( doNoise )
   {
    DDll( my_noiseOffset, my_noiseOffset )
     -= 0.5 * signalSize / ( noiseParam * noiseParam );
   }

  for ( int iOffset = ofsMinDB,
            i       = iOffset - my_dbOffset;
            iOffset < ofsMaxDB; iOffset++, i++ )
   {
                                        //           D2( noise, drift+baseline )
                                        //
    if ( doNoise )
     {
      ddValue = my_db->vDotDp( i, residual );

      DDll( iOffset, my_noiseOffset ) += ddValue;
      DDll( my_noiseOffset, iOffset ) += ddValue;
     }

                                        //  D2( drift+baseline, drift+baseline )
                                        //                              diagonal
                                        //
    DDll( iOffset, iOffset ) -= noiseParam * my_db->DpDotDp( i, i );

                                        //  D2( drift+baseline, drift+baseline )
                                        //                          off-diagonal
                                        //
    for ( int jOffset = ofsMinDB,
              j       = jOffset - my_dbOffset;
              jOffset < iOffset; jOffset++, j++ )
     {
      ddValue = noiseParam * my_db->DpDotDp( i, j );

      DDll( iOffset, jOffset ) -= ddValue;
      DDll( jOffset, iOffset ) -= ddValue;
     }
   }
 }

#endif /* __LLn_n_0_0_H__ */
