/* ----------------------------------------------------------------------------
 *
 * class LPbaseline
 *   implementation
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <brain.model/LPbaseline>

// ------------------------------- construct/reinitialize/incremental initialize
// -----------------------------------------------------------------------------

LPbaseline::LPbaseline( )
 : BasicObjFn( ),
   my_db( 0 ),
   my_dbOffset( 0 ),
   my_scale( 0.0 ),
   my_shift( 0.0 )
 {
  BasicObjFn::setName( "LP Baseline" );
 }

LPbaseline::LPbaseline( const LPbaseline& another )
 : BasicObjFn( another.paramBufSize( ), another.sign( ), another.name( ) ),
   my_db( another.my_db ),
   my_dbOffset( another.my_dbOffset ),
   my_scale( another.my_scale ),
   my_shift( another.my_shift )
 {
 }

LPbaseline&
LPbaseline::operator=( const LPbaseline& rhs )
 {
  if ( this == &rhs ) return *this;

  BasicObjFn::setParamBufSize( rhs.paramBufSize( ) );
  BasicObjFn::setSign( rhs.sign( ) );
  BasicObjFn::setName( rhs.name( ) );

  my_db       = rhs.my_db;
  my_dbOffset = rhs.my_dbOffset;
  my_scale    = rhs.my_scale;
  my_shift    = rhs.my_shift;

  return *this;
 }

LPbaseline::LPbaseline( const dbProfile*                db,
                        const ParameterManager<fpType>& pm,
                              fpType                    scale,
                              fpType                    shift,
                              int                       paramBufSize,
                              fpType                    sign,
                        const string&                   name )
 : BasicObjFn( paramBufSize, sign, name ),
   my_db( db ),
   my_dbOffset( pm.blockOffset( ParamBlkNames::drift ) ),
   my_scale( scale ),
   my_shift( shift )
 {
 }

LPbaseline::LPbaseline( const dbProfile* db,
                              int        dbOffset,
                              fpType     scale,
                              fpType     shift,
                              int        paramBufSize,
                              fpType     sign,
                        const string&    name )
 : BasicObjFn( paramBufSize, sign, name ),
   my_db( db ),
   my_dbOffset( dbOffset ),
   my_scale( scale ),
   my_shift( shift )
 {
 }

void
LPbaseline::setDBprofile( const dbProfile* db )
 {
  my_db = db;
 }

void
LPbaseline::setParamOffset( const ParameterManager<fpType>& pm )
 {
  my_dbOffset = pm.blockOffset( ParamBlkNames::drift );
 }

void
LPbaseline::setParamOffset( int dbOffset )
 {
  my_dbOffset = dbOffset;
 }

void
LPbaseline::setConstants( fpType scale, fpType shift )
 {
  my_scale = scale;
  my_shift = shift;
 }

void
LPbaseline::reinitialize( const dbProfile*                db,
                          const ParameterManager<fpType>& pm,
                                fpType                    scale,
                                fpType                    shift,
                                int                       paramBufSize,
                                fpType                    sign,
                          const string&                   name )
 {
  BasicObjFn::reinitialize( paramBufSize, sign, name );

  setDBprofile( db );
  setParamOffset( pm );
  setConstants( scale, shift );
 }

void
LPbaseline::reinitialize( const dbProfile* db,
                                int        dbOffset,
                                fpType     scale,
                                fpType     shift,
                                int        paramBufSize,
                                fpType     sign,
                          const string&    name )
 {
  BasicObjFn::reinitialize( paramBufSize, sign, name );

  setDBprofile( db );
  setParamOffset( dbOffset );
  setConstants( scale, shift );
 }
