/* ----------------------------------------------------------------------------
 *
 * class LPdrift
 *   implementation
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <brain.model/LPdrift>

// ------------------------------- construct/reinitialize/incremental initialize
// -----------------------------------------------------------------------------

LPdrift::LPdrift( )
 : BasicObjFn( ),
   my_db( 0 ),
   my_dbOffset( 0 ),
   my_noiseOffset( 0 ),
   my_smoothingPar( 0.5 ),
   my_relWeight( 0.0 ),
   my_halfNonNullSize( 0.0 ),
   my_logNormConst( 0.0 )
 {
  BasicObjFn::setName( "LP Drift" );
 }

LPdrift::LPdrift( const LPdrift& another )
 : BasicObjFn( another.paramBufSize( ), another.sign( ), another.name( ) ),
   my_db( another.my_db ),
   my_dbOffset( another.my_dbOffset ),
   my_noiseOffset( another.my_noiseOffset ),
   my_smoothingPar( another.my_smoothingPar ),
   my_relWeight( another.my_relWeight ),
   my_halfNonNullSize( another.my_halfNonNullSize ),
   my_logNormConst( another.my_logNormConst )
 {
 }

LPdrift&
LPdrift::operator=( const LPdrift& rhs )
 {
  if ( this == &rhs ) return *this;

  BasicObjFn::setParamBufSize( rhs.paramBufSize( ) );
  BasicObjFn::setSign( rhs.sign( ) );
  BasicObjFn::setName( rhs.name( ) );

  my_db              = rhs.my_db;
  my_dbOffset        = rhs.my_dbOffset;
  my_noiseOffset     = rhs.my_noiseOffset;
  my_smoothingPar    = rhs.my_smoothingPar;
  my_relWeight       = rhs.my_relWeight;
  my_halfNonNullSize = rhs.my_halfNonNullSize;
  my_logNormConst    = rhs.my_logNormConst;

  return *this;
 }

LPdrift::LPdrift( const dbProfile*                db,
                  const ParameterManager<fpType>& pm,
                        fpType                    smoothingPar,
                        fpType                    relWeight,
                        int                       paramBufSize,
                        fpType                    sign,
                  const string&                   name )
 : BasicObjFn( paramBufSize, sign, name ),
   my_db( db ),
   my_dbOffset( pm.blockOffset( ParamBlkNames::drift ) ),
   my_noiseOffset( pm.blockOffset( ParamBlkNames::noise ) ),
   my_smoothingPar( 0.5 / smoothingPar ),
   my_relWeight( ( relWeight > 0.0 ) ? relWeight : 0.0 ),
   my_halfNonNullSize( 0.5 * ( my_db->basisSize( ) - ( ( my_relWeight == 0.0 ) ? 2 : 1 ) ) ),
   my_logNormConst( 0.0 )
 {
  setLogNormConst( );
 }

LPdrift::LPdrift( const dbProfile* db,
                        int        dbOffset,
                        int        noiseOffset,
                        fpType     smoothingPar,
                        fpType     relWeight,
                        int        paramBufSize,
                        fpType     sign,
                  const string&    name )
 : BasicObjFn( paramBufSize, sign, name ),
   my_db( db ),
   my_dbOffset( dbOffset ),
   my_noiseOffset( noiseOffset ),
   my_smoothingPar( 0.5 / smoothingPar ),
   my_relWeight( ( relWeight > 0.0 ) ? relWeight : 0.0 ),
   my_halfNonNullSize( 0.5 * ( my_db->basisSize( ) - ( ( my_relWeight == 0.0 ) ? 2 : 1 ) ) ),
   my_logNormConst( 0.0 )
 {
  setLogNormConst( );
 }

void
LPdrift::setDBprofile( const dbProfile* db )
 {
  my_db = db;
 }

void
LPdrift::setParamOffset( const ParameterManager<fpType>& pm )
 {
  my_dbOffset    = pm.blockOffset( ParamBlkNames::drift );
  my_noiseOffset = pm.blockOffset( ParamBlkNames::noise );
 }

void
LPdrift::setParamOffset( int dbOffset, int noiseOffset )
 {
  my_dbOffset    = dbOffset;
  my_noiseOffset = noiseOffset;
 }

void
LPdrift::setConstants( fpType smoothingPar, fpType relWeight )
 {
  my_smoothingPar    = 0.5 / smoothingPar;
  my_relWeight       = ( relWeight > 0.0 ) ? relWeight : 0.0;
  my_halfNonNullSize = 0.5 * ( my_db->basisSize( ) - ( ( my_relWeight == 0.0 ) ? 2 : 1 ) );

  setLogNormConst( );
 }

void
LPdrift::reinitialize( const dbProfile*                db,
                       const ParameterManager<fpType>& pm,
                             fpType                    smoothingPar,
                             fpType                    relWeight,
                             int                       paramBufSize,
                             fpType                    sign,
                       const string&                   name )
 {
  BasicObjFn::reinitialize( paramBufSize, sign, name );

  setDBprofile( db );
  setParamOffset( pm );
  setConstants( smoothingPar, relWeight );
 }

void
LPdrift::reinitialize( const dbProfile* db,
                             int        dbOffset,
                             int        noiseOffset,
                             fpType     smoothingPar,
                             fpType     relWeight,
                             int        paramBufSize,
                             fpType     sign,
                       const string&    name )
 {
  BasicObjFn::reinitialize( paramBufSize, sign, name );

  setDBprofile( db );
  setParamOffset( dbOffset, noiseOffset );
  setConstants( smoothingPar, relWeight );
 }

void
LPdrift::setLogNormConst( )
 {
  const fpType* dbParams = 0;
  fpMatrix omega( my_db->DDnormSqrDDDrift( dbParams ) );
                             //
                             // dbProfile needs this redundant argument dbParams

  if ( my_relWeight > 0.0 )
   {
    fpType p;
    for ( size_t i = 0; i < omega.numRows( ); i++ )
     {
      omega( i, i ) += ( my_relWeight * my_db->DDnormSqrDrift( dbParams, i, i ) );
      for ( size_t j = 0; j < i; j++ )
       {
        p = my_relWeight * my_db->DDnormSqrDrift( dbParams, i, j );
        omega( i, j ) += p;
        omega( j, i ) += p;
       }
     }
   }

  // cerr.setf( ios_base::scientific, ios_base::floatfield );
  // cerr.precision( 15 );

  // cerr << "LPdrift::setLogNormConst( ) omega" << endl;
  // for ( size_t j = 0; j < omega.numCols( ); j++ )
  //  for ( size_t i = 0; i < omega.numRows( ); i++ )
  //   cerr << omega[j][i] << endl;

  fpMatrix evecs; fpVector evals; FortranTypes::integer lapackInfo;
  LapackDrivers::RealSymmetric::diagonalize( omega, evals, evecs, lapackInfo, true );
  assert( lapackInfo == 0 );

  // cerr << "LPdrift::setLogNormConst( ) eigenvalues";
  // for ( size_t i = 0; i < evals.size( ); i++ )
  //  cerr << evals[i] << endl;

  size_t iMin = ( my_relWeight == 0.0 ) ? 2 : 1;
                              //
                              // ATTN omega has a null space of dimension one or
                              // two depending on whether relWeight == 0 or > 0.
                              //      The log normalization factor computed here
                              //  disregards the corresponding zero eigenvalues.
                              //         See Brain/doc/formulae.tex for details.
                              //
                              // An alternative to this would be  to discard all
                              // eigenvalues which are numerically zero w. r. t.
                              // some threshold, say 1.0e-10.
                              //
                              // lapack routines return sorted eigenvalues in an
                              // ascending order.

  my_logNormConst = 0.0;
  for ( size_t i = iMin; i < evals.size( ); i++ )
   {
    assert( fabs( evals[i] ) > 0.0 );     // fabs for - signs of numerical zeros
    my_logNormConst += log( fabs( evals[i] ) );
   }
  my_logNormConst *= 0.5;
  my_logNormConst += my_halfNonNullSize * ( log( my_smoothingPar ) - log( M_PI ) );
 }
