/* ----------------------------------------------------------------------------
 *
 * namespace ParamBlkNames
 *   implementation
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <brain.model/ParamBlkNames>

string
ParamBlkNames::paramDescr( const ParameterManager<fpType>& parManager )
 {
  char s[parManager.numParams( )];

  string d = "";

  for ( size_t b = 0; b < parManager.numBlocks( ); b++ )
   {
    if ( b > 0 ) d.append( " " );
    d.append( parManager.blockName( b ) );

    sprintf( s, "%d", parManager.blockOffset( b ) );
    d.append( "," ); d.append( s );

    sprintf( s, "%d", parManager.blockLength( b ) );
    d.append( "," ); d.append( s );
   }

  return d;
 }

string
ParamBlkNames::responseDescr( const ParameterManager<fpType>& parManager, const acProfile& activation )
 {
  char s[parManager.numParams( )];

  string d = "";

  for ( size_t a = 0; a < activation.numActive( ); a++ )
   {
    const acProfile::ConditionBellSet& cbs
     = activation.bellSet( activation.condition( a ) );

    if ( a > 0 ) d.append( " " );
    d.append( cbs.name( ) );
    d.append( "." ); d.append( cbs.tag( ) );
    d.append( "." ); d.append( cbs.bell( )->name( ) );

    sprintf( s, "%d", a );
    d.append( "," ); d.append( s ); d.append( ",1" );
   }

  return d;
 }

string
ParamBlkNames::shapeDescr( const ParameterManager<fpType>& parManager, const acProfile& activation )
 {
  char s[parManager.numParams( )];

  string d = "";

  for ( size_t a = 0; a < activation.numActive( ); a++ )
   {
    const acProfile::ConditionBellSet& cbs
     = activation.bellSet( activation.condition( a ) );

    bool printBellInfo = true;
    for ( size_t b = 0; b < a; b++ )
     printBellInfo =    printBellInfo
                     && (    cbs.tag( )
                          != activation.bellSet( activation.condition( b ) ).tag( ) );

    if ( printBellInfo )
     {
      for ( size_t j = 0; j < cbs.numShapeParams( ); j++ )
       {
        if ( ( a > 0 ) || ( j > 0 ) ) d.append( " " );
        d.append( cbs.bell( )->name( j ) );
        d.append( "." ); d.append( cbs.tag( ) );
        d.append( "." ); d.append( cbs.bell( )->name( ) );

        sprintf( s, "%d", cbs.shapeOffset( ) + j );
        d.append( "," ); d.append( s ); d.append( ",1" );
       }
     }
   }

  return d;
 }
