/* --------------------------------------------------------------------------
 *
 * class UnitModulator
 *   1-parameter identity modulator: does nothing
 *   to the rise characteristics of symmetric up ramps
 *
 * required headers
 *   see include header file UnitModulator
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __UnitModulator_H__
#define __UnitModulator_H__

cpp_using_namespace( BasicTypes );

class UnitModulator
 {
  public:
                             //
                             // basic 1-parameter modulator m( t, a ) over [0,1]
                             //
                             // m( t; a ) = t                -infty < a < +infty
                             //

   static
   inline
   string
   name( );

   static
   inline
   string
   description( );

   inline
   fpType
   operator()( fpType t, fpType a ) const;

   inline
   fpType
   Dt( fpType t, fpType a, size_t dOrder = 1 ) const;

   inline
   fpType
   Da( fpType t, fpType a, size_t dOrder = 1 ) const;

   inline
   fpType
   Dta( fpType t, fpType a, size_t dOrderT = 1, size_t dOrderA = 1 ) const;
 };

string
UnitModulator::name( )
 {
  return "UnitModulator";
 }

string
UnitModulator::description( )
 {
  return "identity modulator";
 }

fpType
UnitModulator::operator()( fpType t, fpType a ) const
 {
  return t;
 }

fpType
UnitModulator::Dt( fpType t, fpType a, size_t dOrder ) const
 {
  if ( dOrder == 0 )
   return (*this)( t, a );
  else if ( dOrder == 1 )
   return 1.0;
  else
   return 0.0;
 }

fpType
UnitModulator::Da( fpType t, fpType a, size_t dOrder ) const
 {
  if ( dOrder == 0 )
   return (*this)( t, a );
  else
   return 0.0;
 }

fpType
UnitModulator::Dta( fpType t, fpType a, size_t dOrderT, size_t dOrderA ) const
 {
  if ( dOrderA == 0 )
   return Dt( t, a, dOrderT );
  else if ( dOrderT == 0 )
   return Da( t, a, dOrderT );
  else
   return 0.0;
 }

#endif /* __UnitModulator_H__ */
