/* ----------------------------------------------------------------------------
 *
 * class acProfile
 *   implementation
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <brain.model/acProfile>

// ----------------------------------------- ConditionBellSet: construct, reinit
// -----------------------------------------------------------------------------

acProfile::ConditionBellSet::ConditionBellSet( const string& name,
                                               const string& tag )
 : my_name( name ),
   my_tag( tag ),
   my_shapeOffset( ),
   my_bell( ),
   my_timeGrid( ),
   my_epochBegin( ),
   my_epochLength( ),
   my_beginIndex( )
 {
 }

acProfile::ConditionBellSet::ConditionBellSet( const ConditionBellSet& another )
 : my_name( another.my_name ),
   my_tag( another.my_tag ),
   my_shapeOffset( another.my_shapeOffset ),
   my_bell( another.my_bell ),
   my_timeGrid( another.my_timeGrid ),
   my_epochBegin( another.my_epochBegin ),
   my_epochLength( another.my_epochLength ),
   my_beginIndex( another.my_beginIndex )
 {
 }

acProfile::ConditionBellSet&
acProfile::ConditionBellSet::operator=( const ConditionBellSet& rhs )
 {
  if ( this == &rhs ) return *this;

  my_name        = rhs.my_name;
  my_tag         = rhs.my_tag;
  my_shapeOffset = rhs.my_shapeOffset;
  my_bell        = rhs.my_bell;
  my_timeGrid    = rhs.my_timeGrid;

  my_epochBegin.resize( rhs.my_epochBegin.size( ) );
  my_epochLength.resize( rhs.my_epochLength.size( ) );
  my_beginIndex.resize( rhs.my_beginIndex.size( ) );

  my_epochBegin  = rhs.my_epochBegin;
  my_epochLength = rhs.my_epochLength;
  my_beginIndex  = rhs.my_beginIndex;

  return *this;
 }

acProfile::ConditionBellSet::ConditionBellSet( )
 : my_name( ),
   my_tag( ),
   my_shapeOffset( ),
   my_bell( ),
   my_timeGrid( ),
   my_epochBegin( ),
   my_epochLength( ),
   my_beginIndex( )
 {
 }

void
acProfile::ConditionBellSet::setName( const string& name )
 {
  my_name = name;
 }

void
acProfile::ConditionBellSet::setTag( const string& tag )
 {
  my_tag = tag;
 }

void
acProfile::ConditionBellSet::setShapeOffset( int shapeOffset )
 {
  my_shapeOffset = shapeOffset;
 }

void
acProfile::ConditionBellSet::setBell( const BasicBell* const theBell )
 {
  my_bell = theBell;
 }

void
acProfile::ConditionBellSet::setEpochData( const fpVector& epochBegin,
                                           const fpVector& epochLength )
 {
  assert( epochBegin.size( ) == epochLength.size( ) );

  my_epochBegin.resize( epochBegin.size( ) );
  my_epochLength.resize( epochLength.size( ) );

  my_epochBegin  = epochBegin;
  my_epochLength = epochLength;
 }

void
acProfile::ConditionBellSet::setTimeGrid( const fpVector* timeGrid, bool init )
 {
  my_timeGrid = timeGrid;
  initialize( init );
 }

void
acProfile::ConditionBellSet::initialize( bool init )
 {
  if ( !init ) return;

  my_beginIndex.resize( numEpochs( ) );

  fpType lowerSupport;
  size_t i, iMax = timeGridSize( ) - 1;

  for ( size_t e = 0; e < numEpochs( ); e++ )
   {
    lowerSupport = my_epochBegin[e];
      //
      // this assumes that the lower support of a bell corresponding to an epoch
      // is the epoch beginning itself,  and does not depend on shape parameters

    i = 0;
    while ( ( i < iMax ) && ( (*my_timeGrid)[i] < lowerSupport ) ) i++;

    my_beginIndex[e] = i;
   }
 }

// ------------------------------- construct/reinitialize/incremental initialize
// -----------------------------------------------------------------------------

acProfile::acProfile( const acProfile& another )
 : my_db( another.my_db ),
   my_sm( another.my_sm ),
   my_cl( another.my_cl )
   // my_sParam2cTag( another.my_sParam2cTag )
 {
 }

acProfile&
acProfile::operator=( const acProfile& rhs )
 {
  if ( &rhs == this ) return *this;

  my_db = rhs.my_db;
  my_sm = rhs.my_sm;

  // my_cl.resize( rhs.my_cl.size( ) );
  // my_sParam2cTag.resize( rhs.my_sParam2cTag.size( ) );
                                           //
                                           // ATTN resize unnecessary for vector

  my_cl = rhs.my_cl;
  // my_sParam2cTag = rhs.my_sParam2cTag;

  return *this;
 }

acProfile::acProfile( )
 : my_db( 0 ),
   my_sm( 0 ),
   my_cl( )
   // my_sParam2cTag( )
 {
 }

void
acProfile::setDBprofile( const dbProfile* db )
 {
  my_db = db;
 }

void
acProfile::setSubmodel( const Submodel* sm )
 {
  my_sm = sm;
 }

void
acProfile::setTimeGrid( const fpVector* timeGrid, bool init )
 {
  for ( size_t c = 0; c < numConditions( ); c++ )
   my_cl[c].setTimeGrid( timeGrid, init );
 }

void
acProfile::addCondition( const string&    name,
                         const fpVector&  epochBegin,
                         const fpVector&  epochLength,
                         const string&    tag,
                         const BasicBell* bell,
                               bool       init )
 {
  size_t c = 0; bool found = false;
  for ( ; ( c < numConditions( ) ) && !found; c++ )
   found = ( name == my_cl[c].name( ) );
 
  if ( found )
   {
    c--;
    my_cl[c].setTag( tag );
    my_cl[c].setEpochData( epochBegin, epochLength );
   }
  else
   {
    my_cl.push_back( ConditionBellSet( name, tag ) );
    my_cl[numConditions( ) - 1].setEpochData( epochBegin, epochLength );
   }
   
  setBell( tag, bell, init );
 }

void
acProfile::setBell( const string& tag, const BasicBell* bell, bool init )
 {
  for ( size_t c = 0; c < numConditions( ); c++ )
   if ( my_cl[c].tag( ) == tag ) my_cl[c].setBell( bell );

  initialize( init );
 }

void
acProfile::initialize( bool init )
 {
  if ( !init ) return;

  for ( size_t c = 0; c < numConditions( ); c++ )
   {
    my_cl[c].initialize( true );
    my_cl[c].setShapeOffset( -1 );
   }
                                      //
                                      //  conditions not in the present submodel
                                      // are assigned a meanlingless offset = -1

  // my_sParam2cTag.resize( numShapeParams( ) );

  size_t c, a, n, ofs = 0;
  bool found;

  for ( size_t active = 0; active < numActive( ); active++ )
   {
    c = condition( active );
    const string& t = my_cl[c].tag( );

    found = false;
    for ( a = 0; ( a < active ) && !found; a++ )
     found = found || ( t == my_cl[condition( a )].tag( ) );
    
    if ( found )
     my_cl[c].setShapeOffset( my_cl[condition( a - 1 )].shapeOffset( ) );
    else
     {
      n = my_cl[c].numShapeParams( );
      // for ( size_t s = ofs; s < ofs + n; s++ ) my_sParam2cTag[s] = t;
      my_cl[c].setShapeOffset( ofs );
      ofs += n;
     }
   }

  // cerr << "acProfile::sParam2cTag" << endl;
  // for ( size_t i = 0; i < my_sParam2cTag.size( ); i++ )
  //  cerr << "i " << i << " tag " << my_sParam2cTag[i] << endl;
 }
