/* ----------------------------------------------------------------------------
 *
 * class InSpec
 *   vmp input specification handler
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <brain.vmp/InSpec>

InSpec::InSpec( const string& data                  /* = "data" */,
                const string& data_chunk            /* = "data.chunk" */,
                const string& data_tdim             /* = "data.dimension.t" */,
                const string& data_zdim             /* = "data.dimension.z" */,
                const string& data_descr            /* = "data.description" */,
                const string& data_missing          /* = "data.missing" */,

                const string& analysis_vange_prefix /* = "analysis.voxel.range." */,
                const string& analysis_descr        /* = "analysis.description" */,

                const string& acq_shots             /* = "acquisition.shots" */,
                const string& acq_tr                /* = "acquisition.tr" */,
                const string& acq_iai               /* = "acquisition.iai" */,
                const string& acq_slice_order       /* = "acquisition.slice.order" */,
                const string& acq_slice_offsets     /* = "acquisition.slice.offsets" */,

                const string& design_conditions     /* = "design.conditions" */,
                const string& design_tags           /* = "design.condition.tags" */,
                const string& design_bells          /* = "design.condition.bells" */,
                const string& design_prefix_ebeg    /* = "design.epoch.begin." */,
                const string& design_prefix_elen    /* = "design.epoch.length." */,

                const string& driftbasis_order      /* = "drift.basis.order" */,
                const string& driftbasis_knots      /* = "drift.basis.knots" */,
                const string& driftbasis_knotcount  /* = "drift.basis.knot.count" */,
                const string& driftbasis_lep        /* = "drift.basis.endknot.left" */,
                const string& driftbasis_rep        /* = "drift.basis.endknot.right" */,
                const string& driftbasis_relc       /* = "drift.basis.relc" */,
                const string& driftbasis_size       /* = "drift.basis.size" */,

                const string& submodels             /* = "submodels" */,
                const string& submodels_list        /* = "submodels.list" */,

                const string& ll_ar                 /* = "ll.arma.ar" */,
                const string& ll_ma                 /* = "ll.arma.ma" */,

                const string& lp_b_scale            /* = "lp.baseline.scale" */,
                const string& lp_b_shift            /* = "lp.baseline.shift" */,

                const string& lp_d_penalty          /* = "lp.drift.norm.penalty" */,
                const string& lp_d_smooth_df        /* = "lp.drift.smooth.df" */,
                const string& lp_d_smooth_fn        /* = "lp.drift.smooth.choice" */,
                const string& lp_d_smooth_it        /* = "lp.drift.smooth.iterations" */,
                const string& lp_d_smooth_tol       /* = "lp.drift.smooth.tolerance" */,

                const string& lp_prefix_n           /* = "lp.noise" */,
                const string& lp_prefix_r           /* = "lp.response." */,
                const string& lp_prefix_s           /* = "lp.shape." */,
                const string& lp_suffix_a_nrs       /* = ".a" */,
                const string& lp_suffix_b_nrs       /* = ".b" */,

                const string& param_prefix_scale    /* = "params.scale." */,
                const string& param_prefix_bound    /* = "params.bounds." */,
                const string& param_suffix_drift    /* = "drift" */,
                const string& param_suffix_noise    /* = "noise" */,
                const string& param_suffix_response /* = "response" */,
                const string& param_suffix_shape    /* = "shape" */,

                const string& min_corrections       /* = "minimizer.corrections" */,
                const string& min_stop_opts         /* = "minimizer.stop.opts" */,
                const string& min_restart_opts      /* = "minimizer.restart.opts" */,
                const string& min_verbosity         /* = "minimizer.verbosity" */,

                const string& init_prefix           /* = "init." */,
                const string& init_scheme           /* = "init.scheme" */,
                const string& init_iterations       /* = "init.iterations" */,

                const string& output_file           /* = "output.file" */,
                const string& output_mode           /* = "output.mode" */,
                const string& output_on_stdout      /* = "output.on.stdout" */,
                const string& output_on_pghmri      /* = "output.on.pghmri" */,
                const string& output_products       /* = "output.products" */,
                const string& product_prefix_type   /* = "" */,
                const string& product_prefix_dims   /* = "" */,
                const string& product_prefix_endian /* = "" */,
                const string& product_prefix_diags  /* = "" */,
                const string& product_suffix_type   /* = ".datatype" */,
                const string& product_suffix_dims   /* = ".leaddims" */,
                const string& product_suffix_endian /* = ".little_endian" */,
                const string& product_suffix_diags  /* = ".diagnostics" */,
                const string& prior_probability_suffix /* = ".prior.probability" */ )
 : my_data( data ), my_data_chunk( data_chunk ), my_data_tdim( data_tdim ), my_data_zdim( data_zdim ),
   my_data_descr( data_descr ), my_data_missing( data_missing ),

   my_analysis_vange_prefix( analysis_vange_prefix ), my_analysis_descr( analysis_descr ),

   my_acq_shots( acq_shots ), my_acq_tr( acq_tr ), my_acq_iai( acq_iai ),
   my_acq_slice_order( acq_slice_order ), my_acq_slice_offsets( acq_slice_offsets ),

   my_design_conditions( design_conditions ),
   my_design_tags( design_tags ), my_design_bells( design_bells ),
   my_design_prefix_ebeg( design_prefix_ebeg ), my_design_prefix_elen( design_prefix_elen ),

   my_driftbasis_order( driftbasis_order ),
   my_driftbasis_knots( driftbasis_knots ), my_driftbasis_knotcount( driftbasis_knotcount ),
   my_driftbasis_lep( driftbasis_lep ), my_driftbasis_rep( driftbasis_rep ),
   my_driftbasis_relc( driftbasis_relc ), my_driftbasis_size( driftbasis_size ),

   my_submodels( submodels ), my_submodels_list( submodels_list ),

   my_ll_ar( ll_ar ), my_ll_ma( ll_ma ),

   my_lp_b_scale( lp_b_scale ), my_lp_b_shift( lp_b_shift ),

   my_lp_d_penalty( lp_d_penalty ),
   my_lp_d_smooth_df(lp_d_smooth_df ), my_lp_d_smooth_fn(lp_d_smooth_fn ),
   my_lp_d_smooth_it(lp_d_smooth_it ), my_lp_d_smooth_tol(lp_d_smooth_tol ),

   my_lp_prefix_n( lp_prefix_n ), my_lp_prefix_r( lp_prefix_r ), my_lp_prefix_s( lp_prefix_s ),
   my_lp_suffix_a_nrs( lp_suffix_a_nrs ), my_lp_suffix_b_nrs( lp_suffix_b_nrs ),

   my_param_prefix_scale( param_prefix_scale ), my_param_prefix_bound( param_prefix_bound ),
   my_param_suffix_drift( param_suffix_drift ), my_param_suffix_noise( param_suffix_noise ),
   my_param_suffix_response( param_suffix_response ), my_param_suffix_shape( param_suffix_shape ),

   my_min_corrections( min_corrections ), my_min_stop_opts( min_stop_opts ),
   my_min_restart_opts( min_restart_opts ), my_min_verbosity( min_verbosity ),

   my_init_prefix( init_prefix ), my_init_scheme( init_scheme ), my_init_iterations( init_iterations ),

   my_output_file( output_file ), my_output_mode( output_mode ),
   my_output_on_stdout( output_on_stdout ), my_output_on_pghmri( output_on_pghmri ),
   my_output_products( output_products ),
   my_product_prefix_type( product_prefix_type ),
   my_product_prefix_dims( product_prefix_dims ),
   my_product_prefix_endian( product_prefix_endian ),
   my_product_prefix_diags( product_prefix_diags ),
   my_product_suffix_type( product_suffix_type ),
   my_product_suffix_dims( product_suffix_dims ),
   my_product_suffix_endian( product_suffix_endian ),
   my_product_suffix_diags( product_suffix_diags ),
   my_prior_probability_suffix( prior_probability_suffix )
 {
 }

void
InSpec::getSpecs_param( const PghMRIhandler& input, Settings& settings )
 {
  assert( input.isOpen( ) );

  string keyD = my_param_prefix_scale + my_param_suffix_drift;
  string keyN = my_param_prefix_scale + my_param_suffix_noise;
  string keyR = my_param_prefix_scale + my_param_suffix_response;
  string keyS = my_param_prefix_scale + my_param_suffix_shape;

  if ( input.hasKey( keyD ) ) settings.params.scales.drift    = input.getFloat( keyD );
  if ( input.hasKey( keyN ) ) settings.params.scales.noise    = input.getFloat( keyN );
  if ( input.hasKey( keyR ) ) settings.params.scales.response = input.getFloat( keyR );
  if ( input.hasKey( keyS ) ) settings.params.scales.shape    = input.getFloat( keyS );

  keyD = my_param_prefix_bound + my_param_suffix_drift;
  keyN = my_param_prefix_bound + my_param_suffix_noise;
  keyR = my_param_prefix_bound + my_param_suffix_response;
  keyS = my_param_prefix_bound + my_param_suffix_shape;

  vector<string> tokens;

  if ( input.hasKey( keyD ) )
   {
    MiscFns::getTokens( input.getString( keyD ), tokens );

    settings.params.types.drift = tokens[0];

    if ( settings.params.types.drift == "lower" )
     settings.params.bounds.drift.first = atof( tokens[1].c_str( ) );
    else if ( settings.params.types.drift == "upper" )
     settings.params.bounds.drift.second = atof( tokens[1].c_str( ) );
    else if ( settings.params.types.drift == "both" )
     {
      settings.params.bounds.drift.first = atof( tokens[1].c_str( ) );
      settings.params.bounds.drift.second = atof( tokens[2].c_str( ) );
     }
   }

  if ( input.hasKey( keyN ) )
   {
    MiscFns::getTokens( input.getString( keyN ), tokens );

    settings.params.types.noise = tokens[0];

    if ( settings.params.types.noise == "lower" )
     settings.params.bounds.noise.first = atof( tokens[1].c_str( ) );
    else if ( settings.params.types.noise == "upper" )
     settings.params.bounds.noise.second = atof( tokens[1].c_str( ) );
    else if ( settings.params.types.noise == "both" )
     {
      settings.params.bounds.noise.first = atof( tokens[1].c_str( ) );
      settings.params.bounds.noise.second = atof( tokens[2].c_str( ) );
     }
   }

  if ( input.hasKey( keyR ) )
   {
    MiscFns::getTokens( input.getString( keyR ), tokens );

    settings.params.types.response = tokens[0];

    if ( settings.params.types.response == "lower" )
     settings.params.bounds.response.first = atof( tokens[1].c_str( ) );
    else if ( settings.params.types.response == "upper" )
     settings.params.bounds.response.second = atof( tokens[1].c_str( ) );
    else if ( settings.params.types.response == "both" )
     {
      settings.params.bounds.response.first = atof( tokens[1].c_str( ) );
      settings.params.bounds.response.second = atof( tokens[2].c_str( ) );
     }
   }

  if ( input.hasKey( keyS ) )
   {
    MiscFns::getTokens( input.getString( keyS ), tokens );

    settings.params.types.shape = tokens[0];

    if ( settings.params.types.shape == "lower" )
     settings.params.bounds.shape.first = atof( tokens[1].c_str( ) );
    if ( settings.params.types.shape == "upper" )
     settings.params.bounds.shape.second = atof( tokens[1].c_str( ) );
    if ( settings.params.types.shape == "both" )
     {
      settings.params.bounds.shape.first = atof( tokens[1].c_str( ) );
      settings.params.bounds.shape.second = atof( tokens[2].c_str( ) );
     }
   }
 }

void
InSpec::getSpecs_minimizer( const PghMRIhandler& input, Settings& settings )
 {
  assert( input.isOpen( ) );

  getSpecs_minimizer( input, settings.minimizer, "" );
 }

void
InSpec::getSpecs_initializer( const PghMRIhandler& input, Settings& settings )
 {
  assert( input.isOpen( ) );

  if ( input.hasKey( my_init_scheme ) )
   settings.initializer.scheme = input.getInt( my_init_scheme );

  if ( input.hasKey( my_init_iterations ) )
   settings.initializer.maxIt = input.getInt( my_init_iterations );

  getSpecs_minimizer( input, settings.initializer.minimizer, my_init_prefix );
 }

void
InSpec::getSpecs_misc( const PghMRIhandler& input, Settings& settings )
 {
  assert( input.isOpen( ) );

  if ( input.hasKey( my_ll_ar ) ) settings.arma.numAR = input.getInt( my_ll_ar );
 }

void
InSpec::getSpecs_output( const PghMRIhandler& input, Settings& settings )
 {
  assert( input.isOpen( ) );

  if ( input.hasKey( my_output_file ) )
   settings.output.fileName = input.getString( my_output_file );

  if ( input.hasKey( my_output_mode ) )
   settings.output.openMode = PghMRIhandler::name2mode( input.getString( my_output_mode ),
                                                        PghMRIhandler::mriWrite );

  if ( input.hasKey( my_output_on_stdout ) )
   {
    string v = input.getString( my_output_on_stdout );
    settings.output.stdout =   ( ( v == "on" ) || ( v == "true" ) )
                             ? true
                             : ( ( ( v == "off" ) || ( v == "false" ) ) ? false : settings.output.stdout );
   }

  if ( input.hasKey( my_output_on_pghmri ) )
   {
    string v = input.getString( my_output_on_pghmri );
    settings.output.pghmri =   ( ( v == "on" ) || ( v == "true" ) )
                             ? true
                             : ( ( ( v == "off" ) || ( v == "false" ) ) ? false : settings.output.pghmri );
   }
 }

void
InSpec::getSpecs_products( const PghMRIhandler& input, Settings& settings, const SubmodelTable& submodelTable )
 {
  assert( input.isOpen( ) );

  int de = PghMRIhandler::endianness( );
  assert( ( de == 1 ) || ( de == 0 ) );

  bool defaultProducts = true, allProducts = false;

  if ( input.hasKey( my_output_products ) )
   {
    vector<string> p;
    MiscFns::getTokens( input.getString( my_output_products ), p );

    // cerr << "(debug) InSpec::getSpecs_products( ) products requested:";
    // for ( size_t i = 0; i < p.size( ); i++ ) cerr << " " << p[i]; cerr << endl;

    defaultProducts = ( find( p.begin( ), p.end( ), "default.products" ) != p.end( ) );
    allProducts     = ( find( p.begin( ), p.end( ), "all.products"     ) != p.end( ) );

    ProductRegistry::expand( settings.output.products, p );
   }

  defaultProducts = defaultProducts || ( settings.output.products.size( ) == 0 );

  // cerr << "(debug) InSpec::getSpecs_products( ) defaultProducts " << defaultProducts << " allProducts " << allProducts << endl;

  if ( allProducts )
   ProductRegistry::expand( settings.output.products, vector<string>( 1, "all.products" ) );
  else if ( defaultProducts )
   ProductRegistry::expand( settings.output.products );

  settings.output.bfRequested =    defaultProducts || allProducts
                                || ( find( settings.output.products.begin( ),
                                           settings.output.products.end( ),
                                           "log.bayes.factor" ) != settings.output.products.end( ) );
  settings.output.mpRequested =    defaultProducts || allProducts
                                || ( find( settings.output.products.begin( ),
                                           settings.output.products.end( ),
                                           "model.probability" ) != settings.output.products.end( ) );
  settings.output.liRequested =    allProducts
                                || ( find( settings.output.products.begin( ),
                                           settings.output.products.end( ),
                                           "log.integral" ) != settings.output.products.end( ) );

  if ( ( settings.output.mpRequested || settings.output.bfRequested ) && !settings.output.liRequested )
   settings.output.products.push_back( "log.integral" );

  if ( settings.output.mpRequested )
   remove( settings.output.products.begin( ), settings.output.products.end( ), "model.probability" );

  if ( settings.output.bfRequested )
   remove( settings.output.products.begin( ), settings.output.products.end( ), "log.bayes.factor" );

  // cerr << "(debug) InSpec::getSpecs_products( ) expanded list:";
  // for ( size_t i = 0; i < settings.output.products.size( ); i++ )
  //  cerr << " " << settings.output.products[i];
  // cerr << endl << "(debug) InSpec::getSpecs_products( ) defaultProducts " << defaultProducts << " allProducts " << allProducts << " mp requested " << settings.output.mpRequested << " bf requested " << settings.output.bfRequested << " li requested " << settings.output.liRequested << endl;

  settings.output.types.resize( settings.output.products.size( ) );
  settings.output.littleE.resize( settings.output.products.size( ) );
  settings.output.leadDims.resize( settings.output.products.size( ) );
  settings.output.diagnostics.resize( settings.output.products.size( ) );

  if ( defaultProducts || allProducts )
   {
    size_t n = find( settings.output.products.begin( ),
                     settings.output.products.end( ), "covariances" ) - settings.output.products.begin( );
    // cerr << "(debug) InSpec::getSpecs_products( ) n " << n << endl;
    settings.output.diagnostics[n] = true;
   }

  string key;
  for ( size_t p = 0; p < settings.output.products.size( ); p++ )
   {
    key = my_product_prefix_type + settings.output.products[p] + my_product_suffix_type;
    settings.output.types[p] = input.hasKey( key ) ? PghMRIhandler::name2type( input.getString( key ) ) : -1;

    key = my_product_prefix_endian + settings.output.products[p] + my_product_suffix_endian;
    settings.output.littleE[p] = input.hasKey( key ) ? input.getInt( key ) : de;

    key = my_product_prefix_dims + settings.output.products[p] + my_product_suffix_dims;
    settings.output.leadDims[p] = input.hasKey( key ) ? input.getString( key ) : "";

    key = my_product_prefix_diags + settings.output.products[p] + my_product_suffix_diags;
    if ( input.hasKey( key ) )
     settings.output.diagnostics[p] = ( input.getString( key ) == "on" ) ? true : false;

    // cerr << "(debug) product " << settings.output.products[p] << " key " << key << " input.hasKey( key ) " << input.hasKey( key ) << " value " << settings.output.diagnostics[p] << endl;
   }

  // ATTN chunk name can be differently set in ModelProbability::configure( )
  //
  key = my_product_prefix_type + "model.probability" + my_product_suffix_type;
  settings.output.mpType = input.hasKey( key ) ? input.getString( key ) : "mriFloat";

  // ATTN chunk name can be differently set in ModelProbability::configure( )
  //
  key = my_product_prefix_endian + "model.probability" + my_product_suffix_endian;
  settings.output.mpLittleE = input.hasKey( key ) ? input.getInt( key ) : de;

  // ATTN chunk name can be differently set in ModelProbability::configure( )
  //
  key = my_product_prefix_type + "log.bayes.factor" + my_product_suffix_type;
  settings.output.bfType = input.hasKey( key ) ? input.getString( key ) : "mriFloat";

  // ATTN chunk name can be differently set in ModelProbability::configure( )
  //
  key = my_product_prefix_endian + "log.bayes.factor" + my_product_suffix_endian;
  settings.output.bfLittleE = input.hasKey( key ) ? input.getInt( key ) : de;

  settings.output.priorProbabilities.resize( submodelTable.size( ) );

  bool hasAll = true;
  for ( SubmodelTable::const_iterator si  = submodelTable.begin( );
                                      si != submodelTable.end( ); ++si )
   hasAll = hasAll && input.hasKey( submodelTable.key( si ) + my_prior_probability_suffix );

  if ( hasAll )
   {
    size_t i = 0; fpType p = 0.0, norm = 0.0;
    for ( SubmodelTable::const_iterator si  = submodelTable.begin( );
                                        si != submodelTable.end( ); ++si, i++ )
     {
      p = input.getFloat( submodelTable.key( si ) + my_prior_probability_suffix );
      assert( p >= 0 ); settings.output.priorProbabilities[i] = p; norm += p;
     }

    if ( norm != 1.0 ) settings.output.priorProbabilities /= norm;
   }
  else
   settings.output.priorProbabilities = 1.0 / submodelTable.size( );
 }

void
InSpec::getSpecs_minimizer( const PghMRIhandler& input, Settings::Minimizer& minimizer, const string& prefix )
 {
  assert( input.isOpen( ) );

  if ( input.hasKey( prefix + my_min_corrections ) )
   minimizer.maxCorrections = input.getInt( prefix + my_min_corrections );

  if ( input.hasKey( prefix + my_min_stop_opts ) )
   {
    vector<string> tokens;
    MiscFns::getTokens( input.getString( prefix + my_min_stop_opts ), tokens );

    if ( tokens.size( ) > 0 )
     minimizer.maxIterations = atoi( tokens[0].c_str( ) );

    if ( tokens.size( ) > 1 )
     minimizer.maxEvaluations = atoi( tokens[1].c_str( ) );

    if ( tokens.size( ) > 2 )
     minimizer.maxPrGradNorm = atof( tokens[2].c_str( ) );

    if ( tokens.size( ) > 3 )
     minimizer.minRelReduceF = atof( tokens[3].c_str( ) ) / DBL_EPSILON;
   }

  if ( input.hasKey( prefix + my_min_restart_opts ) )
   {
    vector<string> ps;
    MiscFns::getTokens( input.getString( prefix + my_min_restart_opts ), ps );

    if ( ps.size( ) > 0 )
     minimizer.maxRestarts = atoi( ps[0].c_str( ) );

    if ( ps.size( ) > 1 )
     minimizer.checkHdet =   ( ( ps[1] == "on" ) || ( ps[1] == "true" ) )
                           ? true
                           : (   ( ( ps[1] == "off" ) || ( ps[1] == "false" ) )
                               ? false
                               : minimizer.checkHdet );

    if ( ps.size( ) > 2 )
     minimizer.stopAtPlusDefH =   ( ( ps[2] == "on" ) || ( ps[2] == "true" ) )
                                ? true
                                : (   ( ( ps[2] == "off" ) || ( ps[2] == "false" ) )
                                    ? false
                                    : minimizer.stopAtPlusDefH );

    if ( ps.size( ) > 3 )
     minimizer.mrrfReduction = atof( ps[3].c_str( ) );
   }

  if ( input.hasKey( prefix + my_min_verbosity ) )
   minimizer.verbosity = input.getInt( prefix + my_min_verbosity );
 }

void
InSpec::initData( const PghMRIhandler&             input,
                        Settings&                  settings,
		        TCrwPghMRI&                tcReader,
                        MissingInfo&               missingInfo,
                        DataNavigator<TCrwPghMRI>& dataNavigator )
 {
  assert( input.isOpen( ) );

  assert( input.hasKey( my_data ) );

  settings.imageData = input.getString( my_data );

  if ( input.hasKey( my_data_descr ) ) settings.dataDescription = input.getString( my_data_descr );
  if ( input.hasKey( my_data_chunk ) ) settings.imageChunk      = input.getString( my_data_chunk );
  if ( input.hasKey( my_data_tdim  ) ) settings.t               = input.getString( my_data_tdim )[0];
  if ( input.hasKey( my_data_zdim  ) ) settings.z               = input.getString( my_data_zdim )[0];

  if ( input.hasKey( my_data_missing ) )
   settings.missingSpecs = input.getString( my_data_missing );
  else
   settings.missingSpecs =   "file "     + settings.imageData
                           + " tname "   + settings.t
                           + " zname "   + settings.z
                           + " data "    + settings.imageChunk
                           + " missing " + settings.missingChunk
                           + " end";

  missingInfo.reinitialize( settings.missingSpecs, false );
  tcReader.readOnly( settings.imageData, settings.imageChunk, settings.t, false );

  if ( input.hasKey( my_analysis_descr  ) ) settings.runDescription = input.getString( my_analysis_descr );

  string key = my_analysis_vange_prefix + "?";
  for ( size_t d = 0; d < tcReader.numDims( ); d++ )
   {
    if ( d != tcReader.tdim( ) )
     {
      key[key.size( ) - 1] = tcReader.names( )[d][0];
      if ( input.hasKey( key ) )
       {
        pair<int,int> rd( tcReader.rangeLower( )[d], tcReader.rangeUpper( )[d] ), range;
        MiscFns::getRange( input.getString( key ), rd, range );
        tcReader.setRange( d, range.first, range.second );
       }
     }
   }

  // cerr << "tcReader" << endl << endl;
  // cerr << "numDims( ) " << tcReader.numDims( ) << " size( ) " << tcReader.size( ) << " anyMove( ) " << tcReader.anyMove( ) << endl;

  // cerr << endl;
  // for ( size_t d = 0; d < tcReader.numDims( ); d++ )
  //  cerr << "dimension "         << d
  //       << " name "             << tcReader.names( )[d]
  //       << " size "             << tcReader.sizes( )[d]
  //       << " order "            << tcReader.order( )[d]
  //       << " index range "      << tcReader.indexLower( )[d] << ":" << tcReader.indexUpper( )[d]
  //       << " navigation range " << tcReader.rangeLower( )[d] << ":" << tcReader.rangeUpper( )[d]
  //       << endl;

  dataNavigator.reinitialize( &tcReader, &missingInfo, string( 1, settings.z ) );

  // cerr << endl << "DataNavigator" << endl << endl;
  // cerr << "numDims( ) " << dataNavigator.numDims( ) << " size( ) " << dataNavigator.size( ) << " anyMove( ) " << dataNavigator.anyMove( ) << " sliceDim " << dataNavigator.sliceDim( ) << endl;

  // cerr << endl;
  // for ( size_t d = 0; d < dataNavigator.numDims( ); d++ )
  //  cerr << "dimension "         << d
  //       << " name "             << dataNavigator.names( )[d]
  //       << " size "             << dataNavigator.sizes( )[d]
  //       << " order "            << dataNavigator.order( )[d]
  //       << " index range "      << dataNavigator.indexLower( )[d] << ":" << dataNavigator.indexUpper( )[d]
  //       << " navigation range " << dataNavigator.rangeLower( )[d] << ":" << dataNavigator.rangeUpper( )[d]
  //       << endl;

  int shots = 1;
  if ( input.hasKey( my_acq_shots ) ) shots = input.getInt( my_acq_shots );
  shots = ( shots > 0 ) ? shots : 0;

  fpType tr = 1;
  if ( input.hasKey( my_acq_tr ) ) tr = input.getFloat( my_acq_tr );
  tr = ( tr > 0 ) ? tr : 0;

  fpType iai = 1;
  if ( input.hasKey( my_acq_iai ) ) iai = input.getFloat( my_acq_iai );
  iai = ( iai > 0 ) ? iai : 0;

  uintVector sliceOrder( missingInfo.numSlices( ) );
  if ( input.hasKey( my_acq_slice_order ) )
   {
    vector<string> tokens;
    MiscFns::getTokens( input.getString( my_acq_slice_order ), tokens );

    vector<string>::const_iterator t = tokens.begin( );

    for ( size_t i = 0; ( t != tokens.end( ) ) && ( i < sliceOrder.size( ) ); ++i, ++t )
     {
      int s = atoi( t->c_str( ) );
      if ( s >= 0 ) sliceOrder[i] = s;
     }
   }

  settings.acquisition.reinitialize( missingInfo.numSlices( ), missingInfo.numInstants( ), tr, iai, shots, sliceOrder );
  settings.timegrids.reinitialize( settings.acquisition, missingInfo );
 }

void
InSpec::initDesign( const PghMRIhandler& input,
                    const dbProfile&     drift,
                          Settings&      settings,
                          acProfile&     activation,
                          SubmodelTable& submodelTable,
                          bool           mustIncludeNullSubmodel )
 {
  assert( input.isOpen( ) );

  activation.setDBprofile( &drift );

  assert( input.hasKey( my_design_conditions ) );

  vector<string> conditions;
  MiscFns::getTokens( input.getString( my_design_conditions ), conditions );

  assert( conditions.size( ) > 0 );

  vector<string> conditionTags( conditions );

  if ( input.hasKey( my_design_tags ) );
   {
    MiscFns::getTokens( input.getString( my_design_tags ), conditionTags );
    assert( conditions.size( ) == conditionTags.size( ) );
   }

  vector<string> conditionBells( conditions.size( ), "v4" );

  if ( input.hasKey( my_design_bells ) );
   {
    MiscFns::getTokens( input.getString( my_design_bells ), conditionBells );
    assert( conditions.size( ) == conditionBells.size( ) );
   }

  vector<string> eBegin, eLength;
  fpVector epochBegin, epochLength;

  for ( size_t c = 0; c < conditions.size( ); c++ )
   {
    assert( input.hasKey( my_design_prefix_ebeg + conditions[c] ) );
    assert( input.hasKey( my_design_prefix_elen + conditions[c] ) );

    MiscFns::getTokens( input.getString( my_design_prefix_ebeg + conditions[c] ), eBegin  );
    MiscFns::getTokens( input.getString( my_design_prefix_elen + conditions[c] ), eLength );

    assert( eBegin.size( ) > 0 );
    assert( eLength.size( ) == eBegin.size( ) );

    epochBegin.resize( eBegin.size( ) );
    epochLength.resize( eBegin.size( ) );

    for ( size_t e = 0; e < eBegin.size( ); e++ )
     {
      epochBegin[e]  = atof( eBegin[e].c_str( )  );
      epochLength[e] = atof( eLength[e].c_str( ) );
     }

    activation.addCondition( conditions[c], epochBegin, epochLength, conditionTags[c],
                             BellRegistry::name2bell( conditionBells[c], "v4" ) );
   }

                                                                    // submodels

  submodelTable.setNumConditions( activation.numConditions( ), conditions );

  if ( input.hasKey( my_submodels ) ) settings.submodelSpecs = input.getString( my_submodels );
  submodelTable.updateWith( settings.submodelSpecs );
  if ( mustIncludeNullSubmodel ) submodelTable.addByKey( "null" );

  assert( submodelTable.size( ) > 0 );
 }

void
InSpec::initDrift( const PghMRIhandler& input,
                         Settings&      settings,
                         BSplineBasis&  driftBasis,
                         dbProfile&     drift )
 {
  assert( input.isOpen( ) );

  int bsOrder = 0;
  if ( input.hasKey( my_driftbasis_order ) ) bsOrder = input.getInt( my_driftbasis_order );
  if ( bsOrder > 0 ) settings.drift.basisOrder = bsOrder;

  assert( input.hasKey( my_driftbasis_knots ) || input.hasKey( my_driftbasis_knotcount ) );

  fpVector knots;

  int leftEndIndex = 0, rightEndIndex;
  bool rightEndLeftContinuous = true;

  if ( input.hasKey( my_driftbasis_knots ) )
   {
    vector<string> tokens;
    MiscFns::getTokens( input.getString( my_driftbasis_knots ), tokens );

    knots.resize( tokens.size( ) );

    bool knotSequenceIsNonDecresing = true;
    for ( size_t i = 0; i < knots.size( ); ++i )
     {
      knots[i] = atof( tokens[i].c_str( ) );
      if ( i > 0 )
       knotSequenceIsNonDecresing =    knotSequenceIsNonDecresing
                                    && ( knots[i] >= knots[i-1] );
     }

    assert( knotSequenceIsNonDecresing );

    if ( input.hasKey( my_driftbasis_lep ) )
     leftEndIndex = input.getInt( my_driftbasis_lep );

    assert( knots[leftEndIndex] == settings.acquisition.initialTime( ) );

    rightEndIndex = knots.size( ) - 1;
    if ( input.hasKey( my_driftbasis_rep ) )
     rightEndIndex = input.getInt( my_driftbasis_rep );

    assert( knots[rightEndIndex] == settings.acquisition.finalTime( ) );

    if ( input.hasKey( my_driftbasis_relc ) )
     rightEndLeftContinuous = ( input.getInt( my_driftbasis_relc ) == 1 );
   }
  else
   {
    int numKnots = input.getInt( my_driftbasis_knotcount );

    assert( numKnots > 0 );

    fpType delta = ( settings.acquisition.finalTime( ) - settings.acquisition.initialTime( ) ) / ( numKnots - 1 );

    knots.resize( numKnots );
    for ( int i = 0; i < numKnots; i++ ) knots[i] = i * delta;

    rightEndIndex = knots.size( ) - 1;
   }

  driftBasis.reinitialize( knots, settings.drift.basisOrder ,
                           leftEndIndex, rightEndIndex, rightEndLeftContinuous );

  drift.setBasis( &driftBasis );
 }

void
InSpec::initBDNpriors( const PghMRIhandler&            input,
                       const dbProfile&                drift,
                       const acProfile&                activation,
                       const ParameterManager<fpType>& parManager,
                             Settings&                 settings,
                             LPbaseline&               lpBaseline,
                             LPdrift&                  lpDrift,
                             LPgamma&                  lpNoise )
 {
  assert( input.isOpen( ) );

  assert( input.hasKey( my_lp_b_scale ) );
  assert( input.hasKey( my_lp_b_shift ) );

  assert( input.hasKey( my_lp_d_smooth_df ) );

  assert( input.hasKey( my_lp_prefix_n + my_lp_suffix_a_nrs ) );
  assert( input.hasKey( my_lp_prefix_n + my_lp_suffix_b_nrs ) );

  lpBaseline.setDBprofile( &drift );
  lpBaseline.setParamOffset( parManager );
  lpBaseline.setConstants( input.getFloat( my_lp_b_scale ), input.getFloat( my_lp_b_shift ) );
                                               //
                                               // ATTN: setParamBufSize( ) later

  settings.drift.df = input.getFloat( my_lp_d_smooth_df );

  if ( input.hasKey( my_lp_d_penalty ) )
   settings.drift.relWeight = input.getFloat( my_lp_d_penalty );

  if ( input.hasKey( my_lp_d_smooth_fn ) )
   settings.drift.smoothFn = input.getInt( my_lp_d_smooth_fn );

  if ( input.hasKey( my_lp_d_smooth_it ) )
   settings.drift.smoothMaxIt = input.getInt( my_lp_d_smooth_it );

  if ( input.hasKey( my_lp_d_smooth_tol ) )
   settings.drift.smoothTol = input.getFloat( my_lp_d_smooth_tol );

  lpDrift.setDBprofile( &drift );
  lpDrift.setParamOffset( parManager );
                                               //
                                               // ATTN: setParamBufSize( ) later

  lpNoise.setNumParams( 1 );
  lpNoise.setConstants( 0, input.getFloat( my_lp_prefix_n + my_lp_suffix_a_nrs ),
                           input.getFloat( my_lp_prefix_n + my_lp_suffix_b_nrs ) );
  lpNoise.setParamOffset( parManager.blockOffset( ParamBlkNames::noise ) );
  lpNoise.setName( settings.objFnNames.noise );
                                               //
                                               // ATTN: setParamBufSize( ) later
 }

void
InSpec::initRSpriors( const PghMRIhandler&       input,
                      const acProfile&           activation,
                            Settings&            settings,
                            vector<LPgamma>&     lpResponse,
                            map<string,LPgamma>& lpShape )
 {
  assert( input.isOpen( ) );

  lpResponse.resize( activation.numConditions( ) );

  string keyA, keyB, prefix;

  for ( size_t c = 0; c < activation.numConditions( ); c++ )
   {
    const acProfile::ConditionBellSet& cbs = activation.bellSet( c );

    prefix = my_lp_prefix_r + cbs.name( );

    keyA = prefix + my_lp_suffix_a_nrs;
    keyB = prefix + my_lp_suffix_b_nrs;

    assert( input.hasKey( keyA ) );
    assert( input.hasKey( keyB ) );

    lpResponse[c].setConstants( 0, input.getFloat( keyA ), input.getFloat( keyB ) );
    lpResponse[c].setName( settings.objFnNames.response + cbs.name( ) );
                        //
                        // ATTN: setParamBufSize( ), setParamOffset( ... ) later

    map<string,LPgamma>::iterator i = lpShape.find( cbs.tag( ) );

    if ( i == lpShape.end( ) )
     {
      prefix = my_lp_prefix_s + cbs.tag( ) + ".";

      lpShape[cbs.tag( )].setNumParams( cbs.numShapeParams( ) );
      lpShape[cbs.tag( )].setName( settings.objFnNames.shape + cbs.tag( ) );

      for ( size_t s = 0; s < cbs.numShapeParams( ); s++ )
       {
        keyA = prefix + cbs.bell( )->name( s ) + my_lp_suffix_a_nrs;
        keyB = prefix + cbs.bell( )->name( s ) + my_lp_suffix_b_nrs;

        assert( input.hasKey( keyA ) );
        assert( input.hasKey( keyB ) );

        lpShape[cbs.tag( )].setConstants( s, input.getFloat( keyA ), input.getFloat( keyB ) );
                        //
                        // ATTN: setParamBufSize( ), setParamOffset( ... ) later
       }
     }
   }
 }

void
InSpec::writeSpecs_data( const Settings&                  settings,
                         const DataNavigator<TCrwPghMRI>& dataNavigator,
                               PghMRIhandler&             output )
 {
  output.setString( my_data, settings.imageData );
  output.setString( my_data_descr, settings.dataDescription );
  output.setString( my_data_chunk, settings.imageChunk );
  output.setString( my_data_tdim, string( 1, settings.t ) );
  output.setString( my_data_zdim, string( 1, settings.z ) );
  output.setString( my_data_missing, settings.missingSpecs );

  output.setInt( my_acq_shots, settings.acquisition.numShots( ) );
  output.setFloat( my_acq_tr, settings.acquisition.tr( ) );
  output.setFloat( my_acq_iai, settings.acquisition.iai( ) );
  output.setString( my_acq_slice_order, MiscFns::asString( settings.acquisition.sliceOrder( ), "%d" ) );
  output.setString( my_acq_slice_offsets, MiscFns::asString( settings.acquisition.sliceOffset( ) ) );

  output.setString( my_analysis_descr, settings.runDescription );

  string key = my_analysis_vange_prefix + "?", val;
  for ( size_t d = 0; d < dataNavigator.numDims( ); d++ )
   {
    if ( d != dataNavigator.tdim( ) )
     {
      key[key.size( ) - 1] = dataNavigator.names( )[d][0];
      char s[MiscFns::nDigits( dataNavigator.rangeUpper( )[d] ) + 1];
      sprintf( s, "%d", dataNavigator.rangeLower( )[d] );
      val = string( s ); val.append( ":" );
      sprintf( s, "%d", dataNavigator.rangeUpper( )[d] );
      val.append( string( s ) );
      output.setString( key, val );
     }
   }
 }

void
InSpec::writeSpecs_design( const Settings&      settings,
                           const acProfile&     activation,
                           const SubmodelTable& submodelTable,
                                 PghMRIhandler& output )
 {
  string conditions, tags, bells;
  
  for ( size_t c = 0; c < activation.numConditions( ); c++ )
   {
    const acProfile::ConditionBellSet& cbs = activation.bellSet( c );

    if ( c > 0 ) { conditions.append( " " ); tags.append( " " ); bells.append( " " ); }

    conditions.append( cbs.name( ) );
    tags.append( cbs.tag( ) );
    bells.append( cbs.bell( )->name( ) );

    output.setString( my_design_prefix_ebeg + cbs.name( ), MiscFns::asString( cbs.epochBegin( ) ) );
    output.setString( my_design_prefix_elen + cbs.name( ), MiscFns::asString( cbs.epochLength( ) ) );
   }

  output.setString( my_design_conditions, conditions );
  output.setString( my_design_tags, tags );
  output.setString( my_design_bells, bells );

  output.setString( my_submodels, settings.submodelSpecs );

  string val; size_t i = 0;
  for ( SubmodelTable::const_iterator si = submodelTable.begin( ); si != submodelTable.end( ); ++si, i++ )
   {
    if ( i > 0 ) val.append( " " ); val.append( submodelTable.key( si ) );
   }

  output.setString( my_submodels_list, val );
 }

void
InSpec::writeSpecs_drift( const Settings& settings, const BSplineBasis& driftBasis, PghMRIhandler& output )
 {
  output.setInt( my_driftbasis_order, driftBasis.order( ) );
  output.setString( my_driftbasis_knots, MiscFns::asString( driftBasis.knots( ) ) );
  output.setInt( my_driftbasis_knotcount, driftBasis.knots( ).size( ) );
  output.setInt( my_driftbasis_size, driftBasis.basisSize( ) );
  output.setInt( my_driftbasis_lep, driftBasis.leftEndPoint( ) );
  output.setInt( my_driftbasis_rep, driftBasis.rightEndPoint( ) );
  output.setInt( my_driftbasis_relc, (int) driftBasis.isRightEndLeftContinuous( ) );
 }

void
InSpec::writeSpecs_BDNpriors( const Settings&      settings,
                              const LPbaseline&    lpBaseline,
                              const LPdrift&       lpDrift,
                              const LPgamma&       lpNoise,
                                    PghMRIhandler& output )
 {
  output.setFloat( my_lp_b_scale, lpBaseline.scale( ) );
  output.setFloat( my_lp_b_shift, lpBaseline.shift( ) );
  output.setFloat( my_lp_d_penalty, settings.drift.relWeight );
  output.setFloat( my_lp_d_smooth_df, settings.drift.df );
  output.setFloat( my_lp_d_smooth_fn, settings.drift.smoothFn );
  output.setFloat( my_lp_d_smooth_it, settings.drift.smoothMaxIt );
  output.setFloat( my_lp_d_smooth_tol, settings.drift.smoothTol );
  output.setFloat( my_lp_prefix_n + my_lp_suffix_a_nrs, lpNoise.constA( 0 ) );
  output.setFloat( my_lp_prefix_n + my_lp_suffix_b_nrs, lpNoise.constB( 0 ) );
 }

void
InSpec::writeSpecs_RSpriors( const                 Settings&            settings,
                             const                 acProfile&           activation,
                             const                 vector<LPgamma>&     lpResponse,
                          /* in_principle_const */ map<string,LPgamma>& lpShape,
                                                   PghMRIhandler&       output )
 {
  string keyA, keyB, prefix;

  for ( size_t c = 0; c < activation.numConditions( ); c++ )
   {
    const acProfile::ConditionBellSet& cbs = activation.bellSet( c );

    prefix = my_lp_prefix_r + cbs.name( );

    output.setFloat( prefix + my_lp_suffix_a_nrs, lpResponse[c].constA( 0 ) );
    output.setFloat( prefix + my_lp_suffix_b_nrs, lpResponse[c].constB( 0 ) );

    prefix = my_lp_prefix_s + cbs.tag( ) + ".";

    for ( size_t s = 0; s < cbs.numShapeParams( ); s++ )
     {
      output.setFloat( prefix + cbs.bell( )->name( s ) + my_lp_suffix_a_nrs, lpShape[cbs.tag( )].constA( 0 ) );
      output.setFloat( prefix + cbs.bell( )->name( s ) + my_lp_suffix_b_nrs, lpShape[cbs.tag( )].constB( 0 ) );
     }
   }
 }

void
InSpec::writeSpecs_param( const Settings& settings, PghMRIhandler& output )
 {
  output.setFloat( my_param_prefix_scale + my_param_suffix_drift, settings.params.scales.drift );
  output.setFloat( my_param_prefix_scale + my_param_suffix_noise, settings.params.scales.noise );
  output.setFloat( my_param_prefix_scale + my_param_suffix_response, settings.params.scales.response );
  output.setFloat( my_param_prefix_scale + my_param_suffix_shape, settings.params.scales.shape );
 
  fpVector bounds; 

  if ( settings.params.types.drift == "lower" )
   {
    bounds.resize( 1 );
    bounds[0] = settings.params.bounds.drift.first;
   }
  else if ( settings.params.types.drift == "upper" )
   {
    bounds.resize( 1 );
    bounds[0] = settings.params.bounds.drift.second;
   }
  else if ( settings.params.types.drift == "both" )
   {
    bounds.resize( 2 );
    bounds[0] = settings.params.bounds.drift.first;
    bounds[1] = settings.params.bounds.drift.second;
   }

  output.setString( my_param_prefix_bound + my_param_suffix_drift,
                      settings.params.types.drift
                    + ( bounds.size( ) > 0 ? ( " " + MiscFns::asString( bounds ) ) : "" ) );

  if ( settings.params.types.noise == "lower" )
   {
    bounds.resize( 1 );
    bounds[0] = settings.params.bounds.noise.first;
   }
  else if ( settings.params.types.noise == "upper" )
   {
    bounds.resize( 1 );
    bounds[0] = settings.params.bounds.noise.second;
   }
  else if ( settings.params.types.noise == "both" )
   {
    bounds.resize( 2 );
    bounds[0] = settings.params.bounds.noise.first;
    bounds[1] = settings.params.bounds.noise.second;
   }

  output.setString( my_param_prefix_bound + my_param_suffix_noise,
                      settings.params.types.noise
                    + ( bounds.size( ) > 0 ? ( " " + MiscFns::asString( bounds ) ) : "" ) );

  if ( settings.params.types.response == "lower" )
   {
    bounds.resize( 1 );
    bounds[0] = settings.params.bounds.response.first;
   }
  else if ( settings.params.types.response == "upper" )
   {
    bounds.resize( 1 );
    bounds[0] = settings.params.bounds.response.second;
   }
  else if ( settings.params.types.response == "both" )
   {
    bounds.resize( 2 );
    bounds[0] = settings.params.bounds.response.first;
    bounds[1] = settings.params.bounds.response.second;
   }

  output.setString( my_param_prefix_bound + my_param_suffix_response,
                      settings.params.types.response
                    + ( bounds.size( ) > 0 ? ( " " + MiscFns::asString( bounds ) ) : "" ) );

  if ( settings.params.types.shape == "lower" )
   {
    bounds.resize( 1 );
    bounds[0] = settings.params.bounds.shape.first;
   }
  else if ( settings.params.types.shape == "upper" )
   {
    bounds.resize( 1 );
    bounds[0] = settings.params.bounds.shape.second;
   }
  else if ( settings.params.types.shape == "both" )
   {
    bounds.resize( 2 );
    bounds[0] = settings.params.bounds.shape.first;
    bounds[1] = settings.params.bounds.shape.second;
   }

  output.setString( my_param_prefix_bound + my_param_suffix_shape,
                      settings.params.types.shape
                    + ( bounds.size( ) > 0 ? ( " " + MiscFns::asString( bounds ) ) : "" ) );
 }

void
InSpec::writeSpecs_minimizer( const Settings& settings, PghMRIhandler& output )
 {
  writeSpecs_minimizer( settings.minimizer, output, "" );
 }

void
InSpec::writeSpecs_initializer( const Settings& settings, PghMRIhandler& output )
 {
  output.setInt( my_init_scheme, settings.initializer.scheme );
  output.setInt( my_init_iterations, settings.initializer.maxIt );

  writeSpecs_minimizer( settings.initializer.minimizer, output, my_init_prefix );
 }

void
InSpec::writeSpecs_misc( const Settings& settings, PghMRIhandler& output )
 {
  output.setInt( my_ll_ar, settings.arma.numAR );
 }

void
InSpec::writeSpecs_output( const Settings& settings, PghMRIhandler& output, const SubmodelTable& submodelTable )
 {
  int de = PghMRIhandler::endianness( );
  assert( ( de == 1 ) || ( de == 0 ) );

  output.setString( my_output_file, settings.output.fileName );
  output.setString( my_output_mode, PghMRIhandler::mode2name( settings.output.openMode, "mriWrite" ) );
  output.setString( my_output_on_stdout, ( settings.output.stdout ? "on" : "off" ) );
  output.setString( my_output_on_pghmri, ( settings.output.pghmri ? "on" : "off" ) );

  string val = "";
  for ( size_t p = 0; p < settings.output.products.size( ); p++ )
   {
    if (    ( ( settings.output.products[p] == "log.integral" ) && settings.output.liRequested )
         ||   ( settings.output.products[p] != "log.integral" ) )
     {
      val.append( settings.output.products[p] ); val.append( " " );

      if ( settings.output.types[p] != -1 )
       output.setString( my_product_prefix_type + settings.output.products[p] + my_product_suffix_type,
                         PghMRIhandler::type2name( settings.output.types[p] ) );

      if ( settings.output.littleE[p] != de )
       output.setInt( my_product_prefix_endian + settings.output.products[p] + my_product_suffix_endian,
                      settings.output.littleE[p] );

      if ( settings.output.leadDims[p] != "" )
       output.setString( my_product_prefix_dims + settings.output.products[p] + my_product_suffix_dims,
                         settings.output.leadDims[p] );

      if ( settings.output.diagnostics[p] )
       output.setString( my_product_prefix_diags + settings.output.products[p] + my_product_suffix_diags, "on" );
     }
   }

  if ( settings.output.mpRequested )
   {
    val.append( "model.probability" ); val.append( " " );
    output.setString( my_product_prefix_type + "model.probability" + my_product_suffix_type, settings.output.mpType );
    output.setInt( my_product_prefix_endian + "model.probability" + my_product_suffix_endian, settings.output.mpLittleE );
   }

  if ( settings.output.bfRequested )
   {
    val.append( "log.bayes.factor" ); val.append( " " );
    output.setString( my_product_prefix_type + "log.bayes.factor" + my_product_suffix_type, settings.output.bfType );
    output.setInt( my_product_prefix_endian + "log.bayes.factor" + my_product_suffix_endian, settings.output.bfLittleE );
   }

  output.setString( my_output_products, MiscFns::stripped( val ) );

  size_t i = 0;
  for ( SubmodelTable::const_iterator si  = submodelTable.begin( );
                                      si != submodelTable.end( ); ++si, i++ )
   output.setFloat( submodelTable.key( si ) + my_prior_probability_suffix,
                    settings.output.priorProbabilities[i] );
 }

void
InSpec::writeSpecs_minimizer( const Settings::Minimizer& minimizer,
                                    PghMRIhandler&       output,
                              const string&              prefix )
 {
  output.setInt( prefix + my_min_corrections, minimizer.maxCorrections );

  string val = ""; char s[128];

  sprintf( s, "%d", minimizer.maxIterations );
  val.append( string( s ) ); val.append( " " );
  sprintf( s, "%d", minimizer.maxEvaluations );
  val.append( string( s ) ); val.append( " " );
  sprintf( s, "%g", minimizer.maxPrGradNorm );
  val.append( string( s ) ); val.append( " " );
  sprintf( s, "%g", minimizer.minRelReduceF * DBL_EPSILON );
  val.append( string( s ) );

  output.setString( prefix + my_min_stop_opts, val );

  val = "";
  sprintf( s, "%d", minimizer.maxRestarts );
  val.append( string( s ) ); val.append( " " );
  val.append( ( minimizer.checkHdet ? "true " : "false " ) );
  val.append( ( minimizer.stopAtPlusDefH ? "true " : "false " ) );
  sprintf( s, "%g", minimizer.mrrfReduction );
  val.append( string( s ) );

  output.setString( prefix + my_min_restart_opts, val );

  output.setInt( prefix + my_min_verbosity, minimizer.verbosity );
 }
