/* ----------------------------------------------------------------------------
 *
 * class InSpec
 *   vmp input specification handler
 *
 * required headers
 *   see include header file InSpec
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __InSpec_H__
#define __InSpec_H__

cpp_using_namespace( std );
cpp_using_namespace( BasicTypes );

struct InSpec
 {
  InSpec( const string& data                  = "data",
          const string& data_chunk            = "data.chunk",
          const string& data_tdim             = "data.dimension.t",
          const string& data_zdim             = "data.dimension.z",
          const string& data_descr            = "data.description",
          const string& data_missing          = "data.missing",
        
          const string& analysis_vange_prefix = "analysis.voxel.range.",
          const string& analysis_descr        = "analysis.description",

          const string& acq_shots             = "acquisition.shots",
          const string& acq_tr                = "acquisition.tr",
          const string& acq_iai               = "acquisition.iai",
          const string& acq_slice_order       = "acquisition.slice.order",
          const string& acq_slice_offsets     = "acquisition.slice.offsets",

          const string& design_conditions     = "design.conditions",
          const string& design_tags           = "design.condition.tags",
          const string& design_bells          = "design.condition.bells",
          const string& design_prefix_ebeg    = "design.epoch.begin.",
          const string& design_prefix_elen    = "design.epoch.length.",

          const string& driftbasis_order      = "drift.basis.order",
          const string& driftbasis_knots      = "drift.basis.knots",
          const string& driftbasis_knotcount  = "drift.basis.knot.count",
          const string& driftbasis_lep        = "drift.basis.endknot.left",
          const string& driftbasis_rep        = "drift.basis.endknot.right",
          const string& driftbasis_relc       = "drift.basis.relc",
          const string& driftbasis_size       = "drift.basis.size",

          const string& submodels             = "submodels",
          const string& submodels_list        = "submodels.list",

          const string& ll_ar                 = "ll.arma.ar",
          const string& ll_ma                 = "ll.arma.ma",

          const string& lp_b_scale            = "lp.baseline.scale",
          const string& lp_b_shift            = "lp.baseline.shift",

          const string& lp_d_penalty          = "lp.drift.norm.penalty",
          const string& lp_d_smooth_df        = "lp.drift.smooth.df",
          const string& lp_d_smooth_fn        = "lp.drift.smooth.choice",
          const string& lp_d_smooth_it        = "lp.drift.smooth.iterations",
          const string& lp_d_smooth_tol       = "lp.drift.smooth.tolerance",

          const string& lp_prefix_n           = "lp.noise",
          const string& lp_prefix_r           = "lp.response.",
          const string& lp_prefix_s           = "lp.shape.",
          const string& lp_suffix_a_nrs       = ".a",
          const string& lp_suffix_b_nrs       = ".b",

          const string& param_prefix_scale    = "params.scale.",
          const string& param_prefix_bound    = "params.bounds.",
          const string& param_suffix_drift    = "drift",
          const string& param_suffix_noise    = "noise",
          const string& param_suffix_response = "response",
          const string& param_suffix_shape    = "shape",

          const string& min_corrections       = "minimizer.corrections",
          const string& min_stop_opts         = "minimizer.stop.opts",
          const string& min_restart_opts      = "minimizer.restart.opts",
          const string& min_verbosity         = "minimizer.verbosity",

          const string& init_prefix           = "init.",
          const string& init_scheme           = "init.scheme",
          const string& init_iterations       = "init.iterations",

          const string& output_file           = "output.file",
          const string& output_mode           = "output.mode",
          const string& output_on_stdout      = "output.on.stdout",
          const string& output_on_pghmri      = "output.on.pghmri",
          const string& output_products       = "output.products",
          const string& product_prefix_type   = "",
          const string& product_prefix_dims   = "",
          const string& product_prefix_endian = "",
          const string& product_prefix_diags  = "",
          const string& product_suffix_type   = ".datatype",
          const string& product_suffix_dims   = ".leaddims",
          const string& product_suffix_endian = ".little_endian",
          const string& product_suffix_diags  = ".diagnostics",
          const string& prior_probability_suffix = ".prior.probability" );

  void
  getSpecs_param( const PghMRIhandler& input, Settings& settings );

  void
  getSpecs_minimizer( const PghMRIhandler& input, Settings& settings );

  void
  getSpecs_initializer( const PghMRIhandler& input, Settings& settings );

  void
  getSpecs_misc( const PghMRIhandler& input, Settings& settings );

  void
  getSpecs_output( const PghMRIhandler& input, Settings& settings );

  void
  getSpecs_products( const PghMRIhandler& input, Settings& settings, const SubmodelTable& submodelTable );

  void
  initData( const PghMRIhandler&             input,
                  Settings&                  settings,
                  TCrwPghMRI&                tcReader,
                  MissingInfo&               missingInfo,
                  DataNavigator<TCrwPghMRI>& dataNavigator );

  void
  initDesign( const PghMRIhandler& input,
              const dbProfile&     drift,
                    Settings&      settings,
                    acProfile&     activation,
                    SubmodelTable& submodelTable,
                    bool           mustIncludeNullSubmodel = true );

  void
  initDrift( const PghMRIhandler& input, Settings& settings, BSplineBasis& driftBasis, dbProfile& drift );

  void
  initBDNpriors( const PghMRIhandler&            input,
                 const dbProfile&                drift,
                 const acProfile&                activation,
                 const ParameterManager<fpType>& parManager,
                       Settings&                 settings,
                       LPbaseline&               lpBaseline,
                       LPdrift&                  lpDrift,
                       LPgamma&                  lpNoise );

  void
  initRSpriors( const PghMRIhandler&       input,
                const acProfile&           activation,
                      Settings&            settings,
                      vector<LPgamma>&     lpResponse,
                      map<string,LPgamma>& lpShape );

  void
  writeSpecs_data( const Settings&                  settings,
                   const DataNavigator<TCrwPghMRI>& dataNavigator,
                         PghMRIhandler&             output );

  void
  writeSpecs_design( const Settings&      settings,
                     const acProfile&     activation,
                     const SubmodelTable& submodelTable,
                           PghMRIhandler& output );

  void
  writeSpecs_drift( const Settings&      settings,
                    const BSplineBasis&  driftBasis,
                          PghMRIhandler& output );

  void
  writeSpecs_BDNpriors( const Settings&      settings,
                        const LPbaseline&    lpBaseline,
                        const LPdrift&       lpDrift,
                        const LPgamma&       lpNoise,
                              PghMRIhandler& output );

  void
  writeSpecs_RSpriors( const                 Settings&            settings,
                       const                 acProfile&           activation,
                       const                 vector<LPgamma>&     lpResponse,
                    /* in_principle_const */ map<string,LPgamma>& lpShape,
                                             PghMRIhandler&       output );

  void
  writeSpecs_param( const Settings& settings, PghMRIhandler& output );

  void
  writeSpecs_minimizer( const Settings& settings, PghMRIhandler& output );

  void
  writeSpecs_initializer( const Settings& settings, PghMRIhandler& output );

  void
  writeSpecs_misc( const Settings& settings, PghMRIhandler& output );

  void
  writeSpecs_output( const Settings& settings, PghMRIhandler& output, const SubmodelTable& submodelTable );

  string
  my_data, my_data_chunk, my_data_tdim, my_data_zdim, my_data_descr, my_data_missing,
  my_analysis_vange_prefix, my_analysis_descr,
  my_acq_shots, my_acq_tr, my_acq_iai, my_acq_slice_order, my_acq_slice_offsets,
  my_design_conditions, my_design_tags, my_design_bells, my_design_prefix_ebeg, my_design_prefix_elen,
  my_driftbasis_order, my_driftbasis_knots, my_driftbasis_knotcount,
  my_driftbasis_lep, my_driftbasis_rep, my_driftbasis_relc, my_driftbasis_size,
  my_submodels, my_submodels_list,
  my_ll_ar, my_ll_ma,
  my_lp_b_scale, my_lp_b_shift,
  my_lp_d_penalty, my_lp_d_smooth_df, my_lp_d_smooth_fn, my_lp_d_smooth_it, my_lp_d_smooth_tol,
  my_lp_prefix_n, my_lp_prefix_r, my_lp_prefix_s, my_lp_suffix_a_nrs, my_lp_suffix_b_nrs,
  my_param_prefix_scale, my_param_prefix_bound, my_param_suffix_drift,
  my_param_suffix_noise, my_param_suffix_response, my_param_suffix_shape,
  my_min_corrections, my_min_stop_opts, my_min_restart_opts, my_min_verbosity,
  my_init_prefix, my_init_scheme, my_init_iterations,
  my_output_file, my_output_mode, my_output_on_stdout, my_output_on_pghmri,
  my_output_products,
  my_product_prefix_type, my_product_prefix_dims, my_product_prefix_endian, my_product_prefix_diags,
  my_product_suffix_type, my_product_suffix_dims, my_product_suffix_endian, my_product_suffix_diags,
  my_prior_probability_suffix;

  private:

   void
   getSpecs_minimizer( const PghMRIhandler&       input,
                             Settings::Minimizer& minimizer,
                       const string&              prefix = "" );

   void
   writeSpecs_minimizer( const Settings::Minimizer& minimizer,
                               PghMRIhandler&       output,
                         const string&              prefix = "" );
 };

#endif /* __InSpec_H__ */
