/* ----------------------------------------------------------------------------
 *
 * Output Products
 *   implementation
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <brain.vmp/products>

// --------------------------------------------------------------- OutputProduct
// -----------------------------------------------------------------------------

OutputProduct::OutputProduct( const OutputProduct& another )
 : my_name( another.my_name ),
   my_chunkName( another.my_chunkName ),
   my_chunkFile( another.my_chunkFile ),
   my_diagsName( another.my_diagsName ),
   my_diagsFile( another.my_diagsFile ),
   my_numLeadDims( another.my_numLeadDims ),
   my_defaultType( another.my_defaultType ),
   my_needs( another.my_needs ), my_products( another.my_products ), my_own( another.my_own ),
   my_dn( another.my_dn ), my_pm( another.my_pm ), my_mn( another.my_mn ),
   my_ac( another.my_ac ), my_pb( another.my_pb ), my_tg( another.my_tg ),
   my_mrh( another.my_mrh ),
   my_configured( another.my_configured ), my_initialized( another.my_initialized ),
   my_stdout( another.my_stdout ), my_pghmri( another.my_pghmri ), my_disablePghMRI( another.my_pghmri ),
   my_diags( another.my_diags ), my_aim( another.my_aim ), my_diagsAIM( another.my_diagsAIM ),
   my_error( another.my_error ),
   my_quantity( another.my_quantity ), my_dataLocation( another.my_dataLocation )
 {
 }

OutputProduct::OutputProduct&
OutputProduct::operator=( const OutputProduct& rhs )
 {
  if ( this == &rhs ) return *this;

  my_name         = rhs.my_name;
  my_chunkName    = rhs.my_chunkName;
  my_chunkFile    = rhs.my_chunkFile;
  my_diagsName    = rhs.my_diagsName;
  my_diagsFile    = rhs.my_diagsFile;
  my_numLeadDims  = rhs.my_numLeadDims;
  my_defaultType  = rhs.my_defaultType;
  my_needs        = rhs.my_needs;
  my_products     = rhs.my_products;
  my_own          = rhs.my_own;

  my_dn = rhs.my_dn; my_pm = rhs.my_pm; my_mn = rhs.my_mn;
  my_ac = rhs.my_ac; my_pb = rhs.my_pb; my_tg = rhs.my_tg;

  my_mrh           = rhs.my_mrh;
  my_configured    = rhs.my_configured;
  my_initialized   = rhs.my_initialized;
  my_stdout        = rhs.my_stdout;
  my_pghmri        = rhs.my_pghmri;
  my_disablePghMRI = rhs.my_disablePghMRI;
  my_diags         = rhs.my_diags;
  my_diagsAIM      = rhs.my_diagsAIM;
  my_error         = rhs.my_error;

  my_quantity.resize( rhs.my_quantity.size( ) );
  my_dataLocation.resize( rhs.my_dataLocation.size( ) );

  my_quantity      = rhs.my_quantity;
  my_dataLocation  = rhs.my_dataLocation;

  return *this;
 }

OutputProduct::OutputProduct( const string&                     name,
                                    size_t                      numLeadDims,
                                    PghMRIhandler::mriArrayType defaultType,
                              const vector<string>&             productsNeeded,
                                    bool                        disablePghMRI )
 : my_name( name ),
   my_numLeadDims( numLeadDims ), my_defaultType( defaultType ),
   my_needs( productsNeeded ), my_own( my_needs.size( ) ),
   my_configured( false ), my_initialized( false ),
   my_stdout( false ), my_pghmri( true ), my_disablePghMRI( disablePghMRI ),
   my_diags( false ), my_error( false )
 {
  setNeeds( productsNeeded );
 }

void
OutputProduct::setNeeds( const vector<string>& productsNeeded )
 {
  my_needs = productsNeeded;

  for ( size_t i = 0; i < my_needs.size( ); i++ )
   {
    // cerr << "(debug) OutputProduct::setNeeds( ) name " << my_name << " need " << my_needs[i] << " " << ProductRegistry::isKnown( my_needs[i] ) << endl;
    assert( ProductRegistry::isKnown( my_needs[i] ) );
   }

  my_own.resize( my_needs.size( ) );
 }

OutputProduct::~OutputProduct( )
 {
  finalize( );

  for ( size_t i = 0; i < my_products.size( ); i++ )
   if ( my_own[i] ) delete my_products[i];

  my_products.resize( 0 );
  my_own.resize( my_needs.size( ), false );
 }

bool
OutputProduct::createProductsNeeded( vector<OutputProduct*>& products, bool own )
 {
  // cerr << "(debug) OutputProduct::createProductsNeeded( ) my_name " << my_name << endl;

  if ( hasProductsNeeded( ) ) return true;

  OutputProduct* p;
  bool success = true;

  for ( size_t i = 0; i < my_needs.size( ); i++ )
   {
    // cerr << "(debug) OutputProduct::createProductsNeeded( ) my_name " << my_name << " " << my_needs[i] << " " << ProductRegistry::isKnown( my_needs[i] );

    p = ProductRegistry::extract( my_needs[i], products );

    // cerr << " ( p != 0 ) " << ( p != 0 );

    if ( p == 0 )
     {
      success = success && ProductRegistry::create( my_needs[i], p, false );

      // cerr << " ( p != 0 ) " << ( p != 0 );

      p->setOutput( false, false );
      products.push_back( p );
      my_own[i] = own;
     }

    // cerr << endl;

    my_products.push_back( p );

    success = success && ( p->createProductsNeeded( products, own ) );
   }

  // cerr << "(debug) OutputProduct::createProductsNeeded( ) my_name " << my_name << " success " << success << endl;

  return success;
 }

void
OutputProduct::configure( const TCreader*                 dataNavigator,
                          const ParameterManager<fpType>* parameterManager,
                          const MinQNB*                   minimizer,
                          const acProfile*                activation,
                                fpVector*                 paramBuf,
                          const TimeGrids*                timeGrids )
 {
  if ( my_initialized ) return;

  my_dn = dataNavigator; my_pm = parameterManager; my_mn = minimizer;
  my_ac = activation; my_pb = paramBuf; my_tg = timeGrids;

  my_configured = my_dn && my_pm && my_mn && my_ac && my_pb && my_tg;
 }

void
OutputProduct::setOutput( bool stdout, bool pghmri )
 {
  my_stdout = stdout; my_pghmri = pghmri && !my_disablePghMRI;
 }

void
OutputProduct::initialize(       PghMRIhandler*              mriHandler,
                           const string&                     chunkPrefix,
                                 PghMRIhandler::mriArrayType chunkType,
                           const string&                     leadDims,
                                 bool                        diagnostics,
                                 bool                        littleEndian )
 {
  // cerr << "(debug) OutputProduct::initialize( ) my_name " << my_name << " my_initialized " << my_initialized << " my_configured " << my_configured << endl;

  if ( my_initialized || !my_configured ) return;

  /*
  cerr << "(debug) OutputProduct::initialize( ) my_name " << my_name << " my_needs.size( ) " << my_needs.size( ) << " my_own.size( ) " << my_own.size( ) << " my_products.size( ) " << my_products.size( ) << endl;

  for ( size_t i = 0; i < my_products.size( ); i++ )
   cerr << "(debug) OutputProduct::initialize( ) my_name " << my_name
        << " ptr " << my_products[i] << " name " << my_products[i]->name( ) << endl;

  for ( size_t i = 0; i < my_needs.size( ); i++ )
   cerr << "(debug) OutputProduct::initialize( ) my_name " << my_name
        << " need " << my_needs[i] << " own " << my_own[i] << " has " << ProductRegistry::has( my_needs[i], my_products ) << endl;
  */

  /* consistency checks (need more?) */

  assert( my_dn != 0);
  for ( size_t i = 0; i < my_dn->numDims( ); i++ )
   assert( my_dn->names( )[i].size( ) == 1 );

  if ( my_mn && my_pm ) assert( my_mn->problemSize( ) == my_pm->numParams( ) );
  if ( my_tg ) assert( my_dn->sizes( )[my_dn->tdim( )] == my_tg->baseGrid( ).size( ) );

  /* products needed by this products */

  bool success = true;

  if ( !hasProductsNeeded( ) )
   success = success && createProductsNeeded( my_products, true );

  // cerr << "(debug) OutputProduct::initialize( ) my_name " << my_name << " success " << success << " hasProductsNeeded( ) " << hasProductsNeeded( ) << endl;

  assert( hasProductsNeeded( ) );

  /* configure and initialize owned products */

  for ( size_t i = 0; i < my_needs.size( ); i++ )
   {
    success = success && ( my_products[i] != 0 );

    if ( my_own[i] && !my_products[i]->isInitialized( ) )
     {
      my_products[i]->configure( my_dn, my_pm, my_mn, my_ac, my_pb, my_tg );
      my_products[i]->setOutput( false, false );
      my_products[i]->initialize( mriHandler, chunkPrefix );
     }

    // success = success && ( my_products[i]->isInitialized( ) );
   }

  /* quantity and diagnostic chunks for this product */

  my_diags = diagnostics;

  // cerr << "(debug) OutputProduct::initialize( ) my_name " << my_name << " my_pghmri " << my_pghmri << " diagnostics " << my_diags << " success " << success << " my_initialized " << my_initialized << endl;

  if ( my_pghmri )
   {
    my_mrh = mriHandler;
    assert( ( my_mrh != 0 ) && my_mrh->isOpen( ) && ( my_mrh->openMode( ) != PghMRIhandler::mriRead ) );

    /* chunk and file names */

    my_chunkName = chunkPrefix  + my_name;
    my_chunkFile = my_chunkName + ".dat";

    my_diagsName = chunkPrefix  + my_name + ".diags";
    my_diagsFile = my_diagsName + ".dat";

    // cerr << "(debug) OutputProduct::initialize( ) my_chunkName " << my_chunkName << " my_chunkFile " << my_chunkFile << " my_diagsName " << my_diagsName << " my_diagsFile " << my_diagsFile << endl;

    /* extents for lead (i. e., product-specific) dimensions */

    uintVector leadSz( leadSizes( ) );
    size_t imax = min( my_numLeadDims, leadSz.size( ) );

    /*
    cerr << "(debug) OutputProduct::initialize( ) leadSz";
    for ( size_t i = 0; i < leadSz.size( ); i++ ) cerr << " " << leadSz[i];
    cerr << endl;
    */

    /* quantity chunk dimensions = data dimensions + quantity dimensions - t-dimention */

    size_t numQdims = my_numLeadDims + my_dn->numDims( ) - 1;

    // cerr << "(debug) OutputProduct::initialize( ) numQdims " << numQdims << endl;

    /* quantity chunk sizes and dimensionwise lower bounds on indices */

    int sizes[numQdims]; int lower[numQdims];
    uintVector sz( numQdims ), lw( numQdims );

    for ( size_t i = 0; i < imax; i++ ) { lw[i] = lower[i] = 0; sz[i] = sizes[i] = leadSz[i]; }
    for ( size_t i = imax; i < my_numLeadDims; i++ ) { lw[i] = lower[i] = 0; sz[i] = sizes[i] = 1; }
    for ( size_t d = 0, i = my_numLeadDims; d < my_dn->numDims( ); d++ )
     if ( d != my_dn->tdim( ) ) { lw[i] = lower[i] = my_dn->indexLower( )[d]; sz[i] = sizes[i] = my_dn->sizes( )[d]; i++; }

    /*
    cerr << "(debug) OutputProduct::initialize( ) sizes";
    for ( size_t i = 0; i < numQdims; i++ ) cerr << " " << sizes[i];
    cerr << endl;

    cerr << "(debug) OutputProduct::initialize( ) lower";
    for ( size_t i = 0; i < numQdims; i++ ) cerr << " " << lower[i];
    cerr << endl;
    */

    my_aim.reinitialize( sz, uintVector( ), lw );
    my_diagsAIM.reinitialize( my_dn->sizesSansT( ), uintVector( ), my_dn->lowerSansT( ) );

    /*
    cerr << "(debug) OutputProduct::initialize( ) my_aim.numDims( ) " << my_aim.numDims( ) << " my_aim.size( ) " << my_aim.size( ) << endl;
    for ( size_t d = 0; d < my_aim.numDims( ); d++ )
     cerr << "(debug) OutputProduct::initialize( ) dimension " << d
          << " size "             << my_aim.sizes( )[d]
          << " order "            << my_aim.order( )[d]
          << " index range "      << my_aim.indexLower( )[d] << ":" << my_aim.indexUpper( )[d]
          << endl;

    cerr << "(debug) OutputProduct::initialize( ) my_diagsAIM.numDims( ) " << my_diagsAIM.numDims( ) << " my_diagsAIM.size( ) " << my_diagsAIM.size( ) << endl;
    for ( size_t d = 0; d < my_diagsAIM.numDims( ); d++ )
     cerr << "(debug) OutputProduct::initialize( ) dimension " << d
          << " size "             << my_diagsAIM.sizes( )[d]
          << " order "            << my_diagsAIM.order( )[d]
          << " index range "      << my_diagsAIM.indexLower( )[d] << ":" << my_diagsAIM.indexUpper( )[d]
          << endl;
    */

    /* data dimensions */

    string data_dimensions( my_dn->nameStr( "", my_dn->dimsSansT( ) ) );
    assert( data_dimensions.size( ) == my_dn->dimsSansT( ).size( ) );
                                    //
                                    // libmri allows one-char names for dimensions

    // cerr << "(debug) OutputProduct::initialize( ) data_dimensions " << data_dimensions << endl;

    /* quantity dimensions */

    string leaddims = ( leadDims == "" ) ? my_dn->names( )[my_dn->tdim( )] : leadDims;

    for ( size_t i = 0; i < data_dimensions.size( ); i++ )
     {
      string::size_type j = leaddims.find( data_dimensions[i] );
      if ( j != string::npos ) leaddims.erase( j, 1 );
     }

    string availdims = PghMRIhandler::availableDimNames( leaddims + data_dimensions );

    imax = min( my_numLeadDims, leaddims.size( ) );
    string qdims = data_dimensions;
    qdims.insert( 0, leaddims.substr( 0, imax ) );
    qdims.insert( imax, availdims.substr( 0, my_numLeadDims - imax ) );

    // cerr << "(debug) OutputProduct::initialize( ) qdims " << qdims << endl;

    /* mode-specific initializations */

    string chunktypename = PghMRIhandler::type2name( chunkType, PghMRIhandler::type2name( my_defaultType ) );

    if ( my_mrh->openMode( ) == PghMRIhandler::mriModifyData )
     {
      assert( PghMRIhandler::matchChunk( *my_mrh, my_chunkName, chunktypename, qdims, sizes, lower ) );

      if ( my_diags )
       assert( PghMRIhandler::matchChunk( *my_mrh, my_diagsName, "uint8", data_dimensions,
                                          sizes + my_numLeadDims, lower + my_numLeadDims ) );
     }
    else
     {
      my_mrh->createChunk( my_chunkName, qdims, chunktypename, my_chunkFile, sizes, lower, littleEndian );

      if ( my_diags )
       my_mrh->createChunk( my_diagsName, data_dimensions,
                            "uint8", my_diagsFile, sizes + my_numLeadDims, lower + my_numLeadDims );
     }

    // cerr << "(debug) OutputProduct::initialize( ) my_name " << my_name << " diagnostics( ) " << OutputProduct::diagnostics( ) << endl;
   }

  /* product-specific sub-initializations */

  subInitialize( chunkPrefix );

  my_initialized = success;

  // cerr << "(debug) OutputProduct::initialize( ) my_name " << my_name << " my_pghmri " << my_pghmri << " diagnostics " << my_diags << " success " << success << " my_configured " << my_configured << " my_initialized " << my_initialized << endl;
 }

void
OutputProduct::subInitialize( const string& chunkPrefix )
 {
 }

void
OutputProduct::subFinalize( )
 {
 }

void
OutputProduct::finalize( )
 {
  my_initialized = false;
  subFinalize( );
 }

// ------------------------------------------------------------ product registry
// -----------------------------------------------------------------------------

void
ProductRegistry::expand( vector<string>& expanded, const vector<string>& productList )
 {
  list<string> l; bool all, individual;

  for ( size_t i = 0; i < productList.size( ); i++ )
   {
    all = ( productList[i] == "all.products" ); individual = true;

    if ( all || ( productList[i] == "default.products" ) )
     {
      if ( isKnown( "estimates"                  ) ) l.push_back( "estimates" );
      if ( isKnown( "standard.errors"            ) ) l.push_back( "standard.errors" );
      if ( isKnown( "covariances"                ) ) l.push_back( "covariances" );
      if ( isKnown( "baseline"                   ) ) l.push_back( "baseline" );
      if ( isKnown( "baseline.stderr"            ) ) l.push_back( "baseline.stderr" );
      if ( isKnown( "fits"                       ) ) l.push_back( "fits" );
      if ( isKnown( "minimization.status"        ) ) l.push_back( "minimization.status" );
      if ( isKnown( "minimization.gradient.norm" ) ) l.push_back( "minimization.gradient.norm" );
      if ( isKnown( "minimization.statistics"    ) ) l.push_back( "minimization.statistics" );
      if ( isKnown( "fit.positive.definite"      ) ) l.push_back( "fit.positive.definite" );
      individual = false;
     }

    if ( all || ( productList[i] == "diagnostic.products" ) )
     {
      if ( isKnown( "minimization.status"        ) ) l.push_back( "minimization.status" );
      if ( isKnown( "minimization.iterations"    ) ) l.push_back( "minimization.iterations" );
      if ( isKnown( "minimization.evaluations"   ) ) l.push_back( "minimization.evaluations" );
      if ( isKnown( "minimization.restarts"      ) ) l.push_back( "minimization.restarts" );
      if ( isKnown( "minimization.gradient.norm" ) ) l.push_back( "minimization.gradient.norm" );
      if ( isKnown( "minimization.statistics"    ) ) l.push_back( "minimization.statistics" );
      if ( isKnown( "log.posterior.max"          ) ) l.push_back( "log.posterior.max" );
      if ( isKnown( "gradient"                   ) ) l.push_back( "gradient" );
      if ( isKnown( "hessian"                    ) ) l.push_back( "hessian" );
      if ( isKnown( "log.determinant"            ) ) l.push_back( "log.determinant" );
      if ( isKnown( "log.integral"               ) ) l.push_back( "log.integral" );
      if ( isKnown( "fit.positive.definite"      ) ) l.push_back( "fit.positive.definite" );
      individual = false;
     }

    if ( all || ( productList[i] == "misc.products" ) )
     {
      if ( isKnown( "profiles.drift"      ) ) l.push_back( "profiles.drift" );
      if ( isKnown( "profiles.activation" ) ) l.push_back( "profiles.activation" );
      if ( isKnown( "residuals"           ) ) l.push_back( "residuals" );
      // if ( isKnown( "log.bayes.factor"    ) ) l.push_back( "log.bayes.factor" );
      individual = false;
     }

    if ( individual && isKnown( productList[i] ) ) l.push_back( productList[i] );
   }

  l.sort( ); unique_copy( l.begin( ), l.end( ), back_inserter( expanded ) );
 }

bool
ProductRegistry::isKnown( const string& productName )
 {
  OutputProduct* p;
  return create( productName, p, true );
 }

bool
ProductRegistry::create( const string& productName, OutputProduct*& p, bool testOnly )
 {
       if ( productName == "minimization.iterations"    ) { if ( testOnly ) return true; p = new Iterations;     return ( p != 0 ); }
  else if ( productName == "minimization.evaluations"   ) { if ( testOnly ) return true; p = new Evaluations;    return ( p != 0 ); }
  else if ( productName == "minimization.restarts"      ) { if ( testOnly ) return true; p = new Restarts;       return ( p != 0 ); }
  else if ( productName == "minimization.status"        ) { if ( testOnly ) return true; p = new Status;         return ( p != 0 ); }
  else if ( productName == "minimization.gradient.norm" ) { if ( testOnly ) return true; p = new PrGradNorm;     return ( p != 0 ); }
  else if ( productName == "gradient"                   ) { if ( testOnly ) return true; p = new Gradient;       return ( p != 0 ); }
  else if ( productName == "estimates"                  ) { if ( testOnly ) return true; p = new Estimates;      return ( p != 0 ); }
  else if ( productName == "baseline"                   ) { if ( testOnly ) return true; p = new Baseline;       return ( p != 0 ); }
  else if ( productName == "log.posterior.max"          ) { if ( testOnly ) return true; p = new LogPostMax;     return ( p != 0 ); }
  else if ( productName == "profiles.drift"             ) { if ( testOnly ) return true; p = new DBprofile;      return ( p != 0 ); }
  else if ( productName == "profiles.activation"        ) { if ( testOnly ) return true; p = new ACprofile;      return ( p != 0 ); }
  else if ( productName == "fits"                       ) { if ( testOnly ) return true; p = new Fits;           return ( p != 0 ); }
  else if ( productName == "residuals"                  ) { if ( testOnly ) return true; p = new Residuals;      return ( p != 0 ); }
  else if ( productName == "hessian"                    ) { if ( testOnly ) return true; p = new Hessian;        return ( p != 0 ); }
  else if ( productName == "covariances"                ) { if ( testOnly ) return true; p = new Covariances;    return ( p != 0 ); }
  else if ( productName == "standard.errors"            ) { if ( testOnly ) return true; p = new StdErrors;      return ( p != 0 ); }
  else if ( productName == "baseline.stderr"            ) { if ( testOnly ) return true; p = new BaselineStdErr; return ( p != 0 ); }
  else if ( productName == "log.determinant"            ) { if ( testOnly ) return true; p = new LogDeterminant; return ( p != 0 ); }
  else if ( productName == "fit.positive.definite"      ) { if ( testOnly ) return true; p = new FitPositiveDefinite; return ( p != 0 ); }
  else if ( productName == "log.integral"               ) { if ( testOnly ) return true; p = new LogIntegral;    return ( p != 0 ); }
  // else if ( productName == "log.bayes.factor"           ) { if ( testOnly ) return true; p = new LogBayesFactor; return ( p != 0 ); }

  return false;
 }

bool
ProductRegistry::add( const string& productName, vector<OutputProduct*>& products )
 {
  if ( has( productName, products ) ) return false;

  OutputProduct* p;
  if ( !create( productName, p, false ) ) return false;

  products.push_back( p );

  return true;
 }

bool
ProductRegistry::del( const string& productName, vector<OutputProduct*>& products )
 {
  vector<OutputProduct*>::iterator
  i = find_if( products.begin( ), products.end( ), nameIs<OutputProduct>( productName ) );

  if ( i == products.end( ) ) return false;

  (*i)->finalize( ); delete (*i); products.erase( i );

  return true;
 }

// ------------------------------------------------------------------ ProductSet
// -----------------------------------------------------------------------------

ProductSet::ProductSet( )
 : OutputProduct( "product.set", 0, -1 ), my_products( )
 {
 }

ProductSet::ProductSet( const ProductSet& another )
 : OutputProduct( "product.set", 0, -1 ), my_products( another.my_products )
 {
  another.my_products.resize( 0 );
 }

ProductSet::~ProductSet( )
 {
  for ( size_t i = 0; i < my_products.size( ); i++ )
   {
    my_products[i]->finalize( );
    delete my_products[i];
    my_products[i] = 0;
   }
 }

ProductSet&
ProductSet::operator=( const ProductSet& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );
  my_products = rhs.my_products;

  rhs.my_products.resize( 0 );

  return *this;
 }

bool
ProductSet::add( const string& productName )
 {
  bool success = true;

  OutputProduct* p = ProductRegistry::extract( productName, my_products );

  if ( p == 0 )
   {
    success = success && ProductRegistry::create( productName, p, false );
    my_products.push_back( p );
   }
  else
   p->setOutput( false, true );

  success = success && ( p->createProductsNeeded( my_products, false ) );

  return success;
 }

bool
ProductSet::del( const string& productName )
 {
  return ProductRegistry::del( productName, my_products );
 }

// ----------------------------------------------------- diagnostics: iterations
// -----------------------------------------------------------------------------

Iterations::Iterations( )
 : OutputProduct( "minimization.iterations", 1, PghMRIhandler::mriShort )
 {
 }

Iterations::Iterations( const Iterations& another )
 : OutputProduct( another )
 {
 }

Iterations::Iterations&
Iterations::operator=( const Iterations& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

// ---------------------------------------------------- diagnostics: evaluations
// -----------------------------------------------------------------------------

Evaluations::Evaluations( )
 : OutputProduct( "minimization.evaluations", 1, PghMRIhandler::mriShort )
 {
 }

Evaluations::Evaluations( const Evaluations& another )
 : OutputProduct( another )
 {
 }

Evaluations::Evaluations&
Evaluations::operator=( const Evaluations& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

// ------------------------------------------------------- diagnostics: restarts
// -----------------------------------------------------------------------------

Restarts::Restarts( )
 : OutputProduct( "minimization.restarts", 1, PghMRIhandler::mriShort )
 {
 }

Restarts::Restarts( const Restarts& another )
 : OutputProduct( another )
 {
 }

Restarts::Restarts&
Restarts::operator=( const Restarts& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

// --------------------------------------------------------- diagnostics: status
// -----------------------------------------------------------------------------

Status::Status( )
 : OutputProduct( "minimization.status", 1, PghMRIhandler::mriUchar )
 {
 }

Status::Status( const Status& another )
 : OutputProduct( another )
 {
 }

Status::Status&
Status::operator=( const Status& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

void
Status::subInitialize( const string& chunkPrefix )
 {
  if ( pghmri( ) && mriHandler( ).isOpen( ) && ( mriHandler( ).openMode( ) != PghMRIhandler::mriRead ) )
   {
    size_t nd = MiscFns::nDigits( MinQNB::statusMsgs.size( ) );

    string fmt = "%";
    cpp_scope_begin
     char ndstr[nd]; sprintf( ndstr, "%d", nd );
     fmt.append( ndstr ); fmt.append( "d" );
    cpp_scope_end

    string key = OutputProduct::name( ) + ".msg." + string( nd, '0' );
    string::size_type m = key.size( ) - nd;
    char s[nd];

    for ( size_t i = 0; i < MinQNB::statusMsgs.size( ); i++ )
     {
      sprintf( s, fmt.c_str( ), i ); key.replace( m, nd, s );
      mriHandler( ).setString( MiscFns::replace( key, " ", "0" ),
                               MiscFns::lowerCase( MiscFns::stripped( MinQNB::statusMsgs[i] ) ) );
     }
   }
 }

// ---------------------------------------- diagnostics: projected gradient norm
// -----------------------------------------------------------------------------

PrGradNorm::PrGradNorm( )
 : OutputProduct( "minimization.gradient.norm", 1, PghMRIhandler::mriFloat )
 {
 }

PrGradNorm::PrGradNorm( const PrGradNorm& another )
 : OutputProduct( another )
 {
 }

PrGradNorm::PrGradNorm&
PrGradNorm::operator=( const PrGradNorm& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

// ------------------------------------------------------- diagnostics: gradient
// -----------------------------------------------------------------------------

Gradient::Gradient( )
 : OutputProduct( "gradient", 1, PghMRIhandler::mriFloat )
 {
 }

Gradient::Gradient( const Gradient& another )
 : OutputProduct( another )
 {
 }

Gradient::Gradient&
Gradient::operator=( const Gradient& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

// -------------------------------------------------------- estimates: parameter
// -----------------------------------------------------------------------------

Estimates::Estimates( )
 : OutputProduct( "estimates", 1, PghMRIhandler::mriFloat )
 {
 }

Estimates::Estimates( const Estimates& another )
 : OutputProduct( another )
 {
 }

Estimates::Estimates&
Estimates::operator=( const Estimates& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

// --------------------------------------------------------- estimates: baseline
// -----------------------------------------------------------------------------

Baseline::Baseline( )
 : OutputProduct( "baseline", 1, PghMRIhandler::mriFloat )
 {
 }

Baseline::Baseline( const Baseline& another )
 : OutputProduct( another )
 {
 }

Baseline::Baseline&
Baseline::operator=( const Baseline& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

// ---------------------------------------------------- log posterior at maximum
// -----------------------------------------------------------------------------

LogPostMax::LogPostMax( )
 : OutputProduct( "log.posterior.max", 1, PghMRIhandler::mriFloat )
 {
 }

LogPostMax::LogPostMax( const LogPostMax& another )
 : OutputProduct( another )
 {
 }

LogPostMax::LogPostMax&
LogPostMax::operator=( const LogPostMax& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  return *this;
 }

// ---------------------------------------------------- base-grid profile: drift
// -----------------------------------------------------------------------------

DBprofile::DBprofile( )
 : OutputProduct( "profiles.drift", 1, PghMRIhandler::mriFloat ), my_dbp( )
 {
 }

DBprofile::DBprofile( const DBprofile& another )
 : OutputProduct( another ), my_dbp( another.my_dbp )
 {
 }

DBprofile::DBprofile&
DBprofile::operator=( const DBprofile& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  my_dbp = rhs.my_dbp;
  return *this;
 }

void
DBprofile::subInitialize( const string& chunkPrefix )
 {
  my_dbp = OutputProduct::drift( );

  if ( timeGrids( ).baseGrid( ).size( ) != my_dbp.timeGridSize( ) )
   {
    my_dbp.setTimeGrid(    timeGrids( ).initialTime( ),
                           timeGrids( ).finalTime( ),
                        &( timeGrids( ).baseGrid( ) ) );
    my_dbp.initialize( );
   }
 }

// ----------------------------------------------- base-grid profile: activation
// -----------------------------------------------------------------------------

ACprofile::ACprofile( )
 : OutputProduct( "profiles.activation", 1, PghMRIhandler::mriFloat ), my_acp( )
 {
 }

ACprofile::ACprofile( const ACprofile& another )
 : OutputProduct( another ), my_acp( another.my_acp )
 {
 }

ACprofile::ACprofile&
ACprofile::operator=( const ACprofile& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  my_acp = rhs.my_acp;
  return *this;
 }

void
ACprofile::subInitialize( const string& chunkPrefix )
 {
  my_acp = OutputProduct::activation( );

  if ( timeGrids( ).baseGrid( ).size( ) != my_acp.timeGridSize( ) )
   {
    my_acp.setTimeGrid( &( timeGrids( ).baseGrid( ) ) );
    my_acp.initialize( );
   }
 }

// ----------------------------------------------------- base-grid profile: fits
// -----------------------------------------------------------------------------

Fits::Fits( )
 : OutputProduct( "fits", 1, PghMRIhandler::mriFloat ), my_acp( ), my_dbp( )
 {
  vector<string> v( 2 );
  v[0] = "profiles.drift"; v[1] = "profiles.activation";
  setNeeds( v );
 }

Fits::Fits( const Fits& another )
 : OutputProduct( another ), my_acp( another.my_acp ), my_dbp( another.my_dbp )
 {
 }

Fits::Fits&
Fits::operator=( const Fits& rhs )
 {
  if ( this == &rhs ) return *this;
  OutputProduct::operator=( rhs );
  my_acp = rhs.my_acp;
  my_dbp = rhs.my_dbp;
  return *this;
 }

void
Fits::subInitialize( const string& chunkPrefix )
 {
  my_acp = static_cast<ACprofile*>( ProductRegistry::extract( "profiles.activation", needs( ) ) );
  my_dbp = static_cast<DBprofile*>( ProductRegistry::extract( "profiles.drift",      needs( ) ) );

  assert( my_acp != 0 ); assert( my_dbp != 0 );
 }

// ------------------------------------------------ base-grid profile: residuals
// -----------------------------------------------------------------------------

Residuals::Residuals( )
 : OutputProduct( "residuals", 1, PghMRIhandler::mriFloat, vector<string>( 1, "fits" ) ),
   my_numAR( ), my_arOfs( ), my_fits( )
 {
 }

Residuals::Residuals( const Residuals& another )
 : OutputProduct( another ),
   my_numAR( another.my_numAR ), my_arOfs( another.my_arOfs ), my_fits( another.my_fits )
 {
 }

Residuals::Residuals&
Residuals::operator=( const Residuals& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  my_numAR = rhs.my_numAR;
  my_arOfs = rhs.my_arOfs;

  my_fits  = rhs.my_fits;

  return *this;
 }

void
Residuals::subInitialize( const string& chunkPrefix )
 {
  my_numAR = parameterManager( ).blockLength( ParamBlkNames::ar );
  my_arOfs = parameterManager( ).blockOffset( ParamBlkNames::ar );

  my_fits = static_cast<Fits*>( ProductRegistry::extract( "fits", needs( ) ) );

  assert( my_fits != 0 );
 }

// ---------------------------------------------------------- hessian at minimum
// -----------------------------------------------------------------------------

Hessian::Hessian( )
 : OutputProduct( "hessian", 1, PghMRIhandler::mriFloat )
 {
 }

Hessian::Hessian( const Hessian& another )
 : OutputProduct( another )
 {
 }

Hessian::Hessian&
Hessian::operator=( const Hessian& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  return *this;
 }

void
Hessian::subInitialize( const string& chunkPrefix )
 {
  if ( pghmri( ) && mriHandler( ).isOpen( ) && ( mriHandler( ).openMode( ) != PghMRIhandler::mriRead ) )
   {
    int n = parameterManager( ).numParams( );
    char s[MiscFns::nDigits( n )]; sprintf( s, "%d", n );
    string mformat = "packed UC dimensions ";
    mformat.append( mriHandler( ).getString( chunkName( ) + ".dimensions" ).substr( 0, numLeadDims( ) ) );
    mformat.append( " size " ); mformat.append( s ); mformat.append( " X " ); mformat.append( s );
    // cerr << "(debug) Hessian::subInitialize( ) mformat " << mformat << endl;
    mriHandler( ).setString( chunkName( ) + ".format", mformat.c_str( ) );
   }
 }

// ----------------------------------------------------------------- covariances
// -----------------------------------------------------------------------------

Covariances::Covariances( double defaultOnError )
 : OutputProduct( "covariances", 1, PghMRIhandler::mriFloat, vector<string>( 1, "hessian" ) ),
   my_default( defaultOnError ), my_hessian( ), my_info1( ), my_info2( )
 {
 }

Covariances::Covariances( const Covariances& another )
 : OutputProduct( another ),
   my_default( another.my_default ), my_hessian( another.my_hessian ),
   my_info1( another.my_info1 ), my_info2( another.my_info2 )
 {
 }

Covariances::Covariances&
Covariances::operator=( const Covariances& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  my_default = rhs.my_default;
  my_hessian = rhs.my_hessian;

  my_info1 = rhs.my_info1;
  my_info2 = rhs.my_info2;

  return *this;
 }

void
Covariances::subInitialize( const string& chunkPrefix )
 {
  my_hessian = static_cast<Hessian*>( ProductRegistry::extract( "hessian", needs( ) ) );
  assert( my_hessian != 0 );

  if ( pghmri( ) && mriHandler( ).isOpen( ) && ( mriHandler( ).openMode( ) != PghMRIhandler::mriRead ) )
   {
    int n = parameterManager( ).numParams( );
    char s[MiscFns::nDigits( n )]; sprintf( s, "%d", n );
    string mformat = "packed UC dimensions ";
    mformat.append( mriHandler( ).getString( chunkName( ) + ".dimensions" ).substr( 0, numLeadDims( ) ) );
    mformat.append( " size " ); mformat.append( s ); mformat.append( " X " ); mformat.append( s );
    // cerr << "(debug) Covariances::subInitialize( ) mformat " << mformat << endl;
    mriHandler( ).setString( chunkName( ) + ".format", mformat.c_str( ) );
   }
 }

// ------------------------------------------------------------- standard errors
// -----------------------------------------------------------------------------

StdErrors::StdErrors( double defaultOnError, double threshold )
 : OutputProduct( "standard.errors", 1, PghMRIhandler::mriFloat, vector<string>( 1, "covariances" ) ),
   my_default( defaultOnError ), my_threshold( threshold ), my_cov( )
 {
 }

StdErrors::StdErrors( const StdErrors& another )
 : OutputProduct( another ),
   my_default( another.my_default ), my_threshold( another.my_threshold ), my_cov( another.my_cov )
 {
 }

StdErrors::StdErrors&
StdErrors::operator=( const StdErrors& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  my_default   = rhs.my_default;
  my_threshold = rhs.my_threshold;
  my_cov       = rhs.my_cov;

  return *this;
 }

void
StdErrors::subInitialize( const string& chunkPrefix )
 {
  my_cov = static_cast<Covariances*>( ProductRegistry::extract( "covariances", needs( ) ) );
  assert( my_cov != 0 );
 }

// ----------------------------------------------------- baseline standard error
// -----------------------------------------------------------------------------

BaselineStdErr::BaselineStdErr( double defaultOnError, double threshold )
 : OutputProduct( "baseline.stderr", 1, PghMRIhandler::mriFloat, vector<string>( 1, "covariances" ) ),
   my_default( defaultOnError ), my_threshold( threshold ), my_cov( )
 {
 }

BaselineStdErr::BaselineStdErr( const BaselineStdErr& another )
 : OutputProduct( another ),
   my_default( another.my_default ), my_threshold( another.my_threshold ), my_cov( another.my_cov )
 {
 }

BaselineStdErr::BaselineStdErr&
BaselineStdErr::operator=( const BaselineStdErr& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  my_default   = rhs.my_default;
  my_threshold = rhs.my_threshold;
  my_cov       = rhs.my_cov;

  return *this;
 }

void
BaselineStdErr::subInitialize( const string& chunkPrefix )
 {
  my_cov = static_cast<Covariances*>( ProductRegistry::extract( "covariances", needs( ) ) );
  assert( my_cov != 0 );
 }

// ------------------------------------------------------------- log determinant
// -----------------------------------------------------------------------------

LogDeterminant::LogDeterminant( double defaultOnError )
 : OutputProduct( "log.determinant", 1, PghMRIhandler::mriFloat, vector<string>( 1, "hessian" ) ),
   my_default( defaultOnError ), my_hessian( )
 {
 }

LogDeterminant::LogDeterminant( const LogDeterminant& another )
 : OutputProduct( another ), my_default( another.my_default ), my_hessian( another.my_hessian )
 {
 }

LogDeterminant::LogDeterminant&
LogDeterminant::operator=( const LogDeterminant& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  my_default = rhs.my_default;
  my_hessian = rhs.my_hessian;

  return *this;
 }

void
LogDeterminant::subInitialize( const string& chunkPrefix )
 {
  my_hessian = static_cast<Hessian*>( ProductRegistry::extract( "hessian", needs( ) ) );
  assert( my_hessian != 0 );
 }

// -------------------------------------------- positive definiteness of the fit
// -----------------------------------------------------------------------------

FitPositiveDefinite::FitPositiveDefinite( )
 : OutputProduct( "fit.positive.definite", 1, PghMRIhandler::mriUchar, vector<string>( 1, "log.determinant" ) ),
   my_logdet( )
 {
 }

FitPositiveDefinite::FitPositiveDefinite( const FitPositiveDefinite& another )
 : OutputProduct( another ), my_logdet( another.my_logdet )
 {
 }

FitPositiveDefinite::FitPositiveDefinite&
FitPositiveDefinite::operator=( const FitPositiveDefinite& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  my_logdet = rhs.my_logdet;

  return *this;
 }

void
FitPositiveDefinite::subInitialize( const string& chunkPrefix )
 {
  my_logdet = static_cast<LogDeterminant*>( ProductRegistry::extract( "log.determinant", needs( ) ) );
  assert( my_logdet != 0 );
 }

// ---------------------------------------------------------------- log integral
// -----------------------------------------------------------------------------

const double LogIntegral::our_ln2pi = log( 2.0 * M_PI );

LogIntegral::LogIntegral( double defaultOnError )
 : OutputProduct( "log.integral", 1, PghMRIhandler::mriDouble ), my_default( defaultOnError ), my_logdet( ), my_logpostmax( )
 {
  vector<string> v( 2 ); v[0] = "log.determinant"; v[1] = "log.posterior.max"; setNeeds( v );
 }

LogIntegral::LogIntegral( const LogIntegral& another )
 : OutputProduct( another ), my_default( another.my_default ), my_logdet( another.my_logdet ), my_logpostmax( another.my_logpostmax )
 {
 }

LogIntegral::LogIntegral&
LogIntegral::operator=( const LogIntegral& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  my_default = rhs.my_default;
  my_logdet = rhs.my_logdet;
  my_logpostmax = rhs.my_logpostmax;

  return *this;
 }

void
LogIntegral::subInitialize( const string& chunkPrefix )
 {
  my_logdet = static_cast<LogDeterminant*>( ProductRegistry::extract( "log.determinant", needs( ) ) );
  assert( my_logdet != 0 );

  my_logpostmax = static_cast<LogPostMax*>( ProductRegistry::extract( "log.posterior.max", needs( ) ) );
  assert( my_logpostmax != 0 );
 }

/* =============================================================================
// ------------------------------------------------------------ log bayes factor
// -----------------------------------------------------------------------------

const double LogBayesFactor::our_lnDBLmax = log( DBL_MAX );

LogBayesFactor::LogBayesFactor( double defaultOnError )
 : OutputProduct( "log.bayes.factor", 1, PghMRIhandler::mriDouble, vector<string>( 1, "log.integral" ) ),
   my_default( defaultOnError ), my_logint( ), my_logintNull( )
 {
 }

LogBayesFactor::LogBayesFactor( const LogBayesFactor& another )
 : OutputProduct( another ), my_default( another.my_default ), my_logint( another.my_logint ), my_logintNull( another.my_logintNull )
 {
 }

LogBayesFactor::LogBayesFactor&
LogBayesFactor::operator=( const LogBayesFactor& rhs )
 {
  if ( this == &rhs ) return *this;

  OutputProduct::operator=( rhs );

  my_default = rhs.my_default;
  my_logint = rhs.my_logint;
  my_logintNull = rhs.my_logintNull;

  return *this;
 }

void
LogBayesFactor::setNullLI( const LogIntegral* nullSubmodelLI )
 {
  my_logintNull = nullSubmodelLI;
 }

void
LogBayesFactor::subInitialize( const string& chunkPrefix )
 {
  my_logint = static_cast<LogIntegral*>( ProductRegistry::extract( "log.integral", needs( ) ) );
  assert( my_logint != 0 );

  if ( activation( ).numActive( ) > 0 ) assert( my_logintNull != 0 );

  if ( pghmri( ) && mriHandler( ).isOpen( ) && ( mriHandler( ).openMode( ) != PghMRIhandler::mriRead ) )
   mriHandler( ).setString( my_chunkName + ".with.respect.to",
                            "submodel " + SubmodelTable::key( my_logingNull->activation( ).submodel( ) ) );
 }
   ============================================================================= */
