#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# csh script check.class.csh
#   script to check if a standard C++ class is supported by a compiler.
#   see the script below for details for the four possibilities tested.
#
# usage
#   usage: check.class.csh class header "declaration line" compilation_command
#
# notes
#   0. return codes
#        0 supported
#        1 all the possibilities tested failed
#   1. not a fool-proof method; results need verification
#   2. depends on the compiler returning a 0 return code for success
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# c++ standard class to check
#
 set class = ${argv[1]}

# c++ standard header for class
#
 set header = ${argv[2]}

# variable declaration line
#
 set declaration = ( ${argv[3]} )

# compilation command (need a.out as the output file)
#
 set compile = ( ${argv[4-]} )

# test filename prefix
#
 set prefix = "cpp.class."

# check possibility 1
#
 echo "\n${0:t}: possibility 1 (class ${class}, header ${header}, declaration ${declaration})\n"

 cat > ${prefix}${class}.cc << @EOF
#include <${header}>
int main ( ) { ${declaration} c; return 0; }
@EOF

 cat ${prefix}${class}.cc; echo ""

 ${compile} ${prefix}${class}.cc

 if ( ${status} == 0 ) then
  set exitstatus = 0
 else
  set exitstatus = 1
 endif

# check possibility 2
#
 if ( ${exitstatus} != 0 ) then
  echo "\n${0:t}: possibility 2 (class ${class}, header ${header}, declaration std::${declaration})\n"

  cat > ${prefix}${class}.cc << @EOF
#include <${header}>
int main ( ) { using namespace std; ${declaration} c; return 0; }
@EOF

  cat ${prefix}${class}.cc; echo ""

  ${compile} ${prefix}${class}.cc

  if ( ${status} == 0 ) then
   set exitstatus = 0
  else
   set exitstatus = 1
  endif
 endif

# check possibility 3
#
 if ( ${exitstatus} != 0 ) then
  echo "\n${0:t}: possibility 3 (class ${class}, header ${header}.h, declaration ${declaration})\n"

  cat > ${prefix}${class}.cc << @EOF
#include <${header}.h>
int main ( ) { ${declaration} c; return 0; }
@EOF

  cat ${prefix}${class}.cc; echo ""

  ${compile} ${prefix}${class}.cc

  if ( ${status} == 0 ) then
   set exitstatus = 0
  else
   set exitstatus = 1
  endif
 endif

# check possibility 4
#
 if ( ${exitstatus} != 0 ) then
  echo "\n${0:t}: possibility 4 (class ${class}, header ${header}.h, declaration std::${declaration})\n"

  cat > ${prefix}${class}.cc << @EOF
#include <${header}.h>
int main ( ) { using namespace std; ${declaration} c; return 0; }
@EOF

  cat ${prefix}${class}.cc; echo ""

  ${compile} ${prefix}${class}.cc

  if ( ${status} == 0 ) then
   set exitstatus = 0
  else
   set exitstatus = 1
  endif
 endif

 /bin/rm -f ${prefix}${class}.* a.out

 exit ${exitstatus}
