#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# csh script check.header.c++
#
# description
#   script to check if a standard c++ header
#   is suported by C++ compiler under the standard name.
#
#   checks for the following two possibilities:
#
#   1. <header>
#   2. <header.h>
# 
# usage
#   usage: check.header.c++ header createstub StubDir compilation_command
#
# notes
#   0. return codes
#        0 header header exits under standard name header
#        1 header header exits under nonstandard name header.h
#        2 neither of these two are supported
#   1. when return code == 1 and createstub == 1,
#      the stub include file StubDir/header is created,
#      with just a simple #include <header.h>
#   2. depends on the compiler returning a 0 return code for success
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# c++ header to check
#
 set header = ${argv[1]}

# create stub or not (1/0)
#
 set createstub = ${argv[2]}

# directory for the stub
#
 set stubdir = ${argv[3]}

# compilation command
#
 set compile = ( ${argv[4-]} )

# test filename prefix
#
 set prefix = "cpp.cppheader."

# check if it exists as ${header}
#
 echo "\n${0:t}: header ${header}, included as ${header})\n"

 cat > ${prefix}${header}.cc << @EOF
#include <${header}>
int main ( ) { return 0; }
@EOF

 cat ${prefix}${header}.cc; echo ""

 ${compile} ${prefix}${header}.cc

 if ( ${status} == 0 ) then
  set exitstatus = 0
 else
  set exitstatus = 1
 endif

# check if it exists as ${header}.h
#
 if ( ${exitstatus} != 0 ) then
  echo "\n${0:t}: header ${header}, included as ${header}.h)\n"

  cat > ${prefix}${header}.cc << @EOF
#include <${header}.h>
int main ( ) { return 0; }
@EOF

  cat ${prefix}${header}.cc; echo ""

  ${compile} ${prefix}${header}.cc

  if ( ${status} == 0 ) then
   set exitstatus = 1
  else
   set exitstatus = 2
  endif
 endif

# create stub
#
 if ( ( ${exitstatus} == 1 ) && ( ${createstub} == 1 ) ) then
  cat > ${stubdir}/${header} << @EOF
#ifndef __cppheader_${header}__
#define __cppheader_${header}__

/* stub for standard c++ header with nonstandard filename */

#undef cpp_missing_header_${header}

#include <${header}.h>

#endif /* __cppheader_${header}__ */
@EOF
 endif

 /bin/rm -f ${prefix}${header}.* a.out

 exit ${exitstatus}
