/* --------------------------------------------------------------------------
 *
 * namespace cppblas_fl
 *   fortran link conventions handler for cppblas.h
 *
 *   different fortran compilers follow different conventions for fortran
 *   function names; e. g. some append an underscore, some capitalize it,
 *   etc. To handle  these  variations in  a transparent manner,  we  use
 *   the value of the definition __FORTRAN_LINK_CONVENTION__ to represent
 *   a convention, which initiates appropriate modifications to  function
 *   names in prototypes and in call:
 *
 *   __FORTRAN_LINK_CONVENTION__     represents
 *
 *                             0     no change
 *                             1     append underscore
 *                           ...     more possibilities as required
 *
 * required headers
 *   see include header file cppblas
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __cppblas_fl_H__
#define __cppblas_fl_H__

#ifndef __FORTRAN_LINK_CONVENTION__
 #define __FORTRAN_LINK_CONVENTION__ 0
#endif

#if ( __FORTRAN_LINK_CONVENTION__ == 1 )
 #define srotg srotg_
 #define drotg drotg_
 #define srotmg srotmg_
 #define drotmg drotmg_
 #define srot srot_
 #define drot drot_
 #define srotm srotm_
 #define drotm drotm_
 #define sswap sswap_
 #define dswap dswap_
 #define cswap cswap_
 #define zswap zswap_
 #define sscal sscal_
 #define dscal dscal_
 #define cscal cscal_
 #define zscal zscal_
 #define csscal csscal_
 #define zdscal zdscal_
 #define scopy scopy_
 #define dcopy dcopy_
 #define ccopy ccopy_
 #define zcopy zcopy_
 #define saxpy saxpy_
 #define daxpy daxpy_
 #define caxpy caxpy_
 #define zaxpy zaxpy_
 #define sdot sdot_
 #define dsdot dsdot_
 #define ddot ddot_
 #define cdotu cdotu_
 #define zdotu zdotu_
 #define cdotc cdotc_
 #define zdotc zdotc_
 #define snrm2 snrm2_
 #define dnrm2 dnrm2_
 #define scnrm2 scnrm2_
 #define dznrm2 dznrm2_
 #define sasum sasum_
 #define dasum dasum_
 #define scasum scasum_
 #define zdasum zdasum_
 #define isamax isamax_
 #define idamax idamax_
 #define icamax icamax_
 #define izamax izamax_
 #define sgemv sgemv_
 #define dgemv dgemv_
 #define cgemv cgemv_
 #define zgemv zgemv_
 #define sgbmv sgbmv_
 #define dgbmv dgbmv_
 #define cgbmv cgbmv_
 #define zgbmv zgbmv_
 #define chemv chemv_
 #define zhemv zhemv_
 #define chbmv chbmv_
 #define zhbmv zhbmv_
 #define chpmv chpmv_
 #define zhpmv zhpmv_
 #define ssymv ssymv_
 #define dsymv dsymv_
 #define ssbmv ssbmv_
 #define dsbmv dsbmv_
 #define sspmv sspmv_
 #define dspmv dspmv_
 #define strmv strmv_
 #define dtrmv dtrmv_
 #define ctrmv ctrmv_
 #define ztrmv ztrmv_
 #define stbmv stbmv_
 #define dtbmv dtbmv_
 #define ctbmv ctbmv_
 #define ztbmv ztbmv_
 #define stpmv stpmv_
 #define dtpmv dtpmv_
 #define ctpmv ctpmv_
 #define ztpmv ztpmv_
 #define strsv strsv_
 #define dtrsv dtrsv_
 #define ctrsv ctrsv_
 #define ztrsv ztrsv_
 #define stbsv stbsv_
 #define dtbsv dtbsv_
 #define ctbsv ctbsv_
 #define ztbsv ztbsv_
 #define stpsv stpsv_
 #define dtpsv dtpsv_
 #define ctpsv ctpsv_
 #define ztpsv ztpsv_
 #define sger sger_
 #define dger dger_
 #define cgeru cgeru_
 #define zgeru zgeru_
 #define cgerc cgerc_
 #define zgerc zgerc_
 #define cher cher_
 #define zher zher_
 #define chpr chpr_
 #define zhpr zhpr_
 #define ssyr ssyr_
 #define dsyr dsyr_
 #define sspr sspr_
 #define dspr dspr_
 #define cher2 cher2_
 #define zher2 zher2_
 #define chpr2 chpr2_
 #define zhpr2 zhpr2_
 #define ssyr2 ssyr2_
 #define dsyr2 dsyr2_
 #define sspr2 sspr2_
 #define dspr2 dspr2_
 #define sgemm sgemm_
 #define dgemm dgemm_
 #define cgemm cgemm_
 #define zgemm zgemm_
 #define ssymm ssymm_
 #define dsymm dsymm_
 #define csymm csymm_
 #define zsymm zsymm_
 #define chemm chemm_
 #define zhemm zhemm_
 #define ssyrk ssyrk_
 #define dsyrk dsyrk_
 #define csyrk csyrk_
 #define zsyrk zsyrk_
 #define cherk cherk_
 #define zherk zherk_
 #define cher2k cher2k_
 #define zher2k zher2k_
 #define ssyr2k ssyr2k_
 #define dsyr2k dsyr2k_
 #define cher2k cher2k_
 #define zher2k zher2k_
 #define strmm strmm_
 #define dtrmm dtrmm_
 #define ctrmm ctrmm_
 #define ztrmm ztrmm_
 #define strsm strsm_
 #define dtrsm dtrsm_
 #define ctrsm ctrsm_
 #define ztrsm ztrsm_

 #define lsame lsame_
 #define lsamen lsamen_
 #define xerbla xerbla_
#endif /* __FORTRAN_LINK_CONVENTION__ == 1 */

#endif /* __cppblas_fl_H__ */
