/* --------------------------------------------------------------------------
 *
 * namespace cpplapack_fl
 *   fortran link conventions handler for cpplapack.h
 *
 *   different fortran compilers follow different conventions for fortran
 *   function names; e. g. some append an underscore, some capitalize it,
 *   etc. To handle  these  variations in  a transparent manner,  we  use
 *   the value of the definition __FORTRAN_LINK_CONVENTION__ to represent
 *   a convention, which initiates appropriate modifications to  function
 *   names in prototypes and in call:
 *
 *   __FORTRAN_LINK_CONVENTION__     represents
 *
 *                             0     no change
 *                             1     append underscore
 *                           ...     more possibilities as required
 *
 * required headers
 *   see include header file cpplapack
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __cpplapack_fl_H__
#define __cpplapack_fl_H__

#ifndef __FORTRAN_LINK_CONVENTION__
 #define __FORTRAN_LINK_CONVENTION__ 0
#endif

#if ( __FORTRAN_LINK_CONVENTION__ == 1 )
 #define cbdsqr cbdsqr_
 #define cgbbrd cgbbrd_
 #define cgbcon cgbcon_
 #define cgbequ cgbequ_
 #define cgbrfs cgbrfs_
 #define cgbsv cgbsv_
 #define cgbsvx cgbsvx_
 #define cgbtf2 cgbtf2_
 #define cgbtrf cgbtrf_
 #define cgbtrs cgbtrs_
 #define cgebak cgebak_
 #define cgebal cgebal_
 #define cgebd2 cgebd2_
 #define cgebrd cgebrd_
 #define cgecon cgecon_
 #define cgeequ cgeequ_
 #define cgees cgees_
 #define cgeesx cgeesx_
 #define cgeev cgeev_
 #define cgeevx cgeevx_
 #define cgegs cgegs_
 #define cgegv cgegv_
 #define cgehd2 cgehd2_
 #define cgehrd cgehrd_
 #define cgelq2 cgelq2_
 #define cgelqf cgelqf_
 #define cgels cgels_
 #define cgelss cgelss_
 #define cgelsx cgelsx_
 #define cgeql2 cgeql2_
 #define cgeqlf cgeqlf_
 #define cgeqpf cgeqpf_
 #define cgeqr2 cgeqr2_
 #define cgeqrf cgeqrf_
 #define cgerfs cgerfs_
 #define cgerq2 cgerq2_
 #define cgerqf cgerqf_
 #define cgesv cgesv_
 #define cgesvd cgesvd_
 #define cgesvx cgesvx_
 #define cgetf2 cgetf2_
 #define cgetrf cgetrf_
 #define cgetri cgetri_
 #define cgetrs cgetrs_
 #define cggbak cggbak_
 #define cggbal cggbal_
 #define cggglm cggglm_
 #define cgghrd cgghrd_
 #define cgglse cgglse_
 #define cggqrf cggqrf_
 #define cggrqf cggrqf_
 #define cggsvd cggsvd_
 #define cggsvp cggsvp_
 #define cgtcon cgtcon_
 #define cgtrfs cgtrfs_
 #define cgtsv cgtsv_
 #define cgtsvx cgtsvx_
 #define cgttrf cgttrf_
 #define cgttrs cgttrs_
 #define chbev chbev_
 #define chbevd chbevd_
 #define chbevx chbevx_
 #define chbgst chbgst_
 #define chbgv chbgv_
 #define chbtrd chbtrd_
 #define checon checon_
 #define cheev cheev_
 #define cheevd cheevd_
 #define cheevx cheevx_
 #define chegs2 chegs2_
 #define chegst chegst_
 #define chegv chegv_
 #define cherfs cherfs_
 #define chesv chesv_
 #define chesvx chesvx_
 #define chetd2 chetd2_
 #define chetf2 chetf2_
 #define chetrd chetrd_
 #define chetrf chetrf_
 #define chetri chetri_
 #define chetrs chetrs_
 #define chgeqz chgeqz_
 #define chpcon chpcon_
 #define chpev chpev_
 #define chpevd chpevd_
 #define chpevx chpevx_
 #define chpgst chpgst_
 #define chpgv chpgv_
 #define chprfs chprfs_
 #define chpsv chpsv_
 #define chpsvx chpsvx_
 #define chptrd chptrd_
 #define chptrf chptrf_
 #define chptri chptri_
 #define chptrs chptrs_
 #define chsein chsein_
 #define chseqr chseqr_
 #define clabrd clabrd_
 #define clacgv clacgv_
 #define clacon clacon_
 #define clacpy clacpy_
 #define clacrm clacrm_
 #define clacrt clacrt_
 #define cladiv cladiv_
 #define claed0 claed0_
 #define claed7 claed7_
 #define claed8 claed8_
 #define claein claein_
 #define claesy claesy_
 #define claev2 claev2_
 #define clags2 clags2_
 #define clagtm clagtm_
 #define clahef clahef_
 #define clahqr clahqr_
 #define clahrd clahrd_
 #define claic1 claic1_
 #define clangb clangb_
 #define clange clange_
 #define clangt clangt_
 #define clanhb clanhb_
 #define clanhe clanhe_
 #define clanhp clanhp_
 #define clanhs clanhs_
 #define clanht clanht_
 #define clansb clansb_
 #define clansp clansp_
 #define clansy clansy_
 #define clantb clantb_
 #define clantp clantp_
 #define clantr clantr_
 #define clapll clapll_
 #define clapmt clapmt_
 #define claqgb claqgb_
 #define claqge claqge_
 #define claqhb claqhb_
 #define claqhe claqhe_
 #define claqhp claqhp_
 #define claqsb claqsb_
 #define claqsp claqsp_
 #define claqsy claqsy_
 #define clar2v clar2v_
 #define clarf clarf_
 #define clarfb clarfb_
 #define clarfg clarfg_
 #define clarft clarft_
 #define clarfx clarfx_
 #define clargv clargv_
 #define clarnv clarnv_
 #define clartg clartg_
 #define clartv clartv_
 #define clascl clascl_
 #define claset claset_
 #define clasr clasr_
 #define classq classq_
 #define claswp claswp_
 #define clasyf clasyf_
 #define clatbs clatbs_
 #define clatps clatps_
 #define clatrd clatrd_
 #define clatrs clatrs_
 #define clatzm clatzm_
 #define clauu2 clauu2_
 #define clauum clauum_
 #define cpbcon cpbcon_
 #define cpbequ cpbequ_
 #define cpbrfs cpbrfs_
 #define cpbstf cpbstf_
 #define cpbsv cpbsv_
 #define cpbsvx cpbsvx_
 #define cpbtf2 cpbtf2_
 #define cpbtrf cpbtrf_
 #define cpbtrs cpbtrs_
 #define cpocon cpocon_
 #define cpoequ cpoequ_
 #define cporfs cporfs_
 #define cposv cposv_
 #define cposvx cposvx_
 #define cpotf2 cpotf2_
 #define cpotrf cpotrf_
 #define cpotri cpotri_
 #define cpotrs cpotrs_
 #define cppcon cppcon_
 #define cppequ cppequ_
 #define cpprfs cpprfs_
 #define cppsv cppsv_
 #define cppsvx cppsvx_
 #define cpptrf cpptrf_
 #define cpptri cpptri_
 #define cpptrs cpptrs_
 #define cptcon cptcon_
 #define cpteqr cpteqr_
 #define cptrfs cptrfs_
 #define cptsv cptsv_
 #define cptsvx cptsvx_
 #define cpttrf cpttrf_
 #define cpttrs cpttrs_
 #define crot crot_
 #define cspcon cspcon_
 #define cspmv cspmv_
 #define cspr cspr_
 #define csprfs csprfs_
 #define cspsv cspsv_
 #define cspsvx cspsvx_
 #define csptrf csptrf_
 #define csptri csptri_
 #define csptrs csptrs_
 #define csrot csrot_
 #define csrscl csrscl_
 #define cstedc cstedc_
 #define cstein cstein_
 #define csteqr csteqr_
 #define csycon csycon_
 #define csymv csymv_
 #define csyr csyr_
 #define csyrfs csyrfs_
 #define csysv csysv_
 #define csysvx csysvx_
 #define csytf2 csytf2_
 #define csytrf csytrf_
 #define csytri csytri_
 #define csytrs csytrs_
 #define ctbcon ctbcon_
 #define ctbrfs ctbrfs_
 #define ctbtrs ctbtrs_
 #define ctgevc ctgevc_
 #define ctgsja ctgsja_
 #define ctpcon ctpcon_
 #define ctprfs ctprfs_
 #define ctptri ctptri_
 #define ctptrs ctptrs_
 #define ctrcon ctrcon_
 #define ctrevc ctrevc_
 #define ctrexc ctrexc_
 #define ctrrfs ctrrfs_
 #define ctrsen ctrsen_
 #define ctrsna ctrsna_
 #define ctrsyl ctrsyl_
 #define ctrti2 ctrti2_
 #define ctrtri ctrtri_
 #define ctrtrs ctrtrs_
 #define ctzrqf ctzrqf_
 #define cung2l cung2l_
 #define cung2r cung2r_
 #define cungbr cungbr_
 #define cunghr cunghr_
 #define cungl2 cungl2_
 #define cunglq cunglq_
 #define cungql cungql_
 #define cungqr cungqr_
 #define cungr2 cungr2_
 #define cungrq cungrq_
 #define cungtr cungtr_
 #define cunm2l cunm2l_
 #define cunm2r cunm2r_
 #define cunmbr cunmbr_
 #define cunmhr cunmhr_
 #define cunml2 cunml2_
 #define cunmlq cunmlq_
 #define cunmql cunmql_
 #define cunmqr cunmqr_
 #define cunmr2 cunmr2_
 #define cunmrq cunmrq_
 #define cunmtr cunmtr_
 #define cupgtr cupgtr_
 #define cupmtr cupmtr_
 #define dbdsqr dbdsqr_
 #define ddisna ddisna_
 #define dgbbrd dgbbrd_
 #define dgbcon dgbcon_
 #define dgbequ dgbequ_
 #define dgbrfs dgbrfs_
 #define dgbsv dgbsv_
 #define dgbsvx dgbsvx_
 #define dgbtf2 dgbtf2_
 #define dgbtrf dgbtrf_
 #define dgbtrs dgbtrs_
 #define dgebak dgebak_
 #define dgebal dgebal_
 #define dgebd2 dgebd2_
 #define dgebrd dgebrd_
 #define dgecon dgecon_
 #define dgeequ dgeequ_
 #define dgees dgees_
 #define dgeesx dgeesx_
 #define dgeev dgeev_
 #define dgeevx dgeevx_
 #define dgegs dgegs_
 #define dgegv dgegv_
 #define dgehd2 dgehd2_
 #define dgehrd dgehrd_
 #define dgelq2 dgelq2_
 #define dgelqf dgelqf_
 #define dgels dgels_
 #define dgelss dgelss_
 #define dgelsx dgelsx_
 #define dgeql2 dgeql2_
 #define dgeqlf dgeqlf_
 #define dgeqpf dgeqpf_
 #define dgeqr2 dgeqr2_
 #define dgeqrf dgeqrf_
 #define dgerfs dgerfs_
 #define dgerq2 dgerq2_
 #define dgerqf dgerqf_
 #define dgesv dgesv_
 #define dgesvd dgesvd_
 #define dgesvx dgesvx_
 #define dgetf2 dgetf2_
 #define dgetrf dgetrf_
 #define dgetri dgetri_
 #define dgetrs dgetrs_
 #define dggbak dggbak_
 #define dggbal dggbal_
 #define dggglm dggglm_
 #define dgghrd dgghrd_
 #define dgglse dgglse_
 #define dggqrf dggqrf_
 #define dggrqf dggrqf_
 #define dggsvd dggsvd_
 #define dggsvp dggsvp_
 #define dgtcon dgtcon_
 #define dgtrfs dgtrfs_
 #define dgtsv dgtsv_
 #define dgtsvx dgtsvx_
 #define dgttrf dgttrf_
 #define dgttrs dgttrs_
 #define dhgeqz dhgeqz_
 #define dhsein dhsein_
 #define dhseqr dhseqr_
 #define dlabad dlabad_
 #define dlabrd dlabrd_
 #define dlacon dlacon_
 #define dlacpy dlacpy_
 #define dladiv dladiv_
 #define dlae2 dlae2_
 #define dlaebz dlaebz_
 #define dlaed0 dlaed0_
 #define dlaed1 dlaed1_
 #define dlaed2 dlaed2_
 #define dlaed3 dlaed3_
 #define dlaed4 dlaed4_
 #define dlaed5 dlaed5_
 #define dlaed6 dlaed6_
 #define dlaed7 dlaed7_
 #define dlaed8 dlaed8_
 #define dlaed9 dlaed9_
 #define dlaeda dlaeda_
 #define dlaein dlaein_
 #define dlaev2 dlaev2_
 #define dlaexc dlaexc_
 #define dlag2 dlag2_
 #define dlags2 dlags2_
 #define dlagtf dlagtf_
 #define dlagtm dlagtm_
 #define dlagts dlagts_
 #define dlahqr dlahqr_
 #define dlahrd dlahrd_
 #define dlaic1 dlaic1_
 #define dlaln2 dlaln2_
 #define dlamch dlamch_
 #define dlamrg dlamrg_
 #define dlangb dlangb_
 #define dlange dlange_
 #define dlangt dlangt_
 #define dlanhs dlanhs_
 #define dlansb dlansb_
 #define dlansp dlansp_
 #define dlanst dlanst_
 #define dlansy dlansy_
 #define dlantb dlantb_
 #define dlantp dlantp_
 #define dlantr dlantr_
 #define dlanv2 dlanv2_
 #define dlapll dlapll_
 #define dlapmt dlapmt_
 #define dlapy2 dlapy2_
 #define dlapy3 dlapy3_
 #define dlaqgb dlaqgb_
 #define dlaqge dlaqge_
 #define dlaqsb dlaqsb_
 #define dlaqsp dlaqsp_
 #define dlaqsy dlaqsy_
 #define dlaqtr dlaqtr_
 #define dlar2v dlar2v_
 #define dlarf dlarf_
 #define dlarfb dlarfb_
 #define dlarfg dlarfg_
 #define dlarft dlarft_
 #define dlarfx dlarfx_
 #define dlargv dlargv_
 #define dlarnv dlarnv_
 #define dlartg dlartg_
 #define dlartv dlartv_
 #define dlaruv dlaruv_
 #define dlas2 dlas2_
 #define dlascl dlascl_
 #define dlaset dlaset_
 #define dlasq1 dlasq1_
 #define dlasq2 dlasq2_
 #define dlasq3 dlasq3_
 #define dlasq4 dlasq4_
 #define dlasr dlasr_
 #define dlasrt dlasrt_
 #define dlassq dlassq_
 #define dlasv2 dlasv2_
 #define dlaswp dlaswp_
 #define dlasy2 dlasy2_
 #define dlasyf dlasyf_
 #define dlatbs dlatbs_
 #define dlatps dlatps_
 #define dlatrd dlatrd_
 #define dlatrs dlatrs_
 #define dlatzm dlatzm_
 #define dlauu2 dlauu2_
 #define dlauum dlauum_
 #define dopgtr dopgtr_
 #define dopmtr dopmtr_
 #define dorg2l dorg2l_
 #define dorg2r dorg2r_
 #define dorgbr dorgbr_
 #define dorghr dorghr_
 #define dorgl2 dorgl2_
 #define dorglq dorglq_
 #define dorgql dorgql_
 #define dorgqr dorgqr_
 #define dorgr2 dorgr2_
 #define dorgrq dorgrq_
 #define dorgtr dorgtr_
 #define dorm2l dorm2l_
 #define dorm2r dorm2r_
 #define dormbr dormbr_
 #define dormhr dormhr_
 #define dorml2 dorml2_
 #define dormlq dormlq_
 #define dormql dormql_
 #define dormqr dormqr_
 #define dormr2 dormr2_
 #define dormrq dormrq_
 #define dormtr dormtr_
 #define dpbcon dpbcon_
 #define dpbequ dpbequ_
 #define dpbrfs dpbrfs_
 #define dpbstf dpbstf_
 #define dpbsv dpbsv_
 #define dpbsvx dpbsvx_
 #define dpbtf2 dpbtf2_
 #define dpbtrf dpbtrf_
 #define dpbtrs dpbtrs_
 #define dpocon dpocon_
 #define dpoequ dpoequ_
 #define dporfs dporfs_
 #define dposv dposv_
 #define dposvx dposvx_
 #define dpotf2 dpotf2_
 #define dpotrf dpotrf_
 #define dpotri dpotri_
 #define dpotrs dpotrs_
 #define dppcon dppcon_
 #define dppequ dppequ_
 #define dpprfs dpprfs_
 #define dppsv dppsv_
 #define dppsvx dppsvx_
 #define dpptrf dpptrf_
 #define dpptri dpptri_
 #define dpptrs dpptrs_
 #define dptcon dptcon_
 #define dpteqr dpteqr_
 #define dptrfs dptrfs_
 #define dptsv dptsv_
 #define dptsvx dptsvx_
 #define dpttrf dpttrf_
 #define dpttrs dpttrs_
 #define drscl drscl_
 #define dsbev dsbev_
 #define dsbevd dsbevd_
 #define dsbevx dsbevx_
 #define dsbgst dsbgst_
 #define dsbgv dsbgv_
 #define dsbtrd dsbtrd_
 #define dspcon dspcon_
 #define dspev dspev_
 #define dspevd dspevd_
 #define dspevx dspevx_
 #define dspgst dspgst_
 #define dspgv dspgv_
 #define dsprfs dsprfs_
 #define dspsv dspsv_
 #define dspsvx dspsvx_
 #define dsptrd dsptrd_
 #define dsptrf dsptrf_
 #define dsptri dsptri_
 #define dsptrs dsptrs_
 #define dstebz dstebz_
 #define dstedc dstedc_
 #define dstein dstein_
 #define dsteqr dsteqr_
 #define dsterf dsterf_
 #define dstev dstev_
 #define dstevd dstevd_
 #define dstevx dstevx_
 #define dsycon dsycon_
 #define dsyev dsyev_
 #define dsyevd dsyevd_
 #define dsyevx dsyevx_
 #define dsygs2 dsygs2_
 #define dsygst dsygst_
 #define dsygv dsygv_
 #define dsyrfs dsyrfs_
 #define dsysv dsysv_
 #define dsysvx dsysvx_
 #define dsytd2 dsytd2_
 #define dsytf2 dsytf2_
 #define dsytrd dsytrd_
 #define dsytrf dsytrf_
 #define dsytri dsytri_
 #define dsytrs dsytrs_
 #define dtbcon dtbcon_
 #define dtbrfs dtbrfs_
 #define dtbtrs dtbtrs_
 #define dtgevc dtgevc_
 #define dtgsja dtgsja_
 #define dtpcon dtpcon_
 #define dtprfs dtprfs_
 #define dtptri dtptri_
 #define dtptrs dtptrs_
 #define dtrcon dtrcon_
 #define dtrevc dtrevc_
 #define dtrexc dtrexc_
 #define dtrrfs dtrrfs_
 #define dtrsen dtrsen_
 #define dtrsna dtrsna_
 #define dtrsyl dtrsyl_
 #define dtrti2 dtrti2_
 #define dtrtri dtrtri_
 #define dtrtrs dtrtrs_
 #define dtzrqf dtzrqf_
 #define dzsum1 dzsum1_
 #define icmax1 icmax1_
 #define ilaenv ilaenv_
 #define izmax1 izmax1_
 #define sbdsqr sbdsqr_
 #define scsum1 scsum1_
 #define sdisna sdisna_
 #define sgbbrd sgbbrd_
 #define sgbcon sgbcon_
 #define sgbequ sgbequ_
 #define sgbrfs sgbrfs_
 #define sgbsv sgbsv_
 #define sgbsvx sgbsvx_
 #define sgbtf2 sgbtf2_
 #define sgbtrf sgbtrf_
 #define sgbtrs sgbtrs_
 #define sgebak sgebak_
 #define sgebal sgebal_
 #define sgebd2 sgebd2_
 #define sgebrd sgebrd_
 #define sgecon sgecon_
 #define sgeequ sgeequ_
 #define sgees sgees_
 #define sgeesx sgeesx_
 #define sgeev sgeev_
 #define sgeevx sgeevx_
 #define sgegs sgegs_
 #define sgegv sgegv_
 #define sgehd2 sgehd2_
 #define sgehrd sgehrd_
 #define sgelq2 sgelq2_
 #define sgelqf sgelqf_
 #define sgels sgels_
 #define sgelss sgelss_
 #define sgelsx sgelsx_
 #define sgeql2 sgeql2_
 #define sgeqlf sgeqlf_
 #define sgeqpf sgeqpf_
 #define sgeqr2 sgeqr2_
 #define sgeqrf sgeqrf_
 #define sgerfs sgerfs_
 #define sgerq2 sgerq2_
 #define sgerqf sgerqf_
 #define sgesv sgesv_
 #define sgesvd sgesvd_
 #define sgesvx sgesvx_
 #define sgetf2 sgetf2_
 #define sgetrf sgetrf_
 #define sgetri sgetri_
 #define sgetrs sgetrs_
 #define sggbak sggbak_
 #define sggbal sggbal_
 #define sggglm sggglm_
 #define sgghrd sgghrd_
 #define sgglse sgglse_
 #define sggqrf sggqrf_
 #define sggrqf sggrqf_
 #define sggsvd sggsvd_
 #define sggsvp sggsvp_
 #define sgtcon sgtcon_
 #define sgtrfs sgtrfs_
 #define sgtsv sgtsv_
 #define sgtsvx sgtsvx_
 #define sgttrf sgttrf_
 #define sgttrs sgttrs_
 #define shgeqz shgeqz_
 #define shsein shsein_
 #define shseqr shseqr_
 #define slabad slabad_
 #define slabrd slabrd_
 #define slacon slacon_
 #define slacpy slacpy_
 #define sladiv sladiv_
 #define slae2 slae2_
 #define slaebz slaebz_
 #define slaed0 slaed0_
 #define slaed1 slaed1_
 #define slaed2 slaed2_
 #define slaed3 slaed3_
 #define slaed4 slaed4_
 #define slaed5 slaed5_
 #define slaed6 slaed6_
 #define slaed7 slaed7_
 #define slaed8 slaed8_
 #define slaed9 slaed9_
 #define slaeda slaeda_
 #define slaein slaein_
 #define slaev2 slaev2_
 #define slaexc slaexc_
 #define slag2 slag2_
 #define slags2 slags2_
 #define slagtf slagtf_
 #define slagtm slagtm_
 #define slagts slagts_
 #define slahqr slahqr_
 #define slahrd slahrd_
 #define slaic1 slaic1_
 #define slaln2 slaln2_
 #define slamch slamch_
 #define slamrg slamrg_
 #define slangb slangb_
 #define slange slange_
 #define slangt slangt_
 #define slanhs slanhs_
 #define slansb slansb_
 #define slansp slansp_
 #define slanst slanst_
 #define slansy slansy_
 #define slantb slantb_
 #define slantp slantp_
 #define slantr slantr_
 #define slanv2 slanv2_
 #define slapll slapll_
 #define slapmt slapmt_
 #define slapy2 slapy2_
 #define slapy3 slapy3_
 #define slaqgb slaqgb_
 #define slaqge slaqge_
 #define slaqsb slaqsb_
 #define slaqsp slaqsp_
 #define slaqsy slaqsy_
 #define slaqtr slaqtr_
 #define slar2v slar2v_
 #define slarf slarf_
 #define slarfb slarfb_
 #define slarfg slarfg_
 #define slarft slarft_
 #define slarfx slarfx_
 #define slargv slargv_
 #define slarnv slarnv_
 #define slartg slartg_
 #define slartv slartv_
 #define slaruv slaruv_
 #define slas2 slas2_
 #define slascl slascl_
 #define slaset slaset_
 #define slasq1 slasq1_
 #define slasq2 slasq2_
 #define slasq3 slasq3_
 #define slasq4 slasq4_
 #define slasr slasr_
 #define slasrt slasrt_
 #define slassq slassq_
 #define slasv2 slasv2_
 #define slaswp slaswp_
 #define slasy2 slasy2_
 #define slasyf slasyf_
 #define slatbs slatbs_
 #define slatps slatps_
 #define slatrd slatrd_
 #define slatrs slatrs_
 #define slatzm slatzm_
 #define slauu2 slauu2_
 #define slauum slauum_
 #define sopgtr sopgtr_
 #define sopmtr sopmtr_
 #define sorg2l sorg2l_
 #define sorg2r sorg2r_
 #define sorgbr sorgbr_
 #define sorghr sorghr_
 #define sorgl2 sorgl2_
 #define sorglq sorglq_
 #define sorgql sorgql_
 #define sorgqr sorgqr_
 #define sorgr2 sorgr2_
 #define sorgrq sorgrq_
 #define sorgtr sorgtr_
 #define sorm2l sorm2l_
 #define sorm2r sorm2r_
 #define sormbr sormbr_
 #define sormhr sormhr_
 #define sorml2 sorml2_
 #define sormlq sormlq_
 #define sormql sormql_
 #define sormqr sormqr_
 #define sormr2 sormr2_
 #define sormrq sormrq_
 #define sormtr sormtr_
 #define spbcon spbcon_
 #define spbequ spbequ_
 #define spbrfs spbrfs_
 #define spbstf spbstf_
 #define spbsv spbsv_
 #define spbsvx spbsvx_
 #define spbtf2 spbtf2_
 #define spbtrf spbtrf_
 #define spbtrs spbtrs_
 #define spocon spocon_
 #define spoequ spoequ_
 #define sporfs sporfs_
 #define sposv sposv_
 #define sposvx sposvx_
 #define spotf2 spotf2_
 #define spotrf spotrf_
 #define spotri spotri_
 #define spotrs spotrs_
 #define sppcon sppcon_
 #define sppequ sppequ_
 #define spprfs spprfs_
 #define sppsv sppsv_
 #define sppsvx sppsvx_
 #define spptrf spptrf_
 #define spptri spptri_
 #define spptrs spptrs_
 #define sptcon sptcon_
 #define spteqr spteqr_
 #define sptrfs sptrfs_
 #define sptsv sptsv_
 #define sptsvx sptsvx_
 #define spttrf spttrf_
 #define spttrs spttrs_
 #define srscl srscl_
 #define ssbev ssbev_
 #define ssbevd ssbevd_
 #define ssbevx ssbevx_
 #define ssbgst ssbgst_
 #define ssbgv ssbgv_
 #define ssbtrd ssbtrd_
 #define sspcon sspcon_
 #define sspev sspev_
 #define sspevd sspevd_
 #define sspevx sspevx_
 #define sspgst sspgst_
 #define sspgv sspgv_
 #define ssprfs ssprfs_
 #define sspsv sspsv_
 #define sspsvx sspsvx_
 #define ssptrd ssptrd_
 #define ssptrf ssptrf_
 #define ssptri ssptri_
 #define ssptrs ssptrs_
 #define sstebz sstebz_
 #define sstedc sstedc_
 #define sstein sstein_
 #define ssteqr ssteqr_
 #define ssterf ssterf_
 #define sstev sstev_
 #define sstevd sstevd_
 #define sstevx sstevx_
 #define ssycon ssycon_
 #define ssyev ssyev_
 #define ssyevd ssyevd_
 #define ssyevx ssyevx_
 #define ssygs2 ssygs2_
 #define ssygst ssygst_
 #define ssygv ssygv_
 #define ssyrfs ssyrfs_
 #define ssysv ssysv_
 #define ssysvx ssysvx_
 #define ssytd2 ssytd2_
 #define ssytf2 ssytf2_
 #define ssytrd ssytrd_
 #define ssytrf ssytrf_
 #define ssytri ssytri_
 #define ssytrs ssytrs_
 #define stbcon stbcon_
 #define stbrfs stbrfs_
 #define stbtrs stbtrs_
 #define stgevc stgevc_
 #define stgsja stgsja_
 #define stpcon stpcon_
 #define stprfs stprfs_
 #define stptri stptri_
 #define stptrs stptrs_
 #define strcon strcon_
 #define strevc strevc_
 #define strexc strexc_
 #define strrfs strrfs_
 #define strsen strsen_
 #define strsna strsna_
 #define strsyl strsyl_
 #define strti2 strti2_
 #define strtri strtri_
 #define strtrs strtrs_
 #define stzrqf stzrqf_
 #define zbdsqr zbdsqr_
 #define zdrot zdrot_
 #define zdrscl zdrscl_
 #define zgbbrd zgbbrd_
 #define zgbcon zgbcon_
 #define zgbequ zgbequ_
 #define zgbrfs zgbrfs_
 #define zgbsv zgbsv_
 #define zgbsvx zgbsvx_
 #define zgbtf2 zgbtf2_
 #define zgbtrf zgbtrf_
 #define zgbtrs zgbtrs_
 #define zgebak zgebak_
 #define zgebal zgebal_
 #define zgebd2 zgebd2_
 #define zgebrd zgebrd_
 #define zgecon zgecon_
 #define zgeequ zgeequ_
 #define zgees zgees_
 #define zgeesx zgeesx_
 #define zgeev zgeev_
 #define zgeevx zgeevx_
 #define zgegs zgegs_
 #define zgegv zgegv_
 #define zgehd2 zgehd2_
 #define zgehrd zgehrd_
 #define zgelq2 zgelq2_
 #define zgelqf zgelqf_
 #define zgels zgels_
 #define zgelss zgelss_
 #define zgelsx zgelsx_
 #define zgeql2 zgeql2_
 #define zgeqlf zgeqlf_
 #define zgeqpf zgeqpf_
 #define zgeqr2 zgeqr2_
 #define zgeqrf zgeqrf_
 #define zgerfs zgerfs_
 #define zgerq2 zgerq2_
 #define zgerqf zgerqf_
 #define zgesv zgesv_
 #define zgesvd zgesvd_
 #define zgesvx zgesvx_
 #define zgetf2 zgetf2_
 #define zgetrf zgetrf_
 #define zgetri zgetri_
 #define zgetrs zgetrs_
 #define zggbak zggbak_
 #define zggbal zggbal_
 #define zggglm zggglm_
 #define zgghrd zgghrd_
 #define zgglse zgglse_
 #define zggqrf zggqrf_
 #define zggrqf zggrqf_
 #define zggsvd zggsvd_
 #define zggsvp zggsvp_
 #define zgtcon zgtcon_
 #define zgtrfs zgtrfs_
 #define zgtsv zgtsv_
 #define zgtsvx zgtsvx_
 #define zgttrf zgttrf_
 #define zgttrs zgttrs_
 #define zhbev zhbev_
 #define zhbevd zhbevd_
 #define zhbevx zhbevx_
 #define zhbgst zhbgst_
 #define zhbgv zhbgv_
 #define zhbtrd zhbtrd_
 #define zhecon zhecon_
 #define zheev zheev_
 #define zheevd zheevd_
 #define zheevx zheevx_
 #define zhegs2 zhegs2_
 #define zhegst zhegst_
 #define zhegv zhegv_
 #define zherfs zherfs_
 #define zhesv zhesv_
 #define zhesvx zhesvx_
 #define zhetd2 zhetd2_
 #define zhetf2 zhetf2_
 #define zhetrd zhetrd_
 #define zhetrf zhetrf_
 #define zhetri zhetri_
 #define zhetrs zhetrs_
 #define zhgeqz zhgeqz_
 #define zhpcon zhpcon_
 #define zhpev zhpev_
 #define zhpevd zhpevd_
 #define zhpevx zhpevx_
 #define zhpgst zhpgst_
 #define zhpgv zhpgv_
 #define zhprfs zhprfs_
 #define zhpsv zhpsv_
 #define zhpsvx zhpsvx_
 #define zhptrd zhptrd_
 #define zhptrf zhptrf_
 #define zhptri zhptri_
 #define zhptrs zhptrs_
 #define zhsein zhsein_
 #define zhseqr zhseqr_
 #define zlabrd zlabrd_
 #define zlacgv zlacgv_
 #define zlacon zlacon_
 #define zlacpy zlacpy_
 #define zlacrm zlacrm_
 #define zlacrt zlacrt_
 #define zladiv zladiv_
 #define zlaed0 zlaed0_
 #define zlaed7 zlaed7_
 #define zlaed8 zlaed8_
 #define zlaein zlaein_
 #define zlaesy zlaesy_
 #define zlaev2 zlaev2_
 #define zlags2 zlags2_
 #define zlagtm zlagtm_
 #define zlahef zlahef_
 #define zlahqr zlahqr_
 #define zlahrd zlahrd_
 #define zlaic1 zlaic1_
 #define zlangb zlangb_
 #define zlange zlange_
 #define zlangt zlangt_
 #define zlanhb zlanhb_
 #define zlanhe zlanhe_
 #define zlanhp zlanhp_
 #define zlanhs zlanhs_
 #define zlanht zlanht_
 #define zlansb zlansb_
 #define zlansp zlansp_
 #define zlansy zlansy_
 #define zlantb zlantb_
 #define zlantp zlantp_
 #define zlantr zlantr_
 #define zlapll zlapll_
 #define zlapmt zlapmt_
 #define zlaqgb zlaqgb_
 #define zlaqge zlaqge_
 #define zlaqhb zlaqhb_
 #define zlaqhe zlaqhe_
 #define zlaqhp zlaqhp_
 #define zlaqsb zlaqsb_
 #define zlaqsp zlaqsp_
 #define zlaqsy zlaqsy_
 #define zlar2v zlar2v_
 #define zlarf zlarf_
 #define zlarfb zlarfb_
 #define zlarfg zlarfg_
 #define zlarft zlarft_
 #define zlarfx zlarfx_
 #define zlargv zlargv_
 #define zlarnv zlarnv_
 #define zlartg zlartg_
 #define zlartv zlartv_
 #define zlascl zlascl_
 #define zlaset zlaset_
 #define zlasr zlasr_
 #define zlassq zlassq_
 #define zlaswp zlaswp_
 #define zlasyf zlasyf_
 #define zlatbs zlatbs_
 #define zlatps zlatps_
 #define zlatrd zlatrd_
 #define zlatrs zlatrs_
 #define zlatzm zlatzm_
 #define zlauu2 zlauu2_
 #define zlauum zlauum_
 #define zpbcon zpbcon_
 #define zpbequ zpbequ_
 #define zpbrfs zpbrfs_
 #define zpbstf zpbstf_
 #define zpbsv zpbsv_
 #define zpbsvx zpbsvx_
 #define zpbtf2 zpbtf2_
 #define zpbtrf zpbtrf_
 #define zpbtrs zpbtrs_
 #define zpocon zpocon_
 #define zpoequ zpoequ_
 #define zporfs zporfs_
 #define zposv zposv_
 #define zposvx zposvx_
 #define zpotf2 zpotf2_
 #define zpotrf zpotrf_
 #define zpotri zpotri_
 #define zpotrs zpotrs_
 #define zppcon zppcon_
 #define zppequ zppequ_
 #define zpprfs zpprfs_
 #define zppsv zppsv_
 #define zppsvx zppsvx_
 #define zpptrf zpptrf_
 #define zpptri zpptri_
 #define zpptrs zpptrs_
 #define zptcon zptcon_
 #define zpteqr zpteqr_
 #define zptrfs zptrfs_
 #define zptsv zptsv_
 #define zptsvx zptsvx_
 #define zpttrf zpttrf_
 #define zpttrs zpttrs_
 #define zrot zrot_
 #define zspcon zspcon_
 #define zspmv zspmv_
 #define zspr zspr_
 #define zsprfs zsprfs_
 #define zspsv zspsv_
 #define zspsvx zspsvx_
 #define zsptrf zsptrf_
 #define zsptri zsptri_
 #define zsptrs zsptrs_
 #define zstedc zstedc_
 #define zstein zstein_
 #define zsteqr zsteqr_
 #define zsycon zsycon_
 #define zsymv zsymv_
 #define zsyr zsyr_
 #define zsyrfs zsyrfs_
 #define zsysv zsysv_
 #define zsysvx zsysvx_
 #define zsytf2 zsytf2_
 #define zsytrf zsytrf_
 #define zsytri zsytri_
 #define zsytrs zsytrs_
 #define ztbcon ztbcon_
 #define ztbrfs ztbrfs_
 #define ztbtrs ztbtrs_
 #define ztgevc ztgevc_
 #define ztgsja ztgsja_
 #define ztpcon ztpcon_
 #define ztprfs ztprfs_
 #define ztptri ztptri_
 #define ztptrs ztptrs_
 #define ztrcon ztrcon_
 #define ztrevc ztrevc_
 #define ztrexc ztrexc_
 #define ztrrfs ztrrfs_
 #define ztrsen ztrsen_
 #define ztrsna ztrsna_
 #define ztrsyl ztrsyl_
 #define ztrti2 ztrti2_
 #define ztrtri ztrtri_
 #define ztrtrs ztrtrs_
 #define ztzrqf ztzrqf_
 #define zung2l zung2l_
 #define zung2r zung2r_
 #define zungbr zungbr_
 #define zunghr zunghr_
 #define zungl2 zungl2_
 #define zunglq zunglq_
 #define zungql zungql_
 #define zungqr zungqr_
 #define zungr2 zungr2_
 #define zungrq zungrq_
 #define zungtr zungtr_
 #define zunm2l zunm2l_
 #define zunm2r zunm2r_
 #define zunmbr zunmbr_
 #define zunmhr zunmhr_
 #define zunml2 zunml2_
 #define zunmlq zunmlq_
 #define zunmql zunmql_
 #define zunmqr zunmqr_
 #define zunmr2 zunmr2_
 #define zunmrq zunmrq_
 #define zunmtr zunmtr_
 #define zupgtr zupgtr_
 #define zupmtr zupmtr_
#endif /* __FORTRAN_LINK_CONVENTION__ == 1 */

#endif /* __cpplapack_fl_H__ */
