/* --------------------------------------------------------------------------
 *
 * namespace cppbvls_fl
 *   fortran link conventions handler
 *
 * description
 *   different fortran compilers follow different conventions for fortran
 *   function names; e. g. some append an underscore, some capitalize it,
 *   etc. To handle  these  variations in  a transparent manner,  we  use
 *   the value of the definition __FORTRAN_LINK_CONVENTION__ to represent
 *   a convention, which initiates appropriate modifications to  function
 *   names in prototypes and in call:
 *
 *   __FORTRAN_LINK_CONVENTION__     represents
 *
 *                             0     no change
 *                             1     append underscore
 *                           ...     more possibilities as required
 *
 * required headers
 *   see include header file cppbvls
 *
 * --------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------- */

#ifndef __cppbvls_fl_H__
#define __cppbvls_fl_H__

#ifndef __FORTRAN_LINK_CONVENTION__
 #define __FORTRAN_LINK_CONVENTION__ 0
#endif

#if ( __FORTRAN_LINK_CONVENTION__ == 1 )
 #define bvls bvls_
#endif /* __FORTRAN_LINK_CONVENTION__ == 1 */

#endif /* __cppbvls_fl_H__ */
