/* --------------------------------------------------------------------------
 *
 * class MissingInfo
 *   implementation
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <missing.info/MissingInfo>

// ------------------------------------------------------------ construct/reinit
// -----------------------------------------------------------------------------

MissingInfo::MissingInfo( size_t numInstants, size_t numSlices,
                          size_t zLowerBound, size_t tLowerBound )
 : my_missing( numInstants, numSlices, false ),
   my_zLower( zLowerBound ),
   my_tLower( tLowerBound )
 {
 }

MissingInfo::MissingInfo( const string& specification, const string& separators )
 : my_missing( ),
   my_zLower( 0 ),
   my_tLower( 0 )
 {
  string err = reinitialize( specification, false, separators );
  if ( err != "" ) cerr << err << endl;
 }

MissingInfo::MissingInfo( const MissingInfo& another )
 : my_missing( another.my_missing ),
   my_zLower( another.my_zLower ),
   my_tLower( another.my_tLower )
 {
 }

MissingInfo&
MissingInfo::operator=( const MissingInfo& rhs )
 {
  if ( this != &rhs )
   {
    my_missing = rhs.my_missing;
    my_zLower  = rhs.my_zLower;
    my_tLower  = rhs.my_tLower;
   }

  return *this;
 }

void
MissingInfo::reinitialize( size_t numInstants, size_t numSlices,
                           size_t zLowerBound, size_t tLowerBound )
 {
  my_missing.resize( numInstants, numSlices, false, false );
  my_zLower = zLowerBound;
  my_tLower = tLowerBound;
 }

string
MissingInfo::reinitialize( const string& specification,
                                 bool    updateOnly,
                           const string& separators )
 {
  // cerr << "# begin reinitialize( \"" << specification << "\", " << updateOnly << ", \"" << separators << "\" )" << endl;

  size_t tsize  = my_missing.numRows( );
  size_t zsize  = my_missing.numCols( );
  size_t zlower = my_zLower;
  size_t tlower = my_tLower;

  char tname = 't';
  char zname = 'z';

  string fname  = "";

  string dchunk = "images";
  string mchunk = "missing";

  bool sizes_specified = false, file_specified = false;

  string errmsg = "";

  vector<string> tokens;
  MiscFns::getTokens( specification, tokens, separators );

  vector<string>::const_iterator t  = tokens.begin( );
  vector<string>::const_iterator op = t;

  int n;

  while ( ( t != tokens.end( ) ) && ( (*op) != "end" ) )
   {
    // cerr << "# token " << *t << " operator " << *op << endl;

    if ( (*op) == "sizes" )
     {
      ++t; n = atoi( (*t).c_str( ) ); tsize = ( n < 0 ) ? 0 : n;
      ++t; n = atoi( (*t).c_str( ) ); zsize = ( n < 0 ) ? 0 : n;
      sizes_specified = true;
      // cerr << "# sizes " << tsize << " " << zsize << endl;
     }
    else if ( (*op) == "tlower" )
     {
      ++t; n = atoi( (*t).c_str( ) ); tlower = ( n < 0 ) ? 0 : n;
      // cerr << "# tlower " << tlower << endl;
     }
    else if ( (*op) == "zlower" )
     {
      ++t; n = atoi( (*t).c_str( ) ); zlower = ( n < 0 ) ? 0 : n;
      // cerr << "# zlower " << zlower << endl;
     }
    else if ( (*op) == "file" )
     {
      ++t; fname = (*t);
      file_specified = true;
      // cerr << "# fname " << fname << endl;
     }
    else if ( (*op) == "tname" )
     {
      ++t; tname = (*t)[0];
      // cerr << "# tname " << tname << endl;
     }
    else if ( (*op) == "zname" )
     {
      ++t; zname = (*t)[0];
      // cerr << "# zname " << zname << endl;
     }
    else if ( (*op) == "data" )
     {
      ++t; dchunk = (*t);
      // cerr << "# dchunk \"" << dchunk << "\"" << endl;
     }
    else if ( (*op) == "missing" )
     {
      ++t; mchunk = (*t);
      // cerr << "# mchunk \"" << mchunk << "\"" << endl;
     }
    else if ( ( (*op) == "+" ) || ( (*op) == "-" ) )
     {
      ++t; ++t; // do this later
     }
    else
     {
      return "invalid operator \"" + (*op) + "\" in missing data specification";
     }

    op = ++t;
   }

  if ( !updateOnly )
   {
    if ( sizes_specified )
     {
      reinitialize( tsize, zsize, zlower, tlower );
      updateOnly = true; // so that my_missing is not resized again when updating from a .mri file 
     }
    else
     {
      my_zLower = zlower;
      my_tLower = tlower;
     }
   }

  if ( file_specified )
   {
    errmsg = updateWith( fname, updateOnly, dchunk, mchunk, tname, zname );
    if ( errmsg != "" ) return errmsg;
   }

  pair<int,int> tDefaults( tLower( ), tUpper( ) );
  pair<int,int> zDefaults( zLower( ), zUpper( ) );

  pair<int,int> tRange, zRange;

  t  = tokens.begin( );
  op = t;

  bool status = true;

  while ( ( t != tokens.end( ) ) && ( (*op) != "end" ) )
   {
    if ( (*op) == "sizes" )
     {
      ++t; ++t;
     }
    else if (    ( (*op) == "tlower" ) || ( (*op) == "zlower"  )
              || ( (*op) == "tname"  ) || ( (*op) == "zname"   )
              || ( (*op) == "data"   ) || ( (*op) == "missing" )
              || ( (*op) == "file"   ) )
     {
      ++t;
     }
    else if ( ( (*op) == "+" ) || ( (*op) == "-" ) )
     {
      ++t; MiscFns::getRange( (*t), tDefaults, tRange );
      ++t; MiscFns::getRange( (*t), zDefaults, zRange );

      // cerr << "# " << (*op) << " " << tRange.first << ":" << tRange.second << " " << zRange.first << ":" << zRange.second << endl;

      status = ( (*op) == "+" ) ? true : false;
      setStatus( tRange.first, tRange.second, zRange.first, zRange.second, status );
     }

    op = ++t;
   }

  // cerr << "# end reinitialize( \"" << specification << "\", " << updateOnly << ", \"" << separators << "\" )" << endl;

  return "";
 }

// ---------------------------------------------------------------------- output
// -----------------------------------------------------------------------------

string
MissingInfo::write2mri( const string&                    mriFile,
                        const string&                    datFile,
                              PghMRIhandler::mriOpenMode openMode,
                        const string&                    missingChunkName,
                              char                       tDimName,
                              char                       zDimName ) const
 {
  // cerr << "# begin write2mri( )" << endl;

  if ( openMode == PghMRIhandler::mriRead ) return "invalid openMode value";

  // cerr << "# mriFile \"" << mriFile << "\" datFile \"" << datFile << "\" openMode " << openMode << endl;

  PghMRIhandler mh( mriFile, openMode, PghMRIhandler::mriIgnoreErrors );

  // cerr << "# mh.error( ) \"" << mh.error( ) << "\"" << endl;

  if ( mh.anyError( ) ) return mh.error( );

  matrix<unsigned char> m( numSlices( ), numInstants( ) );

  for ( size_t s = 0; s < numSlices( ); s++ )
   {
    const bool* b = my_missing[s];

    for ( size_t i = 0; i < numInstants( ); i++ )
     m( s, i ) = static_cast<unsigned char>( b[i] );
   }

  // cerr << "# my_missing";
  // for ( size_t i = 0; i < my_missing.numRows( ) * my_missing.numCols( ); i++ )
  //  cerr << " " << my_missing[0][i];
  // cerr << endl;

  // cerr << "# m";
  // for ( size_t i = 0; i < m.numRows( ) * m.numCols( ); i++ )
  //  cerr << " " << ( m[0][i] ? 1 : 0 );
  // cerr << endl;

  // cerr << "# ok a" << endl;

  mh.setString( missingChunkName, "[chunk]" );
  if ( datFile != "" ) mh.setString( missingChunkName + ".file", datFile );
  mh.setString( missingChunkName + ".datatype", "uint8" );
  mh.setString( missingChunkName + ".dimensions", "zt" );
  mh.setInt( missingChunkName + ".extent.z", numSlices( ) );
  mh.setInt( missingChunkName + ".extent.t", numInstants( ) );
  mh.setInt( missingChunkName + ".size", numSlices( ) * numInstants( ) * sizeof( unsigned char ) );

  if ( my_tLower > 0 )
   {
    int tLower = my_tLower;
    mh.setInt( missingChunkName + ".min.t", tLower );
   }

  if ( my_zLower > 0 )
   {
    int zLower = my_zLower;
    mh.setInt( missingChunkName + ".min.z", zLower );
   }

  // cerr << "# ok b" << endl;

  mh.setChunk( missingChunkName, numInstants( ) * numSlices( ),
               0, PghMRIhandler::mriUchar, m[0] );

  // cerr << "# end write2mri( )" << endl;

  return ( mh.anyError( ) ? mh.error( ) : string( ) );
 }

void
MissingInfo::printInfo( ) const
 {
  printInfo( tLower( ), tUpper( ), zLower( ), zUpper( ) );
 }

void
MissingInfo::printInfo(       size_t   instantMin,
                              size_t   instantMax,
                              size_t   sliceMin,
                              size_t   sliceMax,
                        const string&  hTitle,
                        const string&  vTitle,
                        const char*    symbols,
                              ostream& os ) const
 {
  size_t sMin = max( my_zLower, min( sliceMin, zUpper( ) ) );
  size_t sMax = max( my_zLower, min( sliceMax, zUpper( ) ) );

  size_t iMin = max( my_tLower, min( instantMin, tUpper( ) ) );
  size_t iMax = max( my_tLower, min( instantMax, tUpper( ) ) );

  // cerr << "# sliceMin   " << sliceMin   << " sliceMax   " << sliceMax   << endl;
  // cerr << "# instantMin " << instantMin << " instantMax " << instantMax << endl;
  // cerr << "# tLower( )  " << tLower( )  << " tUpper( )  " << tUpper( )  << endl;
  // cerr << "# zLower( )  " << zLower( )  << " zUpper( )  " << zUpper( )  << endl;
  // cerr << "# sMin       " << sMin       << " sMax       " << sMax       << endl;
  // cerr << "# iMin       " << iMin       << " iMax       " << iMax       << endl;

  os << "Legend: "
     << symbols[false] << " == non-missing"
     << ", "
     << symbols[true] << " == missing"
     << endl
     << endl;

  size_t tWidth = MiscFns::nDigits( iMax );
  if ( vTitle.size( ) > tWidth ) tWidth = vTitle.size( );

  size_t zWidth = MiscFns::nDigits( sMax );

  for ( size_t i = 0; i < tWidth - 1; i++ ) os << ' ';
  os << '\\' << hTitle;

  for ( size_t slice = sMin; slice <= sMax; slice++ )
   os << setw( zWidth + 1 ) << slice;

  os << endl;

  for ( size_t j = 0; j < ( tWidth - vTitle.size( ) ); j++ ) os << ' ';
  os << vTitle << '\\' << endl;

  // for ( size_t instant = 0; instant < numInstants( ); instant++ )
  //  {
  //   cerr << "# " << setw( tWidth ) << instant;
  //   for ( size_t i = 0; i < hTitle.size( ); i++ ) cerr << ' ';
  //   for ( size_t slice = 0; slice < numSlices( ); slice++ )
  //    {
  //     for ( size_t i = 0; i < zWidth; i++ ) cerr << ' ';
  //     cerr << symbols[my_missing( instant, slice )];
  //    }
  //   cerr << endl;
  //  }

  for ( size_t instant = iMin, t = instant - my_tLower; instant <= iMax; instant++, t++ )
   {
    os << setw( tWidth ) << instant;

    for ( size_t i = 0; i < hTitle.size( ); i++ ) os << ' ';

    for ( size_t slice = sMin, z = slice - my_zLower; slice <= sMax; slice++, z++ )
     {
      for ( size_t i = 0; i < zWidth; i++ ) os << ' ';
      os << symbols[my_missing( t, z )];
     }

    os << endl;
    // os << " | " << numMissingSlices( instant ) << endl;
   }
 }

// --------------------------------------------------------------------- private
// -----------------------------------------------------------------------------

string
MissingInfo::updateWith( const string& mriFile,
                               bool    updateOnly,
                         const string& dataChunk,
                         const string& missingChunk,
                               char    tDimName,
                               char    zDimName )
 {
  // cerr << "# begin updateWith( \"" << mriFile << "\", " << updateOnly << ", \"" << dataChunk << "\", \"" << missingChunk << "\", '" << tDimName << "', '" << zDimName << "' )" << endl;

  string expectedDims1( 2, tDimName );
  expectedDims1[1] = zDimName;

  string expectedDims2( 2, tDimName );
  expectedDims2[0] = zDimName;

  string dimensions;
  int nInstants, nSlices, zLower = 0, tLower = 0;

  PghMRIhandler mh( mriFile, PghMRIhandler::mriRead,
                             PghMRIhandler::mriIgnoreErrors );

  if ( mh.anyError( ) ) return mh.error( );

  if ( !(    mh.hasKey( missingChunk )
          && ( mh.getString( missingChunk ) == "[chunk]" ) ) )
   return "missing key/invalid value: missing";

  if ( mh.hasKey( missingChunk + ".dimensions" ) )
   {
    dimensions = mh.getString( missingChunk + ".dimensions" );

    if ( !( ( dimensions == expectedDims1 ) || ( dimensions == expectedDims2 ) ) )
     return "invalid value: missing.dimensions";
   }
  else
   return "missing key: missing.dimensions";

  // cerr << "# dimensions " << dimensions << endl;

  if ( !(    mh.hasKey( missingChunk + ".datatype" )
          && ( mh.getString( missingChunk + ".datatype" ) == "uint8" ) ) )
   return "missing key/invalid value: missing.datatype";

  string iExtentKey = dataChunk    + ".extent.?";
  string mExtentKey = missingChunk + ".extent.?";

  iExtentKey[iExtentKey.size( ) - 1] = tDimName;
  mExtentKey[mExtentKey.size( ) - 1] = tDimName;

  if ( mh.hasKey( mExtentKey ) )
   {
    nInstants = mh.getInt( mExtentKey );

    if ( mh.hasKey( iExtentKey ) && ( nInstants != mh.getInt( iExtentKey ) ) )
     return "inconsistent values: " + iExtentKey + ", " + mExtentKey;
   }
  else if ( mh.hasKey( iExtentKey ) )
   nInstants = mh.getInt( iExtentKey );
  else
   return "missing keys: " + iExtentKey + ", " + mExtentKey;

  // cerr << "# nInstants " << nInstants << endl;

  iExtentKey[iExtentKey.size( ) - 1] = zDimName;
  mExtentKey[mExtentKey.size( ) - 1] = zDimName;

  if ( mh.hasKey( mExtentKey ) )
   {
    nSlices = mh.getInt( mExtentKey );

    if ( mh.hasKey( iExtentKey ) && ( nSlices != mh.getInt( iExtentKey ) ) )
     return "inconsistent values: " + iExtentKey + ", " + mExtentKey;
   }
  else if ( mh.hasKey( iExtentKey ) )
   nSlices = mh.getInt( iExtentKey );
  else
   return "missing keys: " + iExtentKey + ", " + mExtentKey;

  // cerr << "# nSlices " << nSlices << endl;

  iExtentKey = dataChunk    + ".min.?";
  mExtentKey = missingChunk + ".min.?";

  iExtentKey[iExtentKey.size( ) - 1] = tDimName;
  mExtentKey[mExtentKey.size( ) - 1] = tDimName;

  bool hasKeyM = mh.hasKey( mExtentKey );
  bool hasKeyI = mh.hasKey( iExtentKey );

  if ( hasKeyM && hasKeyI )
   {
    tLower = mh.getInt( mExtentKey );

    if ( tLower != mh.getInt( iExtentKey ) )
     return "inconsistent values: " + iExtentKey + ", " + mExtentKey;
   }
  else if ( hasKeyM )
   tLower = mh.getInt( mExtentKey );
  else if ( hasKeyI )
   tLower = mh.getInt( iExtentKey );

  if ( tLower < 0 ) tLower = 0;

  // cerr << "# tLower " << tLower << endl;

  iExtentKey[iExtentKey.size( ) - 1] = zDimName;
  mExtentKey[mExtentKey.size( ) - 1] = zDimName;

  hasKeyM = mh.hasKey( mExtentKey );
  hasKeyI = mh.hasKey( iExtentKey );

  if ( hasKeyM && hasKeyI )
   {
    zLower = mh.getInt( mExtentKey );

    if ( zLower != mh.getInt( iExtentKey ) )
     return "inconsistent values: " + iExtentKey + ", " + mExtentKey;
   }
  else if ( hasKeyM )
   zLower = mh.getInt( mExtentKey );
  else if ( hasKeyI )
   zLower = mh.getInt( iExtentKey );

  if ( zLower < 0 ) zLower = 0;

  // cerr << "# zLower " << zLower << endl;

  if ( updateOnly )
   {
    if ( !(    ( nInstants == static_cast<int>( numInstants( ) ) )
            && ( nSlices   == static_cast<int>( numSlices( )   ) )
            && ( tLower    == static_cast<int>( my_tLower      ) )
            && ( zLower    == static_cast<int>( my_zLower      ) ) ) )
     return "update unsuccessful: mismatching sizes/z-lower/t-lower";
   }

  int size = nInstants * nSlices;

  if ( !(    mh.hasKey( missingChunk + ".size" )
          && ( mh.getInt( missingChunk + ".size" ) == size ) ) )
   return "missing key/inconsistent value: missing.size";

  // cerr << "# nInstants " << nInstants << " nSlices " << nSlices << " size " << size << endl;

  // cerr << "# size " << size << endl;

  unsigned char* buffer
   = static_cast<unsigned char*>( mh.getChunk( missingChunk, size, 0, PghMRIhandler::mriUchar ) );

  // cerr << "# missing (buffer):"; for ( int i = 0; i < size; i++ ) cerr << " " << ( buffer[i] ? 1 : 0 ); cerr << endl;

  if ( !updateOnly ) { my_zLower = zLower; my_tLower = tLower; }
  my_missing.resize( nInstants, nSlices, false, false );

  if ( dimensions == expectedDims2 )                            // zt
   {
    int k = 0;
    for ( int i = 0; i < nInstants; i++ )
     {
      for ( int s = 0; s < nSlices; s++ )
       {
        my_missing( i, s ) = static_cast<bool>( buffer[k] );
        k++;
       }
     }
   }
  else                                                          // tz
   MiscFns::castAndCopy( size, buffer, my_missing[0] );

  // cerr << "# missing (matrix):" << endl; for ( int i = 0; i < nInstants; i++ ) { cerr << "# t = " << i << ","; for ( int s = 0; s < nSlices; s++ ) cerr << " " << my_missing( i, s ); cerr << endl; }

  // cerr << "# end updateWith( \"" << mriFile << "\", " << updateOnly << ", \"" << dataChunk << "\", \"" << missingChunk << "\", '" << tDimName << "', '" << zDimName << "' )" << endl;

  return "";
 }
