/* --------------------------------------------------------------------------
 *
 * class LPflatCS
 *   implementation
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <objective.functions/LPflatCS>

// ------------------------------- construct/reinitialize/incremental initialize
// -----------------------------------------------------------------------------

LPflatCS::LPflatCS( )
 : BasicObjFn( ),
   my_paramOffset( 0 ),
   my_numParams( 1 ),
   my_lowerBound( fpVector( 0.0, 1 ) ),
   my_upperBound( fpVector( 1.0, 1 ) ),
   my_riseLength( fpVector( 0.1, 1 ) ),
   my_fallLength( fpVector( 0.1, 1 ) ),
   my_lowerSupport( fpVector( -0.1, 1 ) )
 {
  BasicObjFn::setName( "log flat prior/compact support" );
 }

LPflatCS::LPflatCS( const LPflatCS& another )
 : BasicObjFn( another.paramBufSize( ), another.sign( ), another.name( ) ),
   my_paramOffset( another.my_paramOffset ),
   my_numParams( another.my_numParams ),
   my_lowerBound( another.my_lowerBound ),
   my_upperBound( another.my_upperBound ),
   my_riseLength( another.my_riseLength ),
   my_fallLength( another.my_fallLength ),
   my_lowerSupport( another.my_lowerSupport )
 {
 }

LPflatCS&
LPflatCS::operator=( const LPflatCS& rhs )
 {
  if ( this == &rhs ) return *this;

  BasicObjFn::setParamBufSize( rhs.paramBufSize( ) );
  BasicObjFn::setSign( rhs.sign( ) );
  BasicObjFn::setName( rhs.name( ) );

  my_paramOffset = rhs.my_paramOffset;
  my_numParams   = rhs.my_numParams;

  my_lowerBound.resize( rhs.my_lowerBound.size( ) );
  my_upperBound.resize( rhs.my_upperBound.size( ) );
  my_riseLength.resize( rhs.my_riseLength.size( ) );
  my_fallLength.resize( rhs.my_fallLength.size( ) );
  my_lowerSupport.resize( rhs.my_lowerSupport.size( ) );

  my_lowerBound   = rhs.my_lowerBound;
  my_upperBound   = rhs.my_upperBound;
  my_riseLength   = rhs.my_riseLength;
  my_fallLength   = rhs.my_fallLength;
  my_lowerSupport = rhs.my_lowerSupport;

  return *this;
 }

LPflatCS::LPflatCS(       int       paramOffset,
                          int       numParams,
                    const fpVector& lowerBound,
                    const fpVector& upperBound,
                    const fpVector& riseFactor,
                    const fpVector& fallFactor,
                          int       paramBufSize,
                          fpType    sign,
                    const string&   name )
 : BasicObjFn( paramBufSize, sign, name ),
   my_paramOffset( paramOffset ),
   my_numParams( ( numParams > 0 ) ? numParams : 1 ),
   my_lowerBound( fpVector( 0.0, my_numParams ) ),
   my_upperBound( fpVector( 1.0, my_numParams ) ),
   my_riseLength( fpVector( 0.1, my_numParams ) ),
   my_fallLength( fpVector( 0.1, my_numParams ) ),
   my_lowerSupport( fpVector( -0.1, my_numParams ) )
 {
  for ( int i = 0; i < my_numParams; i++ )
   setConstants( i, lowerBound[i], upperBound[i],
                    riseFactor[i], fallFactor[i] );
 }

void
LPflatCS::setParamOffset( int paramOffset )
 {
  my_paramOffset = paramOffset;
 }

void
LPflatCS::setNumParams( int numParams )
 {
  my_numParams = ( numParams > 0 ) ? numParams : 1;

  my_lowerBound.resize( my_numParams, 0.0 );
  my_upperBound.resize( my_numParams, 1.0 );

  my_riseLength.resize( my_numParams, 0.1 );
  my_fallLength.resize( my_numParams, 0.1 );

  my_lowerSupport.resize( my_numParams, -0.1 );
 }

void
LPflatCS::setConstants( int    paramIndex,
                        fpType lowerBound, fpType upperBound,
                        fpType riseFactor, fpType fallFactor )
 {
  fpType l, u, r, f;

  l = min( lowerBound, upperBound );
  u = max( lowerBound, upperBound );

  my_lowerBound[paramIndex] = l;
  my_upperBound[paramIndex] = u;

  r = fabs( riseFactor ) * ( u - l );
  f = fabs( fallFactor ) * ( u - l );

  my_riseLength[paramIndex] = r;
  my_fallLength[paramIndex] = f;

  my_lowerSupport[paramIndex] = l - r;
 }

void
LPflatCS::reinitialize(       int       paramOffset,
                              int       numParams,
                        const fpVector& lowerBound,
                        const fpVector& upperBound,
                        const fpVector& riseFactor,
                        const fpVector& fallFactor,
                              int       paramBufSize,
                              fpType    sign,
                        const string&   name )
 {
  BasicObjFn::reinitialize( paramBufSize, sign, name );

  setParamOffset( paramOffset );
  setNumParams( numParams );

  for ( int i = 0; i < my_numParams; i++ )
   setConstants( i, lowerBound[i], upperBound[i],
                    riseFactor[i], fallFactor[i] );
 }

SplineRampC2
LPflatCS::our_ramp;
