/* --------------------------------------------------------------------------
 *
 * class LPgamma
 *   implementation
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <objective.functions/LPgamma>

// ------------------------------- construct/reinitialize/incremental initialize
// -----------------------------------------------------------------------------

LPgamma::LPgamma( )
 : BasicObjFn( ),
   my_paramOffset( 0 ),
   my_numParams( 1 ),
   my_constA( fpVector( 0.0, 1 ) ),
   my_constB( fpVector( 1.0, 1 ) )
 {
  BasicObjFn::setName( "log gamma prior" );
 }

LPgamma::LPgamma( const LPgamma& another )
 : BasicObjFn( another.paramBufSize( ), another.sign( ), another.name( ) ),
   my_paramOffset( another.my_paramOffset ),
   my_numParams( another.my_numParams ),
   my_constA( another.my_constA ),
   my_constB( another.my_constB )
 {
 }

LPgamma&
LPgamma::operator=( const LPgamma& rhs )
 {
  if ( this == &rhs ) return *this;

  BasicObjFn::setParamBufSize( rhs.paramBufSize( ) );
  BasicObjFn::setSign( rhs.sign( ) );
  BasicObjFn::setName( rhs.name( ) );

  my_paramOffset = rhs.my_paramOffset;
  my_numParams   = rhs.my_numParams;

  my_constA.resize( rhs.my_constA.size( ) );
  my_constB.resize( rhs.my_constB.size( ) );

  my_constA = rhs.my_constA;
  my_constB = rhs.my_constB;

  return *this;
 }

LPgamma::LPgamma(       int       paramOffset,
                        int       numParams,
                  const fpVector& constA,
                  const fpVector& constB,
                        int       paramBufSize,
                        fpType    sign,
                  const string&   name )
 : BasicObjFn( paramBufSize, sign, name ),
   my_paramOffset( paramOffset ),
   my_numParams( ( numParams > 0 ) ? numParams : 1 ),
   my_constA( fpVector( 0.0, my_numParams ) ),
   my_constB( fpVector( 1.0, my_numParams ) )
 {
  for ( int i = 0; i < my_numParams; i++ )
   setConstants( i, constA[i], constB[i] );
 }

void
LPgamma::setParamOffset( int paramOffset )
 {
  my_paramOffset = paramOffset;
 }

void
LPgamma::setNumParams( int numParams )
 {
  my_numParams = ( numParams > 0 ) ? numParams : 1;

  my_constA.resize( my_numParams, 0.0 );
  my_constB.resize( my_numParams, 1.0 );
 }

void
LPgamma::setConstants( int paramIndex, fpType constA, fpType constB )
 {
  my_constA[paramIndex] = constA - 1.0;
  my_constB[paramIndex] = constB;
 }

void
LPgamma::reinitialize(       int       paramOffset,
                             int       numParams,
                       const fpVector& constA,
                       const fpVector& constB,
                             int       paramBufSize,
                             fpType    sign,
                       const string&   name )
 {
  BasicObjFn::reinitialize( paramBufSize, sign, name );

  setParamOffset( paramOffset );
  setNumParams( numParams );

  for ( int i = 0; i < my_numParams; i++ )
   setConstants( i, constA[i], constB[i] );
 }
