/* --------------------------------------------------------------------------
 *
 * class LPnormal
 *   implementation
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <objective.functions/LPnormal>

// ------------------------------- construct/reinitialize/incremental initialize
// -----------------------------------------------------------------------------

LPnormal::LPnormal( )
 : BasicObjFn( ),
   my_paramOffset( 0 ),
   my_numParams( 1 ),
   my_mean( fpVector( 0.0, 1 ) ),
   my_precision( fpVector( 1.0, 1 ) )
 {
  BasicObjFn::setName( "log normal prior" );
 }

LPnormal::LPnormal( const LPnormal& another )
 : BasicObjFn( another.paramBufSize( ), another.sign( ), another.name( ) ),
   my_paramOffset( another.my_paramOffset ),
   my_numParams( another.my_numParams ),
   my_mean( another.my_mean ),
   my_precision( another.my_precision )
 {
 }

LPnormal&
LPnormal::operator=( const LPnormal& rhs )
 {
  if ( this == &rhs ) return *this;

  BasicObjFn::setParamBufSize( rhs.paramBufSize( ) );
  BasicObjFn::setSign( rhs.sign( ) );
  BasicObjFn::setName( rhs.name( ) );

  my_paramOffset = rhs.my_paramOffset;
  my_numParams   = rhs.my_numParams;

  my_mean.resize( rhs.my_mean.size( ) );
  my_precision.resize( rhs.my_precision.size( ) );

  my_mean      = rhs.my_mean;
  my_precision = rhs.my_precision;

  return *this;
 }

LPnormal::LPnormal(       int       paramOffset,
                          int       numParams,
                    const fpVector& mean,
                    const fpVector& stdev,
                          int       paramBufSize,
                          fpType    sign,
                    const string&   name )
 : BasicObjFn( paramBufSize, sign, name ),
   my_paramOffset( paramOffset ),
   my_numParams( ( numParams > 0 ) ? numParams : 1 ),
   my_mean( fpVector( 0.0, my_numParams ) ),
   my_precision( fpVector( 1.0, my_numParams ) )
 {
  for ( int i = 0; i < my_numParams; i++ )
   setConstants( i, mean[i], stdev[i] );
 }

void
LPnormal::setParamOffset( int paramOffset )
 {
  my_paramOffset = paramOffset;
 }

void
LPnormal::setNumParams( int numParams )
 {
  my_numParams = ( numParams > 0 ) ? numParams : 1;

  my_mean.resize( my_numParams, 0.0 );
  my_precision.resize( my_numParams, 1.0 );
 }

void
LPnormal::setConstants( int paramIndex, fpType mean, fpType stdev )
 {
  my_mean[paramIndex]      = mean;
  my_precision[paramIndex] = 1.0 / stdev;
 }

void
LPnormal::reinitialize(       int       paramOffset,
                              int       numParams,
                        const fpVector& mean,
                        const fpVector& stdev,
                              int       paramBufSize,
                              fpType    sign,
                        const string&   name )
 {
  BasicObjFn::reinitialize( paramBufSize, sign, name );

  setParamOffset( paramOffset );
  setNumParams( numParams );

  for ( int i = 0; i < my_numParams; i++ )
   setConstants( i, mean[i], stdev[i] );
 }
