/* --------------------------------------------------------------------------
 *
 * class PghMRIhandler
 *   implementation
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <pghmri/PghMRIhandler>

// --------------------------------------------------- type conversion constants
// -----------------------------------------------------------------------------

const PghMRIhandler::mriArrayType PghMRIhandler::mriRaw      = MRI_RAW;
const PghMRIhandler::mriArrayType PghMRIhandler::mriUchar    = MRI_UNSIGNED_CHAR;
const PghMRIhandler::mriArrayType PghMRIhandler::mriShort    = MRI_SHORT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriInt      = MRI_INT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriFloat    = MRI_FLOAT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriDouble   = MRI_DOUBLE;
const PghMRIhandler::mriArrayType PghMRIhandler::mriLong     = MRI_LONG;
const PghMRIhandler::mriArrayType PghMRIhandler::mriLongLong = MRI_LONGLONG;

const PghMRIhandler::mriArrayType PghMRIhandler::mriScalar  = MRI_SCALAR;
const PghMRIhandler::mriArrayType PghMRIhandler::mriComplex = MRI_COMPLEX;
const PghMRIhandler::mriArrayType PghMRIhandler::mriVector  = MRI_VECTOR;

const PghMRIhandler::mriArrayType PghMRIhandler::mriComplexShort    = MRI_COMPLEX_SHORT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriComplexInt      = MRI_COMPLEX_INT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriComplexFloat    = MRI_COMPLEX_FLOAT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriComplexDouble   = MRI_COMPLEX_DOUBLE;
const PghMRIhandler::mriArrayType PghMRIhandler::mriComplexLong     = MRI_COMPLEX_LONG;
const PghMRIhandler::mriArrayType PghMRIhandler::mriComplexLongLong = MRI_COMPLEX_LONGLONG;

const PghMRIhandler::mriArrayType PghMRIhandler::mriVectorShort    = MRI_VECTOR_SHORT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriVectorInt      = MRI_VECTOR_INT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriVectorFloat    = MRI_VECTOR_FLOAT;
const PghMRIhandler::mriArrayType PghMRIhandler::mriVectorDouble   = MRI_VECTOR_DOUBLE;
const PghMRIhandler::mriArrayType PghMRIhandler::mriVectorLong     = MRI_VECTOR_LONG;
const PghMRIhandler::mriArrayType PghMRIhandler::mriVectorLongLong = MRI_VECTOR_LONGLONG;

// ------------------------------------------------------------- file open modes
// -----------------------------------------------------------------------------

const PghMRIhandler::mriOpenMode  PghMRIhandler::mriRead       = MRI_READ;
const PghMRIhandler::mriOpenMode  PghMRIhandler::mriWrite      = MRI_WRITE;
const PghMRIhandler::mriOpenMode  PghMRIhandler::mriModify     = MRI_MODIFY;
const PghMRIhandler::mriOpenMode  PghMRIhandler::mriModifyData = MRI_MODIFY_DATA;

// -------------------------------------------------------------- error handling
// -----------------------------------------------------------------------------

const PghMRIhandler::mriErrorMode PghMRIhandler::mriAbortOnError = MRI_ABORT_ON_ERROR;
const PghMRIhandler::mriErrorMode PghMRIhandler::mriReportErrors = MRI_REPORT_ERRORS;
const PghMRIhandler::mriErrorMode PghMRIhandler::mriIgnoreErrors = MRI_IGNORE_ERRORS;

PghMRIhandler::mriErrorMode
PghMRIhandler::our_errorMode = PghMRIhandler::mriAbortOnError;

char
PghMRIhandler::our_fileNotOpenMsg[] = "PghMRIhandler: no open dataset";

void
PghMRIhandler::setErrorMode( mriErrorMode mode )
 {
  our_errorMode = mode;
  mri_set_error_handling( mode );
 }

// ---------------------------------------- cctor, dtor; explicit open and close
// -----------------------------------------------------------------------------

PghMRIhandler::PghMRIhandler( const string&      fileName,
                                    mriOpenMode  openMode,
                                    mriErrorMode errorMode )
 : my_fileName( ),
   my_openMode( ),
   my_dataset( 0 ),
   my_error( )
 {
  open( fileName, openMode, errorMode );
 }

PghMRIhandler::~PghMRIhandler( )
 {
  close( );
 }

PghMRIhandler::PghMRIhandler( )
 : my_fileName( ),
   my_openMode( ),
   my_dataset( 0 ),
   my_error( )
 {
 }

PghMRIhandler::PghMRIhandler( const PghMRIhandler& another )
 : my_fileName( ),
   my_openMode( ),
   my_dataset( 0 ),
   my_error( )
 {
  open( another.my_fileName, mriRead, our_errorMode );
 }

PghMRIhandler&
PghMRIhandler::operator=( const PghMRIhandler& rhs )
 {
  if ( this != &rhs )
   open( rhs.my_fileName, mriRead, our_errorMode );

  return *this;
 }

void
PghMRIhandler::open( const string&      fileName,
                           mriOpenMode  openMode,
                           mriErrorMode errorMode )
 {
  close( );

  my_fileName = fileName;
  my_openMode = openMode;

  setErrorMode( errorMode );

  if ( my_fileName != "" )
   {
    my_dataset = mri_open_dataset( my_fileName.c_str( ), my_openMode );
    my_error   = mri_error;

    // if ( my_dataset && ( my_openMode == mriWrite ) )
    if ( my_dataset && ( ( my_openMode == mriWrite ) || ( my_openMode == mriModify ) ) )
     {
      setString( "!format", "pgh" );
      setString( "!version", "1.0" );
     }
   }
 }

void
PghMRIhandler::close( )
 {
  if ( my_dataset == 0 ) return;

  mri_close_dataset( my_dataset );
  my_error = mri_error;
  my_dataset = 0;
 }
