/* --------------------------------------------------------------------------
 *
 * class predicate
 *   base that allowes unary/binary compositions of predicates
 *
 * required headers
 *   see include header file predicate
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __predicate_H__
#define __predicate_H__

cpp_using_namespace( std );

template<class LeafType,class ArgType>
struct predicate : public unary_function<ArgType,bool>
 {
  inline LeafType& asLeaf( ){ return static_cast<LeafType&>( *this ); }
  inline const LeafType& asConstLeaf( ) const { return static_cast<const LeafType&>( *this ); }

  inline bool operator( )( const ArgType& a ) const { return asLeaf( )( a ); }

  template<template <typename W> class Op,class P1,class P2>
  struct predicate_composition : public predicate<predicate_composition<Op,P1,P2>,ArgType>
   {
    predicate_composition( const P1& p1, const P2& p2 ) : my_p1( p1 ), my_p2( p2 ) { }
    bool operator( )( const ArgType& a ) const { return my_op( my_p1( a ), my_p2( a ) ); }

    private:
     P1 my_p1; P2 my_p2; Op<bool> my_op;
   };

  template<class P1>
  struct predicate_negate : public predicate<predicate_negate<P1>,ArgType>
   {
    predicate_negate( const P1& p1 ) : my_p1( p1 ) { }
    bool operator( )( const ArgType& a ) const { return !( my_p1( a ) ); }

    private:
     P1 my_p1;
   };

  template<class P2>
  predicate_composition<logical_or,LeafType,P2>
  operator||( const P2& p2 ) const
   {
    return predicate_composition<logical_or,LeafType,P2>( asConstLeaf( ), p2 );
   }

  template<class P2>
  predicate_composition<logical_and,LeafType,P2>
  operator&&( const P2& p2 ) const
   {
    return predicate_composition<logical_and,LeafType,P2>( asConstLeaf( ), p2 );
   }

  predicate_negate<LeafType>
  operator!( ) const
   {
    return predicate_negate<LeafType>( asConstLeaf( ) );
   }
 };

#endif /* __predicate_H__ */
