#!/bin/csh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set comparison = absolute
 set threshold  = 1.0e-6
 set files      = ()

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' numeric comparison of two binary files.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-t|-threshold numeric_threshold]'
    echo ' [-a|-absolute|-r|-relative]'
    echo ''
    echo ' file1:type[:big|:little][:byte_offset]'
    echo ' file2:type[:big|:little][:byte_offset]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination, files contents identical'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified/unknown input file(s)/type(s)/endianness(s)'
    echo ' 3: unknown comparison'
    echo ' 4: numeric mismatch'
    echo ' 5-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' proc.path.csh'
    echo ' mri.cmp.chunk.csh endianness.csh mri.create.chunk.csh'
    echo ''
    echo 'Notes'
    echo ' 1 "absolute" and "relative" refer to the numerical comparison'
    echo '   type:  numeric contents of  the two files  are   considered'
    echo '   identical if c(a,b) <= threshold  for all a  in file 1  and'
    echo '   corresponding b in file 2, where c() is defined as'
    echo '     absolute: c(a,b) := |a - b|'
    echo '     relative: c(a,b) := |a - b|                if a == b == 0'
    echo '                      := |a - b| / max( |a|, |b| )  otherwise.'
    echo ' 2 unspecified endianness ('big'/'little')  defaults  to  that'
    echo '   of the machine that this script is running on.'
    echo ' 3 type is one of standard data types supported by libmri  (or'
    echo '   its od-equivalent name): float64|f8, float32|f4,  int32|d4,'
    echo '   int16|d2, uint8|u1.'
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-t":
   case "-threshold":
    set threshold = "${argv[2]}"; shift; shift; breaksw;

   case "-r":
   case "-relative":
    set comparison = relative; shift; breaksw;

   case "-a":
   case "-absolute":
    set comparison = absolute; shift; breaksw;

   default:
    set files = ( ${files} ${argv[1]} )
    shift
  endsw
 end

# error checks
#
 if ( ${#files} < 2 ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/unknown input file(s)/type(s)/endianness(s)" 1 ""
  exit ${status}
 endif

 set files = ( ${files[1]} ${files[2]} )

 if ( !(    ( "${comparison}" == "absolute" ) \
         || ( "${comparison}" == "relative" ) ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unknown comparison" 1 ""
  exit ${status}
 endif

# file 1: process file specification
#
 set file1 = ( `echo ${files[1]} | sed 's/:/ /g'` )

 # echo file1 $file1

 if ( ${#file1} < 2 ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/unknown input file(s)/type(s)/endianness(s)" 1 ""
  exit ${status}
 endif

 set type1 = ${file1[2]}

 if ( !(    ( "${type1}" == "u1" ) || ( "${type1}" == "uint8"   ) \
         || ( "${type1}" == "d2" ) || ( "${type1}" == "int16"   ) \
         || ( "${type1}" == "d4" ) || ( "${type1}" == "int32"   ) \
         || ( "${type1}" == "f4" ) || ( "${type1}" == "float32" ) \
         || ( "${type1}" == "f8" ) || ( "${type1}" == "float64" ) ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/unknown input file(s)/type(s)/endianness(s)" 1 ""
  exit ${status}
 endif

 # echo type1 $type1

 if ( ${#file1} > 2 ) then
  set end1 = ${file1[3]}
 endif

 # echo end1 $end1

 if ( !( ${?end1} ) ) then
  set end1 = `endianness.csh numeric`
 else if ( ( ${end1} == b ) || ( ${end1} == big ) || ( ${end1} == big_endian ) ) then
  set end1 = 0
 else if ( ( ${end1} == l ) || ( ${end1} == little ) || ( ${end1} == little_endian ) ) then
  set end1 = 1
 else
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/unknown input file(s)/type(s)/endianness(s)" 1 ""
  exit ${status}
 endif

 # echo end1 $end1

 set ofs1 = 0
 if ( ${#file1} > 3 ) then
  set ofs1 = ${file1[4]}
 endif

 # echo ofs1 $ofs1

 set file1 = `echo ${file1[1]} | sed 's/\.mri$//'`

 # echo file1 $file1

# file 2: process file specification
#
 set file2 = ( `echo ${files[2]} | sed 's/:/ /g'` )

 # echo file2 $file2

 if ( ${#file1} < 2 ) then
 if ( ${#file2} < 2 ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/unknown input file(s)/type(s)/endianness(s)" 1 ""
  exit ${status}
 endif

 set type2 = ${file2[2]}

 if ( !(    ( "${type2}" == "u1" ) || ( "${type2}" == "uint8"   ) \
         || ( "${type2}" == "d2" ) || ( "${type2}" == "int16"   ) \
         || ( "${type2}" == "d4" ) || ( "${type2}" == "int32"   ) \
         || ( "${type2}" == "f4" ) || ( "${type2}" == "float32" ) \
         || ( "${type2}" == "f8" ) || ( "${type2}" == "float64" ) ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/unknown input file(s)/type(s)/endianness(s)" 1 ""
  exit ${status}
 endif

 # echo type2 $type2

 if ( ${#file2} > 2 ) then
  set end2 = ${file2[3]}
 endif

 # echo end2 $end2

 if ( !( ${?end2} ) ) then
  set end2 = `endianness.csh numeric`
 else if ( ( ${end2} == b ) || ( ${end2} == big ) || ( ${end2} == big_endian ) ) then
  set end2 = 0
 else if ( ( ${end2} == l ) || ( ${end2} == little ) || ( ${end2} == little_endian ) ) then
  set end2 = 1
 else
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/unknown input file(s)/type(s)/endianness(s)" 1 ""
  exit ${status}
 endif

 # echo end2 $end2

 set ofs2 = 0
 if ( ${#file2} > 3 ) then
  set ofs2 = ${file2[4]}
 endif

 # echo ofs2 $ofs2

 set file2 = `echo ${file2[1]} | sed 's/\.mri$//'`

 # echo file2 $file2

# data files with full paths
#
 set dat1 = `proc.path.csh -verbosity ${verbosity} -physical ${file1}`

 errmsg.csh ${verbosity} ${myname} 7 "failed command" ${status} "proc.path.csh -verbosity ${verbosity} -physical ${file1}"
 if ( ${status} ) exit ${status}

 set dat2 = `proc.path.csh -verbosity ${verbosity} -physical ${file2}`

 errmsg.csh ${verbosity} ${myname} 8 "failed command" ${status} "proc.path.csh -verbosity ${verbosity} -physical ${file2}"
 if ( ${status} ) exit ${status}

# data extents
#
 @ extent1 = ( `ls -l ${file1} | awk '{print $5}'` - ${ofs1} ) / `mri.sizeof.csh ${type1}`

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "ls -l ${file1} | awk '{print $5}'"
 if ( ${status} ) exit ${status}

 @ extent2 = ( `ls -l ${file2} | awk '{print $5}'` - ${ofs2} ) / `mri.sizeof.csh ${type2}`

 errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "ls -l ${file2} | awk '{print $5}'"
 if ( ${status} ) exit ${status}

# create .mri files describing the two files
#
 mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${end1} -offset ${ofs1} -file ${dat1} -mri ${tmp}.1 -chunk images -type ${type1} -dimensions t -extents ${extent1}

 errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${end1} -offset ${ofs1} -file ${dat1} -mri ${tmp}.1 -chunk images -type ${type1} -dimensions t -extents ${extent1}"
 if ( ${status} ) exit ${status}

 mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${end2} -offset ${ofs2} -file ${dat2} -mri ${tmp}.2 -chunk images -type ${type2} -dimensions t -extents ${extent2}

 errmsg.csh ${verbosity} ${myname} 10 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${end2} -offset ${ofs2} -file ${dat2} -mri ${tmp}.2 -chunk images -type ${type2} -dimensions t -extents ${extent2}"
 if ( ${status} ) exit ${status}

# compare
#
 mri.cmp.chunk.csh -verbosity ${verbosity} -threshold ${threshold} -${comparison} ${tmp}.1:images ${tmp}.2:images

 set s = ${status}

 if ( !( ( ${s} == 0 ) || ( ${s} == 4 ) ) ) then
  errmsg.csh ${verbosity} ${myname} 11 "failed command" ${s} "mri.cmp.chunk.csh -verbosity ${verbosity} -threshold ${threshold} -${comparison} ${tmp}.1:images ${tmp}.2:images"
  exit ${s}
 endif

# cleanup
#
 rm -f ${tmp}* >& /dev/null

 exit ${s}
