#!/bin/csh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files suffix
#
 set tmp = .ncmp.tmp

# temporary files prefix
#
# mkdir -p /tmp/${user}
# set tmpdir = /tmp/${user}/
# set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set separator  = ''
 set comment    = '#'
 set comparison = absolute
 set threshold  = 1.0e-6
 set files      = ()

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' text/numeric comparison of two non-binary files: cmp extended'
    echo ' to allow numeric comparison (with respect to a threshold)  of'
    echo ' two non-binary files containing numeric data.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-c|-comment "comment_string"]'
    echo ' [-s|-separator "separator_string"]'
    echo ''
    echo ' [-t|-threshold numeric_threshold]'
    echo ' [-a|-absolute|-r|-relative]'
    echo ''
    echo ' file1 file2'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination, files contents identical'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified input file(s)'
    echo ' 3: unknown comparison'
    echo ' 4: text: mismatch, numeric: match'
    echo ' 5: text: match, numeric: mismatch (w.r.t. numeric threshold)'
    echo ' 6: mismatch in both (numeric: w.r.t. numeric threshold)'
#   echo ''
#   echo 'Required utilities/scripts'
#   echo ' errmsg.csh'
    echo ''
    echo 'Notes'
    echo ' 1 Non-binary files  file1 and file2  are assumed to have  the'
    echo '   following structure:'
    echo '    - text  (non-numeric)  data  preceded by  a comment string'
    echo "      (${comment} by default),  and"
    echo '    - non-commented  numeric  data  separated by  a  separator'
    echo '      (white space by default).'
    echo ' 2 Text portion  of the files  is identified as beginning with'
    echo '   the  designated comment string,  and is compared using cmp.'
    echo ' 3 Numeric portions  are compared  with respect to  a  numeric'
    echo "   threshold (${threshold} by default)."
    echo ' 4 Any non-numeric portions that are not commented may lead to'
    echo '   unreliable results.'
    echo ' 5 "absolute" and "relative" refer to the numerical comparison'
    echo '   type:  numeric contents of  the two files  are   considered'
    echo '   identical if c(a,b) <= threshold  for all a  in file 1  and'
    echo '   corresponding b in file 2, where c() is defined as'
    echo '     absolute: c(a,b) := |a - b|'
    echo '     relative: c(a,b) := |a - b|   if max(|a|,|b|) < threshold'
    echo '                      := |a - b| / max( |a|, |b| )  otherwise.'
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-c":
   case "-comment":
    set comment = "${argv[2]}"; shift; shift; breaksw;

   case "-s":
   case "-separator":
    set separator = '-F"'"${argv[2]}"'"'; shift; shift; breaksw;

   case "-t":
   case "-threshold":
    set threshold = "${argv[2]}"; shift; shift; breaksw;

   case "-r":
   case "-relative":
    set comparison = relative; shift; breaksw;

   case "-a":
   case "-absolute":
    set comparison = absolute; shift; breaksw;

   default:
    set files = ( ${files} ${argv[1]} )
    shift
  endsw
 end

# error checks
#
 if ( ${#files} < 2 ) then
# errmsg.csh ${verbosity} ${myname} 2 "unspecified input file(s)" 1 ""
# exit ${status}
  echo "unspecified input file(s)"
  exit 2
 endif

 set files = ( ${files[1]} ${files[2]} )

 if ( !(    ( "${comparison}" == "absolute" ) \
         || ( "${comparison}" == "relative" ) ) ) then
# errmsg.csh ${verbosity} ${myname} 3 "unknown comparison" 1 ""
# exit ${status}
  echo "unknown comparison"
  exit 2
 endif

# text compare
#
 foreach f ( ${files} )
  grep -q "${comment}" ${f}
  if ( ${status} ) then
   echo -n '' > ${f}${tmp}
  else
   awk -F"${comment}" '{print $2}' ${f} | grep -v '^$' > ${f}${tmp}
  endif
 end

 cmp -s ${files[1]}${tmp} ${files[2]}${tmp}

 set txtcmp = ${status}

 if ( ${txtcmp} ) then
  if ( ${verbosity} > 0 ) echo -n 'text: mismatch'
 else
  if ( ${verbosity} > 0 ) echo -n 'text: match'
 endif

# numeric compare
#
 foreach f ( ${files} )
  awk -F"${comment}" '{print $1}' ${f} | awk ${separator} '{for(i=1;i<=NF;i++)print $i}' | grep -v '^$' > ${f}${tmp}
 end

 if ( "${comparison}" == "relative" ) then
  set nf = ( `paste ${files[1]}${tmp} ${files[2]}${tmp} | awk '{a=$1;b=$2;d=$1-$2;if(a<0)a=-a;if(b<0)b=-b;if(d<0)d=-d;if(b>a)a=b;if(a>'${threshold}')d=d/a;if(d>'${threshold}'){s+=1;if(d>m)m=d}}END{print s" "m}'` )
 else
  set nf = ( `paste ${files[1]}${tmp} ${files[2]}${tmp} | awk '{d=$1-$2;if(d<0)d=-d;if(d>'${threshold}'){s+=1;if(d>m)m=d}}END{print s" "m}'` )
 endif

 if ( "${nf}" == "" ) set nf = ( 0 0 )

 set ncmp = ${nf[1]}
 if ( ${ncmp} ) then
  if ( ${verbosity} > 0 ) then
   echo ', numeric: mismatch'
   echo -n "${nf[1]} numeric difference/s,"
   if ( "${comparison}" == "relative" ) then
    echo " max |relative difference| = ${nf[2]}"
   else
    echo " max |difference| = ${nf[2]}"
   endif
   echo "numeric threshold = ${threshold}"
  endif
 else
  if ( ${verbosity} > 0 ) echo ', numeric: match'
 endif

 rm -f ${files[1]}${tmp} ${files[2]}${tmp}

 if ( ${ncmp} && ${txtcmp} ) exit 5
 if ( ${ncmp}              ) exit 4
 if ( ${txtcmp}            ) exit 3

 exit 0
