#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set adims = ''
#set permute = permute.csh # this is a workaround for a permute bug
 set permute = mri_permute # this bug is gone in 5.1

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Average out a chunk over specified dimensions.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-a|-average_over dimensions]'
    echo ' -i|-infile infile[.mri] -c|-chunk chunk'
    echo ' -m|-mean file[.mri] [-r|-rmse file[.mri]]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3: unspecified input file/chunk'
    echo ' 4: unspecified output file for mean'
    echo ' 6: nonexistent data file'
    echo ' 7: nothing to do'
    echo ' 5,8-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.chunk.info.csh mri.create.chunk.csh'
    echo ' ${permute} mri_subsample mri_interp mri_rpn_math'
    echo ''
    echo 'Notes'
    echo ' 1 Unspecified averaging dimensions default to chunk dimensions.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-a":
   case "-average_over":
    set adims = ${argv[2]}; shift; shift; breaksw;

   case "-m":
   case "-mean":
    set meanf = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-r":
   case "-rmse":
    set rmsf = ${argv[2]}; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# error checks
#
 if ( !( ${?infile} && ${?chunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified input file/chunk" 1 ""
  exit ${status}
 else if ( !( ${?meanf} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified output file" 1 ""
  exit ${status}
 endif

# chunk attributes
#
 mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}"
 if ( ${status} ) exit ${status}

 set chunkinfo = ${tmp}

 set file = ( `grep '^physical_file ' ${chunkinfo} | sed 's/physical_file//'` )

 if ( "${file[2]}" != "exists" ) then
  errmsg.csh ${verbosity} ${myname} 6 "nonexistent data file ${file[1]}" 1 ""
  if ( ${status} ) exit ${status}
 endif

 set file      = ${file[1]}

 set chunk     =   `grep '^chunk '         ${chunkinfo} | sed 's/chunk//'`
 set type      =   `grep '^datatype '      ${chunkinfo} | sed 's/datatype//'`
 set l_endian  =   `grep '^little_endian ' ${chunkinfo} | sed 's/little_endian//'`
 set chunksize =   `grep '^size '          ${chunkinfo} | sed 's/size//'`
 set offset    =   `grep '^offset '        ${chunkinfo} | sed 's/offset//'`
 set dims      = ( `grep '^dimensions '    ${chunkinfo} | sed 's/dimensions//'` )
 set sizes     = ( `grep '^extents '       ${chunkinfo} | sed 's/extents//'`    )

 set chunkdims = `echo ${dims} | sed 's/ //g'`

# default for unspecified averaging dimensions
#
 if ( ${adims} == '' ) then
  set adims = ${chunkdims}
 endif

# dimensions in chunkdims not present in adims
#
 set notadims = `echo ${chunkdims} | sed -e 's/ //g' -e 's/[ '${adims}']//g'`

 if ( "${notadims}" == "${chunkdims}" ) then
  errmsg.csh ${verbosity} ${myname} 7 "nothing to do; chunkdims ${chunkdims}, to average over ${adims}" 1 ""
  if ( ${status} ) exit ${status}
 endif

# get rid of dimensions in adims not in chunkdims
#
 set adims = `echo ${chunkdims} | sed -e 's/ //g' -e 's/[ '${notadims}']//g'`

#echo "chunkdims '$chunkdims'\nnotadims '$notadims'\nadims '$adims'"
#exit

# adims combined size, etc.
#
 set ad = `echo ${adims} | awk '{print substr($1,1,1)}'`

 set asizes = ''
 set asize = 1
 set i = 1
 foreach d ( ${dims} )
  set s = ${sizes[${i}]}
  if ( `echo ${adims} | awk '{print index($1,"'${d}'")}'` != 0 ) then
   @ asize *= ${s}
  else
   set asizes = "${asizes} ${s}"
  endif
# echo i $i d $d asize $asize asizes \"$asizes\"
  @ i += 1
 end

 set asizes = `echo ${asize} ${asizes} | sed 's/ /:/g'`

#echo "asize '$asize'\nnotasizes '$notasizes'\nasizes '$asizes'"
#exit

# permute from order ${chunkdims} to ${adims}${otherdims}
#
 if ( "${chunkdims}" != "${adims}${notadims}" ) then
  ${permute} -memlimit ${chunksize} -input ${infile} -chunk ${chunk} -headerout ${tmp} -order "${adims}${notadims}" >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 8 "failed command" ${status} "${permute} -memlimit ${chunksize} -input ${infile} -chunk ${chunk} -headerout ${tmp} -order '${adims}${notadims}' >& ${tmp}.err"
  if ( ${status} ) exit ${status}

# set mrifile = ${tmp}
  set datfile = ${tmp}.dat
 else
# set mrifile = ${infile}
  set datfile = ${file}
 endif

# mri file describing the permuted file
# ${adims} is represented by a single dimension ${ad}
#
 mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset ${offset} -file ${datfile} -mri ${tmp} -chunk ${chunk} -type ${type} -dimensions ${ad}${notadims} -extents ${asizes}

 errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset ${offset} -file ${datfile} -mri ${tmp} -chunk ${chunk} -type ${type} -dimensions ${ad}${notadims} -extents ${asizes}"
 if ( ${status} ) exit ${status}

#echo "SETUP COMPLETE"; exit
 
# calculate mean
#
 mri_subsample -mean -d ${ad} -e 1 ${tmp} ${meanf} >& ${tmp}.err
 
 errmsg.csh ${verbosity} ${myname} 10 "failed command" ${status} "mri_subsample -mean -d ${ad} -e 1 ${tmp} ${meanf} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# exit if rms error is not requested
#
 if ( !( ${?rmsf} ) ) then
  rm -f ${tmp}* >& /dev/null; exit 0
 endif

# calculate individual squared errors
#
 mri_interp -c -d ${ad} -e ${asize} ${meanf} ${tmp}.mean >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 11 "failed command" ${status} "mri_interp -c -d ${ad} -e ${asize} ${meanf} ${tmp}.mean >& ${tmp}.err"
 if ( ${status} ) exit ${status}

 mri_rpn_math -o ${tmp}.se -c ${chunk} '$1,$2,-,dup,*' ${tmp} ${tmp}.mean >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 12 "failed command" ${status} "mri_rpn_math -o ${tmp}.se -c ${chunk} '"'$1,$2,-,dup,*'"' ${tmp} ${tmp}.mean >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# calculate sum of individual squared errors
#
 mri_subsample -sum -d ${ad} -e 1 ${tmp}.se ${tmp} >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 13 "failed command" ${status} "mri_subsample -sum -d ${ad} -e 1 ${tmp}.se ${tmp} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# rms error
#
 @ n = ${asize} - 1

 mri_rpn_math -o ${rmsf} -c ${chunk} '$1,'${n}',/,0,0,'${n}',==,if_keep,abs,sqrt' ${tmp} >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 14 "failed command" ${status} "mri_rpn_math -o ${rmsf} -c ${chunk} '$1,'${n}',/,0,0,$2,==,if_keep,abs,sqrt' ${tmp} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
