#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set adims = ''
#set permute = permute.csh # this is a workaround for a permute bug
 set permute = mri_permute # this bug is gone in 5.1

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Average out a chunk over specified dimensions.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-a|-average_over dimensions]'
    echo ' -e|-exclusions file[.mri] -ec|-excl_chunk chunk'
    echo ' -m|-mean file[.mri] [-r|-rmse file[.mri]]'
    echo ' -i|-infile infile[.mri] -c|-chunk chunk'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3: unspecified input file/chunk'
    echo ' 4: unspecified exclusions file/chunk'
    echo ' 5: unspecified output file for mean'
    echo ' 6: incompatible averaging dimensions'
    echo ' 7: incompatible averaging and exclusions chunks'
    echo ' 8-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.chunk.info.csh mri.create.chunk.csh mri.rename.chunk.csh'
    echo ' ${permute} mri_subsample mri_interp mri_rpn_math'
    echo ''
    echo 'Notes'
    echo ' 1 Unspecified averaging dimensions default to chunk dimensions.'
    echo ' 2 This script differs  from  mri.average.csh  in that it  allows'
    echo '   specifying a  list of exclusions  from the averaging process,'
    echo '   in the form of a chunk with values 0 or 1,  where 1 indicates'
    echo '   a value to be excluded.  The  averaging and exclusions chunks'
    echo '   are required to be mri_rpn_math-compatible for multiplication'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-a":
   case "-average_over":
    set adims = ${argv[2]}; shift; shift; breaksw;

   case "-m":
   case "-mean":
    set meanf = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-r":
   case "-rmse":
    set rmsf = ${argv[2]}; shift; shift; breaksw;

   case "-e":
   case "-exclusions":
    set exfile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-ec":
   case "-excl_chunk":
    set echunk = ${argv[2]}; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

## ------------
## error checks
## ------------

 if ( !( ${?infile} && ${?chunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified input file/chunk" 1 ""
  exit ${status}
 else if ( !( ${?exfile} && ${?echunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified exclusions file/chunk" 1 ""
  exit ${status}
 else if ( !( ${?meanf} ) ) then
  errmsg.csh ${verbosity} ${myname} 5 "unspecified output file" 1 ""
  exit ${status}
 endif

## ----------------
## chunk attributes
## ----------------

# averaging chunk
#
 mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 8 "failed command" ${status} "mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}"
 if ( ${status} ) exit ${status}

 set dfile     =   `grep '^physical_file ' ${tmp} | awk '{print $2}'`
 set type      =   `grep '^datatype '      ${tmp} | sed 's/datatype//'`
 set l_endian  =   `grep '^little_endian ' ${tmp} | sed 's/little_endian//'`
 set chunksize =   `grep '^size '          ${tmp} | sed 's/size//'`
 set offset    =   `grep '^offset '        ${tmp} | sed 's/offset//'`
 set sizes     = ( `grep '^extents '       ${tmp} | sed 's/extents//'`    )
 set dims      = ( `grep '^dimensions '    ${tmp} | sed 's/dimensions//'` )
 set chunkdims =   `echo ${dims}                  | sed 's/ //g'`

# exclusions chunk
#
 mri.chunk.info.csh -verbosity ${verbosity} -infile ${exfile} -chunk ${echunk} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri.chunk.info.csh -verbosity ${verbosity} -infile ${exfile} -chunk ${echunk} > ${tmp}"
 if ( ${status} ) exit ${status}

 set edfile     =   `grep '^physical_file ' ${tmp} | awk '{print $2}'`
 set etype      =   `grep '^datatype '      ${tmp} | sed 's/datatype//'`
 set el_endian  =   `grep '^little_endian ' ${tmp} | sed 's/little_endian//'`
 set echunksize =   `grep '^size '          ${tmp} | sed 's/size//'`
 set eoffset    =   `grep '^offset '        ${tmp} | sed 's/offset//'`
 set esizes     = ( `grep '^extents '       ${tmp} | sed 's/extents//'`    )
 set edims      = ( `grep '^dimensions '    ${tmp} | sed 's/dimensions//'` )
 set echunkdims =   `echo ${edims}                 | sed 's/ //g'`

## ----------------------------
## process averaging dimensions
## ----------------------------

 if ( ${adims} == '' ) set adims = ${chunkdims}

# at least one of the user-specified averaging dimensions
# must be present in chunkdims
#
 set notadims = `echo ${chunkdims} | sed 's/[ '${adims}']//g'`

 if ( "${notadims}" == "${chunkdims}" ) then
  errmsg.csh ${verbosity} ${myname} 6 "incompatible averaging dimensions" 1 ""
  exit ${status}
 endif

# get rid of dimensions in adims not present in chunkdims
#
 set adims = `echo ${chunkdims} | sed -e 's/ //g' -e 's/[ '${notadims}']//g'`

## ------------------------------------------------
## compatibility of averaging and exclusions chunks
## ------------------------------------------------

# echunkdims must be a subset of chunkdims
#
 if ( "`echo ${echunkdims} | sed 's/[ '${chunkdims}']//g'`" != "" ) then
  errmsg.csh ${verbosity} ${myname} 7 "incompatible averaging and exclusions chunk dimensions" 1 ""
  exit ${status}
 endif

# check sizes
#
 echo ${#dims}  ${dims}  ${sizes}  | awk '{j=(2+$1);for(i=2;i<=(1+$1);i++){print $i" "$j; j++}}' > ${tmp}
 sort ${tmp} > ${tmp}.a

 echo ${#edims} ${edims} ${esizes} | awk '{j=(2+$1);for(i=2;i<=(1+$1);i++){print $i" "$j; j++}}' > ${tmp}
 sort ${tmp} > ${tmp}.e

 grep "^[${echunkdims}]" ${tmp}.a > ${tmp}
 cmp -s ${tmp} ${tmp}.e

 if ( ${status} ) then
  errmsg.csh ${verbosity} ${myname} 8 "incompatible averaging and exclusions chunk sizes" 1 ""
  exit ${status}
 endif

## ------------------------
## process chunk dimensions
## ------------------------

# averaging/other dimensions for the exclusions chunk
#
 set noteadims = `echo ${echunkdims} | sed -e 's/ //g' -e 's/[ '${adims}']//g'`
 set eadims    = `echo ${echunkdims} | sed -e 's/ //g' -e 's/[ '${noteadims}']//g'`

# dimensions in chunkdims not pesent echunkdims
#
 set otheradims  = `echo ${adims} | sed 's/[ '${eadims}']//g'`
 set othernadims = `echo ${notadims} | sed 's/[ '${noteadims}']//g'`

# sort dimensions subsets
#
 set adims = ( `echo ${adims} | awk '{for(i=1;i<=length;i++)printf "%c\n", substr($1,i,1)}' | sort` )
 set adims =   `echo ${adims} | sed 's/ //g'`

 set notadims = ( `echo ${notadims} | awk '{for(i=1;i<=length;i++)printf "%c\n", substr($1,i,1)}' | sort` )
 set notadims =   `echo ${notadims} | sed 's/ //g'`

 set eadims = ( `echo ${eadims} | awk '{for(i=1;i<=length;i++)printf "%c\n", substr($1,i,1)}' | sort` )
 set eadims =   `echo ${eadims} | sed 's/ //g'`

 set noteadims = ( `echo ${noteadims} | awk '{for(i=1;i<=length;i++)printf "%c\n", substr($1,i,1)}' | sort` )
 set noteadims =   `echo ${noteadims} | sed 's/ //g'`

 set otheradims = ( `echo ${otheradims} | awk '{for(i=1;i<=length;i++)printf "%c\n", substr($1,i,1)}' | sort` )
 set otheradims =   `echo ${otheradims} | sed 's/ //g'`

 set othernadims = ( `echo ${othernadims} | awk '{for(i=1;i<=length;i++)printf "%c\n", substr($1,i,1)}' | sort` )
 set othernadims =   `echo ${othernadims} | sed 's/ //g'`

 # echo debug_1 chunkdims \"$chunkdims\" echunkdims \"$echunkdims\" adims \"$adims\"
 # echo debug_1 adims \"$adims\" notadims \"$notadims\"
 # echo debug_1 eadims \"$eadims\" noteadims \"$noteadims\"
 # echo debug_1 otheradims \"$otheradims\" othernadims \"$othernadims\"

## -----------------------------------
## chunk sizes in convenient packaging
## -----------------------------------

 set size_ea = `grep '^['${eadims}']'     ${tmp}.e | awk '{printf " %d", $2}' | awk '{p=1;for(i=1;i<=NF;i++)p*=$i;print p}'`

 set size_oa = `grep '^[ '${otheradims}']' ${tmp}.a | awk '{printf " %d", $2}' | awk '{p=1;for(i=1;i<=NF;i++)p*=$i;print p}'`
 if ( "${size_oa}" == "" ) set size_oa = 1

 @ size_a = ${size_ea} * ${size_oa}

 set extents_nea = `grep '^[ '${noteadims}']'   ${tmp}.e | awk '{if(NR==1)printf "%d", $2;else printf ":%d", $2}'`
 set extents_ona = `grep '^[ '${othernadims}']' ${tmp}.a | awk '{if(NR==1)printf "%d", $2;else printf ":%d", $2}'`
 set extents_oa  = `grep '^[ '${otheradims}']'  ${tmp}.a | awk '{if(NR==1)printf "%d", $2;else printf ":%d", $2}'`

 # echo debug_2 size_ea \"$size_ea\"
 # echo debug_2 size_oa \"$size_oa\"
 # echo debug_2 size_a \"$size_a\"
 # echo debug_2 extents_nea \"$extents_nea\"
 # echo debug_2 extents_ona \"$extents_ona\"
 # echo debug_2 extents_oa \"$extents_oa\"

## ------------------------------------------
## name for the composite averaging dimension
## ------------------------------------------

 set ad = `echo ${eadims} | awk '{print substr($1,1,1)}'`

 # echo debug_3 ad \"$ad\"

## ------------------------
## prepare exclusions chunk
## ------------------------

# permute from order ${echunkdims} to ${eadims}${noteadims}
#
 set newdims = ${eadims}${noteadims}

 # echo debug_4 newdims \"$newdims\"

 if ( "${echunkdims}" != "${newdims}" ) then
  ${permute} -memlimit ${echunksize} -input ${exfile} -chunk ${echunk} -headerout ${tmp}.e -order ${newdims} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 12 "failed command" ${status} "${permute} -memlimit ${echunksize} -input ${exfile} -chunk ${echunk} -headerout ${tmp}.e -order ${newdims} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

# set emrifile = ${tmp}.e
  set edatfile = .dat
  set eofs     = 0
 else
# set emrifile = ${exfile}
  set edatfile = ${edfile}
  set eofs     = ${eoffset}
 endif

 # echo debug_5 edatfile \"$edatfile\"

# mri file describing the permuted file with
# ${eadims} represented by a single dimension ${ad}
#
 set newdims = ${ad}${noteadims}

 # echo debug_6 newdims \"$newdims\"

 set extents = ${size_ea}
 if ( ${extents_nea} != '' ) set extents = ${extents}':'${extents_nea}

 # echo debug_6 extents \"$extents\"

 mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${el_endian} -offset ${eofs} -file ${edatfile} -chunk ${chunk} -type ${etype} -dimensions ${newdims} -extents ${extents} -mri ${tmp}.e

 errmsg.csh ${verbosity} ${myname} 13 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${el_endian} -offset ${eofs} -file ${edatfile} -chunk ${chunk} -type ${etype} -dimensions ${newdims} -extents ${extents} -mri ${tmp}.e"
 if ( ${status} ) exit ${status}

# normalization factors for mean
#
 mri_subsample -sum -d ${ad} -e 1 ${tmp}.e ${tmp} >& ${tmp}.err
 
 errmsg.csh ${verbosity} ${myname} 14 "failed command" ${status} "mri_subsample -sum -d ${ad} -e 1 ${tmp}.e ${tmp} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

 mri_rpn_math -o ${tmp}.n ${size_ea}',$1,-,'${size_oa}',*' -c ${chunk} ${tmp} >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 15 "failed command" ${status} "mri_rpn_math -o ${tmp}.n '${size_ea}"',$1,-,'${size_oa}',*'"' -c ${chunk} ${tmp} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

## -----------------------
## prepare averaging chunk
## -----------------------

# permute from order ${chunkdims} to ${eadims}${noteadims}${othernadims}${otheradims}
#
 set newdims = ${eadims}${noteadims}${othernadims}${otheradims}

 # echo debug_7 newdims \"$newdims\"

 if ( "${chunkdims}" != "${newdims}" ) then
  ${permute} -memlimit ${chunksize} -input ${infile} -chunk ${chunk} -headerout ${tmp} -order ${newdims} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 16 "failed command" ${status} "${permute} -memlimit ${chunksize} -input ${infile} -chunk ${chunk} -headerout ${tmp} -order ${newdims} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

# set mrifile = ${tmp}
  set datfile = .dat
  set ofs     = 0
 else
# set mrifile = ${infile}
  set datfile = ${dfile}
  set ofs     = ${offset}
 endif

 # echo debug_7 datfile \"$datfile\"

# mri file describing the permuted file with
# ${eadims} represented by a single dimension ${ad}
#
 set newdims = ${ad}${noteadims}${othernadims}${otheradims}

 # echo debug_8 newdims \"$newdims\"

 set extents = ${size_ea}
 if ( ${extents_nea} != '' ) set extents = ${extents}':'${extents_nea}
 if ( ${extents_ona} != '' ) set extents = ${extents}':'${extents_ona}
 if ( ${extents_oa}  != '' ) set extents = ${extents}':'${extents_oa}

 # echo debug_8 extents \"$extents\"

 mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset ${ofs} -file ${datfile} -chunk ${chunk} -type ${type} -dimensions ${newdims} -extents ${extents} -mri ${tmp}

 errmsg.csh ${verbosity} ${myname} 17 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset ${ofs} -file ${datfile} -chunk ${chunk} -type ${type} -dimensions ${newdims} -extents ${extents} -mri ${tmp}"
 if ( ${status} ) exit ${status}

# multiply the two chunks together
#
 mri_rpn_math -o ${tmp}.a '$1,1,$2,-,*' -c ${chunk} ${tmp} ${tmp}.e >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 18 "failed command" ${status} "mri_rpn_math -o ${tmp}.a '"'$1,1,$2,-,*'"' -c ${chunk} ${tmp} ${tmp}.e >& ${tmp}.err"
 if ( ${status} ) exit ${status}

## ----
## mean
## ----

# permute dimensions to ${ad}${otheradims}${noteadims}${othernadims}
#
 set newdims = ${ad}${otheradims}${noteadims}${othernadims}

 # echo debug_9 newdims \"$newdims\"

 ${permute} -memlimit ${chunksize} -input ${tmp}.a -chunk ${chunk} -headerout ${tmp} -order ${newdims} >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 19 "failed command" ${status} "${permute} -memlimit ${chunksize} -input ${tmp}.a -chunk ${chunk} -headerout ${tmp} -order ${newdims} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# create .mri file with all ${adims} represented by ${ad}
#
 set newdims = ${ad}${noteadims}${othernadims}

 # echo debug_10 newdims \"$newdims\"

 set extents = ${size_a}
 if ( ${extents_nea} != '' ) set extents = ${extents}':'${extents_nea}
 if ( ${extents_ona} != '' ) set extents = ${extents}':'${extents_ona}

 # echo debug_10 extents \"$extents\"

 mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset 0 -file '.dat' -chunk ${chunk} -type ${type} -dimensions ${newdims} -extents ${extents} -mri ${tmp}

 errmsg.csh ${verbosity} ${myname} 20 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset 0 -file '.dat' -chunk ${chunk} -type ${type} -dimensions ${newdims} -extents ${extents} -mri ${tmp}"
 if ( ${status} ) exit ${status}

# sum up
#
 mri_subsample -sum -d ${ad} -e 1 ${tmp} ${tmp}.s >& ${tmp}.err
 
 errmsg.csh ${verbosity} ${myname} 21 "failed command" ${status} "mri_subsample -sum -d ${ad} -e 1 ${tmp} ${tmp}.s >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# normalize
#
 mri_rpn_math -o ${meanf} '$1,$2,/,0.0,$2,0.0,==,if_keep' -c ${chunk} ${tmp}.s ${tmp}.n >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 22 "failed command" ${status} "mri_rpn_math -o ${meanf} '"'$1,$2,/,0.0,$2,0.0,==,if_keep'"' -c ${chunk} ${tmp}.s ${tmp}.n >& ${tmp}.err"
 if ( ${status} ) exit ${status}

## ---------
## rms error
## ---------

# exit if rms error is not requested
#
 if ( !( ${?rmsf} ) ) then
  rm -f ${tmp}* >& /dev/null; exit 0
 endif

# individual squared errors
#
 mri_interp -c -d ${ad} -e ${size_ea} ${meanf} ${tmp} >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 24 "failed command" ${status} "mri_interp -c -d ${ad} -e ${size_ea} ${meanf} ${tmp} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

 mri_rpn_math -o ${tmp}.s -c ${chunk} '$1,$2,-,dup,*,1,$3,-,*' ${tmp}.a ${tmp} ${tmp}.e >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 25 "failed command" ${status} "mri_rpn_math -o ${tmp}.s -c ${chunk} '"'$1,$2,-,dup,*,1,$3,-,*'"' ${tmp}.a ${tmp} ${tmp}.e >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# permute dimensions to ${ad}${otheradims}${noteadims}${othernadims}
#
 set newdims = ${ad}${otheradims}${noteadims}${othernadims}

 # echo debug_11 newdims \"$newdims\"

 ${permute} -memlimit ${chunksize} -input ${tmp}.s -chunk ${chunk} -headerout ${tmp} -order ${newdims} >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 23 "failed command" ${status} "${permute} -memlimit ${chunksize} -input ${tmp}.s -chunk ${chunk} -headerout ${tmp} -order ${newdims} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# create .mri file with all ${adims} represented by ${ad}
#
 set newdims = ${ad}${noteadims}${othernadims}

 # echo debug_12 newdims \"$newdims\"

 set extents = ${size_a}
 if ( ${extents_nea} != '' ) set extents = ${extents}':'${extents_nea}
 if ( ${extents_ona} != '' ) set extents = ${extents}':'${extents_ona}

 # echo debug_12 extents \"$extents\"

 mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset 0 -file '.dat' -chunk ${chunk} -type ${type} -dimensions ${newdims} -extents ${extents} -mri ${tmp}

 errmsg.csh ${verbosity} ${myname} 24 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset 0 -file '.dat' -chunk ${chunk} -type ${type} -dimensions ${newdims} -extents ${extents} -mri ${tmp}"
 if ( ${status} ) exit ${status}

# sum up squared errors
#
 mri_subsample -sum -d ${ad} -e 1 ${tmp} ${tmp}.s >& ${tmp}.err
 
 errmsg.csh ${verbosity} ${myname} 25 "failed command" ${status} "mri_subsample -sum -d ${ad} -e 1 ${tmp} ${tmp}.s >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# normalize/rms error
#
# ATTN abs() is introduced before sqrt() to avoid
# problems with numerically small negative variances
#
 mri_rpn_math -o ${rmsf} '$1,$2,1,-,/,0,1,$2,<=,if_keep,abs,sqrt' -c ${chunk} ${tmp}.s ${tmp}.n >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 23 "failed command" ${status} "mri_rpn_math -o ${rmsf} '"'$1,$2,1,-,/,0,1,$2,<=,if_keep,abs,sqrt'"' -c ${chunk} ${tmp}.s ${tmp}.n >& ${tmp}.err"
 if ( ${status} ) exit ${status}

## -------
## cleanup
## -------

 rm -f ${tmp}* >& /dev/null; exit 0
