#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Print complete chunk information.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' -i|-infile file[.mri] -c|-chunk chunk'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.datfile.csh'
    echo ' mri.chunk.sizes.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 In addition to standard pghmri  chunk descriptors,'
    echo '   this looks for keys chunk.min.{dim} that specifies'
    echo '   the minimum index value for that dimension.'
    echo ' 2 Index ranges are 0-based.'
    echo ' 3 See mri.datfile.csh for a note on portability of the'
    echo '   full chunk file name reported by this script.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# chunk attributes
#
 set chunkfile = ( `mri.datfile.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk}` )

 errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "mri.datfile.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk}"
 if ( ${status} ) exit ${status}

 set chunktype = `mri_printfield -input ${infile} -field ${chunk}.datatype`

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_printfield -input ${infile} -field ${chunk}.datatype"
 if ( ${status} ) exit ${status}

 set chunkofs = `mri_printfield -input ${infile} -field ${chunk}.offset`

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri_printfield -input ${infile} -field ${chunk}.offset"
 if ( ${status} ) exit ${status}

 set chunksize = `mri_printfield -input ${infile} -field ${chunk}.size`

 errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "mri_printfield -input ${infile} -field ${chunk}.size"
 if ( ${status} ) exit ${status}

 set l_endian = `mri_printfield -input ${infile} -field ${chunk}.little_endian`

 if ( ${status} ) set l_endian = 0

# print chunk information
#
 echo "mri           ${infile}.mri\nchunk         ${chunk}\ndatatype      ${chunktype}\ntypesize      `mri.sizeof.csh ${chunktype}`\nfile          ${chunkfile[2]}\nphysical_file ${chunkfile[1]} ${chunkfile[3]}\noffset        ${chunkofs}\nlittle_endian ${l_endian}\nsize          ${chunksize}\n"

# dimensionwise sizes, lower and upper bounds on indices
#
 mri.chunk.sizes.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk}

# other chunk headers
#
 # echo "^${chunk} \n^${chunk}.min.[a-z]\n^${chunk}.extent.[a-z]\n^${chunk}.datatype\n^${chunk}.dimensions\n^${chunk}.file\n^${chunk}.little_endian\n^${chunk}.offset\n^${chunk}.size" > ${tmp}
 # mri.find.key.csh -verbosity ${verbosity} -infile ${infile} -operation beginswith "${chunk}" | pretty.print.csh -s '=' - | grep -v -f ${tmp}

 echo ''
 mri.find.key.csh -verbosity ${verbosity} -infile ${infile} -operation beginswith "${chunk}" | pretty.print.csh -s '=' - | grep -v -e "^${chunk}.min.[a-z]" -e "^${chunk}.extent.[a-z]" -e "^${chunk}.datatype" -e "^${chunk}.dimensions" -e "^${chunk}.file" -e "^${chunk}.little_endian" -e "^${chunk}.offset" -e "^${chunk}.size" -e "^${chunk} "

# if status == 1, this is most likely
# because grep found nothing to print,
# which is not a problem at all.
#
 if ( ${status} == 1 ) set status = 0

# cleanup
#
# rm -f ${tmp}* >& /dev/null; exit 0
