#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Dimensionwise chunk sizes and index bounds.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-n|-nontrivial]'
    echo ' -i|-infile file[.mri] -c|-chunk chunk'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 In addition to standard pghmri  chunk descriptors,'
    echo '   this looks for keys chunk.min.{dim} that specifies'
    echo '   the minimum index value for that dimension.'
    echo ' 2 Index ranges are 0-based.'
    echo ' 3 -nontrivial prints only dimensions with extents => 1.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-n":
   case "-nontrivial":
    set nontrivial; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# dimensionwise sizes, lower and upper bounds on indices
#
 set chunkdims = `mri_printfield -input ${infile} -field ${chunk}.dimensions`

 errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "mri_printfield -input ${infile} -field ${chunk}.dimensions"
 if ( ${status} ) exit ${status}

 set numdims = ${%chunkdims}

 set dims  = ()
 set sizes = ()
 set lower = ()
 set upper = ()

 set i = 1
 while ( ${i} <= ${numdims} )
  set d = `echo ${chunkdims} | awk '{print substr( $1, '${i},' 1 )}'`

  set s = `mri_printfield -input ${infile} -field ${chunk}.extent.${d}`

  errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_printfield -input ${infile} -field ${chunk}.extent.${d}"
  if ( ${status} ) exit ${status}

  if ( ${?nontrivial} && ( ${s} <= 1 ) ) goto skipdim

  if ( ${s} == "" ) then
   set s = 0
   set l = 0
   set u = -1
  else
   set l = `mri_printfield -input ${infile} -field ${chunk}.min.${d}`
   if ( ${status} ) set l = 0

   @ u  = ${l} + ${s}
   @ u -= 1
  endif

  set dims  = ( ${dims}  ${d} )
  set sizes = ( ${sizes} ${s} )
  set lower = ( ${lower} ${l} )
  set upper = ( ${upper} ${u} )

  skipdim:

  @ i += 1
 end

 echo "numdims ${numdims}\ndimensions ${dims}\nextents ${sizes}\nlower ${lower}\nupper ${upper}" | pretty.print.csh -

# 0-based index of the leading nontrivial (i. e., extent > 1)
# dimension. If all sizes == 1, this is defined to be the very first.
#
 set leaddim = `echo ${sizes} | awk '{for(i=1;(i<=NF)&&($i==1);i++); if(i>NF)print 0; else print i-1}'`

 echo "\nleading_nontrivial_dimension_index ${leaddim}"
 echo "leading_nontrivial_dimension" `echo ${leaddim} ${dims} | awk '{print substr($2,$1,1)}'`

# cleanup
#
# rm -f ${tmp}* >& /dev/null; exit 0
