#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set comparison = absolute
 set threshold  = 1.0e-6
 set notext     = ''
 set dirs = ()

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' comparison of text and chunkwise contents of'
    echo ' two *directories* containing .mri files with'
    echo ' the same names.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-t|-threshold numeric_threshold]'
    echo ' [-a|-absolute|-r|-relative]'
    echo ''
    echo ' [-n|-notext]'
    echo ''
    echo ' directory1 directory2'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination; text contents match, chunks match to within threshold'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified/non-existent directories'
    echo ' 3: missing .mri files in the second directory'
    echo ' 4: comparison failed for at least one pair of .mri files'
    echo ' 5-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.cmp.csh'
    echo ''
    echo 'Notes'
    echo ' 1 see  mri.cmp.csh -help for more details on command line arguments.'
    echo ' 2 all files names in the first directory should exist in the second.'
#   echo ''
#   echo 'Defaults'
#   echo " threshold: ${threshold}, comparison: ${comparison}"
#   echo ''
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-t":
   case "-threshold":
    set threshold = "${argv[2]}"; shift; shift; breaksw;

   case "-r":
   case "-relative":
    set comparison = relative; shift; breaksw;

   case "-a":
   case "-absolute":
    set comparison = absolute; shift; breaksw;

   case "-n":
   case "-notext":
    set notext = "-notext"; shift; breaksw;

   default:
    set dirs = ( ${dirs} ${argv[1]} ); shift; breaksw;
  endsw
 end

# error checks
#
 echo dirs \"$dirs\"

 if ( ${#dirs} < 2 ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/non-existent directories" 1 ""
  exit ${status}
 else if ( !( -d ${dirs[1]} ) || !( -d ${dirs[2]} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/non-existent directories" 1 ""
  exit ${status}
 endif

# filewise comparison
#
 pushd ${dirs[1]} >& /dev/null
 ls *.mri | sort >& ${tmp}
 popd >& /dev/null

 pushd ${dirs[2]} >& /dev/null
 ls *.mri | sort >& ${tmp}.1
 popd >& /dev/null

 head -`cat ${tmp} | wc -l` ${tmp}.1 > ${tmp}.2
 cmp -s ${tmp} ${tmp}.2

 if ( ${status} ) then
  if ( ${verbosity} > 0 ) echo "missing .mri files in the second directory"
  exit 3
 endif

 set exitstatus = 0

 foreach f ( `cat ${tmp}` )
  if ( ${verbosity} > 0 ) echo "file ${f}.mri:"

  mri.cmp.csh -verbosity ${verbosity} ${notext} -${comparison} -threshold ${threshold} ${dirs[1]}/${f} ${dirs[2]}/${f}

  set s = ${status}
  if ( ( ${s} == 3 ) || ( ${s} == 4 ) || ( ${s} == 5 ) ) then
   set exitstatus = 4
  else if ( ${s} != 0 ) then
   errmsg.csh ${verbosity} ${myname} 5 "failed command" ${s} "mri.cmp.csh -verbosity ${verbosity} ${notext} -${comparison} -threshold ${threshold} ${dirs[1]}/${f} ${dirs[2]}/${f}"
   exit ${status}
  endif
 end

# cleanup
#
 rm -f ${tmp}* >& /dev/null

 exit ${exitstatus}
