#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set files = ()

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' comparison of text contents of two .mri files.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' file1[.mri] file2[.mri]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination; text contents match'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified/non-existent input file(s)'
    echo ' 3: non-identical text contents'
    echo ' 4-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.list.txt.csh'
    echo ''
    echo 'Notes'
    echo ' 1 .file and .offset keys for a chunk are ignored.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   default:
    set files = ( ${files} ${argv[1]} ); shift; breaksw;
  endsw
 end

# error checks
#
 if ( ${#files} < 2 ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/non-existent input file(s)" 1 ""
  exit ${status}
 endif

 set files = ( `echo ${files[1]} ${files[2]} | sed -e 's/\.mri / /g' -e 's/\.mri$//'` )

 if ( !( -f ${files[1]}.mri ) || !( -f ${files[2]}.mri ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/non-existent input file(s)" 1 ""
  exit ${status}
 endif

# list the two files
#
 mri.list.txt.csh -verbosity ${verbosity} ${files[1]} | grep -v -e '\.file \=' -e '\.offset \=' | sort > ${tmp}.1

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri.list.txt.csh -verbosity ${verbosity} ${files[1]} | grep -v -e '\.file \=' -e '\.offset \=' | sort > ${tmp}.1"
 if ( ${status} ) exit ${status}

 mri.list.txt.csh -verbosity ${verbosity} ${files[2]} | grep -v -e '\.file \=' -e '\.offset \=' | sort > ${tmp}.2

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri.list.txt.csh -verbosity ${verbosity} ${files[2]} | grep -v -e '\.file \=' -e '\.offset \=' | sort > ${tmp}.2"
 if ( ${status} ) exit ${status}

# compare
#
 cmp -s ${tmp}.1 ${tmp}.2
 set s = ${status}

 if ( ${s} ) then
  set s = 3
  if ( ${verbosity} > 0 ) echo 'text contents differ'
 endif

# cleanup
#
 rm -f ${tmp}* >& /dev/null

 exit ${s}
