#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# misc
#
 set file   = '.dat'
 set offset = 0

 set l_endian = `endianness.csh numeric`

 set description = ''

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Create an empty chunk and the corresponding .mri header.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' Form 1: explicit specification'
    echo ''
    echo ' [-H|-headeronly [-le|-little_endian flag] [-o|-offset offset] [-f|-file file]]'
    echo ' [-l|-lower a:b:c:d:...]'
    echo ' [-D|-description description]'
    echo ' -c|-chunk chunk -t|-type mritype'
    echo ' -d|-dimensions dimensions -e|-extents a:b:c:d:...'
    echo ' -m|-mri outfile[.mri]'
    echo ''
    echo ' Form 2: copy all settings for specified chunk'
    echo ''
    echo ' [-l|-lower a:b:c:d:...]'
    echo ' [-D|-description description]'
    echo ' [-t|-type mritype]'
    echo ' -i|-infile infile -c|-chunk chunk'
    echo ' -m|-mri outfile[.mri]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3: unspecified .mri file name'
    echo ' 4: unspecified chunk name'
    echo ' 6: unspecified chunk dimensions'
    echo ' 7: unspecified chunk extents'
    echo ' 8: unspecified/unknown chunk type'
    echo ' 5,9-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' endianness.csh mri.sizeof.csh mri.type2c.csh mri.chunk.info.csh'
    echo ' mri_from_ascii mri_type_convert'
    echo ''
    echo 'Notes'
    echo ' If -headeronly is specified, only a .mri header is created.'
    echo ' In this case,  the .file, .offset,  and .little_endian keys'
    echo ' can be specified (these are ignored otherwise).'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-H":
   case "-headeronly":
    set headeronly; shift; breaksw;

   case "-m":
   case "-mri":
    set mri = `echo ${argv[2]} | sed 's/\.mri$//'`;
    set mri = ${mri}.mri
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-t":
   case "-type":
    set type = ${argv[2]}; shift; shift; breaksw;

   case "-d":
   case "-dimensions":
    set dimensions = ${argv[2]}; shift; shift; breaksw;

   case "-e":
   case "-extents":
    set extents = ${argv[2]}; shift; shift; breaksw;

   case "-l":
   case "-lower":
    set lower = ${argv[2]}; shift; shift; breaksw;

   case "-D":
   case "-description":
    set description = "${argv[2]}"; shift; shift; breaksw;

   case "-o":
   case "-offset":
    set offset = ${argv[2]}; shift; shift; breaksw;

   case "-f":
   case "-file":
    set file = ${argv[2]}; shift; shift; breaksw;

   case "-le":
   case "-little_endian":
    set l_endian = ${argv[2]}; shift; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# error checks
#
 if ( !( ${?mri} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified .mri file" 1 ""
  exit ${status}
 else if ( !( ${?chunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified chunk name" 1 ""
  exit ${status}
 endif

 if ( ${?infile} ) then
  mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}

  errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}"
  if ( ${status} ) exit ${status}

  set dimensions = ( `grep '^dimensions ' ${tmp} | sed 's/dimensions//'` )
  set dimensions = `echo ${dimensions} | sed 's/ //g'`

  set extents = ( `grep '^extents '    ${tmp} | sed -e 's/extents//'` )
  set extents = `echo ${extents} | sed 's/ /:/g'`

  if ( !( ${?type} ) ) then
   set type = "`grep '^datatype ' ${tmp} | sed -e 's/datatype//'`"
  endif

  if ( !( ${?lower} ) ) then
   set lower = ( `grep '^lower '    ${tmp} | sed -e 's/lower//'` )
   set lower = `echo ${lower} | sed 's/ /:/g'`
  endif

  if ( !( ${?description} ) ) then
   set description = ( `grep '^description ' ${tmp} | sed 's/description//'` )
  endif
 else
  if ( !( ${?dimensions} ) ) then
   errmsg.csh ${verbosity} ${myname} 6 "unspecified chunk dimensions" 1 ""
   exit ${status}
  else if ( !( ${?extents} ) ) then
   errmsg.csh ${verbosity} ${myname} 7 "unspecified chunk extents" 1 ""
   exit ${status}
  else if ( !( ${?type} ) ) then
   errmsg.csh ${verbosity} ${myname} 8 "unspecified/unknown chunk type" 1 ""
   exit ${status}
  endif
 endif

# datatype size
#
 set typesize = `mri.sizeof.csh ${type}`

 if ( ${typesize} == 0 ) then
  errmsg.csh ${verbosity} ${myname} 8 "unspecified/unknown chunk type ${type}" 1 ""
  exit ${status}
 endif

 set numdims = ${%dimensions}

# create header only
#
 if ( ${?headeronly} ) then
  set extents = ( `echo ${extents} | sed -e 's/^:/1:/' -e 's/:$/:1/' -e 's/::/:1:/g' | awk -F: '{for ( i = 1; i <= NF; i++ ) print $i}'` )

  if ( ${#extents} < ${numdims} ) then
   set extents = ( ${extents} `awk '{for ( i = '${#extents}' + 1; i <= '${numdims}'; i++ ) print 1}'` )
  endif

  set size  = `echo ${extents} | awk '{s = 1; for ( i = 1; i <= NF; i++ ) s *= $i; print s}'`
  @   size *= ${typesize}

  echo "\!format = pgh\n\!version = 1.0\n${chunk} = [chunk]\n${chunk}.datatype = ${type}\n${chunk}.dimensions = ${dimensions}\n${chunk}.offset = ${offset}\n${chunk}.size = ${size}" > ${mri}

  if ( ${file} != '' ) echo "${chunk}.file = ${file}" >> ${mri}
  echo "${chunk}.little_endian = ${l_endian}"         >> ${mri}

  echo ${dimensions} ${extents} | awk '{for ( i = 1; i <= length( $1 ); i++ ) print "'${chunk}'.extent."substr($1,i,1)" = "$(i+1)}' >> ${mri}

 else

 # create chunk header + empty chunk using mri_from_ascii
 #
  echo ${numdims} | awk '{for ( i = 0; i <= $1; i++ ) printf " %d", 0; printf "\n"}' | mri_from_ascii -d ${dimensions} -e ${extents} -c ${chunk} -i ${dimensions} -o ${tmp} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "echo ${numdims} | awk '{for ( i = 0; i <= $1; i++ ) printf " %d", 0; print "\n"}' | mri_from_ascii -d ${dimensions} -e ${extents} -c ${chunk} -i ${dimensions} -o ${tmp} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

 # type convert using mri_type_convert
 #
  mri_type_convert -c ${chunk} -`mri.type2c.csh ${type} | sed 's/unsigned//'` ${tmp} ${mri} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 10 "failed command" ${status} "mri_type_convert -c ${chunk} -`mri.type2c.csh ${type} | sed 's/unsigned//'` ${tmp} ${mri} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

 endif

 if ( ${?lower} ) then
  set lower = ( `echo ${lower} | sed -e 's/^:/1:/' -e 's/:$/:1/' -e 's/::/:1:/g' | awk -F: '{for ( i = 1; i <= NF; i++ ) print $i}'` )

  if ( ${#lower} < ${numdims} ) then
   set lower = ( ${lower} `awk '{for ( i = '${#lower}' + 1; i <= '${numdims}'; i++ ) print 0}'` )
  endif

  echo ${dimensions} ${lower} | awk '{for ( i = 1; i <= length( $1 ); i++ ) print "'${chunk}'.min."substr($1,i,1)" = "$(i+1)}' >> ${mri}
 endif

 if ( "${description}" != '' ) echo ${chunk}.description = \"${description}\" >> ${mri}

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
