#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Construct full name of the data file for'
    echo ' a data chunk as described in a .mri file'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' -i|-infile file[.mri] -c|-chunk chunk'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3: unspecified input file/chunk'
    echo ' 4: invalid chunk?'
    echo ' 5-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' proc.path.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 The output of this script is  (1)  the  full name of the'
    echo '   data file,  (2) the value of the field  chunk.file,  and'
    echo '   (3) whether this file exists or not.'
    echo ' 2 The full name may not be portable across systems; e. g.,'
    echo '   when  a  file system  is mounted on  two computers under'
    echo '   different names.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# error checks
#
 if ( !( ${?infile} && ${?chunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified input file/chunk" 1 ""
  exit ${status}
 endif

 set chunkval = `mri_printfield -input ${infile} -field ${chunk}`

 errmsg.csh ${verbosity} ${myname} 4 "invalid chunk ${chunk}" ${status} "mri_printfield -input ${infile} -field ${chunk}"
 if ( ${status} ) exit ${status}

# extract directory path from ${infile}
#
 set infile = `proc.path.csh -verbosity ${verbosity} -split ${infile}`

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "proc.path.csh -verbosity ${verbosity} -split ${infile}"
 if ( ${status} ) exit ${status}

 # echo infile \"$infile\"

 set mripath = ${infile[1]}'/'
 set infile  = `echo ${infile} | sed 's/ /\//'`

 # echo mripath \"$mripath\"
 # echo infile \"$infile\"

# data file
#
 set chunkfile = `mri_printfield -input ${infile} -field ${chunk}.file`

 # echo chunkfile \"$chunkfile\"

 switch ( ${chunkfile} )
  case '':
   # echo 0
   set physfile = ${infile}.mri; breaksw;

  default:
   # echo 1
   set firstchar = `echo ${chunkfile} | awk '{print substr( $1, 1, 1 )}'`

   switch ( ${firstchar} )
    case '/':
     # echo 1.1
     # set physfile = ${mripath}${chunkfile}; breaksw; # can't remember why I did this
     set physfile = ${chunkfile}; breaksw;

    case '.':
     # echo 1.2
     set first2 = `echo ${chunkfile} | awk '{print substr( $1, 1, 2 )}'`
     set first3 = `echo ${chunkfile} | awk '{print substr( $1, 1, 3 )}'`

     if ( ( "${first2}" == './'  ) || ( "${first3}" == '../' ) ) then
      set physfile = ${mripath}${chunkfile}
     else
      set physfile = ${infile}${chunkfile} # extension; .dat, e. g.
     endif

     breaksw;

    default:
     # echo 1.3
     set physfile = ${mripath}${chunkfile}; breaksw;
   endsw

   breaksw;
 endsw

 # echo physfile \"$physfile\"

 if ( -f ${physfile} ) then
  set msg = 'exists'
 else
  set msg = 'does_not_exist'
 endif

 echo ${physfile} \'${chunkfile}\' ${msg}
