#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# misc
#
 set operation = beginswith

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( "${argv[1]}" )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Print key-value pairs in a  .mri file for keys that'
    echo ' begin with, end with, are, or contain given string.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-o|-operation operation]'
    echo ' -i|-infile file[.mri]'
    echo ' string'
    echo ''
    echo 'operation'
    echo ' beginswith, b, endswith, e, has, h, is, i'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: nonexistent infile.mri'
    echo ' 3: invalid operation'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ''
    echo 'Notes'
    echo ' May not work properly for complicated search patterns.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-o":
   case "-operation":
    set operation = ${argv[2]}; shift; shift; breaksw;

   default:
    set string = "${argv[1]}"; shift; breaksw;
  endsw
 end

 if ( !( -f ${infile}.mri ) ) then
  errmsg.csh 1 ${myname} 2 "nonexistent ${infile}.mri" 1 ""
  exit ${status}
 endif

 set linecount = `grep -n '' ${infile}.mri | head -1 | awk -F: '{print $1}'`
 if ( "${linecount}" == '' ) set linecount = `wc -l ${infile}.mri | awk '{print $1}'`

 switch ( "${operation}" )
  case "b":
  case "beginswith":
   head -${linecount} ${infile}.mri | awk -F= '{k=$1;v=$0;sub(k"[ \t]*=[ \t]*","",v);gsub("^[ \t]*","",v);gsub("[ \t]*$","",v);gsub("^[ \t]*","",k);gsub("[ \t]*$","",k);if(match(k,"^'"${string}"'")!=0)print k" = "v}'
   breaksw

  case "e":
  case "endswith":
   head -${linecount} ${infile}.mri | awk -F= '{k=$1;v=$0;sub(k"[ \t]*=[ \t]*","",v);gsub("^[ \t]*","",v);gsub("[ \t]*$","",v);gsub("^[ \t]*","",k);gsub("[ \t]*$","",k);if(match(k,"'"${string}"'$")!=0)print k" = "v}'
   breaksw

  case "i":
  case "is":
   head -${linecount} ${infile}.mri | awk -F= '{k=$1;v=$0;sub(k"[ \t]*=[ \t]*","",v);gsub("^[ \t]*","",v);gsub("[ \t]*$","",v);gsub("^[ \t]*","",k);gsub("[ \t]*$","",k);if(match(k,"^'"${string}"'$")!=0)print k" = "v}'
   breaksw

  case "h":
  case "has":
   head -${linecount} ${infile}.mri | awk -F= '{k=$1;v=$0;sub(k"[ \t]*=[ \t]*","",v);gsub("^[ \t]*","",v);gsub("[ \t]*$","",v);gsub("^[ \t]*","",k);gsub("[ \t]*$","",k);if(index(k,"'"${string}"'")!=0)print k" = "v}'
   breaksw

  default:
   errmsg.csh 1 ${myname} 3 "invalid operation ${operation}" 1 ""
   exit ${status}
 endsw
