#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# misc
#
 set infile = ''
 set chunk  = ''

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Print dimensionwise size and index range for a chunk.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' -i|-infile file[.mri] -c|-chunk chunk'
    echo ' [dimensions]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.chunk.info.csh'
    echo ''
    echo 'Notes'
    echo ' 1 In addition to standard pghmri  chunk descriptors,'
    echo '   this looks for keys chunk.min.{dim} that dpecifies'
    echo '   the minimum index value for that dimension.'
    echo ' 2 Index ranges are 0-based.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   default:
    set dimensions = ${argv[1]}; shift; breaksw;
  endsw
 end

# chunk attributes
#
 mri.chunk.info.csh -verbosity ${verbosity} -infile "${infile}" -chunk "${chunk}" > ${tmp}

 errmsg.csh ${verbosity} ${myname} 2 "failed command" ${status} "mri.chunk.info.csh -verbosity ${verbosity} -infile "${infile}" -chunk "${chunk}" > ${tmp}"
 if ( ${status} ) exit ${status}

# dimensionwise sizes, lower and upper bounds on indices
#
 set dims  = ( `grep '^dimensions ' ${tmp} | sed 's/dimensions//'` )
 set sizes = ( `grep '^extents '    ${tmp} | sed 's/extents//'`    )
 set lower = ( `grep '^lower '      ${tmp} | sed 's/lower//'`      )
 set upper = ( `grep '^upper '      ${tmp} | sed 's/upper//'`      )

 if ( !( ${?dimensions} ) ) set dimensions = `echo ${dims} | sed 's/ //g'`

 set i = 1
 while ( ${i} <= ${#dims} )
  echo ${dims[${i}]} ${sizes[${i}]} ${lower[${i}]} ${upper[${i}]} | grep '^['${dimensions}']'
  @ i += 1
 end

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
