#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# misc
#
 set chunk  = missing
 set zlabel = "z/"
 set tlabel = "/t"

 set infile = ''
 set zrange = ''
 set trange = ''

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Display the "missing" chunk in a tabular form'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-t|-trange [n][:[m]]] [-z|-zrange [n][:[m]]]'
    echo ' file[.mri]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 3: invalid "missing" chunk'
    echo ' 2,4,5: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' proc.rangeopts.csh mri.chunk.info.csh'
    echo ''
    echo 'Notes'
    echo ' Datatype assumed to be "uint8", dimensions assume to be "zt"'
#   echo ''
#   echo 'Defaults'
#   echo " t,z ranges: entire"
#   echo ''
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-t":
   case "-trange":
    set trange = ${argv[2]}; shift; shift; breaksw;

   case "-z":
   case "-zrange":
    set zrange = ${argv[2]}; shift; shift; breaksw;

   default:
    set infile = `echo ${argv[1]} | sed 's/\.mri$//'`;
    shift; breaksw;
  endsw
 end

# dimensionwise sizes, lower and upper bounds on indices
#
 mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 2 "failed command" ${status} "mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}"
 if ( ${status} ) exit ${status}

 set chunktype = `grep '^datatype '   ${tmp} | awk '{print $2}'`
 set chunkdims = `grep '^dimensions ' ${tmp} | awk '{print $2$3}'`

 if ( ( "${chunktype}" != "uint8" ) || ( "${chunkdims}" != "zt" ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "invalid 'missing' chunk" 1 ""
  exit ${status}
 endif

 set chunkofs  = `grep '^offset '        ${tmp} | awk '{print $2}'`
 set chunkfile = `grep '^physical_file ' ${tmp} | awk '{print $2}'`

#set dims  = ( `grep '^dimensions ' ${tmp} | sed 's/dimensions//'` )
 set sizes = ( `grep '^extents '    ${tmp} | sed 's/extents//'`    )

 set lower = ( `grep '^lower ' ${tmp} | sed 's/lower//'` )
 set upper = ( `grep '^upper ' ${tmp} | sed 's/upper//'` )

# ranges
#
 set zrange = ( `proc.rangeopts.csh -verbosity ${verbosity} -numeric -defaults ${lower[1]}:${upper[1]} ${zrange}` )

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "proc.rangeopts.csh -verbosity ${verbosity} -numeric -defaults ${lower[1]}:${upper[1]} ${zrange}"
 if ( ${status} ) exit ${status}

 set trange = ( `proc.rangeopts.csh -verbosity ${verbosity} -numeric -defaults ${lower[2]}:${upper[2]} ${trange}` )

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "proc.rangeopts.csh -verbosity ${verbosity} -numeric -defaults ${lower[2]}:${upper[2]} ${trange}"
 if ( ${status} ) exit ${status}

 set numZ  = `echo ${trange} ${zrange} | awk '{t=($2-$1+1); z=($4-$3+1); print t*z" "t" "z}'`
 set numTZ = ${numZ[1]}; shift numZ
 set numT  = ${numZ[1]}; shift numZ

# print missing chunk
#
 set tSize = ${%trange[2]}
 if ( ${tSize} < ${%tlabel} ) set tSize = ${%tlabel}

 set zSize = ${%zrange[2]}
 if ( ${zSize} < ${%zlabel} ) set zSize = ${%zlabel}
 @ zSize += 1

 printf '%'${tSize}'s' ${zlabel}
 echo '' | awk '{printf " "; for (z=0;z<'${numZ}';z++) printf "%'${zSize}'d", z; printf "\n"}'

 printf '%'${tSize}'s\n' ${tlabel}

 set zcount = ( `echo '' | awk '{for (i=1;i<='${numZ}';i++) printf " %d", 0}'` )

 set t = ${trange[1]}
 while ( ${t} <= ${trange[2]} )
  set ofs = `echo '' | awk '{print '${t}' * '${sizes[1]}' + '${zrange[1]}' + '${chunkofs}'}'`
  set m   = ( `od -An -vtu1 -j${ofs} -N${numZ} ${chunkfile}` )

  set tcount = `echo ${m} | awk '{s=0;for(i=1;i<=NF;i++)s+=$i;print s}'`
  set zcount = ( `echo ${zcount} ${m} | awk '{j='${numZ}';for (i=1;i<='${numZ}';i++) print $i+$(j+i)}'` )

  printf "%${tSize}d " ${t}
  printf "%${zSize}c" `echo ${m} | sed -e 's/0/\./g' -e 's/1/X/g'`
  printf "   %${zSize}d\n" ${tcount}

  @ t += 1
 end

 set tcount = `echo ${zcount} | awk '{s=0;for(i=1;i<=NF;i++)s+=$i;print s}'`

 printf "\nslicewise counts"
 printf " %d" ${zcount}
 printf "\ntotal missing %d of %d\n" ${tcount} ${numTZ}

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
