#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
#set permute = permute.csh # this is a workaround for a permute bug
 set permute = mri_permute # this bug is gone in 5.1

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Matrix-vector multiplication.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' -m|-matrix matrix[.mri] -mc|-matchunk matrix_chunk -md|-matdims matrix_row_col_dimensions'
    echo ' -x|-vector vector[.mri] -xc|-vecchunk vector_chunk -xd|-vecdim  vector_dimension'
    echo ' -o|-outfile outfile[.mri] [-oc|-outchunk outchunk]'
    echo ''
    echo ' [-rq] [-RQ]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3: unspecified matrix file/chunk/dimensions'
    echo ' 4: unspecified vector file/chunk/dimensions'
    echo ' 5: invalid number for matrix dimensions'
    echo ' 6: invalid number for vector dimensions'
    echo ' 7: unspecified output file'
    echo ' 8: incompatible matrix/vector chunk dimensions'
    echo ' 9: matrix needs to be square for -RQ or -rq"
    echo '10-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.rename.chunk.csh'
    echo ' ${permute} mri_rpn_math mri_subsample mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 -rq and -RQ respectively compute (v^T M v) and'
    echo '   (v^T M v / v^T v) instead of (M v).'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-x":
   case "-vector":
    set vfile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-xc":
   case "-vecchunk":
    set vchunk = ${argv[2]}; shift; shift; breaksw;

   case "-xd":
   case "-vecdim":
    set vdim = ${argv[2]}; shift; shift; breaksw;

   case "-m":
   case "-matrix":
    set mfile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-mc":
   case "-matchunk":
    set mchunk = ${argv[2]}; shift; shift; breaksw;

   case "-md":
   case "-matdims":
    set mdims = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set outfile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-oc":
   case "-outchunk":
    set outchunk = ${argv[2]}; shift; shift; breaksw;

   case "-rq":
    set rq; shift; breaksw;

   case "-RQ":
    set rq RQ; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# error checks
#
 if ( !( ${?mfile} && ${?mchunk} && ${?mdims} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified matrix file/chunk/dimensions" 1 ""
  exit ${status}
 else if ( !( ${?vfile} && ${?vchunk} && ${?vdim} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified vector file/chunk/dimensions" 1 ""
  exit ${status}
 else if ( ${%mdims} != 2 ) then
  errmsg.csh ${verbosity} ${myname} 5 "invalid number for matrix dimensions in ${mdims}; expecting 2" 1 ""
  exit ${status}
 else if ( ${%vdim} != 1 ) then
  errmsg.csh ${verbosity} ${myname} 6 "invalid number for vector dimensions in ${vdim}; expecting 1" 1 ""
  exit ${status}
 else if ( !( ${?outfile} ) ) then
  errmsg.csh ${verbosity} ${myname} 7 "unspecified output file" 1 ""
  exit ${status}
 endif

# reverse matrix dimensions order from row/col to col/row
#
 set mdims = ( `echo ${mdims} | awk '{c=substr($1,2,1); r=substr($1,1,1); printf "%c %c %c%c", c, r, c, r}'` )
 set cdim  = ${mdims[1]}; set rdim  = ${mdims[2]};
 shift mdims; shift mdims;

# matrix/vector size compatibility
#
 set rsize = `mri_printfield -input ${mfile} -field ${mchunk}.extent.${rdim}`
 set csize = `mri_printfield -input ${mfile} -field ${mchunk}.extent.${cdim}`
 set vsize = `mri_printfield -input ${vfile} -field ${vchunk}.extent.${vdim}`

 if ( ${csize} != ${vsize} ) then
  errmsg.csh ${verbosity} ${myname} 8 "matrix/vector column dimensions mismatch" 1 ""
  exit ${status}
 endif

# if -RQ or -rq is specified, the matrix needs to be square
#
 if ( ( ${?RQ} || ${?rq} ) && ( ${csize} != ${rsize} ) ) then
  errmsg.csh ${verbosity} ${myname} 9 "matrix needs to be square for -RQ or -rq" 1 ""
  exit ${status}
 endif

# matrix/vector chunk compatibility
#
 set mchunkdims = `mri_printfield -input ${mfile} -field ${mchunk}.dimensions`

 errmsg.csh ${verbosity} ${myname} 10 "failed command" ${status} "mri_printfield -input ${mfile} -field ${mchunk}.dimensions"
 if ( ${status} ) exit ${status}

 set commondims = `echo ${mchunkdims} | sed 's/[ '${mdims}']//g'`
 set notmdims   = ( `echo ${commondims} | awk '{for(i=1;i<=length;i++)printf "%c\n", substr($1,i,1)}' | sort` )

 set vchunkdims = `mri_printfield -input ${vfile} -field ${vchunk}.dimensions`

 errmsg.csh ${verbosity} ${myname} 11 "failed command" ${status} "mri_printfield -input ${vfile} -field ${vchunk}.dimensions"
 if ( ${status} ) exit ${status}

 set notvdims = `echo ${vchunkdims} | sed 's/[ '${vdim}']//g'`
 set notvdims = ( `echo ${notvdims} | awk '{for(i=1;i<=length;i++)printf "%c\n", substr($1,i,1)}' | sort` )

 if ( "${notvdims}" == "" ) then
  set situation = 0
  set morder    = ${mdims}${commondims}
  set vorder    = ${cdim}
 else if ( "${notvdims}" == "${notmdims}" ) then
  set situation = 1
  set morder    = ${commondims}${mdims}
  set vorder    = ${commondims}${cdim}
 else
  errmsg.csh ${verbosity} ${myname} 12 "incompatible matrix/vector chunk dimensions" 1 ""
  exit ${status}
 endif

# prepare matrix chunk
#
 mri.rename.chunk.csh -verbosity ${verbosity} -infile ${mfile} -chunk ${mchunk} -to images -o ${tmp}.m

 errmsg.csh ${verbosity} ${myname} 13 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${mfile} -chunk ${mchunk} -to images -o ${tmp}.m"
 if ( ${status} ) exit ${status}

 if ( "${mchunkdims}" == "${morder}" ) then
  set pmfile = ${tmp}.m
 else
  ${permute} -m `mri_printfield -input ${tmp}.m -field images.size` -i ${tmp}.m -h ${tmp}.pm -o ${morder} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 14 "failed command" ${status} "${permute} -m `mri_printfield -input ${tmp}.m -field images.size` -i ${tmp}.m -h ${tmp}.pm -o ${morder} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  set pmfile = ${tmp}.pm
 endif

# prepare vector chunk
#
 mri.rename.chunk.csh -verbosity ${verbosity} -infile ${vfile} -chunk ${vchunk} -to images -o ${tmp}

 errmsg.csh ${verbosity} ${myname} 15 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${vfile} -chunk ${vchunk} -to images -o ${tmp}"
 if ( ${status} ) exit ${status}

 sed -e "s/dimensions \= ${vdim}/dimensions \= ${cdim}/" -e "s/extent.${vdim}/extent.${cdim}/" ${tmp}.mri > ${tmp}.v.mri

 if ( "${vchunkdims}" == "${vorder}" ) then
  set pvfile = ${tmp}.v
 else
  ${permute} -m `mri_printfield -input ${tmp}.v -field images.size` -i ${tmp}.v -h ${tmp}.pv -o ${vorder} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 16 "failed command" ${status} "${permute} -m `mri_printfield -input ${tmp}.v -field images.size` -i ${tmp}.v -h ${tmp}.pv -o ${vorder} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  set pvfile = ${tmp}.pv
 endif

# multiplication
#
 mri_rpn_math -o ${tmp} '$1,$2,*' ${pmfile} ${pvfile} >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 17 "failed command" ${status} "mri_rpn_math -o ${tmp} '"'$1,$2,*'"' ${pmfile} ${pvfile} >& ${tmp}.err"
 if ( ${status} ) exit ${status}

 mri_subsample -sum -e 1 -d ${cdim} ${tmp} ${tmp}.mv >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 18 "failed command" ${status} "mri_subsample -sum -e 1 -d ${cdim} ${tmp} ${tmp}.mv >& ${tmp}.err"
 if ( ${status} ) exit ${status}

 set pmfile = ${tmp}.mv

# compute Rayleigh quotient if requested
#
 if ( ${?rq} ) then
  if ( ${situation} == 0 ) then
   sed -e "s/dimensions \= ${cdim}/dimensions \= ${rdim}/" -e "s/extent.${cdim}/extent.${rdim}/" ${pvfile}.mri > ${tmp}.mri
  else
   sed -e "s/dimensions \= ${commondims}${cdim}/dimensions \= ${commondims}${rdim}/" -e "s/extent.${cdim}/extent.${rdim}/" ${pvfile}.mri > ${tmp}.mri
  endif

  mv -f ${tmp}.mri ${pvfile}.mri

  # numerator
  #
  mri_rpn_math -o ${tmp} '$1,$2,*' ${tmp}.mv ${pvfile} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 19 "failed command" ${status} "mri_rpn_math -o ${tmp} '"'$1,$2,*'"' ${tmp}.mv ${pvfile} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  mri_subsample -sum -e 1 -d ${rdim} ${tmp} ${tmp}.mv >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 20 "failed command" ${status} "mri_subsample -sum -e 1 -d ${rdim} ${tmp} ${tmp}.mv >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  set pmfile = ${tmp}.mv

  if ( ${?RQ} ) then
   # denominator
   #
   mri_rpn_math -o ${tmp} '$1,dup,*' ${pvfile} >& ${tmp}.err

   errmsg.csh ${verbosity} ${myname} 21 "failed command" ${status} "mri_rpn_math -o ${tmp} '"'$1,dup,*'"' ${pvfile} >& ${tmp}.err"
   if ( ${status} ) exit ${status}

   mri_subsample -sum -e 1 -d ${rdim} ${tmp} ${tmp}.vv >& ${tmp}.err

   errmsg.csh ${verbosity} ${myname} 22 "failed command" ${status} "mri_subsample -sum -e 1 -d ${rdim} ${tmp} ${tmp}.vv >& ${tmp}.err"
   if ( ${status} ) exit ${status}

   # calculate sum_of_matrix_elements/vector_dimension for zero-norm cases
   #
   mri.average.csh -verbosity ${verbosity} -infile ${tmp}.pm -chunk images -average_over ${rdim}${cdim} -mean ${tmp}

   errmsg.csh ${verbosity} ${myname} 23 "failed command" ${status} "mri.average.csh -verbosity ${verbosity} -infile ${tmp}.pm -chunk images -average_over ${rdim}${cdim} -mean ${tmp}"
   if ( ${status} ) exit ${status}

   # rearrange dimensions ordering
   #
   sed -e 's/'${rdim}${commondims}'/'${commondims}${cdim}'/g' -e 's/'${cdim}${commondims}'/'${commondims}${cdim}'/g' ${tmp}.mri > ${tmp}.p.mri
   mv -f ${tmp}.p.mri ${tmp}.mri

   # average has been divided by an extra factor of row/col size: need to adjust
   #
   mri_rpn_math -o ${tmp}.am '$1,'${rsize}',*' ${tmp} >& ${tmp}.err

   errmsg.csh ${verbosity} ${myname} 24 "failed command" ${status} "mri_rpn_math -o ${tmp}.am '"'$1,'${rsize}',*'"' ${tmp} >& ${tmp}.err"
   if ( ${status} ) exit ${status}

   # numerator/denominator; replace zero-norm cases by the correct limiting value
   #
   mri_rpn_math -o ${tmp} '$1,$2,/,$3,0.0,$2,==,if_keep' ${tmp}.mv ${tmp}.vv ${tmp}.am >& ${tmp}.err

   errmsg.csh ${verbosity} ${myname} 25 "failed command" ${status} "mri_rpn_math -o ${tmp} '"'$1,$2,/,$3,0.0,$2,==,if_keep'"' ${tmp}.mv ${tmp}.vv ${tmp}.am >& ${tmp}.err"
   if ( ${status} ) exit ${status}

   set pmfile = ${tmp}
  endif
 endif

# permute the product back to ${mdims}${commondims}
#
 if ( ${situation} == 1 ) then
  ${permute} -m `mri_printfield -input ${pmfile} -field images.size` -i ${pmfile} -h ${tmp}.pmv -o ${mdims}${commondims} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 26 "failed command" ${status} "${permute} -m `mri_printfield -input ${pmfile} -field images.size` -i ${pmfile} -h ${tmp}.pmv -o ${mdims}${commondims} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  set pmfile = ${tmp}.pmv
 endif

# rename product chunk to ${outchunk}
#
 if ( !( ${?outchunk} ) ) set outchunk = ${mchunk}

 mri.rename.chunk.csh -verbosity ${verbosity} -infile ${pmfile} -chunk images -to ${outchunk} -o ${tmp}

 errmsg.csh ${verbosity} ${myname} 27 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${pmfile} -chunk images -to ${outchunk} -o ${tmp}"
 if ( ${status} ) exit ${status}

 if ( ${?rq} || ${?RQ} ) then
  set dims = `grep "${outchunk}.dimensions" ${tmp}.mri | awk -F'=' '{print $2}' | sed 's/[ '${rdim}${cdim}']//g'`

  grep -v -e "${outchunk}.format" -e "${outchunk}.file" -e "${outchunk}.extent.${rdim}" -e "${outchunk}.extent.${cdim}" -e "${outchunk}.dimensions" ${tmp}.mri > ${outfile}.mri

  echo "${outchunk}.dimensions = ${dims}" >> ${outfile}.mri
  echo "${outchunk}.file = .dat"          >> ${outfile}.mri
 else
  grep -v "${outchunk}.file" ${tmp}.mri  > ${outfile}.mri
  echo "${outchunk}.file = .dat"        >> ${outfile}.mri
 endif

 mv -f ${pmfile}.dat ${outfile}.dat
 
# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
