#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# misc
#
 set indspecs  = ''
 set infile    = ''
 set chunk     = ''
 set chunkinfo = '' # preprocessed chunk info (mrichunkinfo format) if available

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Calculate byte offset corresponding to the given set of coordinate'
    echo ' indices for data in .mri format. Try "proc.indspecs.csh -help" for'
    echo ' details on index specification. Appropriate defaults are used  for'
    echo ' unspecified index values. Out-of-range indices cause an error exit.'
    echo ' Indices and byte offsets are both 0-based.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-d|-detailed] [-tc|-timecourse]'
    echo ' -i|-infile file[.mri] -c|-chunk chunk [indspecs]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' proc.indspecs.csh'
    echo ''
    echo 'Notes'
    echo ' 1 If -detailed is unspecified, this prints byte_offset record_offset.'
    echo ' 2 In the -timecourse mode, index corresponding to  leading nontrivial'
    echo '   dimension is set to 0 (or minimum allowed).'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-d":
   case "-detailed":
    set detailed; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-ci":
   case "-chunkinfo":
    set chunkinfo = ${argv[2]}; shift; shift; breaksw;

   case "-tc":
   case "-timecourse":
    set timecourse; shift; breaksw;

   default:
    set indspecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# dimensionwise sizes, lower and upper bounds on indices
#
 proc.indspecs.csh -verbosity ${verbosity} -detailed -chunkinfo "${chunkinfo}" -infile "${infile}" -chunk "${chunk}" ${indspecs} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 2 "failed command" ${status} "proc.indspecs.csh -verbosity ${verbosity} -detailed -chunkinfo "'"'${chunkinfo}'"'" -infile "'"'${infile}'"'" -chunk "'"'${chunk}'"'" ${indspecs} > ${tmp}"
 if ( ${status} ) exit ${status}

 set dims     = ( `grep '^dimensions '   ${tmp} | sed 's/dimensions//'` )
 set sizes    = ( `grep '^extents '      ${tmp} | sed 's/extents//'`    )
 set lower    = ( `grep '^lower '        ${tmp} | sed 's/lower//'`      )
 set upper    = ( `grep '^upper '        ${tmp} | sed 's/upper//'`      )
 set indices  = ( `grep '^indices '      ${tmp} | sed 's/indices//'`    )
 set typesize = ( `grep '^typesize '     ${tmp} | awk '{print $2}'`     )
 set chunkofs = ( `grep '^offset '       ${tmp} | awk '{print $2}'`     )

 set numdims = ${#dims}

 if ( ${?timecourse} ) then
  @ leadntdim = 1 + `grep leading_nontrivial_dimension_index ${tmp} | awk '{print $2}'`
  set indices[${leadntdim}] = ${lower[${leadntdim}]}
 endif

# record offset
#
 set offset = `echo ${numdims} ${indices} ${lower} ${sizes} | awk '{v=1;o=0;j=2+$1;k=j+$1;for(i=2;i<=($1+1);i++){o+=v*($i-$j);v*=$k;j++;k++};print o}'`

# same calculation the csh way
#
#set offset = 0
#set volume = 1
#set i      = 1
#while ( ${i} <= ${numdims} )
# @ value = ${indices[${i}]} - ${lower[${i}]}
# @ offset += ${value} * ${volume}
# @ volume *= ${sizes[${i}]}
# @ i += 1
#end

# byte offset
#
 set boffset = `echo ${offset} ${typesize} ${chunkofs} | awk '{print $1*$2+$3}'`

# same calculation the csh way
#
#@ boffset  = ${offset} * ${typesize}
#@ boffset += ${chunkofs}

# -detailed: print offsets, indices, sizes, lower and upper bounds
# otherwise: byte offset, record offset
#
 if ( ${?detailed} ) then
  cat ${tmp}
  echo "\nrec_offset    ${offset}\nbyte_offset   ${boffset}"
 else
  echo ${boffset} ${offset}
 endif

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
