#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set function  = ""

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo " print chunk indices and values for values satisfying a"
    echo " criterion with respect to a threshold;  no restriction"
    echo " on the number or names of chunk dimensions."
    echo ""
    echo " criteria (used as 'threshold criterion value')"
    echo "  <, >, <=, >=, ==, !="
    echo ""
    echo " function"
    echo "  any mri_rpn_math single-operand function that returns"
    echo "  a single value."
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-f|-function function]'
    echo " -t|-threshold threshold -C|-criterion 'criterion'"
    echo ' -i|-infile file[.mri] -c|-chunk chunk'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3: unspecified input file/chunk'
    echo ' 4: unspecified criterion/threshold'
    echo ' 5-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri_rpn_math'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-f":
   case "-function":
    set function = "${argv[2]}"; shift; shift; breaksw;

   case "-t":
   case "-threshold":
    set threshold = "${argv[2]}"; shift; shift; breaksw;

   case "-C":
   case "-criterion":
    set criterion = "${argv[2]}"; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# error checks
#
 if ( !( ${?infile} && ${?chunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified input file/chunk" 1 ""
  exit ${status}
 else if ( !( ${?criterion} && ${?threshold} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified criterion/threshold" 1 ""
  exit ${status}
 endif

# chunk dimensions
#
 set chunkdims = `mri_printfield -input ${infile} -field ${chunk}.dimensions`

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri_printfield -input ${infile} -field ${chunk}.dimensions"
 if ( ${status} ) exit ${status}

 set chunkdims = ( `echo ${chunkdims} | awk '{for(i=1;i<=length;i++)printf "%c ", substr($1,i,1)}'` )
 set numdims   = ${#chunkdims}

# construct scripts for mri_rpn_math and awk
#
 if ( ${function} != "" ) set function = "${function},"

 set rpnscript = ''
 set awkscript = '{print '

 set d = 1
 foreach dim ( ${chunkdims} )
  # index lower bound as specified through the key chunk.min.?
  #
  set lb = `mri_printfield -nofail -input ${infile} -field ${chunk}.min.${dim}`
  if ( ${lb} == 0  ) set lb = ''
  if ( ${lb} != '' ) set lb = ','${lb}',+'

  set rpnscript = "${rpnscript}"'$'${dim}${lb}','
  if ( ${d} == 1 ) then
   set awkscript = "${awkscript}"'"'${dim}'="$'${d}
  else
   set awkscript = "${awkscript}"'",'${dim}'="$'${d}
  endif
  @ d += 1
 end

 @ numdims += 1

 set rpnscript = "${rpnscript}"'$1,dup,'"${function}${threshold},${criterion}"',if_print_'${numdims}',0.0'
 set awkscript = "${awkscript}"'" "$'${numdims}'}'

 # echo rpnscript \"${rpnscript}\"
 # echo awkscript \"${awkscript}\"

# print thresholded values along with their coordinates
#
 mri_rpn_math -o ${tmp} "${rpnscript}" -c ${chunk} ${infile} |& grep -v -e '^script:' -e '^#' | awk "${awkscript}"

 errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "mri_rpn_math '${rpnscript}' -c ${chunk} ${infile} |& grep -v -e '^script:' -e '^#' | awk '${awkscript}'"
 if ( ${status} ) exit ${status}

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
