#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# misc
#
 set tcopt    = ''
 set indspecs = ''

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Extract and print a single record from data in .mri format.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-tc|-timecourse]'
    echo ' [-o|-outfile outfile]'
    echo ' -i|-infile infile[.mri] -c|-chunk chunk'
    echo ' [indspecs]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified input file/chunk'
    echo ' 3: nonexistent data file'
    echo ' 4-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' endianness.csh'
    echo ' mri.offset.csh'
    echo ' mri.extract.record.csh'
    echo ' mri_rpn_math'
    echo ''
    echo 'Notes'
    echo ' 1 Refer to  extract.record.csh  for details on  -timecourse.'
    echo ' 2 Refer to proc.indspecs.csh for index specification format.'
    echo ' 3 stdout assumed if outfile not specified.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-tc":
   case "-timecourse":
    set tcopt = '-timecourse'; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set outfile = ${argv[2]}; shift; shift; breaksw;

   default:
    set indspecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# error checks
#
 if ( !( ${?infile} && ${?chunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified input file/chunk" 1 ""
  exit ${status}
 endif

# endianness
#
 set me = `endianness.csh numeric`
 set de = `mri_printfield -nofail -input ${infile} -field ${chunk}.little_endian`
 if ( ${de} == "" ) set de = 0

# if data endianness matches machine endianness, use od
#
 if ( ${de} == ${me} ) then
  mri.offset.csh -verbosity ${verbosity} -detailed ${tcopt} -infile ${infile} -chunk ${chunk} ${indspecs} > ${tmp}

  errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri.offset.csh -verbosity ${verbosity} -detailed ${tcopt} -infile ${infile} -chunk ${chunk} ${indspecs} > ${tmp}"
  if ( ${status} ) exit ${status}

  # cat ${tmp}

  set odtype   = `grep datatype ${tmp} | awk '{print $2}'`
  set odtype   = `mri.type2od.csh ${odtype}`
  set tsize    = `grep typesize ${tmp} | awk '{print $2}'`

  # echo odtype \"$odtype\" tsize \"$tsize\"

  set datafile = ( `grep physical_file ${tmp}` )

  # echo datafile \"$datafile\"

  if ( "${datafile[3]}" != "exists" ) then
   errmsg.csh ${verbosity} ${myname} 3 "nonexistent data file ${datafile[2]}" 1 ""
   exit ${status}
  endif
  set datafile = ${datafile[2]}

  # echo datafile \"$datafile\"

  set extent = 1
  if ( "${tcopt}" == "-timecourse" ) then
   set leadntdim = `grep leading_nontrivial_dimension_index ${tmp} | awk '{print $2}'`
   set extent = `grep extents ${tmp} | awk '{print $('${leadntdim}'+2)}'`
  endif

  @ datasize = ${extent} * ${tsize}

  # echo extent \"$extent\" datasize \"$datasize\"

  set offset = `grep byte_offset ${tmp} | awk '{print $2}'`

  # echo offset \"$offset\"

  alias sf "awk '"'{for(i=1;i<=NF;i++)print $i}'"'"

  # which sf

  if ( ${?outfile} ) then
   od -An -vt${odtype} -j${offset} -N${datasize} ${datafile} | sf | head -${extent} > ${outfile}
  else
   od -An -vt${odtype} -j${offset} -N${datasize} ${datafile} | sf | head -${extent}
  endif
 else
  # extract record/time course
  #
  mri.extract.record.csh -verbosity ${verbosity} ${tcopt} -infile ${infile} -chunk ${chunk} -outfile ${tmp} ${indspecs}

  errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri.extract.record.csh -verbosity ${verbosity} ${tcopt} -infile ${infile} -chunk ${chunk} -outfile ${tmp} ${indspecs}"
  if ( ${status} ) exit ${status}

  # print using mri_rpn_math
  #
  ( mri_rpn_math -c ${chunk} '$1,1,if_print_1,0' ${tmp} |& grep -i -e '^[0-9]' -e '^-' -e '^nan' -e '^inf' > ${tmp}.out ) >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "( mri_rpn_math -c ${chunk} '$1,1,if_print_1,0' ${tmp} |& grep -i -e '^[0-9]' -e '^-' -e '^nan' -e '^inf' > ${tmp}.out ) >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  # dump to screen if outfile is not specified
  #
  if ( ${?outfile} ) then
   mv ${tmp}.out ${outfile}
  else
   cat ${tmp}.out
  endif
 endif

# cleanup
#
 rm -f ${tmp}* mri_rpn_math_out.{mri,dat} >& /dev/null; exit 0
