#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${0}
    echo ' Generate a new .mri file with the specified chunk renamed.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-cd|-copydata]'
    echo ' -i|-infile infile[.mri] -o|-outfile outfile[.mri]'
    echo ' -c|-chunk chunk -to newchunkname'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command line argument'
    echo ' 3: unspecified input/output file/chunk'
    echo ' 4: failed utility/script'
    echo ' 5: nonexistent chunk file'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.datfile.csh mri.find.key.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 If  -cd  option is specified, then the file containing'
    echo '   actual chunk  is copied to  {outfile}.dat;  the  .file'
    echo '   field in the output is then set to ".dat".  Otherwise,'
    echo '   physical chunk is not copied;  the .file  field in the'
    echo '   output is made to point to the original data file.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-cd":
   case "-copydata":
    set copydata; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set outfile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-to":
    set newchunk = ${argv[2]}; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# error checks
#
 if ( !( ${?infile} && ${?chunk} && ${?outfile} && ${?newchunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified input/output file/chunk" 1 ""
  exit ${status}
 endif

# chunk file
#
 set chunkfile = ( `mri.datfile.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk}` )

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri.datfile.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk}"
 if ( ${status} ) exit ${status}

# copy data if necessary
#
 if ( ${?copydata} ) then
  if ( "${chunkfile[3]}" == "does_not_exist" ) then
   errmsg.csh ${verbosity} ${myname} 5 "nonexistent file ${chunkfile[3]}" 1 ""
   exit ${status}
  endif

  cp ${chunkfile[1]} ${outfile}.dat
  set chunkfile = ".dat"
 endif

# generate output mri file describing the renamed chunk
#
 echo '\!format = pgh\n\!version = 1.0' > ${outfile}.mri

 mri.find.key.csh -verbosity ${verbosity} -operation beginswith -infile ${infile}.mri ${chunk} | grep -v "^${chunk}.file" | sed 's/'${chunk}'/'${newchunk}'/' >> ${outfile}.mri

 echo "${newchunk}.file = ${chunkfile[1]}" >> ${outfile}.mri
