#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set fiasco_args = ''

 set infile = ''
 set chunk  = ''

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Change chunk name to "images" and execute fiasco utility mri_to_img'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-f|-fiasco "additional arguments for mri_to_img (except -o)"]'
    echo ' [-o|-outfile file[.ps]]'
    echo ' -i|-infile file[.mri] -c|-chunk chunk'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command-line argument'
    echo ' 3: no mri file specified'
    echo ' 4: no chunk specified'
    echo ' 5-7: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.rename.chunk.csh'
    echo ' mri_to_img'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set outfile = `echo ${argv[2]} | sed 's/\.ps$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-f":
   case "-fiasco":
    set fiasco_args = "${argv[2]}"; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# rename chunk to "images"
#
 mri.rename.chunk.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} -outfile ${tmp} -to images

 errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} -outfile ${tmp} -to images"
 if ( ${status} ) exit ${status}

# call fiasco utility
#
 mri_to_img ${fiasco_args} ${tmp} -o ${tmp}.ps >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_to_img ${fiasco_args} ${tmp} -o ${tmp}.ps >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# finishing touches
#
 set infilebase = `basename ${infile}`

 if ( !( ${?outfile} ) ) set outfile = `basename ${infile}`
 set outfile = ${outfile}.ps

 set sFrom = `echo ${tmp} | sed -e 's/\//\\\//g' -e 's/\./\\\./g'`
 set sTo   = `echo ${infilebase} | sed -e 's/\//\\\//g' -e 's/\./\\\./g'`

 sed 's/'${sFrom}'/'${sTo}'/g' ${tmp}.ps > ${outfile}

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
