#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
#set permute = permute.csh # this is a workaround for a permute bug
 set permute = mri_permute # this bug is gone in 5.1

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Expand a  real symmetric matrix  in a  packed'
    echo ' triangular format to its full symmetric form.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' -i|-infile infile[.mri] -c|-chunk matrix_chunk'
    echo ' -d|-dimension matrix_dimension -f|-format packed_format_name'
    echo ' -o|-outfile outfile[.mri] [-oc|-outchunk chunk]'
    echo ' -od|-outdims output_row_col_dims'
    echo ''
    echo ' Packed matrix format names'
    echo '  uc, ur, lc, lr'
    echo ''
    echo '  u == upper triangular, c == column major'
    echo '  l == lower triangular, r == row major'
    echo ''
    echo 'Termination status codes'
    echo '  0: normal termination'
    echo '  1: abnormal termination, except'
    echo '  2: unknown command-line argument'
    echo '  3: unspecified input file/chunk'
    echo '  4: unspecified/invalid packed-matrix dimension'
    echo '  5: unspecified/invalid packed-matrix format'
    echo '  6: unspecified output file/dimensions'
    echo '  7: invalid number of output matrix dimensions'
    echo '  8: unknown packed-matrix format'
    echo ' 10: nonexistent data file'
    echo ' 11: invalid packed-matrix dimension'
    echo ' 12: invalid packed matrix size'
    echo ' 13: invalid output matrix dimensions'
    echo ' 9,14-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.chunk.info.csh mri.create.chunk.csh'
    echo ' ${permute} mri_type_convert mri_from_ascii mri_rpn_math mri_subsample'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-d":
   case "-dimension":
    set mdim = ${argv[2]}; shift; shift; breaksw;

   case "-f":
   case "-format":
    set mformat = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set outfile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-oc":
   case "-outchunk":
    set outchunk = ${argv[2]}; shift; shift; breaksw;

   case "-od":
   case "-outdims":
    set outdims = ${argv[2]}; shift; shift; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

# error checks
#
 if ( !( ${?infile} && ${?chunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified input file/chunk" 1 ""
  exit ${status}
 else if ( !( ${?mdim} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified/invalid packed-matrix dimension" 1 ""
  exit ${status}
 else if ( ${%mdim} != 1 ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified/invalid packed-matrix dimension" 1 ""
  exit ${status}
 else if ( !( ${?mformat} ) ) then
  errmsg.csh ${verbosity} ${myname} 5 "unspecified/invalid packed-matrix format" 1 ""
  exit ${status}
 endif

 set mformat = `echo ${mformat} | awk '{print tolower($0)}'`

 switch ( ${mformat} )
  case 'uc': case 'ur': case 'lc': case 'lr':
   breaksw

  default:
   errmsg.csh ${verbosity} ${myname} 5 "unspecified/invalid packed-matrix format" 1 ""
   exit ${status}
 endsw

 if ( !( ${?outfile} && ${?outdims} ) ) then
  errmsg.csh ${verbosity} ${myname} 6 "unspecified output file/dimensions" 1 ""
  exit ${status}
 else if ( ${%outdims} != 2 ) then
  errmsg.csh ${verbosity} ${myname} 7 "invalid number of output matrix dimensions in ${outdims}; expecting: 2" 1 ""
  exit ${status}
 endif

 if ( !( ${?outchunk} ) ) set outchunk = ${chunk}

# chunk attributes
#
 mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri.chunk.info.csh -verbosity ${verbosity} -infile ${infile} -chunk ${chunk} > ${tmp}"
 if ( ${status} ) exit ${status}

 set chunkinfo = ${tmp}

 set file = ( `grep '^physical_file ' ${chunkinfo} | sed 's/physical_file//'` )

 if ( "${file[2]}" != "exists" ) then
  errmsg.csh ${verbosity} ${myname} 10 "nonexistent data file ${file[1]}" 1 ""
  if ( ${status} ) exit ${status}
 endif

 set file      = ${file[1]}

 set chunk     =   `grep '^chunk '         ${chunkinfo} | sed 's/chunk//'`
 set type      =   `grep '^datatype '      ${chunkinfo} | sed 's/datatype//'`
 set typesize  =   `grep '^typesize '      ${chunkinfo} | sed 's/typesize//'`
 set l_endian  =   `grep '^little_endian ' ${chunkinfo} | sed 's/little_endian//'`
 set chunksize =   `grep '^size '          ${chunkinfo} | sed 's/size//'`
 set offset    =   `grep '^offset '        ${chunkinfo} | sed 's/offset//'`
 set dims      = ( `grep '^dimensions '    ${chunkinfo} | sed 's/dimensions//'` )
 set sizes     = ( `grep '^extents '       ${chunkinfo} | sed 's/extents//'`    )

 set chunkdims = `echo ${dims} | sed 's/ //g'`

 set mdimidx   = `echo ${chunkdims} | awk '{print index($1,"'${mdim}'")}'`

 if ( ${mdimidx} == 0 ) then
  errmsg.csh ${verbosity} ${myname} 11 "no dimension ${mdim} in matrix chunk dimensions ${chunkdims}" 1 ""
  exit ${status}
 endif

# packed-matrix size, matrix row/col size
#
 set msize = ${sizes[${mdimidx}]}
 set n     = `echo ${msize} | awk '{print 0.5 * ( sqrt( 1 + 8 * $1 ) - 1 )}'`

 if ( ${msize} != `echo ${n} | awk '{print 0.5 * $1 * ( $1 + 1 )}'` ) then
  errmsg.csh ${verbosity} ${myname} 12 "invalid packed matrix size ${msize}; expected form: n(n+1)/2 for integer n > 0" 1 ""
  exit ${status}
 endif

# chunk dimensions other than mdim
#
 set notmdims = `echo ${chunkdims} | sed -e 's/[ '${mdim}']//g'`

# error check: outdims other than mdim should not occur in chunkdims
#
 if ( `echo ${notmdims} | grep "[${outdims}]"` != `` ) then
  errmsg.csh ${verbosity} ${myname} 13 "invalid output dimensions ${outdims}; matrix chunk dimensions ${chunkdims}" 1 ""
  exit ${status}
 endif

# construct Joel's qrc matrix
#
 echo -n '' > ${tmp}.qrc

 switch ( ${mformat} )
  # UT i <= j, rowwise traversal
  # LT i >= j, columnwise traversal
  #
  case 'ur':
  case 'lc':
   set k = 0
   set i = 0
   while ( ${i} < ${n} )
    set j = ${i}
    while ( ${j} < ${n} )
     echo "${k} ${i} ${j} 1.0\n${k} ${j} ${i} 1.0" >> ${tmp}.qrc
     @ j += 1
     @ k += 1
    end
    @ i += 1
   end
   breaksw

  # LT i >= j, rowwise traversal
  # UT i <= j, columnwise traversal
  #
  case 'uc':
  case 'lr':
   set k = 0
   set i = 0
   while ( ${i} < ${n} )
    set j = 0
    while ( ${j} <= ${i} )
     echo "${k} ${i} ${j} 1.0\n${k} ${j} ${i} 1.0" >> ${tmp}.qrc
     @ j += 1
     @ k += 1
    end
    @ i += 1
   end
   breaksw

# # LT i >= j, columnwise traversal
# #
# case 'lc':
#  set k = 0
#  set j = 0
#  while ( ${j} < ${n} )
#   set i = ${j}
#   while ( ${i} < ${n} )
#    echo "${k} ${i} ${j} 1.0\n${k} ${j} ${i} 1.0" >> ${tmp}.qrc
#    @ i += 1
#    @ k += 1
#   end
#   @ j += 1
#  end
#  breaksw

# # UT i <= j, columnwise traversal, zero-based indices
# #
# case 'uc':
#  set k = 0
#  set j = 0
#  while ( ${j} < ${n} )
#   set i = 0
#   while ( ${i} <= ${j} )
#    echo "${k} ${i} ${j} 1.0\n${k} ${j} ${i} 1.0" >> ${tmp}.qrc
#    @ i += 1
#    @ k += 1
#   end
#   @ j += 1
#  end
#  breaksw

  default:
   exit -1 # should not occur
 endsw

# temporary dimension names for intermediate processing: avoid chunkdims, mdim
#
 set tmpdims = `echo abcdefghijklmnopqrstuvwxyz | sed "s/[${mdim}${chunkdims}]//g" | awk '{print substr($1,1,1)substr($1,2,1)}'`

 cat ${tmp}.qrc | mri_from_ascii -d ${mdim}${tmpdims} -e ${msize}:${n}:${n} -c images -i ${mdim}${tmpdims} -o ${tmp}.qrc >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 14 "failed command" ${status} "cat ${tmp}.qrc | mri_from_ascii -d ${mdim}${tmpdims} -e ${msize}:${n}:${n} -c images -i ${mdim}${tmpdims} -o ${tmp}.qrc >& ${tmp}.err"
 if ( ${status} ) exit ${status}

 mri_type_convert -c images -`mri.type2c.csh ${type} | sed 's/unsigned//'` ${tmp}.qrc ${tmp}.qrc.1 >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 15 "failed command" ${status} "mri_type_convert -c images -`mri.type2c.csh ${type} | sed 's/unsigned//'` ${tmp}.qrc ${tmp}.qrc.1 >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# permute matrix file from order ${chunkdims} to ${mdim}${notmdims}
#
 if ( "${chunkdims}" != "${mdim}${notmdims}" ) then
  ${permute} -memlimit ${chunksize} -input ${infile} -chunk ${chunk} -headerout ${tmp} -order ${mdim}${notmdims} >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 16 "failed command" ${status} "${permute} -memlimit ${chunksize} -input ${infile} -chunk ${chunk} -headerout ${tmp} -order ${mdim}${notmdims} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

# set matrixmri = ${tmp}
  set matrixdat = ${tmp}.dat
 else
# set matrixmri = ${infile}
  set matrixdat = ${file}
 endif

# create a template .mri file to describe a single record (packed matrix)
#
 @ recsize = ${msize} * ${typesize}

# record-by-record conversion of packed traingular to fully symmetric form
#
 rm -f ${outfile}.dat

 set ofs     = ${offset}
 @   ofsmax  = ${chunksize} - ${msize} * ${typesize}
 @   ofsmax += ${offset}
 while ( ${ofs} <= ${ofsmax} )
  echo "\!format = pgh\n\!version = 1.0\nimages = [chunk]\nimages.datatype = ${type}\nimages.dimensions = ${mdim}\nimages.file = ${matrixdat}\nimages.offset = ${ofs}\nimages.little_endian = ${l_endian}\nimages.size = ${recsize}\nimages.extent.${mdim} = ${msize}" > ${tmp}.q.mri

  mri_rpn_math -o ${tmp}.qrc.mri -c images '$1,$2,*' ${tmp}.qrc.1 ${tmp}.q >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 17 "failed command" ${status} "mri_rpn_math -o ${tmp}.qrc.mri -c images '"'$1,$2,*'"' ${tmp}.qrc.1 ${tmp}.q >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  mri_subsample -sum -d ${mdim} -e 1 ${tmp}.qrc ${tmp}.rc >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 18 "failed command" ${status} "mri_subsample -sum -d ${mdim} -e 1 ${tmp}.qrc ${tmp}.rc >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  cat ${tmp}.rc.dat >> ${outfile}.dat

  @ ofs += ${recsize}
 end

# create ${outfile}.mri
#
 set sizes = ( `echo ${mdimidx} ${sizes} | awk '{for ( i = 2; i <= NF; i++ ) if ( (i-1) != $1 ) printf " %d", $i; printf "\n"}' | sed -e 's/^ //' -e 's/ /:/g'` )

 mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset 0 -file .dat -mri ${outfile} -chunk ${outchunk} -type ${type} -dimensions ${outdims}${notmdims} -extents ${n}:${n}:${sizes}

 errmsg.csh ${verbosity} ${myname} 19 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -headeronly -little_endian ${l_endian} -offset 0 -file .dat -mri ${outfile} -chunk ${outchunk} -type ${type} -dimensions ${mdim}${notmdims} -extents ${n}:${n}:${sizes}"
 if ( ${status} ) exit ${status}

 echo "${outchunk}.format = "'"'"full symmetric dimensions ${outdims} size ${n} X ${n}"'"' >> ${outfile}.mri

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
