#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' generate an eps image depicting the location of a pixel'
    echo ' over a square grid. Origin at the upper left corner,  x'
    echo ' increases to the right, y increses downwards.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-g|-grid]'
    echo ' -o|-out outfile -n gridsize -x x -y y'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unknown command line argument'
    echo ' 3: unspecified output file'
    echo ' 4: unspecified grid size'
    echo ' 5: grid size <= 0'
    echo ' 6: unspecified x-coordinate'
    echo ' 7: unspecified y-coordinate'
    echo ' 8: out-of-range value for x and/or y'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-g":
   case "-grid":
    set grid; shift; breaksw;

   case "-n":
    set numBoxes = ${argv[2]}; shift; shift; breaksw;

   case "-x":
    set x = ${argv[2]}; shift; shift; breaksw;

   case "-y":
    set y = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set outfile = `echo ${argv[2]} | sed 's/\.eps$//'`; shift; shift;
    set outfile = ${outfile}.eps; breaksw;

   default:
    errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
    exit ${status}
  endsw
 end

 if ( !( ${?outfile} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified output file" 1 ""
  exit ${status}
 else if ( !( ${?numBoxes} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified grid size" 1 ""
  exit ${status}
 else if ( ${numBoxes} < 1 ) then
  errmsg.csh ${verbosity} ${myname} 5 "grid size <= 0" 1 ""
  exit ${status}
 else if ( !( ${?x} ) ) then
  errmsg.csh ${verbosity} ${myname} 6 "unspecified x-coordinate" 1 ""
  exit ${status}
 else if ( !( ${?y} ) ) then
  errmsg.csh ${verbosity} ${myname} 7 "unspecified y-coordinate" 1 ""
  exit ${status}
 else if (    ( ${x} <  0           ) || ( ${y} <  0           )        \
           || ( ${x} >= ${numBoxes} ) || ( ${y} >= ${numBoxes} ) ) then
  errmsg.csh ${verbosity} ${myname} 8 "out-of-range x and/or y" 1 ""
  exit ${status}
 endif

 set x0  = 406
 set y0  = 280
 set len = 4592

 set scale = `echo ${len} ${numBoxes} | awk '{print $1/$2}'`
 set psize = `echo 72.0   ${numBoxes} | awk '{print $1/$2}'`

# headers
#
cat > ${outfile} << @EOF
%!PS-Adobe-2.0 EPSF-2.0
%%Title: ${outfile}
%%Creator: ${myname} for ${user}; thanks to gnuplot 3.7 patchlevel 1
%%CreationDate: `date`
%%DocumentFonts: (none)
%%BoundingBox: 50 50 320 309
%%Orientation: Portrait
%%EndComments
/gnudict 256 dict def
gnudict begin
/Color false def
/Solid false def
/gnulinewidth 5.000 def
/userlinewidth gnulinewidth def
/vshift -46 def
/dl {10 mul} def
/hpt_ 31.5 def
/vpt_ 31.5 def
/hpt hpt_ def
/vpt vpt_ def
/M {moveto} bind def
/L {lineto} bind def
/R {rmoveto} bind def
/V {rlineto} bind def
/vpt2 vpt 2 mul def
/hpt2 hpt 2 mul def
/UP { dup vpt_ mul /vpt exch def hpt_ mul /hpt exch def
      /hpt2 hpt 2 mul def /vpt2 vpt 2 mul def } def
/AL { stroke userlinewidth 2 div setlinewidth } def
/UL { dup gnulinewidth mul /userlinewidth exch def
      10 mul /udl exch def } def
/DL { Color {setrgbcolor Solid {pop []} if 0 setdash }
 {pop pop pop Solid {pop []} if 0 setdash} ifelse } def
/PL { stroke userlinewidth setlinewidth } def
/LT0 { PL [] 1 0 0 DL } def
/LTa { AL [1 udl mul 2 udl mul] 0 setdash 0 0 0 setrgbcolor } def
/BoxF { stroke [] 0 setdash exch hpt sub exch vpt add M
        0 vpt2 neg V  hpt2 0 V  0 vpt2 V
        hpt2 neg 0 V  closepath fill } def
/x0 ${x0} def
/y0 ${y0} def
/n ${len} def
end
%%EndProlog
gnudict begin
gsave
50 50 translate
0.05 0.05 scale
0 setgray
newpath
${psize} UP
LT0
@EOF

# box
#
 set i = 0

 set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${y0}' )}'`
 echo "x0 ${k} M n 0 V" >> ${outfile}
 set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${x0}' )}'`
 echo "${k} y0 M 0 n V" >> ${outfile}

 set i = ${numBoxes}

 set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${x0}' )}'`
 echo "${k} y0 M 0 n V" >> ${outfile}
 set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${y0}' )}'`
 echo "x0 ${k} M n 0 V" >> ${outfile}

# grid or crosswire
#
 echo "1 UL LTa" >> ${outfile}

 if ( ${?grid} ) then
  set i = 1
  while ( ${i} < ${numBoxes} )
   set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${y0}' )}'`
   echo "x0 ${k} M n 0 V" >> ${outfile}
   set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${x0}' )}'`
   echo "${k} y0 M 0 n V" >> ${outfile}
   @ i += 1
  end
 else
  set i = `echo ${numBoxes} | awk '{print int( 0.5*$1 )}'`

  set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${y0}' )}'`
  echo "x0 ${k} M n 0 V" >> ${outfile}
  set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${x0}' )}'`
  echo "${k} y0 M 0 n V" >> ${outfile}

  if ( `echo ${i} | awk '{print 2*$1}'` != ${numBoxes} ) then
   @ i += 1
   set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${y0}' )}'`
   echo "x0 ${k} M n 0 V" >> ${outfile}
   set k = `echo ${i} | awk '{print int( $1 * '${scale}' + '${x0}' )}'`
   echo "${k} y0 M 0 n V" >> ${outfile}
  endif
 endif

# pixel to be marked
#
 set x = `echo ${x} | awk '{print int( ( $1 + 0.5 ) * '${scale}' + '${x0}' )}'`
 set y = `echo ${y} | awk '{print int( ( '${numBoxes}' - $1 - 0.5 ) * '${scale}' + '${y0}' )}'`

 echo "${x} ${y} BoxF" >> ${outfile}

# trailers
#
 echo 'stroke' >> ${outfile}
 echo 'grestore' >> ${outfile}
 echo 'end' >> ${outfile}
 echo 'showpage' >> ${outfile}
 echo '%%Trailer' >> ${outfile}
