#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  set first = `echo ${argv[1]} | awk '{print substr( $1, 1, 1 )}'`

  if ( "${first}" == '-' ) then
   switch ( ${argv[1]} )
    case "-h":
    case "-help":
     echo ${myname}
     echo ' Process a path as follows:'
     echo '  a. expand any initial ./ or ../ in terms of pwd'
     echo '  b. remove all redundant ./'
     echo '  c. optionally get an alternate/compact representation'
     echo '     of a physically existing path (-physical)'
     echo '  d. optionally split it into basename/dirname (-split)'
     echo ''
     echo 'Command line'
     echo ' [-h|-help]'
     echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
     echo ''
     echo ' [-p|-physical] [-s|-split] [path]'
     echo ''
     echo 'Termination status codes'
     echo ' 0: normal termination'
     echo ' 1: abnormal termination, except'
     echo ' 2: unknown command line argument'
     echo ' 3: -physical specified but path nonexistent'
     echo ''
     echo 'Required utilities/scripts'
     echo ' errmsg.csh'
     echo ''
     echo 'Notes'
     echo ' 1 The path returned may not be portable'
     echo '   across all hosts on a network.'
#    echo ''
#    echo 'Defaults'
#    echo ' path: ./, physical: false, split: false'
#    echo ''
#    echo " verbosity: ${verbosity} (maximum ${verbmax})"
#    echo " temporary files prefix: ${tmp}"
     exit 0;

    case "-q":
    case "-quiet":
     set verbosity = 0; shift; breaksw;

    case "-v":
    case "-verbosity":
     set verbosity = ${argv[2]}; shift; shift; breaksw;

    case "-V":
    case "-verbose":
     set verbosity = ${verbmax}; shift; breaksw;

    case "-p":
    case "-physical":
     set physical; shift; breaksw;

    case "-v":
    case "-verbose":
     set verbosity = 1; shift; breaksw;

    case "-s":
    case "-split":
     set split; shift; breaksw;

    default:
     errmsg.csh ${verbosity} ${myname} 2 "unknown command-line argument ${argv[1]}" 1 ""
     exit ${status}
   endsw
  else
   set destin = ${argv[1]}; shift;
  endif
 end

# process path
#
 if ( !( ${?destin} ) ) then
  set destin = ./
 else if (    ( `echo ${destin} | awk '{print substr( $1, 1, 1 )}'` != '.' ) \
           && ( `echo ${destin} | awk '{print substr( $1, 1, 1 )}'` != '/' ) ) then
  set destin = ./${destin}
 endif

 if ( ${?physical} ) then
  if ( -e ${destin} ) then
   # process a physically existing path
   #
   set dir  = `dirname  ${destin}`
   set base = `basename ${destin}`
   pushd ${dir} >& /dev/null
   set dir = `pwd`
   popd >& /dev/null
  else
   errmsg.csh ${verbosity} ${myname} 3 "-physical specified, path ${destin} nonexistent." 1 ""
   exit ${status}
  endif
 else
  # fully expand any leading ./ or ../ using of pwd
  #
   if (    ( `echo ${destin} | awk '{print substr( $1, 1, 2 )}'` == './' ) \
        || ( "${destin}" == '.' ) ) then
    set pwd = `pwd | sed 's/\//\\\//g'`
    set destin = `echo ${destin} | sed 's/^\./'${pwd}'/'`
   else if (    ( `echo ${destin} | awk '{print substr( $1, 1, 3 )}'` == '../' ) \
             || ( "${destin}" == '..' ) ) then
    pushd .. >& /dev/null
    set pwd = `pwd | sed 's/\//\\\//g'`
    popd >& /dev/null
    set destin = `echo ${destin} | sed 's/^\.\./'${pwd}'/'`
   endif

   # echo destin \"$destin\"

  # change all 'd1/./d2' to 'd1/d2'
  #
   set destin = `echo ${destin} | sed -e 's/\/\.\//\//g' -e 's/\.\///g'`

   # echo destin \"$destin\"

   set dir  = `dirname  ${destin}`
   set base = `basename ${destin}`

   # echo dir \"$dir\"
   # echo base \"$base\"
 endif

 if ( ${?split} ) then
  echo ${dir} ${base}
 else
  echo ${dir}/${base}
 endif
