#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Process range specification of the form [a][:[b]],'
    echo ' substitute appropriate defaults if necessary.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-n|-numeric]'
    echo ' -d|-defaults lower:upper'
    echo ' [[a][:[b]]]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: invalid defaults'
    echo ' 3: nonnumeric defaults'
    echo ' 4: nonnumeric rangespecs'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ''
    echo 'Notes'
    echo ' If -numeric option is specified, the range specification'
    echo ' is  treated  as  a  numerical  entity,  and defaults are'
    echo ' substituted also  when the range specification is out of'
    echo ' bounds with respect to the defaults.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-d":
   case "-defaults":
    set defaults = ( `echo ${argv[2]} | awk -F: '{print $1" "$2}'` );
    shift; shift; breaksw;

   case "-n":
   case "-numeric":
    set numeric; shift; breaksw;

   default:
    set rangespecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# process defaults and rearrange if necessary
#
 if ( !( ${?defaults} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "invalid defaults" 1 ""
  exit ${status}
 else if ( ${#defaults} != 2 ) then
  errmsg.csh ${verbosity} ${myname} 2 "invalid defaults" 1 ""
  exit ${status}
 endif

 if ( ${?numeric} ) then
  set defaults = ( `echo ${defaults} | sed -e 's/\+//g' -e 's/\- / /g'` )
  echo ${defaults} | sed 's/ //g' | grep -qv '[^-0-9]'
  if ( ${status} ) then
   errmsg.csh ${verbosity} ${myname} 3 "non-numeric defaults" 1 ""
   exit ${status}
  endif
  if ( ${defaults[2]} < ${defaults[1]} ) set defaults = ( ${defaults[2]} ${defaults[1]} )
 endif

 if ( !( ${?rangespecs} ) ) then
  echo ${defaults}
  exit 0
 else if ( "${rangespecs}" == '' ) then
  echo ${defaults}
  exit 0
 endif

# process range specs
#
 if ( ${?numeric} ) then
  set rangespecs = `echo ${rangespecs} | sed -e 's/\+//g' -e 's/-:/:/g'`
  echo ${rangespecs} | sed 's/ //g' | grep -qv '[^-:0-9]'
  if ( ${status} ) then
   errmsg.csh ${verbosity} ${myname} 4 "non-numeric rangespecs" 1 ""
   exit ${status}
  endif
 endif

 set cpos = `echo ${rangespecs} | awk '{print index( $1, ":" )}'`
 set len  = ${%rangespecs}

 switch ( ${cpos} )
  case 0:                                                              # 'a', ''
   if ( "${len}" == "0" ) then
    set range = ( ${defaults[1]} ${defaults[2]} )
   else
    set range = ( ${rangespecs} ${defaults[2]} )
   endif
   breaksw;

  case 1:                                                              # ':',':b'
   if ( ${len} == 1 ) then
    set range = ( ${defaults[1]} ${defaults[2]} )
   else
    set range = ( ${defaults[1]} `echo ${rangespecs} | awk -F: '{print $2}'` )
   endif
   breaksw;

  case ${len}:                                                         # 'a:'
   set range = ( `echo ${rangespecs} | awk -F: '{print $1}'` ${defaults[2]} );
   breaksw;

  default:
   set range = ( `echo ${rangespecs} | awk -F: '{print $1" "$2}'` );
   breaksw;
 endsw

# impose bounds
#
 if ( ${?numeric} ) then
  if ( ${range[2]} < ${range[1]} ) set range = ( ${range[2]} ${range[1]} )

  if ( ( ${range[1]} < ${defaults[1]} ) || ( ${range[1]} >= ${defaults[2]} ) ) set range[1] = ${defaults[1]}
  if ( ( ${range[2]} < ${defaults[1]} ) || ( ${range[2]} >= ${defaults[2]} ) ) set range[2] = ${defaults[2]}
 endif

 echo ${range}
