#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
# mkdir -p /tmp/${user}
# set tmpdir = /tmp/${user}/
# set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# defaults for covariances
#
 set covdims  = rw          # for fully expanded matrices # ATTN can't have fixed dimension names
 set covchunksuffix = .covariances

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Expand vmp  covariance matrices from packed'
    echo ' triangular to full symmetric storage format.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' -s|-submodel submodel_key'
    echo ' -o|-output expanded_covariances_file'
    echo ' vmp_output_file'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified/invalid vmp output file'
    echo ' 3: unspecified/invalid submodel'
    echo ' 4: unknown covariance format'
    echo ' 5: non-symmetric covariance matrix?'
    echo ' 6-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.tri2sym.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo " 1 stdout output: input (packed), output (full symmetric), chunk, dimensions."
    echo " 2 Assuming vmp output header conventions, this script does the following:"
    echo "   if the matrix storage format for covariances is"
    echo "    (a) full symmetric: do nothing"
    echo "    (b) packed triangular:"
    echo "         (b1) if (infile).full exists, is in the full symmetric format,"
    echo "              and the sizes match: do nothing"
    echo "         (b2) otherwise: expand packed triangular to full symmetric"
    echo "        This is not a fool-proof checking scheme, but it should work"
    echo "        for common/expected patterns of usage."
    echo "    (c) other: unknown, an error condition"
    echo " 3 If -o output is unspecified, the expanded covariances chunk is"
    echo "   added to the vmp output file."
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-s":
   case "-submodel":
    set submodel = ${argv[2]}; shift; shift; breaksw;

   default:
    set vmpout = `echo ${argv[1]} | sed 's/\.mri$//'`; shift; breaksw;
  endsw
 end

# error checks
#
 if ( !( ${?vmpout} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid vmp output file" 1 ""
  exit ${status}
 endif

 if ( !( -f ${vmpout}.mri ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid vmp output file" 1 ""
  exit ${status}
 endif

 if ( !( ${?submodel} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid submodel" 1 ""
  exit ${status}
 endif

 if ( "${submodel}" != "`vmp.list.submodels.csh -s ${submodel} ${vmpout}`" ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid submodel" 1 ""
  exit ${status}
 endif

# expand covariances from packed triangular to full symmetric format if necessary
#
 set covchunk  = ${submodel}${covchunk}
 set covformat = `mri_printfield -input ${infile} -field ${covchunk}.format`

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri_printfield -input ${infile} -field ${covchunk}.format"
 if ( ${status} ) exit ${status}

 set covformat = `echo ${covformat} | awk '{print tolower($0)}'`

 set format = `echo ${covformat} | awk '{print $1}'`
 set dims   = `echo ${covformat} | awk '{print $4}'`
 set sizes  = `echo ${covformat} | awk '{print $6" "$8}'`

 switch ( "${format}" )
  case 'packed':
   set format = `echo ${covformat} | awk '{print $2}'`

   switch ( "${format}" )
    case "uc": case "lc": case "lr": case "ur":
     if ( -f ${infile}.full.mri ) then
      set covformat2 = `mri_printfield -input ${infile}.full -field ${covchunk}.format`

      if ( ${status} == 0 ) then
       set covformat2 = `echo ${covformat2} | awk '{print tolower($0)}'`

       set format2 = `echo ${covformat2} | awk '{print $1" "$2}'`
       set dims2   = `echo ${covformat2} | awk '{print $4}'`
       set sizes2  = `echo ${covformat2} | awk '{print $6" "$8}'`

       if ( ( "${format2}" == "full symmetric" ) && ( "${sizes}" == "${sizes2}" ) ) then
        echo "${infile} ${infile}.full ${covchunk} ${covdims}"
        exit 0
       endif
      endif
     endif

     mri.tri2sym.csh -verbosity ${verbosity} -infile ${infile} -chunk ${covchunk} -dimension ${dims} -format ${format} -outfile ${infile}.full -outchunk ${covchunk} -outdims ${covdims}

     errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "mri.tri2sym.csh -verbosity ${verbosity} -infile ${infile} -chunk ${covchunk} -dimension ${dims} -format ${format} -outfile ${infile}.full -outchunk ${covchunk} -outdims ${covdims}"
     if ( ${status} ) exit ${status}

     grep -v -e "^\!" -e "^${covchunk} " -e "^${covchunk}.datatype" -e "^${covchunk}.dimensions" -e "^${covchunk}.extent" -e "^${covchunk}.file" -e "^${covchunk}.size" -e "^${covchunk}.little_endian" -e "^${covchunk}.format" -e "^${covchunk}.min.${dims}" -e "^${covchunk}.offset" ${infile}.mri >> ${infile}.full.mri

     echo "${infile} ${infile}.full ${covchunk} ${covdims}"
     exit 0

    default:
     errmsg.csh ${verbosity} ${myname} 3 "unrecognized format for covariance matrix" 1 ""
     exit ${status}
   endsw

  case 'full':
   set format = `echo ${covformat} | awk '{print $2}'`

   switch ( ${format} )
    case 'symmetric':
     echo "${infile} ${infile} ${covchunk} ${dims}"
     exit 0
    case 'general':
     errmsg.csh ${verbosity} ${myname} 4 "non-symmetric covariance matrix?" 1 ""
     exit ${status}
    default:
     errmsg.csh ${verbosity} ${myname} 3 "unrecognized format for covariance matrix" 1 ""
     exit ${status}
   endsw

  default:
   errmsg.csh ${verbosity} ${myname} 3 "unrecognized format for covariance matrix" 1 ""
   exit ${status}
 endsw

# cleanup
#
#rm -f ${tmp}* >& /dev/null; exit 0
