#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# vmp output prefix, submodels to process
#
 set prefix = './'
 set submodels = ()

# plot limits for response
#
 set plresp = '0.0:0.02'

# model probabilities file/chunk name
#
 set mp = model.probabilities

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Create postscript images for standard vmp results'
    echo ' for specified submodels using default settings.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-p|-prefix any_path/prefix]'
    echo ' [submodel1 submodel2 ...]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' plot.baseline.csh plot.noiselevel.csh'
    echo ' plot.param.csh plot.modelp.csh plot.diags.csh'
#   echo ''
#   echo 'Defaults'
#   echo " prefix: '${prefix}'"
#   echo " submodels: all submodels in the model.probabilities file"
#   echo " plot limits for response estimates: ${plresp}"
#   echo ''
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   default:
    set submodels = ( ${submodels} ${argv[1]} ); shift; breaksw;
  endsw
 end

# submodels
#
 set fitsubmodels = ( `mri_printfield -input ${prefix}${mp} -field submodel.list` )

 errmsg.csh ${verbosity} ${myname} 2 "failed command" ${status} "mri_printfield -input ${prefix}${mp} -field submodel.list"
 if ( ${status} ) exit ${status}

 set sm        = ( ${submodels} )
 set submodels = ''
 foreach s ( ${sm} )
  if ( `echo '' | awk '{print index("'"${fitsubmodels}"'","'${s}'")}'` != 0 ) set submodels = ( ${submodels} ${s} )
 end

 if ( "${submodels}" == "" ) set submodels = ( ${fitsubmodels} average )

# plot diagnostics
#
 plot.diags.csh -verbosity ${verbosity} -prefix ${prefix} -diags all ${submodels}

 errmsg.csh ${verbosity} ${myname} 2 "failed command" ${status} "plot.diags.csh -verbosity ${verbosity} -prefix ${prefix} -diags all ${submodels}"
 if ( ${status} ) exit ${status}

# plot model probabilities
#
 plot.modelp.csh -verbosity ${verbosity} -prefix ${prefix} ${submodels}

 errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "plot.modelp.csh -verbosity ${verbosity} -prefix ${prefix} ${submodels}"
 if ( ${status} ) exit ${status}

# plot baseline
#
 plot.baseline.csh -verbosity ${verbosity} -norelerr -prefix ${prefix} ${submodels}

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "plot.baseline.csh -verbosity ${verbosity} -prefix ${prefix} ${submodels}"
 if ( ${status} ) exit ${status}

# plot noise level
#
 plot.noiselevel.csh -verbosity ${verbosity} -norelerr -prefix ${prefix} ${submodels}

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "plot.noiselevel.csh -verbosity ${verbosity} -prefix ${prefix} ${submodels}"
 if ( ${status} ) exit ${status}

# plot response
#
 plot.param.csh -verbosity ${verbosity} -norelerr -plotlimits ${plresp} -paramname response -prefix ${prefix} ${submodels}

 errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "plot.param.csh -verbosity ${verbosity} -norelerr -plotlimits ${plresp} -paramname response -prefix ${prefix} ${submodels}"
 if ( ${status} ) exit ${status}

# plot shape
#
 plot.param.csh -verbosity ${verbosity} -norelerr -paramname shape -prefix ${prefix} ${submodels}

 errmsg.csh ${verbosity} ${myname} 7 "failed command" ${status} "plot.param.csh -verbosity ${verbosity} -norelerr -paramname shape -prefix ${prefix} ${submodels}"
 if ( ${status} ) exit ${status}

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
