#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# vmp output prefix, submodels to process
#
 set prefix    = './'
 set submodels = ''

# diagnostics list
#
 set diags     = ( npdef                      iters      evals       gnorm           status                )
 set chunks    = ( log.determinant.lapackinfo iterations evaluations  gradient.norm  status                )
 set names     = ( 'semi/indefinite fits'     iterations evaluations 'gradient norm' 'minimization status' )
 set limits    = ( '-n 0 -x 1'                ''         ''          ''              ''                    )
 set requested = ( false                      false      false       false           false                 )

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Create images for available vmp diagnostics.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-p|-prefix any_path/prefix]'
    echo ' -d|-diags diag1,diag2,diag3,...'
    echo ' submodel1 submodel2 ...'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.to.img.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' Available diagnostics'
    echo '  name  description'
    set d = 1
    while ( ${d} <= ${#diags} )
     printf "%6s  %s\n" "${diags[${d}]}" "${names[${d}]}"
     @ d += 1
    end
    printf "%6s  %s\n" "all" "all of the above"
#   echo ''
#   echo 'Defaults'
#   echo " prefix: '${prefix}'"
#   echo ''
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-d":
   case "-diags":
    set dlst = ( `echo ${argv[2]} | sed 's/,/ /g'` )
    shift; shift;

    foreach d ( ${dlst} )
     if ( ${d} == all ) then
      set requested = ( `echo ${#diags} | awk '{for(i=0;i<$1;i++)printf " true"}'` )
      break
     else
      set i = 1
      while ( ${i} <= ${#diags} )
       if ( "${d}" == "${diags[${i}]}" ) set requested[${i}] = true
       @ i += 1
      end
     endif
    end

    breaksw;

   default:
    set submodels = ( ${submodels} ${argv[1]} ); shift; breaksw;
  endsw
 end

# error checks
#
 if ( "${submodels}" == "" ) then
# errmsg.csh ${verbosity} ${myname} 0 "nothing to plot" 1 ""
# exit ${status}
  exit 0
 endif

# unspecified diags default
#
 if ( !( ${?dlst} ) ) set requested = ( `echo ${#diags} | awk '{for(i=0;i<$1;i++)printf " true"}'` )

# loop over submodels
#
 foreach submodel ( ${submodels} )
  # currently, no diagnostics for "average"
  #
  if ( "${submodel}" == "average" ) goto label_skipaverage

  # loops over diagnostics
  #
  set d = 1
  while ( ${d} <= ${#chunks} )
   if ( ${requested[${d}]} == true ) then
    set chunk = ${chunks[${d}]}
    set cfile = ${prefix}${submodel}.${chunk}.mri

    if ( -f ${cfile} ) then
     set slices = `mri_printfield -input ${cfile} -field ${chunk}.extent.z`

     errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "mri_printfield -input ${cfile} -field ${chunk}.extent.z"
     if ( ${status} ) exit ${status}

     setenv F_PS_TITLE1 `mri_printfield -input ${cfile} -field imagedata.description`
     setenv F_PS_TITLE2 "submodel: ${submodel}, slices: ${slices}"
     setenv F_PS_TITLE3 "diagnostic: ${names[${d}]}"

     mri.to.img.csh -verbosity ${verbosity} -fiasco "-mosaic ${limits[${d}]} -g -r -t -c -p" -infile ${cfile} -chunk ${chunk} -outfile ${submodel}.diag_${diags[${d}]}.ps

     errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri.to.img.csh -verbosity ${verbosity} -fiasco '-mosaic ${limits[${d}]} -g -r -t -c -p' -infile ${cfile} -chunk ${chunk} -outfile ${submodel}.diag_${diags[${d}]}.ps"
     if ( ${status} ) exit ${status}
    endif
   endif

   @ d += 1
  end

  label_skipaverage:
 end

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
