#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# vmp output prefix, submodels to process
#
 set prefix     = './'
 set submodels  = ''

# probabilities and diagnostics file/chunk names
#
 set mp   = model.probabilities
 set diag = log.determinant.lapackinfo

# misc
#
 set plotlimits = '0.0:1.0'

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Make model probability images from vmp output.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    # echo ' [-1|-oneMinus]'
    echo ' [-pl|-plotlimits [lower][:[upper]]]'
    echo ' [-p|-prefix any_path/prefix]'
    echo ' [submodel1 submodel2 ...]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.rename.chunk.csh proc.rangeopts.csh'
    echo ' mri_printfield mri_rpn_math mri_to_img'
    echo ''
    echo 'Notes'
    echo " 1 For null submodel, this plots 1 - model_probability."
    echo " 2 This  script  uses  log.determinant.diagnostics   for"
    echo "   null and submodel to mask off any pixels with nonzero"
    echo "   diagnostics."
#   echo ''
#   echo 'Defaults'
#   echo " prefix: '${prefix}', plotlimits: '${plotlimits}'"
#   echo " submodels: all submodels in the model.probabilities file"
#   echo ''
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-pl":
   case "-plotlimits":
    set plotlimits = ${argv[2]}; shift; shift; breaksw;

   # case "-1":
   # case "-oneMinus":
   #  set oneMinus; shift; breaksw;

   default:
    set submodels = ( ${submodels} ${argv[1]} ); shift; breaksw;
  endsw
 end

# submodels
#
 set fitsubmodels = ( `mri_printfield -input ${prefix}${mp} -field submodel.list` )

 errmsg.csh ${verbosity} ${myname} 2 "failed command" ${status} "mri_printfield -input ${prefix}${mp} -field submodel.list"
 if ( ${status} ) exit ${status}

 set sm        = ( ${submodels} )
 set submodels = ''
 foreach s ( ${sm} )
  if ( `echo '' | awk '{print index("'"${fitsubmodels}"'","'${s}'")}'` != 0 ) set submodels = ( ${submodels} ${s} )
 end

 if ( "${submodels}" == "" ) set submodels = ( ${fitsubmodels} )

# convert plot range options into a form that can be passed on to mri_to_img
#
 set plotlimits = ( `proc.rangeopts.csh -verbosity ${verbosity} -defaults '0.0:1.0' ${plotlimits}` )

 errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "proc.rangeopts.csh -verbosity ${verbosity} -defaults '0.0:1.0' ${plotlimits}"
 if ( ${status} ) exit ${status}

 if ( "${plotlimits[1]}" != "auto" ) then
  set plotlimits[1] = "-n ${plotlimits[1]}"
 else
  set plotlimits[1] = ""
 endif

 if ( "${plotlimits[2]}" != "auto" ) then
  set plotlimits[2] = "-x ${plotlimits[2]}"
 else
  set plotlimits[2] = ""
 endif

# loop over submodels
#
 foreach submodel ( ${submodels} )
  if ( ${submodel} == average ) goto label_skipaverage

  # null submodel
  #
  set null = `echo ${submodel} | awk '{for ( i = 0; i < length; i++ ) printf "0"}'`

  # plot 1 - p instead of p for null
  #
  unset oneMinus
  if ( ${submodel} == ${null} ) set oneMinus

  # plot p or ( 1 - p )?
  #
  # arg 2 of if_keep is used to mask out voxels with any diagnostics.
  # for such voxels, p (or 1-p) is set to 0.0 to mask it on the image.
  #
  set addprefix = ''
  if ( ${?oneMinus} ) then
   # set addprefix = 1-.
   set rpnscript = 1.0,\$1,1.0,\$2,if_keep,-
  else
   # set addprefix = ''
   set rpnscript = \$1,0.0,\$2,if_keep
  endif

  # slices
  #
  set slices = `mri_printfield -input ${prefix}${mp} -field ${mp}.extent.z`

  errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_printfield -input ${prefix}${mp} -field ${mp}.extent.z"
  if ( ${status} ) exit ${status}

  # image titles
  #
  setenv F_PS_TITLE1 `mri_printfield -input ${prefix}${mp} -field imagedata.description`
  setenv F_PS_TITLE2 "submodel: ${submodel}, slices: ${slices}"
  setenv F_PS_TITLE3 "posterior probability"
  if ( ${?oneMinus} ) setenv F_PS_TITLE3 "1 - ${F_PS_TITLE3}"

  # prepare diagnostics
  #
  if ( ( -f ${prefix}${submodel}.${diag}.mri ) && ( -f ${prefix}${null}.${diag}.mri ) ) then

   mri_rpn_math -o ${tmp} -c ${diag} '$1,$2,+' ${prefix}${submodel}.${diag} ${prefix}${null}.${diag} >& ${tmp}.err

   errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri_rpn_math -o ${tmp} -c ${diag} "'"'$1,$2,+'"'" ${prefix}${submodel}.${diag} ${prefix}${null}.${diag} >& ${tmp}.err"
   if ( ${status} ) exit ${status}

   mri.rename.chunk.csh -verbosity ${verbosity} -copydata -infile ${tmp} -chunk ${diag} -outfile ${tmp}.diag -to images

   errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -copydata -infile ${tmp} -chunk ${diag} -outfile ${tmp}.diag -to images"
   if ( ${status} ) exit ${status}

  else if ( -f ${prefix}${submodel}.${diag}.mri ) then

   mri.rename.chunk.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${diag} -chunk ${diag} -outfile ${tmp}.diag -to images

   errmsg.csh ${verbosity} ${myname} 7 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${diag} -chunk ${diag} -outfile ${tmp}.diag -to images"
   if ( ${status} ) exit ${status}

  else if ( -f ${prefix}${null}.${diag}.mri ) then

   mri.rename.chunk.csh -verbosity ${verbosity} -infile ${prefix}${null}.${diag} -chunk ${diag} -outfile ${tmp}.diag -to images

   errmsg.csh ${verbosity} ${myname} 8 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${prefix}${null}.${diag} -chunk ${diag} -outfile ${tmp}.diag -to images"
   if ( ${status} ) exit ${status}

  else

   set numX = `mri_printfield -input ${prefix}${mp} -field ${mp}.extent.x`

   errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri_printfield -input ${prefix}${mp} -field ${mp}.extent.x"
   if ( ${status} ) exit ${status}

   set numY = `mri_printfield -input ${prefix}${mp} -field ${mp}.extent.y`

   errmsg.csh ${verbosity} ${myname} 10 "failed command" ${status} "mri_printfield -input ${prefix}${mp} -field ${mp}.extent.y"
   if ( ${status} ) exit ${status}

   mri.create.chunk.csh -verbosity ${verbosity} -offset 0 -file .dat -mri ${tmp}.diag -chunk images -type uint8 -dimensions xyz -extents ${numX}:${numY}:${slices}

   errmsg.csh ${verbosity} ${myname} 11 "failed command" ${status} "mri.create.chunk.csh -verbosity ${verbosity} -offset 0 -file .dat -mri ${tmp}.diag -chunk images -type uint8 -dimensions xyz -extents ${numX}:${numY}:${slices}"
   if ( ${status} ) exit ${status}

  endif

  # rename model probabilities chunk to images
  #
  mri.rename.chunk.csh -verbosity ${verbosity} -infile ${prefix}${mp} -chunk ${mp} -outfile ${tmp} -to images

  errmsg.csh ${verbosity} ${myname} 12 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${prefix}${mp} -chunk ${mp} -outfile ${tmp} -to images"
  if ( ${status} ) exit ${status}

  # get submodel offset in ${mp} chunk
  #
  set offset = `mri_printfield -input ${prefix}${mp} -field submodel.index.${submodel}`

  errmsg.csh ${verbosity} ${myname} 13 "failed command" ${status} "mri_printfield -input ${prefix}${mp} -field submodel.index.${submodel}"
  if ( ${status} ) exit ${status}

  # extract model probabilities for submodel
  #
  mri_subset -d s -s ${offset} -l 1 ${tmp} ${tmp}.mp >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 14 "failed command" ${status} "mri_subset -d s -s ${offset} -l 1 ${tmp} ${tmp}.mp >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  # process probabilities (calculate 1-p if requested) and
  # mask out problematic voxels (if diagnostics != 0, set
  # probability to something out of range; see rpnscript)
  #
  mri_rpn_math -o ${tmp} -c images ${rpnscript} ${tmp}.mp ${tmp}.diag >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 15 "failed command" ${status} "mri_rpn_math -o ${tmp} -c images ${rpnscript} ${tmp}.mp ${tmp}.diag >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  # prepare image
  #
  mri_to_img -o ${submodel}.${addprefix}${mp}.ps -mosaic -t -c -p -r -g ${plotlimits} ${tmp} >& ${tmp}.err
 
  errmsg.csh ${verbosity} ${myname} 16 "failed command" ${status} "mri_to_img -o ${submodel}.${addprefix}${mp}.ps -mosaic -t -c -p -r -g ${plotlimits} ${tmp} >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  label_skipaverage:
 end

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
