#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# vmp output prefix, submodels to process
#
 set prefix    = './'
 set submodels = ''

# misc
#
 set plotlimits = 'auto:auto'

 set echunk = estimates
 set schunk = standard.errors

 set zero = 1.0e-8 # numerical zero for relative error calculation

 set plot_estimate
 set plot_stderr
 set plot_relerr

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Make parameter estimate/stderr images from vmp output.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-ne|-noestimate] [-ns|-nostderr] [-nr|-norelerr]'
    echo ' [-pl|-plotlimits [lower][:[upper]]]'
    echo ' [-p|-prefix any_path/prefix]'
    echo ' -pn|-paramname parameter_name'
    echo ' submodel1 submodel2 ...'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified parameter name'
    echo ' 3: incompatible estimates/standard errors files?'
    echo ' 4-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.to.img.csh mri.rename.chunk.csh proc.rangeopts.csh'
    echo ' mri_printfield mri_rpn_math mri_to_img'
#   echo ''
#   echo 'Defaults'
#   echo " prefix: '${prefix}'"
#   echo ''
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-pl":
   case "-plotlimits":
    set plotlimits = ${argv[2]}; shift; shift; breaksw;

   case "-pn":
   case "-paramname":
    set paramname = ${argv[2]}; shift; shift; breaksw;

   case "-ne":
   case "-noestimate":
    unset plot_estimate; shift; breaksw;

   case "-ns":
   case "-nostderr":
    unset plot_stderr; shift; breaksw;

   case "-nr":
   case "-norelerr":
    unset plot_relerr; shift; breaksw;

   default:
    set submodels = ( ${submodels} ${argv[1]} ); shift; breaksw;
  endsw
 end

# error checks
#
 if ( !( ${?plot_estimate} || ${?plot_stderr} || ${?plot_relerr} ) || ( "${submodels}" == "" ) ) then
# errmsg.csh ${verbosity} ${myname} 0 "nothing to plot" 1 ""
# exit ${status}
  exit 0
 else if ( !( ${?paramname} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified parameter name" 1 ""
  exit ${status}
 endif

# convert plot range options into a form that can be passed on to mri_to_img
#
 set plotlimits = ( `proc.rangeopts.csh -verbosity ${verbosity} -defaults "auto:auto" ${plotlimits}` )

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "proc.rangeopts.csh -verbosity ${verbosity} -defaults "auto:auto" ${plotlimits}"
 if ( ${status} ) exit ${status}

 if ( "${plotlimits[1]}" == "auto" ) then
  set plotlimits[1] = ""
 else
  set plotlimits[1] = "-n ${plotlimits[1]}"
 endif

 if ( "${plotlimits[2]}" == "auto" ) then
  set plotlimits[2] = ""
 else
  set plotlimits[2] = "-x ${plotlimits[2]}"
 endif

# loop over submodels
#
 foreach submodel ( ${submodels} )
  # estimates file name
  #
  set efile = ${prefix}${submodel}.${echunk}

  # image titles
  #
  set slices = `mri_printfield -input ${efile} -field ${echunk}.extent.z`

  errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri_printfield -input ${efile} -field ${echunk}.extent.z"
  if ( ${status} ) exit ${status}

  setenv F_PS_TITLE1 `mri_printfield -input ${efile} -field imagedata.description`
  setenv F_PS_TITLE2 "submodel: ${submodel}, slices: ${slices}"

  # extract parameter estimates
  #
  extract.param.csh -verbosity ${verbosity} -prefix ${tmp} -infile ${efile} -chunk ${echunk} ${paramname} > ${tmp}.e

  # skip plotting if this parameter name is nonexistent
  #
  set estat = ${status}
  if ( ${estat} ) goto label_skipE

  set elines = `wc -l ${tmp}.e | awk '{print $1}'`

  # plot estimates
  #
  if ( !( ${?plot_estimate} ) ) goto label_skipE

  set i = 1
  while ( ${i} <= ${elines} )
   set pblk  = ( `head -${i} ${tmp}.e | tail -1` )
   set pfile = ${pblk[1]}
   set pname = ${pblk[2]}
   set pidx  = ${pblk[4]}
   set pblk  = ${pblk[5]}

   setenv F_PS_TITLE3 "${pname}"

   mri.to.img.csh -verbosity ${verbosity} -infile ${pfile} -chunk ${echunk} -outfile ${submodel}.${pblk},${pidx}.ps -fiasco "-mosaic -t -c -p -r -g ${plotlimits}"

   errmsg.csh ${verbosity} ${myname} 7 "failed command" ${status} "mri.to.img.csh -verbosity ${verbosity} -infile ${pfile} -chunk ${echunk} -outfile ${submodel}.${pblk},${pidx}.ps -fiasco '-mosaic -t -c -p -r -g ${plotlimits}'"
   if ( ${status} ) exit ${status}

   @ i += 1
  end

  label_skipE:

  # standard errors file name
  #
  set sfile = ${prefix}${submodel}.${schunk}

  # extract standard errors
  #
  extract.param.csh -verbosity ${verbosity} -prefix ${tmp} -infile ${sfile} -chunk ${schunk} ${paramname} > ${tmp}.s

  # skip plotting if this parameter name is nonexistent
  #
  set sstat = ${status}
  if ( ${sstat} ) goto label_skipS

  set slines = `wc -l ${tmp}.s | awk '{print $1}'`

  # plot standard errors
  #
  if ( !( ${?plot_stderr} ) ) goto label_skipS

  set i = 1
  while ( ${i} <= ${slines} )
   set pblk  = ( `head -${i} ${tmp}.s | tail -1` )
   set pfile = ${pblk[1]}
   set pname = ${pblk[2]}
   set pidx  = ${pblk[4]}
   set pblk  = ${pblk[5]}

   setenv F_PS_TITLE3 "stderror for ${pname}"

   mri.to.img.csh -verbosity ${verbosity} -infile ${pfile} -chunk ${schunk} -outfile ${submodel}.${pblk},${pidx}.se.ps -fiasco "-mosaic -t -c -p -r -g ${plotlimits}"

   errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri.to.img.csh -verbosity ${verbosity} -infile ${pfile} -chunk ${schunk} -outfile ${submodel}.${pblk},${pidx}.se.ps -fiasco '-mosaic -t -c -p -r -g ${plotlimits}'"
   if ( ${status} ) exit ${status}

   @ i += 1
  end

  label_skipS:

  # relative error
  #
  if ( !( ${?plot_relerr} ) || ${estat} || ${sstat} ) goto label_skipR

  # error check
  #
  if ( ${elines} != ${slines} ) then
   errmsg.csh ${verbosity} ${myname} 3 "incompatible estimates/standard errors files?" 1 ""
   exit ${status}
  endif

  set i = 1
  while ( ${i} <= ${elines} )
   set pblk  = ( `head -${i} ${tmp}.e | tail -1` )
   set pfile = ${pblk[1]}
   set pname = ${pblk[2]}
   set pgofs = ${pblk[3]}
   set pidx  = ${pblk[4]}
   set pblk  = ${pblk[5]}

   set spblk  = ( `head -${i} ${tmp}.s | tail -1` )
   set spfile = ${spblk[1]}
   set spname = ${spblk[2]}
   set spgofs = ${spblk[3]}
   set spidx  = ${spblk[4]}
   set spblk  = ${spblk[5]}

   if ( ( ${pblk} != ${spblk} ) || ( ${pgofs} != ${spgofs} ) || ( ${pidx} != ${spidx} ) || ( ${pname} != ${spname} ) ) then
    errmsg.csh ${verbosity} ${myname} 3 "incompatible estimates/standard errors files?" 1 ""
    exit ${status}
   endif

   setenv F_PS_TITLE3 "relerror for ${pname}"

   # rename estimate/stderr chunks to "images" for relative error calculation
   #
   mri.rename.chunk.csh -verbosity ${verbosity} -infile ${pfile} -chunk ${echunk} -outfile ${tmp}.e -to images

   errmsg.csh ${verbosity} ${myname} 10 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${pfile} -chunk ${echunk} -outfile ${tmp}.e -to images"
   if ( ${status} ) exit ${status}

   mri.rename.chunk.csh -verbosity ${verbosity} -infile ${spfile} -chunk ${schunk} -outfile ${tmp}.s -to images

   errmsg.csh ${verbosity} ${myname} 11 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${spfile} -chunk ${schunk} -outfile ${tmp}.s -to images"
   if ( ${status} ) exit ${status}

   # compute relative error := ( abs(mean) > zero ) ? stderr/abs(mean) : stderr
   #
   mri_rpn_math -o ${tmp} -c images '$1,$2,abs,/,$1,$2,abs,'${zero}',>,if_keep' ${tmp}.s ${tmp}.e >& ${tmp}.err

   errmsg.csh ${verbosity} ${myname} 12 "failed command" ${status} "mri_rpn_math -o ${tmp} -c images '"'$1,$2,abs,/,$1,$2,abs,'${zero}',>,if_keep'"' ${tmp}.s ${tmp}.e >& ${tmp}.err"
   if ( ${status} ) exit ${status}

   # plot relative error
   #
   mri_to_img -o ${submodel}.${pblk},${pidx}.re.ps -mosaic -t -c -p -r -g ${plotlimits} ${tmp} >& ${tmp}.err

   errmsg.csh ${verbosity} ${myname} 13 "failed command" ${status} "mri_to_img -o ${submodel}.${pblk},${pidx}.re.ps -mosaic -t -c -p -r -g ${plotlimits} ${tmp} >& ${tmp}.err"
   if ( ${status} ) exit ${status}

   @ i += 1
  end

  label_skipR:
 end

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
