#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set indspecs  = ''
 set prefix    = './'
 set submodels = ()
 set pretty

# standard file/chunk names
#
 set f_modelp = model.probabilities

 set f_est    = estimates
 set f_se     = standard.errors
 set f_bl     = baseline
 set f_blse   = baseline.stderr

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Produce,  for  a  voxel,  a submodelwise summary of vmp'
    echo ' parameter estimates: parameter name, estimate, standard'
    echo ' error, relative error.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-n|-nopretty]'
    echo ' [-o|-out output_file]'
    echo ' [-p|-prefix any_directory_prefix]'
    echo ' [-s|-submodels s1,s2,s3,...]'
    echo ' [indspecs]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: invalid voxel specification'
    echo ' 3: submodel information not available'
    echo ' 4-: failed command/utility'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.print.record.csh param.block.info.csh proc.indspecs.csh pretty.print.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 Refer to proc.indspecs.csh for index specification format.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-s":
   case "-submodels":
    set submodels = ( `echo ${argv[2]} | sed 's/,/ /g'` ); shift; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-out":
    set out = ${argv[2]}; shift; shift; breaksw;

   case "-n":
   case "-nopretty":
    unset pretty; shift; breaksw;

   default:
    set indspecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# voxel
#
 proc.indspecs.csh -verbosity ${verbosity} -infile ${prefix}${f_modelp} -chunk ${f_modelp} ${indspecs} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 2 "invalid voxel" ${status} ""
 if ( ${status} ) exit ${status}

 set indspecs = ( `grep '^[xyz]' ${tmp} | sort | awk '{print $1"="$2}'` )
 set indspecs = `echo ${indspecs} | sed 's/ /,/g'`

# submodels
#
 if ( "${submodels}" == "" ) then
  set fitsubmodels = ( `mri_printfield -input ${prefix}${f_modelp} -field submodel.list` )

  errmsg.csh ${verbosity} ${myname} 3 "submodel information not available; failed command:" ${status} "mri_printfield -input ${prefix}${f_modelp} -field submodel.list"
  if ( ${status} ) exit ${status}

  set sm        = ( ${submodels} )
  set submodels = ''
  foreach s ( ${sm} )
   if ( `echo '' | awk '{print index("'"${fitsubmodels}"'","'${s}'")}'` != 0 ) set submodels = ( ${submodels} ${s} )
  end

  if ( "${submodels}" == "" ) set submodels = ( ${fitsubmodels} )
 endif

# overwrite any existing file with the same name
#
 echo -n '' > ${tmp}

# submodelwise parameter estimates, standard and relative errors for voxel
#
 foreach submodel ( ${submodels} )
  echo '' >> ${tmp}
  if ( ${?pretty} ) echo 'Submodel ParameterName Estimate StandardError RelativeError\n' >> ${tmp}

  # parameter names
  #
  param.block.info.csh -verbosity ${verbosity} -block all -infile ${prefix}${submodel}.${f_est} | sort -n -k2 >& ${tmp}.pi

  errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "param.block.info.csh -verbosity ${verbosity} -block all -infile ${prefix}${submodel}.${f_est} | sort -n -k2 >& ${tmp}.pi"
  if ( ${status} ) goto label_skip_estimates

  # estimates
  #
  set estimate = `mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${submodel}.${f_est} -chunk estimates -outfile ${tmp}.e ${indspecs}`

  # errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${submodel}.${f_est} -chunk estimates -outfile ${tmp}.e ${indspecs}"
  if ( ${status} ) goto label_skip_estimates

  # standard errors
  #
  set stderr = `mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${submodel}.${f_se} -chunk ${f_se} -outfile ${tmp}.s ${indspecs}`

  errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${submodel}.${f_se} -chunk ${f_se} -outfile ${tmp}.s ${indspecs}"
  if ( ${status} ) goto label_skip_estimates

  # consistency check
  #
  set numP = ( `wc -l ${tmp}.pi` )
  set numE = ( `wc -l ${tmp}.e`  )
  set numS = ( `wc -l ${tmp}.s`  )

  if ( ( ${numE[1]} != ${numS[1]} ) || ( ${numE[1]} != ${numP[1]} ) ) then
   errmsg.csh ${verbosity} ${myname} 7 "inconsistent estimates/stderrs/paramblock.descriptions?" 1 ""
   goto label_skip_estimates
  endif

  # relative errors
  #
  paste -d' ' ${tmp}.e ${tmp}.s | awk '{if(($1!=0.0)&&(match($0,"(NA)")==0))printf "%g\n",$2/$1;else print "(NA)"}' > ${tmp}.r

  # noise level: using first parameter (noise precision) in the noise block
  #
  set esr = ( '(NA)' '(NA)' '(NA)' )

  set ofs = ( `grep "^noise" ${tmp}.pi | head -1 | awk '{print 1+$2}'` )
  if ( !( ${status} ) ) then
   set e   = `head -${ofs} ${tmp}.e | tail -1`
   set s   = `head -${ofs} ${tmp}.s | tail -1`
   set esr = ( `echo ${e} ${s} | awk '{if($1!=0.0){e=1.0/sqrt($1); s=0.5*$2*e*e*e; printf "%g %g %g", e, s, s/e} else print "(NA) (NA) (NA)"}'` )
  endif

  paste -d' ' ${tmp}.pi ${tmp}.e ${tmp}.s ${tmp}.r | awk '{printf "'${submodel}' %s %g %g %g\n", $1, $3, $4, $5}' >> ${tmp}
  echo ${submodel} noise.level ${esr[1]} ${esr[2]} ${esr[3]} >> ${tmp}

  label_skip_estimates:

  # baseline
  #
  set baseline = '(NA)'
  if ( -f ${prefix}${submodel}.${f_bl}.mri ) then
   set baseline = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_bl} -chunk ${f_bl} ${indspecs}`

   errmsg.csh ${verbosity} ${myname} 8 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_bl} -chunk ${f_bl} ${indspecs}"
   # if ( ${status} ) goto label_skip_baseline

   if ( ${status} ) set baseline = '(NA)'
  endif

  set baselinese = '(NA)'
  if ( -f ${prefix}${submodel}.${f_blse}.mri ) then
   set baselinese = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_blse} -chunk ${f_blse} ${indspecs}`

   errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_blse} -chunk ${f_blse} ${indspecs}"
   # if ( ${status} ) goto label_skip_baseline

   if ( ${status} ) set baselinese = '(NA)'
  endif

  set esr = ( `echo ${baseline} ${baselinese} | awk '{printf "%g %g",$1,$2;if(($1!=0.0)&&(match($0,"(NA)")==0))printf " %g",$2/$1;else print " (NA)"}'` )

  echo ${submodel} baseline ${esr[1]} ${esr[2]} ${esr[3]} >> ${tmp}

  # label_skip_baseline:
 end

# pretty format
#
 if ( ${?pretty} ) then
  pretty.print.csh ${tmp} > ${tmp}.1
  mv ${tmp}.1 ${tmp}
 endif

# screendump if no output file is specified
#
 if ( ${?out} ) then
  echo -n '' > ${out}
  if ( ${?pretty} ) echo "Voxel ${indspecs}" > ${out}
  cat ${tmp} >> ${out}
 else
  if ( ${?pretty} ) echo "Voxel ${indspecs}"
  cat ${tmp}
 endif

# cleanup
#
 rm -f ${tmp}* >& /dev/null
