#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set indspecs  = ''
 set prefix    = './'
 set submodels = ()
 set pretty

# standard file/chunk names
#
 set f_modelp    = model.probabilities
 set f_pdef      = log.determinant.lapackinfo

 set f_status    = status
 set f_iters     = iterations
 set f_evals     = evaluations
 set f_rstrts    = restarts
 set f_gnorm     = gradient.norm

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Produce, for a voxel, a submodelwise summary of minimization'
    echo ' related diagnostics:  status code,  iterations, evaluations,'
    echo ' restarts, infinity norm of the projected gradient,  positive'
    echo ' definiteness.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-n|-nopretty]'
    echo ' [-o|-out output_file]'
    echo ' [-p|-prefix any_directory_prefix]'
    echo ' [-s|-submodels s1,s2,s3,...]'
    echo ' [indspecs]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: invalid voxel specification'
    echo ' 3: submodel information not available'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.print.record.csh proc.indspecs.csh pretty.print.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 Refer to proc.indspecs.csh for index specification format.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-s":
   case "-submodels":
    set submodels = ( `echo ${argv[2]} | sed 's/,/ /g'` ); shift; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-out":
    set out = ${argv[2]}; shift; shift; breaksw;

   case "-n":
   case "-nopretty":
    unset pretty; shift; breaksw;

   default:
    set indspecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# voxel
#
 proc.indspecs.csh -verbosity ${verbosity} -infile ${prefix}${f_modelp} -chunk ${f_modelp} ${indspecs} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 2 "invalid voxel" ${status} ""
 if ( ${status} ) exit ${status}

 set indspecs = ( `grep '^[xyz]' ${tmp} | sort | awk '{print $1"="$2}'` )
 set indspecs = `echo ${indspecs} | sed 's/ /,/g'`

# submodels
#
 if ( "${submodels}" == "" ) then
  set fitsubmodels = ( `mri_printfield -input ${prefix}${f_modelp} -field submodel.list` )

  errmsg.csh ${verbosity} ${myname} 3 "submodel information not available; failed command:" ${status} "mri_printfield -input ${prefix}${f_modelp} -field submodel.list"
  if ( ${status} ) exit ${status}

  set sm        = ( ${submodels} )
  set submodels = ''
  foreach s ( ${sm} )
   if ( `echo '' | awk '{print index("'"${fitsubmodels}"'","'${s}'")}'` != 0 ) set submodels = ( ${submodels} ${s} )
  end

  if ( "${submodels}" == "" ) set submodels = ( ${fitsubmodels} )
 endif

# headers
#
 echo -n '' > ${tmp}
 if ( ${?pretty} ) echo 'Submodel Status Iterations Evaluations Restarts |ProjGrad|_inf PlusDef\n' > ${tmp}

# minimization diagnostics
#
 foreach submodel ( ${submodels} )
  if ( ${submodel} == "average" ) goto label_skip_submodel

  # positive definiteness
  #
  set pdef = '(NA)'
  if ( -f ${prefix}${submodel}.${f_pdef}.mri ) then
   set pdef = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_pdef} -chunk ${f_pdef} ${indspecs}`

   if ( !( ${status} ) ) then
    if ( ${pdef} == 0 ) then
     set pdef = yes
    else
     set pdef = no
    endif
   endif
  endif

  # minimization status
  #
  set ms = '(NA)'
  if ( -f ${prefix}${submodel}.${f_status}.mri ) then
   set ms = `mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${submodel}.${f_status} -chunk ${f_status} ${indspecs}`

   if ( ${status} ) set ms = '(NA)'
  endif

  # iterations
  #
  set iters = '(NA)'
  if ( -f ${prefix}${submodel}.${f_iters}.mri ) then
   set iters = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_iters} -chunk ${f_iters} ${indspecs}`

   if ( ${status} ) set iters = '(NA)'
  endif

  # evaluations
  #
  set evals = '(NA)'
  if ( -f ${prefix}${submodel}.${f_evals}.mri ) then
   set evals = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_evals} -chunk ${f_evals} ${indspecs}`

   if ( ${status} ) set evals = '(NA)'
  endif

  # restarts
  #
  set rstrts = '(NA)'
  if ( -f ${prefix}${submodel}.${f_rstrts}.mri ) then
   set rstrts = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_rstrts} -chunk ${f_rstrts} ${indspecs}`

   if ( ${status} ) set rstrts = '(NA)'
  endif

  # infinity norm of projected gradient
  #
  set gnorm = '(NA)'
  if ( -f ${prefix}${submodel}.${f_gnorm}.mri ) then
   set gnorm = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${submodel}.${f_gnorm} -chunk ${f_gnorm} ${indspecs}`

   if ( ${status} ) set gnorm = '(NA)'
  endif

  echo ${submodel} ${ms} ${iters} ${evals} ${rstrts} ${gnorm} ${pdef} >> ${tmp}

  label_skip_submodel:
 end

# pretty format
#
 if ( ${?pretty} ) then
  pretty.print.csh ${tmp} > ${tmp}.1
  mv ${tmp}.1 ${tmp}
 endif

# ATTN: print explanation of status codes as well?

# screendump if no output file is specified
#
 if ( ${?out} ) then
  echo -n '' > ${out}
  if ( ${?pretty} ) echo "Voxel ${indspecs}\n" > ${out}
  cat ${tmp} >> ${out}
 else
  if ( ${?pretty} ) echo "Voxel ${indspecs}\n"
  cat ${tmp}
 endif

# cleanup
#
 rm -f ${tmp}* >& /dev/null
