#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set indspecs  = ''
 set prefix    = './'
 set submodels = ()
 set pretty

# standard file/chunk names
#
 set f_modelp = model.probabilities
 set f_grad   = gradient

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Produce, for a voxel, a submodelwise summary of minimization'
    echo ' related  diagnostics:  parameterwise  gradients  at maximum.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-n|-nopretty]'
    echo ' [-o|-out output_file]'
    echo ' [-p|-prefix any_directory_prefix]'
    echo ' [-s|-submodels s1,s2,s3,...]'
    echo ' [indspecs]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: invalid voxel specification'
    echo ' 3: submodel information not available'
    echo ' 4-: failed command/utility'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.print.record.csh param.block.info.csh proc.indspecs.csh pretty.print.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 Refer to proc.indspecs.csh for index specification format.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-s":
   case "-submodels":
    set submodels = ( `echo ${argv[2]} | sed 's/,/ /g'` ); shift; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-out":
    set out = ${argv[2]}; shift; shift; breaksw;

   case "-n":
   case "-nopretty":
    unset pretty; shift; breaksw;

   default:
    set indspecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# voxel
#
 proc.indspecs.csh -verbosity ${verbosity} -infile ${prefix}${f_modelp} -chunk ${f_modelp} ${indspecs} > ${tmp}

 errmsg.csh ${verbosity} ${myname} 2 "invalid voxel" ${status} ""
 if ( ${status} ) exit ${status}

 set indspecs = ( `grep '^[xyz]' ${tmp} | sort | awk '{print $1"="$2}'` )
 set indspecs = `echo ${indspecs} | sed 's/ /,/g'`

# submodels
#
 if ( "${submodels}" == "" ) then
  set fitsubmodels = ( `mri_printfield -input ${prefix}${f_modelp} -field submodel.list` )

  errmsg.csh ${verbosity} ${myname} 3 "submodel information not available; failed command:" ${status} "mri_printfield -input ${prefix}${f_modelp} -field submodel.list"
  if ( ${status} ) exit ${status}

  set sm        = ( ${submodels} )
  set submodels = ''
  foreach s ( ${sm} )
   if ( `echo '' | awk '{print index("'"${fitsubmodels}"'","'${s}'")}'` != 0 ) set submodels = ( ${submodels} ${s} )
  end

  if ( "${submodels}" == "" ) set submodels = ( ${fitsubmodels} )
 endif

# header
#
 echo -n '' > ${tmp}
 if ( ${?pretty} ) echo 'Submodel ParameterName Gradient' > ${tmp}

# submodelwise parameter estimates, standard and relative errors for voxel
#
 foreach submodel ( ${submodels} )
  # parameter names
  #
  param.block.info.csh -verbosity ${verbosity} -block all -infile ${prefix}${submodel}.estimates | sort -n -k2 | awk '{print $1}' >& ${tmp}.p

  errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "param.block.info.csh -verbosity ${verbosity} -block all -infile ${prefix}${submodel}.estimates | sort -n -k2 >& ${tmp}.p"
  if ( ${status} ) goto label_skip_submodel

  # gradients
  #
  mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${submodel}.${f_grad} -chunk ${f_grad} -outfile ${tmp}.g ${indspecs}

  errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${submodel}.${f_grad} -chunk ${f_grad} -outfile ${tmp}.g ${indspecs}"
  if ( ${status} ) goto label_skip_submodel

  # consistency check
  #
  set numP = ( `wc -l ${tmp}.p` )
  set numG = ( `wc -l ${tmp}.g` )

  if ( ${numG[1]} != ${numP[1]} ) then
   errmsg.csh ${verbosity} ${myname} 6 "inconsistent gradients/paramblock.descriptions?" 1 ""
   goto label_skip_submodel
  endif

  # print to temporary file
  #
  echo '' >> ${tmp}
  paste -d' ' ${tmp}.p ${tmp}.g | sed 's/^/'${submodel}' /' >> ${tmp}

  label_skip_submodel:
 end

# pretty format
#
 if ( ${?pretty} ) then
  pretty.print.csh ${tmp} > ${tmp}.1
  mv ${tmp}.1 ${tmp}
 endif

# screendump if no output file is specified
#
 if ( ${?out} ) then
  echo -n '' > ${out}
  if ( ${?pretty} ) echo "Voxel ${indspecs}\n" > ${out}
  cat ${tmp} >> ${out}
 else
  if ( ${?pretty} ) echo "Voxel ${indspecs}\n"
  cat ${tmp}
 endif

# cleanup
#
 rm -f ${tmp}* >& /dev/null
