#!/bin/tcsh -f

# defaults
#
 set myname = `basename ${0}`

 mkdir -p /tmp/`whoami`
 set tmpdir = /tmp/`whoami`/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

 set verbmax   = 1
 set verbosity = ${verbmax}

# "cleanup" quantity images using a threshold on another quantity:
# (output) := (quantity)    if (reference) >= (threshold)
#          := (replacewith) otherwise
#
 set quantity    = `echo $1 | sed 's/\.mri$//'`
 set qchunk      = $2
 set reference   = `echo $3 | sed 's/\.mri$//'`
 set rchunk      = $4
 set outfile     = `echo $5 | sed 's/\.mri$//'`
 set threshold   = $6
 set replacewith = $7

# rename chunks
#
 mrirenchunk.csh -verbosity ${verbosity} -infile ${quantity} -outfile ${tmp}.q -chunk ${qchunk} -to images

 errmsg.csh ${verbosity} ${myname} 2 "failed command" ${status} "mrirenchunk.csh -verbosity ${verbosity} -infile ${quantity} -outfile ${tmp}.q -chunk ${qchunk} -to images"
 if ( ${status} ) exit ${status}

 mrirenchunk.csh -verbosity ${verbosity} -infile ${reference} -outfile ${tmp}.r -chunk ${rchunk} -to images

 errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "mrirenchunk.csh -verbosity ${verbosity} -infile ${reference} -outfile ${tmp}.r -chunk ${rchunk} -to images"
 if ( ${status} ) exit ${status}

# permute dimensions appropriately: bring common dimensions to the front
# assume that otherwise chunk dimensions are mri_rpn_math-compatible
# (look this up in mrimatvec.csh)
#
 ...

# multiply quantity by I{reference>=threshold}
#
 mri_rpn_math -o ${tmp} -c images ${replacewith}',$1,'${threshold}',$2,>=,if_keep' ${tmp}.q ${tmp}.r >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_rpn_math -o ${tmp} -c images '${replacewith}"',$1,'${threshold}',$2,>=,if_keep'"' ${tmp}.q ${tmp}.r >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# permute back to original order
#
 ... (creates ${outfile}.dat)

# construct appropriate header file from original quantity header
#
 strings ${quantity}.mri | grep -v "${quantity}.file" > ${outfile}.mri
 echo "${quantity}.file = .dat" >> ${outfile}.mri

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
