#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set asmlist   = ();

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Process a  voxel  list  file  (e. g.,  brain stderr output)'
    echo ' to produce voxelwise results summaries using  voxel.summary.csh.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ' [-s|-submodels s1,s2,s3,...]'
    echo ' -b|-brain BRAIN_input_file'
    echo ' voxel_list_file'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified/invalid voxel list file'
    echo ' 3: unspecified BRAIN output file'
    echo ' 4-: failed script/utility'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' proc.vlist.csh voxel.summary.csh'
    echo ''
    echo 'Notes'
    echo ' Refer to proc.vlist.csh for the  voxel list  and'
    echo ' additional submodel list formats.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo ' additional submodels: none'
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-s":
   case "-submodels":
    set asmlist = ${argv[2]}; shift; shift; breaksw;

   case "-b":
   case "-brain":
    set brainout = ${argv[2]};
    shift; shift; breaksw;

   default:
    set voxelfile = ${argv[1]};
    shift; breaksw;
  endsw
 end

# error checks
#
 if ( !( ${?voxelfile} ) || !( -f ${voxelfile} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid voxel list file" 1 ""
  exit ${status}
 else if ( !( ${?brainout} ) || !( -f ${brainout} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid BRAIN output file" 1 ""
  exit ${status}
 endif

# convert voxel list into a convenient format
#
 proc.vlist.csh -verbosity ${verbosity} -submodels \"${asmlist}\" ${voxelfile} > ${tmp}.vlist

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "proc.vlist.csh -verbosity ${verbosity} -submodels \"${asmlist}\" ${voxelfile} > ${tmp}.vl"
 if ( ${status} ) exit ${status}

 set lines = `wc -l ${tmp}.vl | awk '{print $1}'`

# process voxel list
#
 set i = 1
 while ( ${i} <= ${lines} )
  set submodels = ( `head -${i} ${tmp}.vl | tail -1` )
  set voxel     = ${submodels[1]}; shift submodels;
  set indspecs  = ${submodels[1]}; shift submodels;

  if ( ${verbosity} > 0 ) echo -n "voxel ${voxel}; submodels"

  foreach s ( ${submodels} )
   if ( ${verbosity} > 0 ) echo -n " ${s}"

   voxel.summary.csh -verbosity ${verbosity} -brain ${brainout} -submodel ${s} ${indspecs} >& ${tmp}.err

   errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "voxel.summary.csh -verbosity ${verbosity} -brain ${brainout} -submodel ${s} ${indspecs} >& ${tmp}.err"
   if ( ${status} ) exit ${status}
  end

  if ( ${verbosity} > 0 ) echo ""

  @ i += 1
 end

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
