#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set process_summary
 set cleanup

 set indspecs = ''

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Produce a BRAIN results summary for the specified'
    echo ' voxel and submodel (as TeX+PostScript).'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-np|-noprocess] [-nc|-nocleanup]'
    echo ' -s|-submodel submodel -b|-brain BRAIN_input_file'
    echo ' [indspecs]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: gnuplot/Splus/latex/dvips not available'
    echo ' 3: unspecified/invalid BRAIN input file'
    echo ' 4: unspecified/invalid submodel'
    echo ' 5: inconsistent BRAIN output'
    echo ' 6-: failed script/utility'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' tc.to.ps.csh tcfit.to.ps.csh pixel.to.eps.csh'
    echo ' mri.print.record.csh param.block.info.csh proc.indspecs.csh'
    echo ' Splus gnuplot latex dvips'
    echo ''
    echo 'Notes'
    echo ' 1 Refer to proc.indspecs.csh for index specification format.'
    echo " 2 Unfortunately,  there doesn't seem to be any way of making"
    echo '   latex  exit  on error  with a  nonzero termination status.'
    echo ' 3 Output file name(s)  begin with a prefix constructed using'
    echo '   voxel indices and submodel (of the form x.y.z_s.*).'
    echo ' 4 -noprocess implies -nocleanup'
#   echo ''
#   echo 'Defaults'
#   echo ' image datachunk name: images'
#   echo ' process summary using latex; cleanup all the mess.'
#   echo ''
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-nc":
   case "-nocleanup":
    unset cleanup; shift; breaksw;

   case "-np":
   case "-noprocess":
    unset process_summary; unset cleanup; shift; breaksw;

   case "-s":
   case "-submodel":
    set submodel = ${argv[2]}; shift; shift; breaksw;

   case "-b":
   case "-brain":
    set master = ${argv[2]}; shift; shift; breaksw;

   default:
    set indspecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# check availability: gnuplot, Splus, latex, dvips
#
 foreach a ( gnuplot Splus )
  if ( `which ${a} | awk '{print index($0,"Command not found")}'` != 0 ) then
   errmsg.csh ${verbosity} ${myname} 2 "${a} not available" 1 ""
   exit ${status}
  endif
 end

 if ( ${?process_summary} ) then
  foreach a ( latex dvips )
   if ( `which ${a} | awk '{print index($0,"Command not found")}'` != 0 ) then
    errmsg.csh ${verbosity} ${myname} 2 "${a} not available" 1 ""
    exit ${status}
   endif
  end
 endif

# error checks
#
 if ( !( ${?master} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid BRAIN output file" 1 ""
  exit ${status}
 else if ( !( -f ${master} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid BRAIN output file" 1 ""
  exit ${status}
 else if ( !( ${?submodel} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified/invalid submodel" 1 ""
  exit ${status}
 endif

# data: chunkname assumed to be images
#
 set dchunk = images
 set dfile  = `mri_printfield -input ${master} -field imagedata`

 errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "mri_printfield -input ${master} -field imagedata"
 if ( ${status} ) exit ${status}

# voxel indices
#
 proc.indspecs.csh -verbosity ${verbosity} -infile ${dfile} -chunk images ${indspecs} >& ${tmp}

 errmsg.csh ${verbosity} ${myname} 7 "failed command" ${status} "proc.indspecs.csh -verbosity ${verbosity} -infile ${dfile} -chunk images ${indspecs} >& ${tmp}"
 if ( ${status} ) exit ${status}

 set indices = (            `grep '^x' ${tmp} | awk '{print $2}'` )
 set indices = ( ${indices} `grep '^y' ${tmp} | awk '{print $2}'` )
 set indices = ( ${indices} `grep '^z' ${tmp} | awk '{print $2}'` )

# construct a name for the voxel
#
 set voxel = ${indices[1]}.${indices[2]}.${indices[3]}

# prefix for files generated by this script
#
 set oprefix = ${voxel}_${submodel}

# output files path/prefix
#
 set prefix = `mri_printfield -input ${master} -field output.prefix`

 errmsg.csh ${verbosity} ${myname} 8 "failed command" ${status} "mri_printfield -input ${master} -field output.prefix"
 if ( ${status} ) exit ${status}

# data/run description
#
 set description  = `mri_printfield -input ${master} -field imagedata.description`
 if ( "${description}" == '(none available)' ) set description = ''
 set description2 = `mri_printfield -input ${master} -field thisrun.description`
 if ( "${description2}" != '(none available)' ) set description = "${description}; ${description2}"

# experimental conditions and condition tags
#
 set conditions = ( `mri_printfield -input ${master} -field design.conditions` )

 errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri_printfield -input ${master} -field design.conditions"
 if ( ${status} ) exit ${status}

 set ctags = ( `mri_printfield -input ${master} -field design.conditions.tags` )

 errmsg.csh ${verbosity} ${myname} 10 "failed command" ${status} "mri_printfield -input ${master} -field design.conditions.tags"
 if ( ${status} ) exit ${status}

# submodel file prefix
#
 set smprefix = ${submodel}

 if ( "${smprefix}" == "average" ) then
  set average
  set submodel = `echo ${#conditions} | awk '{for ( i = 0; i < $1; i++ ) printf "1"}'`
 endif

# number of conditions: total/active
#
 set numconditions = ${%submodel}

 if ( ${numconditions} != ${#conditions} ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified/invalid submodel" 1 ""
  exit ${status}
 endif

 set numactive = `echo ${submodel} | awk '{for(i=1;i<=length;i++)s+=substr($1,i,1);print s}'`

# miscellaneous
#
 set iai = `mri_printfield -input ${master} -field acquisition.iai`
 if ( ${status} ) set iai = 1.0

 set tmax = `mri_printfield -input ${dfile} -field ${dchunk}.extent.t`

 errmsg.csh ${verbosity} ${myname} 11 "failed command" ${status} "mri_printfield -input ${dfile} -field ${dchunk}.extent.t"
 if ( ${status} ) exit ${status}

 @ tmax -= 1 # 0-based

 set numX = `mri_printfield -input ${dfile} -field ${dchunk}.extent.x`

 errmsg.csh ${verbosity} ${myname} 12 "failed command" ${status} "mri_printfield -input ${dfile} -field ${dchunk}.extent.x"
 if ( ${status} ) exit ${status}

# generate settings for gnuplot
#
 echo 'set xtics ("0" 0,\\' > ${oprefix}.gpt

 foreach c ( ${conditions} )
  set epochs = ( `mri_printfield -input ${master} -field design.epochs.begin.${c}` )

  errmsg.csh ${verbosity} ${myname} 13 "failed command" ${status} "mri_printfield -input ${master} -field design.epochs.begin.${c}"
  if ( ${status} ) exit ${status}

  set i = 1
  while ( ${i} <= ${#epochs} )
   set tic = `echo ${epochs[${i}]} | awk '{print $1/'${iai}'}'`
   echo -n '"" '${tic}',' >> ${oprefix}.gpt
   @ i += 1
  end

  echo '\\' >> ${oprefix}.gpt
 end

 echo '"'${tmax}'" '${tmax}');' >> ${oprefix}.gpt
 echo 'set grid x;'             >> ${oprefix}.gpt

# non-critical errors flag
#
 set errstatus = 0

# generate voxel location indicator
#
 set vlocplot = ${oprefix}.vloc.eps

 set msg = "`pixel.to.eps.csh -verbosity ${verbosity} -outfile ${vlocplot} -n ${numX} -x ${indices[1]} -y ${indices[2]}`"

 if ( ${status} ) then
  errmsg.csh ${verbosity} ${myname} 0 "failed command" ${status} "pixel.to.eps.csh -verbosity ${verbosity} -outfile ${vlocplot} -n ${numX} -x ${indices[1]} -y ${indices[2]}"

  set errstatus = 30
  set vlocplot  = ${tmp}.sad.eps
 endif

# generate data+fit plot
#
 set fitplot = ${oprefix}.fit.eps

 echo 'set xlabel "t";'          >> ${oprefix}.gpt
 echo 'set ylabel "data+fit";'   >> ${oprefix}.gpt

 set msg = `tcfit.to.ps.csh -verbosity ${verbosity} -datafile ${dfile} -datachunk ${dchunk} -fitfile ${prefix}${smprefix}.fit -fitchunk fit -outfile ${fitplot} -gnuplot ${oprefix}.gpt ${indspecs}`

 if ( ${status} ) then
  errmsg.csh ${verbosity} ${myname} 0 "failed command" ${status} "tcfit.to.ps.csh -verbosity ${verbosity} -datafile ${dfile} -datachunk ${dchunk} -fitfile ${prefix}${smprefix}.fit -fitchunk fit -outfile ${fitplot} -gnuplot ${oprefix}.gpt ${indspecs}"

  set errstatus = 30
  set fitplot   = ${tmp}.sad.eps
 endif

# generate residuals plot
#
 set residplot = ${oprefix}.resid.eps

 echo 'set nogrid; set grid y;'  >> ${oprefix}.gpt
 echo 'set ylabel "residuals";'  >> ${oprefix}.gpt

 set msg = `tc.to.ps.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.residuals -chunk residuals -outfile ${residplot} -gnuplot ${oprefix}.gpt ${indspecs}`

 if ( ${status} ) then
  errmsg.csh ${verbosity} ${myname} 0 "failed command" ${status} "tc.to.ps.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.residuals -chunk residuals -outfile ${residplot} -gnuplot ${oprefix}.gpt ${indspecs}"

  set errstatus = 30
  set residplot = ${tmp}.sad.eps
 endif

# generate residuals acf and histogram
#
 set rhistplot = ${oprefix}.rhist.eps
 set rqqplot   = ${oprefix}.rqq.eps
 set racfnplot = ${oprefix}.racfn.eps
 set rsdplot   = ${oprefix}.spden.eps

 set msg = `mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${smprefix}.residuals -chunk residuals -outfile ${oprefix}.resid.txt ${indspecs}`

 if ( ${status} ) then
  errmsg.csh ${verbosity} ${myname} 0 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${smprefix}.residuals -chunk residuals -outfile ${oprefix}.resid.txt ${indspecs}"

  set errstatus = 30
  set rhistplot = ${tmp}.sad.eps
  set rqqplot   = ${tmp}.sad.eps
  set racfnplot = ${tmp}.sad.eps
  set rsdplot   = ${tmp}.sad.eps
 else
  echo '# Function from Venables & Ripley,'                               > ${oprefix}.S
  echo '# Modern Applied Statistics with S-Plus, 3rd ed.'                >> ${oprefix}.S
  echo '#'                                                               >> ${oprefix}.S
  echo 'nclass.FD <- function( x )'                                      >> ${oprefix}.S
  echo ' {'                                                              >> ${oprefix}.S
  echo '  r <- quantile( x, c( 0.25, 0.75 ) )'                           >> ${oprefix}.S
  echo '  names( r ) <- NULL # to avoid the label 75%'                   >> ${oprefix}.S
  echo '  h <- 2 * ( r[2] - r[1] ) * length( x )^{-1/3}'                 >> ${oprefix}.S
  echo '  ceiling( diff( range( x ) / h ) )'                             >> ${oprefix}.S
  echo ' }'                                                              >> ${oprefix}.S
  echo ''                                                                >> ${oprefix}.S
  echo '# Spectral density analysis functions from' >> ${oprefix}.S
  echo '# the book' >> ${oprefix}.S
  echo '# by Sam Efromovich; http://math.unm.edu/~efrom/book1/' >> ${oprefix}.S
  echo '#' >> ${oprefix}.S
  echo 'ab.vec <- function( f = NA )' >> ${oprefix}.S
  echo ' {' >> ${oprefix}.S
  echo '  n   <- length( f ) + 1.' >> ${oprefix}.S
  echo '  f   <- c( f, 0. )' >> ${oprefix}.S
  echo '  vec <- c( 0., 0. )' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( all( f > 0. ) )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    vec <- c( vec, 1., n )' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '  else' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    seq.pos <- ( 1.:n )[f > 0.]' >> ${oprefix}.S
  echo '    seq.neg <- ( 1.:n )[f <= 0.]' >> ${oprefix}.S
  echo '    seq.neg <- seq.neg[seq.neg > 1.]' >> ${oprefix}.S
  echo '    a <- 1.' >> ${oprefix}.S
  echo '    while ( length( seq.pos ) * length( seq.neg ) > 0. )' >> ${oprefix}.S
  echo '     {' >> ${oprefix}.S
  echo '      if ( f[a + 1.] > 0. )' >> ${oprefix}.S
  echo '       {' >> ${oprefix}.S
  echo '        b <- min( seq.neg ) - 1.' >> ${oprefix}.S
  echo '       }' >> ${oprefix}.S
  echo '      else' >> ${oprefix}.S
  echo '       {' >> ${oprefix}.S
  echo '        a <- min( seq.pos )' >> ${oprefix}.S
  echo '        seq.neg <- seq.neg[seq.neg > a]' >> ${oprefix}.S
  echo '        b <- min( seq.neg ) - 1.' >> ${oprefix}.S
  echo '       }' >> ${oprefix}.S
  echo '      vec <- c( vec, a, b )' >> ${oprefix}.S
  echo '      seq.pos <- seq.pos[seq.pos > b]' >> ${oprefix}.S
  echo '      seq.neg <- seq.neg[seq.neg > b]' >> ${oprefix}.S
  echo '      a <- b' >> ${oprefix}.S
  echo '     }' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( vec[length( vec )] == n )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    vec[length( vec )] <- n - 1.' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  vec' >> ${oprefix}.S
  echo ' }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo 'rem.bump1 <- function( f = NA, AREASQ = NA, coef = 1. )' >> ${oprefix}.S
  echo ' {' >> ${oprefix}.S
  echo '  n   <- length( f )' >> ${oprefix}.S
  echo '  vec <- ab.vec( f )' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( length( vec ) > 2. )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    vec <- vec[ - c( 1., 2. )]' >> ${oprefix}.S
  echo '    k   <- length( vec ) / 2.' >> ${oprefix}.S
  echo '    for ( s in 1.:k )' >> ${oprefix}.S
  echo '     {' >> ${oprefix}.S
  echo '      if ( sum( ( f[vec[2. * s - 1.]:vec[2. * s]] )^2. ) / n <= coef * AREASQ )' >> ${oprefix}.S
  echo '       {' >> ${oprefix}.S
  echo '        f[vec[2. * s - 1.]:vec[2. * s]] <- 0.' >> ${oprefix}.S
  echo '       }' >> ${oprefix}.S
  echo '     }' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  f' >> ${oprefix}.S
  echo ' }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo 'negden <- function( f = NA, delta = 0.01, FLAGBUMP = 1., cB = 2. )' >> ${oprefix}.S
  echo ' {' >> ${oprefix}.S
  echo '  flag <- 0.' >> ${oprefix}.S
  echo '  f1   <- f' >> ${oprefix}.S
  echo '  k    <- length( f )' >> ${oprefix}.S
  echo '  AREA <- ( k/( k - 1. ) ) * mean( f ) - ( f[1.] + f[k] )/( 2. * ( k - 1. ) )' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( all( f >= 0. ) )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    flag <- 1.' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( all( f <= 2. * delta ) | ( AREA <= 2. * delta ) )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    flag <- 2.' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '   while ( flag == 0. )' >> ${oprefix}.S
  echo '    {' >> ${oprefix}.S
  echo '     f <- f - delta' >> ${oprefix}.S
  echo '     f[f < 0.] <- 0.' >> ${oprefix}.S
  echo '     int <- ( k/( k - 1. ) ) * mean( f ) - ( f[1.] + f[k] )/( 2. * ( k - 1. ) )' >> ${oprefix}.S
  echo '     if ( int <= AREA )' >> ${oprefix}.S
  echo '      {' >> ${oprefix}.S
  echo '       if ( int > ( 10. * delta ) )' >> ${oprefix}.S
  echo '        {' >> ${oprefix}.S
  echo '         f <- f * ( AREA/int )' >> ${oprefix}.S
  echo '        }' >> ${oprefix}.S
  echo '       flag <- 1.' >> ${oprefix}.S
  echo '      }' >> ${oprefix}.S
  echo '    }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '   if ( FLAGBUMP == 1. )' >> ${oprefix}.S
  echo '    {' >> ${oprefix}.S
  echo '     AREASQ <- mean( ( f - f1 )^2. )' >> ${oprefix}.S
  echo '     f      <- rem.bump1( f = f, AREASQ = AREASQ, coef = cB )' >> ${oprefix}.S
  echo '    }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '   if ( flag == 1. )' >> ${oprefix}.S
  echo '    {' >> ${oprefix}.S
  echo '     if ( mean( f ) > ( 10. * delta ) )' >> ${oprefix}.S
  echo '      {' >> ${oprefix}.S
  echo '       f <- f * ( AREA/mean( f ) )' >> ${oprefix}.S
  echo '      }' >> ${oprefix}.S
  echo '    }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '   f[f < 0.] <- 0.' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '   f' >> ${oprefix}.S
  echo ' }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo 'Updiag <- function( J = 2. )' >> ${oprefix}.S
  echo ' {' >> ${oprefix}.S
  echo '  m <- matrix( 1., nrow = J, ncol = J )' >> ${oprefix}.S
  echo '  m[row( m ) > col( m )] <- 0.' >> ${oprefix}.S
  echo '  m' >> ${oprefix}.S
  echo ' }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo 'estspden <- function( X   = NA, TT  = NA,  FLAGMIS = 0., knots = 100.,' >> ${oprefix}.S
  echo '                      cJ0 = 4., cJ1 = 0.5, cJM     = 6., cT    = 4., cB = 2., FLAGNEG = 1. )' >> ${oprefix}.S
  echo ' {' >> ${oprefix}.S
  echo '  n <- length( X )' >> ${oprefix}.S
  echo '  if ( FLAGMIS == 1. )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    n <- sum( TT )' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  JMAX   <- ceiling( cJ0 + cJ1 * logb( n + 3. ) )' >> ${oprefix}.S
  echo '  theta1 <- acf( X, lag.max = cJM * JMAX + 1., type = "covariance", plot = F )$acf[, 1., 1.]' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( FLAGMIS == 1. )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    cc     <- acf( TT, cJM * JMAX + 1., type = "covariance", plot = F )$acf[, 1., 1.] + mean( TT )^2.' >> ${oprefix}.S
  echo '    theta1 <- ( theta1 + mean( X )^2. )/cc' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  d <- 2. * sum( theta1[1.:JMAX]^2. ) - theta1[1.]^2.' >> ${oprefix}.S
  echo '  thetasq1 <- theta1^2. - d/n' >> ${oprefix}.S
  echo '  thetasq1[thetasq1 < 0.] <- 0.' >> ${oprefix}.S
  echo '  theta <- theta1[1.:JMAX]' >> ${oprefix}.S
  echo '  thetasq <- thetasq1[1.:JMAX]' >> ${oprefix}.S
  echo '  error <- matrix( d/n - thetasq, nrow = 1. ) %*% Updiag( JMAX )' >> ${oprefix}.S
  echo '  J <- order( error )[1.]' >> ${oprefix}.S
  echo '  theta <- theta[1.:J]' >> ${oprefix}.S
  echo '  thetasq <- thetasq[1.:J]' >> ${oprefix}.S
  echo '  arg <- outer( seq( 0., pi, len = knots ), ( 0.:( cJM * JMAX ) ) )' >> ${oprefix}.S
  echo '  bas <- cos( arg )/pi' >> ${oprefix}.S
  echo '  theta <- ( theta * thetasq )/( thetasq + ( d/n ) )' >> ${oprefix}.S
  echo '  theta[1.] <- theta[1.]/2.' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( cJM <= 1. )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    if ( JMAX > J )' >> ${oprefix}.S
  echo '     {' >> ${oprefix}.S
  echo '      theta <- c( theta, rep( 0., JMAX - J + 1. ) )' >> ${oprefix}.S
  echo '     }' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( cJM > 1. )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    rest.theta <- theta1[( J + 1. ):( cJM * JMAX + 1. )]' >> ${oprefix}.S
  echo '    rest.theta[rest.theta^2. < ( cT * d * logb( n + 3. ) )/n] <- 0.' >> ${oprefix}.S
  echo '    theta <- c( theta, rest.theta )' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  f <- bas %*% theta' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  if ( FLAGNEG == 1. )' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    f <- negden( f, FLAGBUMP = 1., cB = cB )' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '  else' >> ${oprefix}.S
  echo '   {' >> ${oprefix}.S
  echo '    f[f < 0.] <- 0.' >> ${oprefix}.S
  echo '   }' >> ${oprefix}.S
  echo '' >> ${oprefix}.S
  echo '  f' >> ${oprefix}.S
  echo ' }' >> ${oprefix}.S
  echo ''                                                                >> ${oprefix}.S
  echo 'residuals <- scan( "'${oprefix}.resid.txt'" );'                  >> ${oprefix}.S
  echo ''                                                                >> ${oprefix}.S
  echo 'postscript( ); pscript( onefile = F, horizo = F, print = F );'   >> ${oprefix}.S
  echo 'acf( residuals ); dev.off( );'                                   >> ${oprefix}.S
  echo ''                                                                >> ${oprefix}.S
  echo 'postscript( ); pscript( onefile = F, horizo = F, print = F );'   >> ${oprefix}.S
  echo 'hist( residuals, nclass = nclass.FD( residuals ) ); dev.off( );' >> ${oprefix}.S
  echo ''                                                                >> ${oprefix}.S
  echo 'postscript( ); pscript( onefile = F, horizo = F, print = F );'   >> ${oprefix}.S
  echo 'qqnorm( residuals ); qqline( residuals ); dev.off( );'           >> ${oprefix}.S
  echo ''                                                                >> ${oprefix}.S
  echo 'sp <- estspden( residuals );'                                    >> ${oprefix}.S
  echo 'f  <- seq( 0.0, 0.5, length = length( sp ) );'                   >> ${oprefix}.S
  echo ''                                                                >> ${oprefix}.S
  echo 'postscript( ); pscript( onefile = F, horizo = F, print = F );'   >> ${oprefix}.S
  echo "plot( f, sp, type = 'l', ylab = 'Spectral Density' );"           >> ${oprefix}.S
  echo 'dev.off( );'                                                     >> ${oprefix}.S

  Splus < ${oprefix}.S >& ${oprefix}.S.out

  if ( ${status} || ( `grep -ci problem ${oprefix}.S.out` != 0 ) ) then
   errmsg.csh ${verbosity} ${myname} 0 "failed command" 1 "Splus < ${oprefix}.S >& ${oprefix}.S.out"

   set errstatus = 30
   set rhistplot = ${tmp}.sad.eps
   set rqqplot   = ${tmp}.sad.eps
   set racfnplot = ${tmp}.sad.eps
   set rsdplot   = ${tmp}.sad.eps
  else
   set rfiles = ( `cat ${oprefix}.S.out | grep "^Generated postscript file" | awk '{print $4}' | sed -e 's/^"//' -e 's/"\.$//'` )

   mv ${rfiles[1]} ${racfnplot}
   mv ${rfiles[2]} ${rhistplot}
   mv ${rfiles[3]} ${rqqplot}
   mv ${rfiles[4]} ${rsdplot}

   unset rfiles
  endif
 endif

# if any of the plots could not be generated,
# substitute a sad face for that.
#
 if ( ${errstatus} ) then
  echo 'set nokey; set noborder; set noxtics; set noytics;'              > ${tmp}.gpt
  echo 'set samples 49; set xrange [-1:1]; set yrange [-1:1];'          >> ${tmp}.gpt
  echo "set terminal postscript eps; set output '"${tmp}".sad.eps';"    >> ${tmp}.gpt
  echo 'a=9.0/4.0;b=5.0/16.0;'                                          >> ${tmp}.gpt
  echo 'pl sqrt(1.0-x*x) w l lt 1 lw 3, -sqrt(1.0-x*x) w l lt 1 lw 3,\' >> ${tmp}.gpt
  echo '   -0.5*(1.5-sqrt(a-4*(b+x*x))) w l lt 1 lw 3,\'                >> ${tmp}.gpt
  echo '    0.5*(0.5+sqrt(0.25-4.0*(x*x+x+0.25))) w l lt 1 lw 3,\'      >> ${tmp}.gpt
  echo '    0.5*(0.5-sqrt(0.25-4.0*(x*x+x+0.25))) w l lt 1 lw 3,\'      >> ${tmp}.gpt
  echo '    0.5*(0.5+sqrt(0.25-4.0*(x*x-x+0.25))) w l lt 1 lw 3,\'      >> ${tmp}.gpt
  echo '    0.5*(0.5-sqrt(0.25-4.0*(x*x-x+0.25))) w l lt 1 lw 3'        >> ${tmp}.gpt

  gnuplot ${tmp}.gpt >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 14 "failed command" ${status} "gnuplot ${tmp}.gpt >& ${tmp}.err"
  if ( ${status} ) exit ${status}
 endif

# model probability, positive definiteness
#
 set mp   = 'N/A'
 set pdef = 'N/A'

 if ( !( ${?average} ) ) then
  set mp = `mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}model.probabilities -chunk model.probabilities ${indspecs}`

  if ( ${status} ) then
   errmsg.csh ${verbosity} ${myname} 0 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}model.probabilities -chunk model.probabilities ${indspecs}"

   set errstatus = 30
   set mp        = 'N/A'
  else
   set index = `mri_printfield -input ${prefix}model.probabilities -field submodel.index.${submodel}`

   if ( ${status} ) then
    errmsg.csh ${verbosity} ${myname} 0 "failed command" ${status} "mri_printfield -input ${prefix}model.probabilities -field submodel.index.${submodel}"

    set errstatus = 30
    set mp        = 'N/A'
   else
    @   index += 1
    set mp     = ${mp[${index}]}
   endif
  endif

  set pdef = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.log.determinant.lapackinfo -chunk log.determinant.lapackinfo ${indspecs}`

  if ( ${status} ) then
   errmsg.csh ${verbosity} ${myname} 0 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.log.determinant.lapackinfo -chunk log.determinant.lapackinfo ${indspecs}"

   set errstatus = 30
   set pdef      = 'N/A'
  else if ( ${pdef} == 0 ) then
   set pdef = yes
  else
   set pdef = no
  endif
 endif

# parameter blocks, names, estimates, standard errors
#
 param.block.info.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.estimates >& ${tmp}.bi

 errmsg.csh ${verbosity} ${myname} 15 "failed command" ${status} "param.block.info.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.estimates >& ${tmp}.bi"
 if ( ${status} ) exit ${status}

 param.block.info.csh -verbosity ${verbosity} -block all -infile ${prefix}${smprefix}.estimates >& ${tmp}.pn

 errmsg.csh ${verbosity} ${myname} 16 "failed command" ${status} "param.block.info.csh -verbosity ${verbosity} -block all -infile ${prefix}${smprefix}.estimates >& ${tmp}.pn"
 if ( ${status} ) exit ${status}

 set estimate = `mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${smprefix}.estimates -chunk estimates -outfile ${tmp}.estimates ${indspecs}`

 errmsg.csh ${verbosity} ${myname} 17 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${smprefix}.estimates -chunk estimates -outfile ${tmp}.estimates ${indspecs}"
 if ( ${status} ) exit ${status}

 set stderr = `mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${smprefix}.standard.errors -chunk standard.errors -outfile ${tmp}.stderrs ${indspecs}`

 errmsg.csh ${verbosity} ${myname} 18 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${prefix}${smprefix}.standard.errors -chunk standard.errors -outfile ${tmp}.stderrs ${indspecs}"
 if ( ${status} ) exit ${status}

 set numP = ( `wc -l ${tmp}.pn`     )
 set numE = ( `wc -l ${tmp}.estimates` )
 set numS = ( `wc -l ${tmp}.stderrs`   )

 if ( ( ${numE[1]} != ${numS[1]} ) || ( ${numE[1]} != ${numP[1]} ) ) then
  errmsg.csh ${verbosity} ${myname} 5 "inconsistent estimates/stderrs/paramblock.description?" 1 ""
  exit ${status}
 endif

 set estimate = ( `cat ${tmp}.estimates` )
 set stderr   = ( `cat ${tmp}.stderrs`   )
 set pnames   = ( `cat ${tmp}.pn | awk '{print $1}' | sed -e 's/noise,//' -e 's/response,//' -e 's/shape,//'` )

# baseline
#
 set baseline = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.baseline -chunk baseline ${indspecs}`

 errmsg.csh ${verbosity} ${myname} 19 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.baseline -chunk baseline ${indspecs}"
 if ( ${status} ) exit ${status}

 set baselinese = `mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.baseline.stderr -chunk baseline.stderr ${indspecs}`

 errmsg.csh ${verbosity} ${myname} 20 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -infile ${prefix}${smprefix}.baseline.stderr -chunk baseline.stderr ${indspecs}"
 if ( ${status} ) exit ${status}

# more error checks
#
 set stat = 0

 grep -q 'noise' ${tmp}.bi
 @ stat += ${status}

 if ( ${numactive} > 0 ) then
  grep -q 'response' ${tmp}.bi
  @ stat += ${status}

  set resplen = ( `grep response ${tmp}.bi` )
  if ( ${numactive} != ${resplen[3]} ) @ stat += 1
 endif

 if ( ${stat} ) then
  errmsg.csh ${verbosity} ${myname} 5 "missing/inconsistent parameter blocks noise/response?" 1 ""
  exit ${status}
 endif

# generate tex summary
#
 echo '\\documentclass[11pt]{report}'                                                      > ${oprefix}.tex
 echo '\usepackage{epsf}'                                                                 >> ${oprefix}.tex
 echo ''                                                                                  >> ${oprefix}.tex
 echo '\\setlength{\\textheight}{10.0in}'                                                 >> ${oprefix}.tex
 echo '\\setlength{\\textwidth}{7.5in}'                                                   >> ${oprefix}.tex
 echo '\\setlength{\\topmargin}{-76pt}'                                                   >> ${oprefix}.tex
 echo '\\setlength{\\oddsidemargin}{-53pt}'                                               >> ${oprefix}.tex
 echo '\\setlength{\\evensidemargin}{-53pt}'                                              >> ${oprefix}.tex
 echo ''                                                                                  >> ${oprefix}.tex
 echo '\\pagestyle{empty}'                                                                >> ${oprefix}.tex
 echo ''                                                                                  >> ${oprefix}.tex
 echo '\\begin{document}'                                                                 >> ${oprefix}.tex
 echo ''                                                                                  >> ${oprefix}.tex
 echo '\\small'                                                                           >> ${oprefix}.tex
 echo ''                                                                                  >> ${oprefix}.tex
 echo '\\begin{tabular}{cc}'                                                              >> ${oprefix}.tex
 echo ' \\begin{minipage}{3.75in}'                                                        >> ${oprefix}.tex
 echo ''                                                                                  >> ${oprefix}.tex
 echo '  \\begin{tabular}{cc}'                                                            >> ${oprefix}.tex
 echo '   \\begin{minipage}{1.0in}'                                                       >> ${oprefix}.tex
 echo '    \\centerline{\\epsfxsize 1.0in \\epsfysize 1.0in \\epsfbox{'${vlocplot}'}}'    >> ${oprefix}.tex
 echo '   \\end{minipage}'                                                                >> ${oprefix}.tex
 echo '   &'                                                                              >> ${oprefix}.tex
 echo '   \\begin{minipage}{2.75in}'                                                      >> ${oprefix}.tex
 if ( "${description}" != "" ) echo '    \\noindent{\\bf '${description}'}\\\\'           >> ${oprefix}.tex
 echo '    \\noindent{\\bf Voxel: '${voxel}', Submodel: '${submodel}'}\\\\'               >> ${oprefix}.tex
 if ( ${?average} ) then
  echo '    \\noindent{\\bf Model-Averaged Results}\\\\'                                  >> ${oprefix}.tex
 else
  echo '    \\noindent{\\bf Model Probability: '${mp}'}\\\\'                              >> ${oprefix}.tex
  echo '    \\noindent{\\bf Positive Definite: '${pdef}'}'                                >> ${oprefix}.tex
 endif
 echo '   \\end{minipage}'                                                                >> ${oprefix}.tex
 echo '  \\end{tabular}'                                                                  >> ${oprefix}.tex
 echo ''                                                                                  >> ${oprefix}.tex
 echo '  \\vspace{0.1in}'                                                                 >> ${oprefix}.tex
 echo ''                                                                                  >> ${oprefix}.tex
 echo '  \\centerline{\\epsfxsize 4.0in \\epsfysize 3.0in \\epsfbox{'${fitplot}'}}'       >> ${oprefix}.tex

 echo ''                                                                                  >> ${oprefix}.tex
 echo '  \\begin{center}'                                                                 >> ${oprefix}.tex
 echo '   \\begin{tabular}{rlll}'                                                         >> ${oprefix}.tex
 echo '    & {\\bf Estimate} & {\\bf StdError } & {\\bf RelError}\\\\'                    >> ${oprefix}.tex
 echo '    & & & \\\\'                                                                    >> ${oprefix}.tex

# baseline
#
 # set esr = ( `echo ${baseline} ${baselinese} | awk '{printf "%12.5e %12.5e",$1,$2;if($1!=0.0)printf " %12.5e",$2/$1;else print " -"}'` )
 set esr = ( `echo ${baseline} ${baselinese} | awk '{printf "%g %g",$1,$2;if($1!=0.0)printf " %g",$2/$1;else print " -"}'` )

 echo '    {\\bf baseline} & '${esr[1]}' & '${esr[2]}' & '${esr[3]}'\\\\'                 >> ${oprefix}.tex

# noise level: first parameter in the noise block
#
 @ ofs = 1 + `grep '^noise ' ${tmp}.bi | awk '{print $2}'`

 # set esr = ( `echo ${estimate[${ofs}]} ${stderr[${ofs}]} | awk '{if($1!=0.0){e=1.0/sqrt($1); s=0.5*$2*e*e*e; printf "%12.5e %12.5e %12.5e", e, s, s/e} else print "-- -- --"}'` )
 set esr = ( `echo ${estimate[${ofs}]} ${stderr[${ofs}]} | awk '{if($1!=0.0){e=1.0/sqrt($1); s=0.5*$2*e*e*e; printf "%g %g %g", e, s, s/e} else print "-- -- --"}'` )

 echo '    {\\bf noise level} & '${esr[1]}' & '${esr[2]}' & '${esr[3]}'\\\\'               >> ${oprefix}.tex

# rest of the noise parameters, if any
#
 set len = `grep '^noise ' ${tmp}.bi | awk '{print $3}'`

 if ( ${len} > 1 ) then
  echo '    {\\bf Noise} & & & \\\\'                                                       >> ${oprefix}.tex

  set p = 2; @ofs += 1;
  while ( ${p} <= ${len} )
   # set esr = ( `echo ${estimate[${ofs}]} ${stderr[${ofs}]} | awk '{printf "%12.5e %12.5e",$1,$2;if($1!=0.0)printf " %12.5e",$2/$1;else print " --"}'` );
   set esr = ( `echo ${estimate[${ofs}]} ${stderr[${ofs}]} | awk '{printf "%g %g",$1,$2;if($1!=0.0)printf " %g",$2/$1;else print " --"}'` );

   echo '    {'${pnames[${ofs}]}'} & '${esr[1]}' & '${esr[2]}' & '${esr[3]}'\\\\'          >> ${oprefix}.tex

   @ p   += 1
   @ ofs += 1
  end
 endif

# rest of the parameter blocks
#
 foreach b ( `awk '{print $1}' ${tmp}.bi | grep -v -e 'drift' -e 'noise'` )
  set len = ( `grep "^${b} " ${tmp}.bi | awk '{print $2" "$3}'` )
  @ ofs = 1 + ${len[1]}
  shift len

  if ( ${len} > 0 ) then
   echo '    {\\bf '${b}'} & & & \\\\' >> ${oprefix}.tex

   set p = 1
   while ( ${p} <= ${len} )
    # set esr = ( `echo ${estimate[${ofs}]} ${stderr[${ofs}]} | awk '{printf "%12.5e %12.5e",$1,$2;if($1!=0.0)printf " %12.5e",$2/$1;else print " --"}'` );
    set esr = ( `echo ${estimate[${ofs}]} ${stderr[${ofs}]} | awk '{printf "%g %g",$1,$2;if($1!=0.0)printf " %g",$2/$1;else print " --"}'` );

    if ( "${b}" == "${pnames[${ofs}]}" ) then
     echo '    {'${p}'} & '${esr[1]}' & '${esr[2]}' & '${esr[3]}'\\\\' >> ${oprefix}.tex
    else
     echo '    {'${pnames[${ofs}]}'} & '${esr[1]}' & '${esr[2]}' & '${esr[3]}'\\\\' >> ${oprefix}.tex
    endif

    @ p   += 1
    @ ofs += 1
   end
  endif
 end

 echo '   \\end{tabular}'                                                                  >> ${oprefix}.tex
 echo '  \\end{center}'                                                                    >> ${oprefix}.tex
 echo ' \\end{minipage}'                                                                   >> ${oprefix}.tex

# residuals plots
#
 echo ' &'                                                                                 >> ${oprefix}.tex
 echo ' \\begin{minipage}{3.75in}'                                                         >> ${oprefix}.tex
 echo '  \\centerline{\\epsfxsize 3.5in \\epsfysize 2.0in \\epsfbox{'${residplot}'}}'      >> ${oprefix}.tex
 echo '  \\centerline{\\epsfxsize 3.5in \\epsfysize 1.5in \\epsfbox{'${rhistplot}'}}'      >> ${oprefix}.tex
 echo '  \\centerline{\\epsfxsize 3.5in \\epsfysize 1.5in \\epsfbox{'${rqqplot}'}}'        >> ${oprefix}.tex
 echo '  \\centerline{\\epsfxsize 3.5in \\epsfysize 2.5in \\epsfbox{'${racfnplot}'}}'      >> ${oprefix}.tex
 echo '  \\centerline{\\epsfxsize 3.5in \\epsfysize 2.5in \\epsfbox{'${rsdplot}'}}'        >> ${oprefix}.tex
 echo ' \\end{minipage}'                                                                   >> ${oprefix}.tex
 echo '\\end{tabular}'                                                                     >> ${oprefix}.tex
 echo ''                                                                                   >> ${oprefix}.tex
 echo '\\end{document}'                                                                    >> ${oprefix}.tex

# process tex summary
#
# unfortunately, there doesn't seem to be any way of making
# latex exit on error with a nonzero termination status.
#
 if ( ${?process_summary} ) then
  latex ${oprefix}; dvips ${oprefix} -o ${oprefix}.ps;
 endif

# cleanup
#
 rm -f ${tmp}* ${oprefix}.{aux,log,dvi} >& /dev/null

 if ( ${?cleanup} ) then
  rm -f ${oprefix}.{S,S.out,tex,gpt} ${oprefix}.resid.txt                                   \
        ${fitplot} ${residplot} ${rhistplot} ${rqqplot} \
        ${racfnplot} ${rsdplot} ${vlocplot}
 endif

 exit ${errstatus}
