#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set indspecs = ''

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Plot time course for the specified voxel.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-g|-gnuplot gnuplot_settings_file]'
    echo ' -o|-outfile file[.ps|.eps]'
    echo ' -i|-infile infile[.mri] -c|-chunk chunk'
    echo ' [indspecs]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: gnuplot not available'
    echo ' 3: unspecified input file/chunk'
    echo ' 4: unspecified output file'
    echo ' 5-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.print.record.csh'
    echo ' mri_rpn_math gnuplot'
    echo ''
    echo 'Notes'
    echo ' 1 Refer to  extract.record.csh  for definition of  a time course.'
    echo ' 2 Refer  to  proc.indspecs.csh  for  index specification  format.'
    echo ' 3 gnuplot_settings_file  can  contain any set of gnuplot commands'
    echo '   except  terminal,  output,  key  and  xrange (overridden).  See'
    echo '   gnuplot documentation, or "help set" within gnuplot. Especially'
    echo '   useful are "set xtics" specifications for generating customized'
    echo '   tick marks on the axes.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-i":
   case "-infile":
    set infile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-c":
   case "-chunk":
    set chunk = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set psfile = ${argv[2]}; shift; shift; breaksw;

   case "-g":
   case "-gnuplot":
    set gptopts = ${argv[2]}; shift; shift; breaksw;

   default:
    set indspecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# check availability: gnuplot
#
 which gnuplot >& /dev/null

 if ( ${status} ) then
  errmsg.csh ${verbosity} ${myname} 2 "gnuplot not available" 1 ""
  exit ${status}
 endif

# error checks
#
 if ( !( ${?infile} && ${?chunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified input file/chunk" 1 ""
  exit ${status}
 else if ( !( ${?psfile} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified output file" 1 ""
  exit ${status}
 endif

# ps or eps?
#
 echo ${psfile} | grep -q '\.eps$'
 if ( ${status} == 0 ) then
  set eps    = eps
  set ext    = .eps
 else
  set eps    = ""
  set ext    = .ps
  set psfile = `echo ${psfile} | sed 's/\.ps$//'`${ext}
 endif

# extract and ascii-convert time course
#
 mri.print.record.csh.csh -verbosity ${verbosity} -timecourse -infile ${infile} -chunk ${chunk} -outfile ${tmp}.tc ${indspecs}

 errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri.print.record.csh.csh -verbosity ${verbosity} -timecourse -infile ${infile} -chunk ${chunk} -outfile ${tmp}.tc ${indspecs}"
 if ( ${status} ) exit ${status}

 @ tmax = `wc -l ${tmp}.tc | awk '{print $1}'`

# gnuplot script: user-specified portion
#
 if ( ${?gptopts} ) then
  if ( ( "${gptopts}" != "" ) && ( -f ${gptopts} ) ) then
   echo '# user-specified gnuplot settings'  > ${tmp}.gpt
   echo '#'                                 >> ${tmp}.gpt
   cat ${gptopts}                           >> ${tmp}.gpt
  endif
 endif

# gnuplot script: the rest
#
cat >> ${tmp}.gpt << @EOF

# portion of the gnuplot script generated by tc2ps.csh
#
set nokey
set pointsize 0.4
set xrange [0:${tmax}]

set terminal postscript ${eps}
set output "${psfile}"

plot "${tmp}.tc" w p 7
@EOF

# generate postscript
#
# ATTN: gnuplot cannot gracefully handle nans and infs in the data
#
 gnuplot ${tmp}.gpt >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "gnuplot ${tmp}.gpt >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# clean up
#
 rm -f ${tmp}* >& /dev/null; exit 0
