#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# misc
#
 set indspecs = ''

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Plot time course + fit for the specified voxel.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-g|-gnuplot gnuplot_settings_file]'
    echo ' -d|-datafile datafile[.mri] -D|-datachunk chunk'
    echo ' -f|-fitfile fitfile[.mri] -F|-fitchunk chunk'
    echo ' -o|-outfile file[.ps|.eps]'
    echo ' [indspecs]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: gnuplot not available'
    echo ' 3: unspecified data file/chunk'
    echo ' 4: unspecified fit file/chunk'
    echo ' 5: unspecified output file'
    echo ' 8: mismatching t-dimensions in data and fit'
    echo ' 6,7,9-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.print.record.csh'
    echo ' mri_rpn_math gnuplot'
    echo ''
    echo 'Notes'
    echo ' 1 Refer to extract.record.csh for definition of a time course.'
    echo ' 2 Refer  to  tc.to.ps.csh  for  more  on   gnuplot_settings_file.'
    echo ' 3 Refer to  proc.indspecs.csh  for index specification format.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-d":
   case "-datafile":
    set dfile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-D":
   case "-datachunk":
    set dchunk = ${argv[2]}; shift; shift; breaksw;

   case "-f":
   case "-fitfile":
    set ffile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-F":
   case "-fitchunk":
    set fchunk = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set psfile = ${argv[2]}; shift; shift; breaksw;

   case "-g":
   case "-gnuplot":
    set gptopts = ${argv[2]}; shift; shift; breaksw;

   default:
    set indspecs = ${argv[1]}; shift; breaksw;
  endsw
 end

# check availability: gnuplot
#
 which gnuplot >& /dev/null

 if ( ${status} ) then
  errmsg.csh ${verbosity} ${myname} 2 "gnuplot not available" 1 ""
  exit ${status}
 endif

# error checks
#
 if ( !( ${?dfile} && ${?dchunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified data file/chunk" 1 ""
  exit ${status}
 else if ( !( ${?ffile} && ${?fchunk} ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "unspecified fit file/chunk" 1 ""
  exit ${status}
 else if ( !( ${?psfile} ) ) then
  errmsg.csh ${verbosity} ${myname} 5 "unspecified output file" 1 ""
  exit ${status}
 endif

# ps or eps?
#
 echo ${psfile} | grep -q '\.eps$'
 if ( ${status} == 0 ) then
  set eps    = eps
  set ext    = .eps
 else
  set eps    = ""
  set ext    = .ps
  set psfile = `echo ${psfile} | sed 's/\.ps$//'`${ext}
 endif

# data: extract and ascii-convert time course
#
 mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${dfile} -chunk ${dchunk} -outfile ${tmp}.data ${indspecs}

 errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${dfile} -chunk ${dchunk} -outfile ${tmp}.data ${indspecs}"
 if ( ${status} ) exit ${status}

 @ tmax = `wc -l ${tmp}.data | awk '{print $1}'`

# fit: extract and ascii-convert time course
#
 mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${ffile} -chunk ${fchunk} -outfile ${tmp}.fit ${indspecs}

 errmsg.csh ${verbosity} ${myname} 7 "failed command" ${status} "mri.print.record.csh -verbosity ${verbosity} -timecourse -infile ${ffile} -chunk ${fchunk} -outfile ${tmp}.fit ${indspecs}"
 if ( ${status} ) exit ${status}

 @ tmaxf = `wc -l ${tmp}.fit | awk '{print $1}'`

# compatibility check
#
 if ( ${tmax} != ${tmaxf} ) then
  errmsg.csh ${verbosity} ${myname} 8 "mismatching t-dimensions; data: ${tmax}, fit: ${tmaxf}" 1 ""
  exit ${status}
 endif

# gnuplot script: user-specified portion
#
 if ( ${?gptopts} ) then
  if ( ( "${gptopts}" != "" ) && ( -f ${gptopts} ) ) then
   echo '# user-specified gnuplot settings'  > ${tmp}.gpt
   echo '#'                                 >> ${tmp}.gpt
   cat ${gptopts}                           >> ${tmp}.gpt
  endif
 endif

# gnuplot script: the rest
#
cat >> ${tmp}.gpt << @EOF

# portion of the gnuplot script generated by $0
#
set nokey
set pointsize 0.4
set xrange [0:${tmax}]

set terminal postscript ${eps}
set output "${psfile}"

plot "${tmp}.data" with points 7, "${tmp}.fit" with lines 1
@EOF

# generate postscript
#
 gnuplot ${tmp}.gpt >& ${tmp}.err

 errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "gnuplot ${tmp}.gpt >& ${tmp}.err"
 if ( ${status} ) exit ${status}

# clean up
#
 rm -f ${tmp}* >& /dev/null; exit 0
