#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# output prefix
#
 set prefix = ''
 set product = 'estimates'
 set paramname = 'all'

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Extract vmp parameter estimates or standard errors.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-P|-product vmp_output_product_name]'
    echo ' [-d|-dimension product_chunk_dimension]'
    echo ' [-p|-prefix prefix_for_extracted_param_files]'
    echo ' [-n|-name parameter_or_block_name]'
    echo ' -s|-submodel submodel_key'
    echo ' vmp_output[.mri]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified/invalid vmp output file'
    echo ' 3: unspecified/invalid submodel'
    echo ' 4: invalid vmp output product name'
    echo ' 5: invalid parameter name'
    echo ' 6-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' vmp.param.info.csh'
    echo ' mri_subset'
    echo ''
    echo 'Notes'
    echo " 0 Parameter name  is  either  a parameter block name  or  a  complete"
    echo "   parameter name as returned by"
    echo "    vmp.param.info -b all (vmp_output)"
    echo " 1 Unspecified parameter name amounts to extracting *all* paramters."
    echo " 2 Trailing spaces are significant in a parameter name. For example,"
    echo "   'drift,1 ' extracts that one parameter alone, while 'drift,1' will"
    echo "   extract parameters 'drift,1 ', 'drift,10', 'drift,11', etc."
    echo " 3 Valid vmp output product names: estimates standard.errors gradient."
    echo " 4 -d|-dimension specifies the chunk dimension over which to extract."
    echo "    If unspecified, this defaults to the leading nontrivial dimension."
    echo " 5 Produces one file per parameter;  file/parameter names  are printed"
    echo "   on stdout, along with global and within-block parameter offsets and"
    echo "   block name."
    echo " 6 Output file names are of the form"
    echo "    (output_prefix)(submodel).(param_block_name),(param_index).(chunk_name).mri"
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = ${argv[2]}; shift; shift; breaksw;

   case "-P":
   case "-product":
    set product = ${argv[2]}; shift; shift; breaksw;

   case "-n":
   case "-name":
    set paramname = "${argv[2]}"; shift; shift; breaksw;

   case "-d":
   case "-dimension":
    set dimension = "${argv[2]}"; shift; shift; breaksw;

   case "-s":
   case "-submodel":
    set submodel = ${argv[2]}; shift; shift; breaksw;

   default:
    set vmpout = `echo ${argv[1]} | sed 's/\.mri$//'`; shift; breaksw;
  endsw
 end

# error checks
#
 if ( !( ${?vmpout} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid vmp output file" 1 ""
  exit ${status}
 endif

 if ( !( -f ${vmpout}.mri ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid vmp output file" 1 ""
  exit ${status}
 endif

 if ( !( ${?submodel} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid submodel" 1 ""
  exit ${status}
 endif

 if ( "${submodel}" != "`vmp.list.submodels.csh -s ${submodel} ${vmpout}`" ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid submodel" 1 ""
  exit ${status}
 endif

 if ( !( ( "${product}" == "estimates" ) || ( "${product}" == "standard.errors" ) || ( "${product}" == "gradient" ) ) ) then
  errmsg.csh ${verbosity} ${myname} 4 "invalid vmp output product name" 1 ""
  exit ${status}
 endif

# full chunk name for vmp output product quantity
#
 set chunk = ${submodel}.${product}

# parameter/block names
#
 vmp.param.info.csh -verbosity ${verbosity} -outfile ${tmp}.bi -submodel ${submodel} ${vmpout}

 errmsg.csh ${verbosity} ${myname} 6 "failed command" ${status} "vmp.param.info.csh -verbosity ${verbosity} -outfile ${tmp}.bi -submodel ${submodel} ${vmpout}"
 if ( ${status} ) exit ${status}

 vmp.param.info.csh -verbosity ${verbosity} -outfile ${tmp}.pn -block all -submodel ${submodel} ${vmpout}

 errmsg.csh ${verbosity} ${myname} 7 "failed command" ${status} "vmp.param.info.csh -verbosity ${verbosity} -outfile ${tmp}.pn -block all -submodel ${submodel} ${vmpout}"
 if ( ${status} ) exit ${status}

# full name(s) and offset(s) of parameter(s) to extract
#
 if ( "${paramname}" == "all" ) then
  set pnames   = ( `awk '{print $1}' ${tmp}.pn` )
  set poffsets = ( `awk '{print $2}' ${tmp}.pn` )
 else
  set pnames   = ( `grep "^${paramname}" ${tmp}.pn | awk '{print $1}'` )
  set poffsets = ( `grep "^${paramname}" ${tmp}.pn | awk '{print $2}'` )

  if ( ( "${pnames}" == "" ) || ( "${poffsets}" == "" ) ) then
   errmsg.csh ${verbosity} ${myname} 5 "invalid parameter name" 1 ""
   exit ${status}
  endif
 endif

 # echo paramname \"${paramname}\"
 # echo pnames ${pnames}
 # echo poffsets ${poffsets}

# rename chunk to images
#
 mri.rename.chunk.csh -verbosity ${verbosity} -infile ${vmpout} -chunk ${chunk} -outfile ${tmp} -to images

 errmsg.csh ${verbosity} ${myname} 8 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${vmpout} -chunk ${chunk} -outfile ${tmp} -to images"
 if ( ${status} ) exit ${status}

# leading nontrivial dimension
#
 if ( !( ${?dimension} ) ) then
  set dimension = `mri.chunk.sizes.csh -infile ${vmpout} -chunk ${chunk} | grep '^leading_nontrivial_dimension ' | awk '{print $2}'`

  errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri.chunk.sizes.csh -infile ${vmpout} -chunk ${chunk}"
  if ( ${status} ) exit ${status}
 endif

# loop over parameters to extract
#
 set p = 1
 foreach pname ( ${pnames} )
  set pofs  = ${poffsets[${p}]}
  set block = `echo ${pname} | awk -F, '{print $1}'`
  set bofs  = `grep ${block} ${tmp}.bi | awk '{print $2}'`
  @   pidx  = ${pofs} - ${bofs}

  set ofile = ${prefix}${submodel}.${block},${pidx}.${chunk}

  # echo pname \"$pname\" pofs \"$pofs\" block \"$block\" bofs \"$bofs\" pidx \"$pidx\" ofile \"$ofile\"

  # extract parameter
  #
  mri_subset -d ${dimension} -s ${pofs} -l 1 ${tmp} ${tmp}.p >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 10 "failed command" ${status} "mri_subset -d ${dimension} -s ${pofs} -l 1 ${tmp} ${tmp}.p >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  # rename "images" back to ${chunk} if necessary
  #
  # mri.rename.chunk.csh -verbosity ${verbosity} -infile ${tmp}.p -chunk images -outfile ${ofile} -to ${chunk} -copydata

  # errmsg.csh ${verbosity} ${myname} 9 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${tmp}.p -chunk images -outfile ${ofile} -to ${chunk} -copydata"
  # if ( ${status} ) exit ${status}

  mv ${tmp}.p.dat ${ofile}.dat

  if ( "${chunk}" == "images" ) then
   grep -v -e 'history\.' ${tmp}.p.mri > ${ofile}.mri
  else
   grep -v -e 'history\.' ${tmp}.p.mri | sed 's/^images/'${chunk}'/g' > ${ofile}.mri
  endif

  echo "${chunk}.description = "'"'"parameter ${pname} from ${vmpout} offset ${pofs} offset_within_block ${pidx}"'"' >> ${ofile}.mri

  echo ${ofile} ${pname} ${pofs} ${pidx} ${block}

  @ p += 1
 end

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
