#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
# mkdir -p /tmp/${user}
# set tmpdir = /tmp/${user}/
# set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' List all the submodel keys in a vmp output file.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' Form 1: [-d|-description] vmp_output_file[.mri]'
    echo ' Form 2: [-s|-submodel submodel_key] vmp_output_file[.mri]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: invalid/unspecified vmp output file'
    echo ' 3: nonexistent submodel (for Form 2)'
    echo ' 4-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 Form 1 lists all submodels, with optional description.'
    echo ' 2 Form 2 checks if the submodel exists in the vmp output'
    echo '   file. If present, it is echoed back.'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-d":
   case "-description":
    set description; shift; breaksw;

   case "-s":
   case "-submodel":
    set submodel = ${argv[2]}; shift; shift; breaksw;

   default:
    set vmpout = `echo ${argv[1]} | sed 's/\.mri$//'`; shift; breaksw;
  endsw
 end

# error checks
#
 if ( !( ${?vmpout} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "invalid/unspecified vmp output file" 1 ""
  exit ${status}
 endif

 if ( !( -f ${vmpout}.mri ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "invalid/unspecified vmp output file" 1 ""
  exit ${status}
 endif

# submodels list
#
 set smlist = "`mri_printfield -input ${vmpout} -field submodels.list`"

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_printfield -input ${vmpout} -field submodels.list"
 if ( ${status} ) exit ${status}

# check if the specified submodel exists in the list
#
 if ( ${?submodel} ) then
  set smlist = `echo ${smlist} | sed 's/ /\\n/g'`
  set smlist = `echo ${smlist} | grep '^'${submodel}'$'`
  if ( "${smlist}" == "${submodel}" ) then
   if ( ${?description} ) then
    echo ${submodel}':' `mri_printfield -input ${vmpout} -field ${submodel}.submodel.description`
   else
    echo ${submodel}
   endif
   exit 0
  endif
  exit 3
 endif

# list submodels
#
 if ( ${?description} ) then
  foreach s ( ${smlist} )
   set d = "`mri_printfield -input ${vmpout} -field ${s}.submodel.description`"

   errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_printfield -input ${vmpout} -field ${s}.submodel.description"
   if ( ${status} ) exit ${status}

   echo ${s}':' ${d}
  end
 else
  echo $smlist
 endif

# cleanup
#
# rm -f ${tmp}* >& /dev/null; exit 0
