#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Generate  gnuplot  tic  settings for timecourse'
    echo ' or data/fit plots using vmp input specification'
    echo ' or output (.mri) file.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-o|-outfile gpt_settings_file]'
    echo ' [-s|-submodel submodel]'
    echo ' vmp_input_or_output_file'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified/invalid vmp input/output file'
    echo ' 3-: failed script/utility'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 1 This script looks for  the key  "design.conditions",  and  for  each'
    echo '   condition in "conditions", the keys "design.epoch.begin.(condition)"'
    echo '   and "design.epoch.length.(condition)". Also: "acquisition.iai".'
    echo ' 2 tic settings generated are in units of iai.'
    echo ' 3 Unspecified submodel defaults to 11111...'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-s":
   case "-submodel":
    set submodel = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set outfile = ${argv[2]}; shift; shift; breaksw;

   default:
    set vmpout = `echo ${argv[1]} | sed 's/\.mri$//'`; shift; breaksw;
  endsw
 end

# error checks
#
 if ( !( ${?vmpout} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid vmp input/output file" 1 ""
  exit ${status}
 endif

 if ( !( -f ${vmpout}.mri ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid vmp input/output file" 1 ""
  exit ${status}
 endif

# experimental conditions and condition tags
#
 set conditions = ( `mri_printfield -input ${vmpout} -field design.conditions` )

 errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "mri_printfield -input ${vmpout} -field design.conditions"
 if ( ${status} ) exit ${status}

# iai
#
 set iai = `mri_printfield -input ${vmpout} -field acquisition.iai`
 if ( ${status} != 0 ) set iai = 1.0

# submodel
#
 if ( !( ${?submodel} ) ) then
  set submodel = ( `echo ${#conditions} | awk '{for ( i = 1; i <= $0; i++ ) print 1}'` )
 endif

 set submodel = ( `echo ${submodel} | sed -e 's/0/0 /g' -e 's/1/1 /g'` )

 if ( ${#submodel} != ${#conditions} ) then
  set submodel = ( `echo ${#conditions} | awk '{for ( i = 1; i <= $0; i++ ) print 1}'` )
 endif

# generate settings for gnuplot (assume initial t to be 0)
#
 echo 'set grid x'       > ${tmp}.gpt
 echo -n 'set xtics ( ' >> ${tmp}.gpt

# extract epoch beginning times and lengths conditionwise
#
 set msg  = "# submodel: ${submodel}\n# conditions:"
 set tmax = 0

 set i = 1
 while ( ${i} <= ${#conditions} )
  if ( ${submodel[${i}]} == 1 ) then
   set msg = "${msg} ${conditions[${i}]}"

   set epochs = ( `mri_printfield -input ${vmpout} -field design.epoch.begin.${conditions[${i}]}` )

   errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_printfield -input ${vmpout} -field design.epoch.begin.${conditions[${i}]}"
   if ( ${status} ) exit ${status}

   set tics = `echo ${epochs} | awk '{for(i=1;i<=NF;i++)printf "\"\" %d, ", $i/'${iai}'}'`

   if ( ${i} > 1 ) echo -n '            ' >> ${tmp}.gpt
   echo "${tics} \\" >> ${tmp}.gpt

   set lengths = ( `mri_printfield -input ${vmpout} -field design.epoch.length.${conditions[${i}]}` )

   errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri_printfield -input ${vmpout} -field design.epoch.length.${conditions[${i}]}"
   if ( ${status} ) exit ${status}

   set tmax = `echo ${tmax} ${lengths} | awk '{for(i=1;i<=NF;i++)s+=$i;print s}'`
  endif

  @ i += 1
 end

 set tmax = `echo ${tmax} | awk '{print $1/'${iai}'}'`

 echo '            "0" 0, "'${tmax}'" '${tmax}' )' >> ${tmp}.gpt
 echo "${msg}" >> ${tmp}.gpt

# screen dump if outfile not specified
#
 if ( ${?outfile} ) then
  mv ${tmp}.gpt ${outfile}
 else
  cat ${tmp}.gpt
  rm -f ${tmp}* >& /dev/null; exit 0
 endif
