#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = 0

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${myname}
    echo ' Print vmp parameter block information.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-b|-block vmp_parameter_block_name]'
    echo ' [-o|-outfile outfile]'
    echo ' -s|-submodel submodel_key'
    echo ' vmp_output[.mri]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2: unspecified/invalid vmp output file'
    echo ' 3: unspecified/invalid submodel'
    echo ' 4-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri_printfield'
    echo ''
    echo 'Notes'
    echo ' 0 -block takes one of the vmp block names ("drift",'
    echo '   "noise", "response", "shape", "ar", or "all").'
    echo ' 1 This scripts accesses the keys'
    echo '    (submodel_kay).paramblock.description'
    echo '   and'
    echo '    (submodel_key).paramblock.description.{blockname}'
    echo '   where blockname is any parameter block name from'
    echo '   paramblock.description, or "all".'
    echo ' 2 Although this  parameter  description  format is'
    echo '   recursive in principle, this script assumes that'
    echo '   there are only two levels of these descriptions.'
    echo ' 3 If  -block  option is unspecified, this produces'
    echo '   the top-level description of the parameter block'
    echo '   in the name--offset--length format. If -block is'
    echo '   specified,  this produces a  detailed listing in'
    echo '   the name--offset format.  Either list is  sorted'
    echo '   in increasing order of the offset.'
    echo ' 4 Does not check consistency of block descriptions'
#   echo ''
#   echo 'Defaults'
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-s":
   case "-submodel":
    set submodel = ${argv[2]}; shift; shift; breaksw;

   case "-o":
   case "-outfile":
    set outfile = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-b":
   case "-block":
    set block = ${argv[2]}; shift; shift; breaksw;

   default:
    set vmpout = `echo ${argv[1]} | sed 's/\.mri$//'`; shift; breaksw;
  endsw
 end

# error checks
#
 if ( !( ${?vmpout} ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid vmp output file" 1 ""
  exit ${status}
 endif

 if ( !( -f ${vmpout}.mri ) ) then
  errmsg.csh ${verbosity} ${myname} 2 "unspecified/invalid vmp output file" 1 ""
  exit ${status}
 endif

 if ( !( ${?submodel} ) ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid submodel" 1 ""
  exit ${status}
 endif

 if ( "${submodel}" != "`vmp.list.submodels.csh -s ${submodel} ${vmpout}`" ) then
  errmsg.csh ${verbosity} ${myname} 3 "unspecified/invalid submodel" 1 ""
  exit ${status}
 endif

# reset temporary file
#
 echo -n '' > ${tmp}

# block description
#
 set blockdscr = "`mri_printfield -input ${vmpout} -field ${submodel}.paramblock.description`"

 errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri_printfield -input ${vmpout} -field ${submodel}.paramblock.description"
 if ( ${status} ) exit ${status}

 if ( !( ${?block} ) ) then
  set blocks = ( ${blockdscr} )
  foreach b ( ${blocks} )
   set info = ( `echo ${b} | awk -F, '{print $1" "$2" "$3}'` )
   echo ${info[1]} ${info[2]} ${info[3]} >> ${tmp}
  end
 else
  set blocknames = ()
  if ( "${block}" == "all" ) then
   set blocks = ( ${blockdscr} )
   foreach b ( ${blocks} )
    set blocknames = ( ${blocknames} `echo ${b} | awk -F, '{print $1}'` )
   end
  else
   set blocknames = ( ${block} )
  endif

  set blockdscr2 = "`echo ${blockdscr} | sed 's/ /\\n/g'`"

# remove nonexistent blocks from the list
#
  foreach block ( ${blocknames} )
   echo ${blockdscr} | grep -q "${block},"
   if ( ${status} ) shift blocknames
  end

  if ( ${#blocknames} == 0 ) exit 0

# process block list
#
  foreach block ( ${blocknames} )
#  echo ${blockdscr} | grep -q "${block},"

#  errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "echo ${blockdscr} | grep -q '${block},'"
#  if ( ${status} ) exit ${status}

   set blocklength = ( `echo ${blockdscr2} | grep "${block}," | awk -F, '{print $1" "$2" "$3}'` )
   set blockname   = ${blocklength[1]}; shift blocklength;
   set blockoffset = ${blocklength[1]}; shift blocklength;

  # print individual parameter names and offsets in the full parameter block
  #
   set paramlst = ( `mri_printfield -input ${vmpout} -field ${submodel}.paramblock.description.${block}` )

   if ( ${status} == 0 ) then
    foreach p ( ${paramlst} )
     @   offset = ${blockoffset} + `echo ${p} | awk -F, '{print $2}'`
     set name   = `echo ${p} | awk -F, '{print $1}'`
     set length = `echo ${p} | awk -F, '{print $3}'`
     if ( ${length} == 1 ) then
      echo ${block},${name} ${offset} >> ${tmp}
     else
      set p = 1
      while ( ${p} <= ${length} )
       @ ofs  = ${offset} + ${p}
       @ ofs -= 1

       echo ${block},${name},${p} ${ofs} >> ${tmp}

       @ p += 1
      end
     endif
    end
   else
    if ( ${blocklength} == 1 ) then
     echo ${block} ${blockoffset} >> ${tmp}
    else
     set p = 1
     while ( ${p} <= ${blocklength} )
      @ ofs  = ${blockoffset} + ${p}
      @ ofs -= 1

      echo ${block},${p} ${ofs} >> ${tmp}

      @ p += 1
     end
    endif
   endif
  end
 endif

# dump sorted list onto screen/move to outfile
#
 if ( ${?outfile} ) then
  mv -f ${tmp} ${outfile}
 else
  cat ${tmp}
 endif

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
