/* ----------------------------------------------------------------------------
 *
 * class TChandler
 *   implementation
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <timecourse.handlers/TChandler>

// ------------------------------------------------------------- construct/reinit
// ------------------------------------------------------------------------------

TChandler::TChandler( )
 : ArrayNavigator( ), my_tdim( ), my_names( ), my_dimsSansT( ),
   my_isLeadDimT( ), my_leadDimNT( )
 {
 }

TChandler::~TChandler( )
 {
 }

TChandler::TChandler( const TChandler& another )
 : ArrayNavigator( another ),
   my_tdim( another.my_tdim ),
   my_names( another.my_names ),
   my_dimsSansT( another.my_dimsSansT ),
   my_isLeadDimT( another.my_isLeadDimT ),
   my_leadDimNT( another.my_leadDimNT )
 {
 }

TChandler&
TChandler::operator=( const TChandler& rhs )
 {
  if ( this == &rhs ) return *this;

  ArrayNavigator::operator=( rhs );

  my_names.resize( rhs.my_names.size( ) );
  my_dimsSansT.resize( rhs.my_dimsSansT.size( ) );

  my_tdim       = rhs.my_tdim;
  my_names      = rhs.my_names;
  my_dimsSansT  = rhs.my_dimsSansT;
  my_isLeadDimT = rhs.my_isLeadDimT;
  my_leadDimNT  = rhs.my_leadDimNT;

  return *this;
 }

TChandler::TChandler( const uintVector&     sizes,
                            size_t          tDimension,
                      const uintVector&     indexOrder,
                      const LocationType&   lowerBounds,
                      const vector<string>& names,
                            bool            deleteTrivial )
 : ArrayNavigator( ), my_tdim( ), my_names( ), my_dimsSansT( ),
   my_isLeadDimT( ), my_leadDimNT( )
 {
  reinitialize( sizes, tDimension, indexOrder, lowerBounds, names, deleteTrivial );
 }

void
TChandler::reinitialize( const uintVector&     sizes,
                               size_t          tDimension,
                         const uintVector&     indexOrder,
                         const LocationType&   lowerBounds,
                         const vector<string>& names,
                               bool            deleteTrivial )
 {
  // cerr << "(debug) TChandler::reinitialize( ) tDimension " << tDimension << " deleteTrivial " << deleteTrivial << endl;

  uintVector     sz( sizes );
  LocationType   lw( lowerBounds );
  vector<string> nm( names );
  uintVector     io( indexOrder );

  if ( tDimension >= sizes.size( ) ) tDimension = 0;

  if ( deleteTrivial )
   {
    size_t n = 1;
    for ( size_t d = 0; d < tDimension; d++ ) n += ( sizes[d] > 1 );
    for ( size_t d = tDimension + 1; d < sizes.size( ); d++ ) n += ( sizes[d] > 1 );

    sz.resize( n );
    if ( nm.size( ) > 0 ) nm.resize( n );
    if ( lw.size( ) > 0 ) lw.resize( n );
    if ( io.size( ) > 0 ) io.resize( n );

    size_t i = 0;
    for ( size_t d = 0; d < tDimension; d++ )
     if ( sizes[d] > 1 )
      {
       sz[i] = sizes[d];
       if ( nm.size( ) > 0 ) nm[i] = names[d];
       if ( lw.size( ) > 0 ) lw[i] = lowerBounds[d];
       if ( io.size( ) > 0 ) io[i] = indexOrder[d];
       i++;
      }

    size_t tp = i;

    sz[i] = sizes[tDimension];
    if ( nm.size( ) > 0 ) nm[i] = names[tDimension]; 
    if ( lw.size( ) > 0 ) lw[i] = lowerBounds[tDimension]; 
    if ( io.size( ) > 0 ) io[i] = indexOrder[tDimension]; 
    i++;

    for ( size_t d = tDimension + 1; d < sizes.size( ); d++ )
     if ( sizes[d] > 1 )
      {
       sz[i] = sizes[d];
       if ( nm.size( ) > 0 ) nm[i] = names[d];
       if ( lw.size( ) > 0 ) lw[i] = lowerBounds[d];
       if ( io.size( ) > 0 ) io[i] = indexOrder[d];
       i++;
      }

    tDimension = tp;
   }

  /*
  cerr << "(debug) TChandler::reinitialize( ) sz";
  for ( size_t i = 0; i < sz.size( ); i++ )
   cerr << " " << sz[i];
  cerr << endl;

  cerr << "(debug) TChandler::reinitialize( ) nm";
  for ( size_t i = 0; i < nm.size( ); i++ )
   cerr << " " << nm[i];
  cerr << endl;

  cerr << "(debug) TChandler::reinitialize( ) lw";
  for ( size_t i = 0; i < lw.size( ); i++ )
   cerr << " " << lw[i];
  cerr << endl;

  cerr << "(debug) TChandler::reinitialize( ) io";
  for ( size_t i = 0; i < io.size( ); i++ )
   cerr << " " << io[i];
  cerr << endl;

  cerr << "(debug) TChandler::reinitialize( ) tDimension " << tDimension << endl;
  */

  ArrayNavigator::reinitialize( sz, io, lw );

  my_tdim = ( tDimension < numDims( ) ) ? tDimension : 0;

  // cerr << "(debug) TChandler::reinitialize( ) my_tdim " << my_tdim << " ranges " << rangeLower( )[my_tdim] << " " << rangeUpper( )[my_tdim] << endl;

  my_names.resize( numDims( ) );
  my_names[my_tdim] = "t"; setNames( nm );

  /*
  for ( size_t i = 0; i < sz.size( ); i++ )
   cerr << "(debug) TChandler::reinitialize( ) name " << my_names[i] << " size " << ArrayNavigator::sizes( )[i] << " order " << order( )[i] << " lower " << indexLower( )[i] << endl;
  */

  my_dimsSansT.resize( numDims( ) - 1 );

  for ( size_t d = 0, i = 0; d < numDims( ); d++ )
   if ( d != my_tdim ) { my_dimsSansT[i] = d; i++; }

  /*
  cerr << "(debug) TChandler::reinitialize( ) my_dimsSansT";
  for ( size_t i = 0; i < my_dimsSansT.size( ); i++ )
   cerr << " " << my_dimsSansT[i]
        << "," << my_names[my_dimsSansT[i]]
        << "," << ArrayNavigator::sizes( )[my_dimsSansT[i]];
  cerr << endl;
  */

  my_isLeadDimT = true;
  for ( size_t d = 0; d < my_tdim; d++ )
   my_isLeadDimT = my_isLeadDimT && ( ArrayNavigator::sizes( )[d] == 1 );

  // cerr << "(debug) TChandler::reinitialize( ) my_isLeadDimT " << my_isLeadDimT << endl;

  size_t d = 0;
  for ( ; ( d < my_dimsSansT.size( ) ) && ( ArrayNavigator::sizes( )[my_dimsSansT[d]] == 1 ); d++ );
  my_leadDimNT = ( d < my_dimsSansT.size( ) - 1 ) ? my_dimsSansT[d] : numDims( );

  // cerr << "(debug) TChandler::reinitialize( ) d " << d << " my_leadDimNT " << my_leadDimNT << endl;
 }

void
TChandler::setNames( const vector<string>& names, bool excludeT )
 {
  if ( my_names.size( ) != numDims( ) )
   my_names.resize( numDims( ) );

  size_t ns = min( names.size( ), my_names.size( ) );

  for ( size_t d = 0; d < ns; d++ )
   setName( d, names[d], excludeT );
  for ( size_t d = ns; d < my_names.size( ); d++ )
   setName( d, string( ), excludeT );
 }

void
TChandler::setName( size_t d, const string& name, bool excludeT )
 {
  if (    ( my_names.size( ) == 0              )
       || ( d                >= numDims( )     )
       || ( excludeT         && ( d == my_tdim ) ) ) return;

  bool found = false;
  for ( size_t i = 0; ( i < d ) && ( !found ); i++ ) found = ( my_names[i] == name );

  if ( !( found || ( name.size( ) == 0 ) ) ) { my_names[d] = name; return; }

  if ( d == my_tdim ) { my_names[d] = "t"; return; }

  size_t n = MiscFns::nDigits( d );
  char nm[n+1];
  sprintf( nm, "d%d", d );

  my_names[d] = string( nm );
 }
