/* ----------------------------------------------------------------------------
 *
 * class TCreader
 *   base class for time-course data readers.
 *   see TChandler.h for more details.
 *
 * required headers
 *   see include header file TCreader
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __TCreader_H__
#define __TCreader_H__

cpp_using_namespace( std );
cpp_using_namespace( BasicTypes );

class TCreader : public TChandler
 {
  public:

   virtual
   ~TCreader( ) = 0;

  // ---------------------------------- retrieve time course at current location
  // ---------------------------------------------------------------------------

     // virtual functions  are  unavoidable in this design.  Short of an elegant
     // solution, all type conversions to the requested type are handled through
     // variants of retrieve( ) for five basic numerical types  (the most common
     // case, double, is incorporated in retrieve() and timeCourse()). Templates
     // would have been ideal for this purpose  (virtual templates are illegal).
     // All retrieve( ) variants  retrieve time course at  the  current-location
     // into timeCourse, invoke seek(), report success, assuming that timeCourse
     // is properly sized.

   inline
   const varray<double>&
   timeCourse( ) const;

   virtual
   bool
   retrieve( varray<double>& timeCourse ) const = 0;

   virtual
   bool
   retrieveFloat( varray<float>& timeCourse ) const = 0;

   virtual
   bool
   retrieveInt( varray<int>& timeCourse ) const = 0;

   virtual
   bool
   retrieveShort( varray<short>& timeCourse ) const = 0;

   virtual
   bool
   retrieveUchar( varray<unsigned char>& timeCourse ) const = 0;

  protected:

  // ---------------------------------------------------------- construct/reinit
  // ---------------------------------------------------------------------------

   TCreader( );

   TCreader( const TCreader& another );
   TCreader( const TChandler& another );
   TCreader( const TCrw& another );

   TCreader& operator=( const TCreader& rhs );
   TCreader& operator=( const TChandler& rhs );
   TCreader& operator=( const TCrw& rhs );

   TCreader( const uintVector&     sizes,
                   size_t          tDimension    = 0,
             const uintVector&     indexOrder    = LocationType( ),
             const LocationType&   lowerBounds   = LocationType( ),
             const vector<string>& names         = vector<string>( ),
                   bool            deleteTrivial = true );

   void
   reinitialize( const uintVector&     sizes,
                       size_t          tDimension    = 0,
                 const uintVector&     indexOrder    = LocationType( ),
                 const LocationType&   lowerBounds   = LocationType( ),
                 const vector<string>& names         = vector<string>( ),
                       bool            deleteTrivial = true );

  private:

   mutable varray<double> my_timeCourse;
 };

// ------------------------------------ retrieve time course at current location
// -----------------------------------------------------------------------------

const varray<double>&
TCreader::timeCourse( ) const
 {
  if ( anyMove( ) )
   {
    size_t t = tdim( );
    size_t l = rangeLower( )[t];

    moveTo( t, l );
    seek( );

    my_timeCourse.resize( rangeUpper( )[t] - l + 1 );
    retrieve( my_timeCourse );
   }

  return my_timeCourse;
 }

#endif /* __TCreader_H__ */
