/* ----------------------------------------------------------------------------
 *
 * class TCrwPghMRI
 *   implementation
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <timecourse.handlers/TCrwPghMRI>

TCrwPghMRI::TCrwPghMRI( )
 : TCrw( ),
   my_chunkName( ),
   my_tskip( ),
   my_mriHandler( )
 {
 }

TCrwPghMRI::TCrwPghMRI( const TCrwPghMRI& another )
 : TCrw( another ),
   my_chunkName( another.my_chunkName ),
   my_tskip( another.my_tskip ),
   my_mriHandler( another.my_mriHandler )
 {
 }

TCrwPghMRI&
TCrwPghMRI::operator=( const TCrwPghMRI& rhs )
 {
  if ( this != &rhs )
   {
    TCrw::operator=( rhs );

    my_chunkName  = rhs.my_chunkName;
    my_tskip      = rhs.my_tskip;
    my_mriHandler = rhs.my_mriHandler;
   }

  return *this;
 }

TCrwPghMRI::TCrwPghMRI( const string&                     fileName,
                        const string&                     chunkName,
                              char                        tDimName,
                              bool                        deleteTrivial,
                              PghMRIhandler::mriErrorMode errorMode )
 : TCrw( ), my_chunkName( ), my_tskip( ), my_mriHandler( )
 {
  readOnly( fileName, chunkName, tDimName, deleteTrivial, errorMode );
 }

TCrwPghMRI::TCrwPghMRI( const string&                     fileName,
                              char                        tDimName,
                        const string&                     chunkName,
                              bool                        deleteTrivial,
                              PghMRIhandler::mriErrorMode errorMode )
 : TCrw( ), my_chunkName( ), my_tskip( ), my_mriHandler( )
 {
  modifyData( fileName, tDimName, chunkName, deleteTrivial, errorMode );
 }

TCrwPghMRI::TCrwPghMRI( const string&                     fileName,
                        const string&                     dimensions,
                        const uintVector&                 sizes,
                              char                        tDimName,
                        const string&                     chunkName,
                        const LocationType&               chunkLower,
                              PghMRIhandler::mriArrayType chunkType,
                        const string&                     chunkFile,
                              bool                        littleEndian,
                              PghMRIhandler::mriErrorMode errorMode )
 : TCrw( ), my_chunkName( ), my_tskip( ), my_mriHandler( )
 {
  write( fileName, dimensions, sizes, tDimName,
         chunkName, chunkLower, chunkType, chunkFile, littleEndian, errorMode );
 }

TCrwPghMRI::TCrwPghMRI( const string&                     fileName,
                        const TChandler*                  tch,
                        const string&                     chunkName,
                              PghMRIhandler::mriArrayType chunkType,
                        const string&                     chunkFile,
                              bool                        littleEndian,
                              PghMRIhandler::mriErrorMode errorMode )
 : TCrw( ), my_chunkName( ), my_tskip( ), my_mriHandler( )
 {
  string dimensions = tch->nameStr( "" );
  write( fileName, dimensions, tch->sizes( ), tch->names( )[tch->tdim( )][0],
         chunkName, tch->indexLower( ), chunkType, chunkFile, littleEndian, errorMode );
 }

void
TCrwPghMRI::readOnly( const string&                     fileName,
                      const string&                     chunkName,
                            char                        tDimName,
                            bool                        deleteTrivial,
                            PghMRIhandler::mriErrorMode errorMode )
 {
  my_chunkName = chunkName;
  my_mriHandler.open( fileName, PghMRIhandler::mriRead, errorMode );
                                           //
                                           // automatically closes any open file

  assert( my_mriHandler.hasKey( chunkName ) );
  assert( my_mriHandler.getString( chunkName ) == "[chunk]" );

  openRM( tDimName, deleteTrivial );
 }

void
TCrwPghMRI::modifyData( const string&                     fileName,
                              char                        tDimName,
                        const string&                     chunkName,
                              bool                        deleteTrivial,
                              PghMRIhandler::mriErrorMode errorMode )
 {
  my_chunkName = chunkName;
  my_mriHandler.open( fileName, PghMRIhandler::mriModifyData, errorMode );
                                           //
                                           // automatically closes any open file

  assert( my_mriHandler.hasKey( chunkName ) );
  assert( my_mriHandler.getString( chunkName ) == "[chunk]" );

  openRM( tDimName, deleteTrivial );
 }

void
TCrwPghMRI::write( const string&                     fileName,
                   const string&                     dimensions,
                   const uintVector&                 sizes,
                         char                        tDimName,
                   const string&                     chunkName,
                   const LocationType&               chunkLower,
                         PghMRIhandler::mriArrayType chunkType,
                   const string&                     chunkFile,
                         bool                        littleEndian,
                         PghMRIhandler::mriErrorMode errorMode )
 {
  my_chunkName = chunkName;
  my_mriHandler.open( fileName, PghMRIhandler::mriWrite, errorMode );
                                           //
                                           // automatically closes any open file

  assert( dimensions.size( ) == sizes.size( ) );

  vector<string> names( dimensions.size( ) );

  bool noneRepeated = true, nonzeroSizes = true;
  for ( size_t i = 0; i < sizes.size( ); i++ )
   {
    names[i] = dimensions[i];

    for ( size_t j = 0; j < i; j++ )
     noneRepeated = noneRepeated && ( dimensions[i] != dimensions[j] );

    nonzeroSizes = nonzeroSizes && ( sizes[i] > 0 );
   }

  assert( noneRepeated && nonzeroSizes );

  string::size_type tpos = dimensions.find( tDimName );

  TCrw::reinitialize( sizes, tpos, uintVector( ), chunkLower, names, false );

  my_tskip = 1;
  for ( size_t d = 0; d < tdim( ); d++ ) my_tskip *= TCrw::sizes( )[d];

  varray<int> sz( numDims( ) ), lw( numDims( ) );
  for ( size_t i = 0; i < numDims( ); i++ )
   {
    sz[i] = TCrw::sizes( )[i];
    lw[i] = TCrw::indexLower( )[i];
   }

  my_mriHandler.createChunk( chunkName, dimensions,
                             PghMRIhandler::type2name( chunkType, "float64" ),
                             chunkFile, sz( 0 ), lw( 0 ), littleEndian );
 }

void
TCrwPghMRI::write( const string&                     fileName,
                   const TChandler*                  tch,
                   const string&                     chunkName,
                         PghMRIhandler::mriArrayType chunkType,
                   const string&                     chunkFile,
                         bool                        littleEndian,
                         PghMRIhandler::mriErrorMode errorMode )
 {
  string dimensions = tch->nameStr( "" );
  write( fileName, dimensions, tch->sizes( ), tch->names( )[tch->tdim( )][0],
         chunkName, tch->indexLower( ), chunkType, chunkFile, littleEndian, errorMode );
 }

void
TCrwPghMRI::openRM( char tDimName, bool deleteTrivial )
 {
  // cerr << "(debug) begin TCrwPghMRI::openRM( )" << endl;

  string dimensions = my_mriHandler.getString( my_chunkName + ".dimensions" );

  /*
  for ( size_t i = 0; i < dimensions.size( ); i++ )
   {
    cerr << "(debug) " << dimensions[i]
         << " size "
         << my_mriHandler.getInt( my_chunkName + ".extent." + dimensions[i] );
    if ( my_mriHandler.hasKey( my_chunkName + ".min." + dimensions[i] ) )
     cerr << " lower "
          << my_mriHandler.getInt( my_chunkName + ".min." + dimensions[i] );
    cerr << endl;
   }
  */

  string::size_type tpos = dimensions.find( tDimName );

  // cerr << "(debug) tpos " << tpos << endl;

  assert( tpos != string::npos );

  uintVector     sizes( dimensions.size( ) );
  vector<string> names( dimensions.size( ) );
  LocationType   lower( dimensions.size( ) );

  int l;
  string keyE = my_chunkName + ".extent.?";
  string keyM = my_chunkName + ".min.?";

  for ( size_t d = 0; d < sizes.size( ); d++ )
   {
    keyE[keyE.size( ) - 1] = dimensions[d];
    keyM[keyM.size( ) - 1] = dimensions[d];

    names[d] = dimensions[d];
    sizes[d] = my_mriHandler.getInt( keyE );

    l = 0;
    if ( my_mriHandler.hasKey( keyM ) ) l = my_mriHandler.getInt( keyM );

    lower[d] = ( l < 0 ) ? 0 : l;
   }

  // for ( size_t i = 0; i < sizes.size( ); i++ )
  //  cerr << "(debug) name " << names[i] << " size " << sizes[i] << " lower " << lower[i] << endl;

  TCrw::reinitialize( sizes, tpos, uintVector( ), lower, names, deleteTrivial );

  my_tskip = 1;
  for ( size_t d = 0; d < tdim( ); d++ ) my_tskip *= TCrw::sizes( )[d];

  // cerr << "(debug) my_tskip " << my_tskip << endl;

  // cerr << "(debug) end TCrwPghMRI::openRM( )" << endl;
 }
