/* ----------------------------------------------------------------------------
 *
 * class TCrwPghMRI_W
 *   implementation
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#include <timecourse.handlers/TCrwPghMRI_W>

PghMRIhandler TCrwPghMRI_W::my_mri;

TCrwPghMRI_W::TCrwPghMRI_W( )
 : TCrw( ),
   my_chunkName( ),
   my_tskip( ),
   my_mriHandler( )
 {
 }

TCrwPghMRI_W::TCrwPghMRI_W( PghMRIhandler* mriHandler )
 : TCrw( ),
   my_chunkName( ),
   my_tskip( ),
   my_mriHandler( mriHandler )
 {
 }

TCrwPghMRI_W::TCrwPghMRI_W( const TCrwPghMRI_W& another )
 : TCrw( another ),
   my_chunkName( another.my_chunkName ),
   my_tskip( another.my_tskip ),
   my_mriHandler( another.my_mriHandler )
 {
 }

TCrwPghMRI_W&
TCrwPghMRI_W::operator=( const TCrwPghMRI_W& rhs )
 {
  if ( this != &rhs )
   {
    TCrw::operator=( rhs );

    my_chunkName  = rhs.my_chunkName;
    my_tskip      = rhs.my_tskip;
    my_mriHandler = rhs.my_mriHandler;
   }

  return *this;
 }

TCrwPghMRI_W::TCrwPghMRI_W(       PghMRIhandler* mriHandler,
                            const string&        chunkName,
                                  char           tDimName,
                                  bool           deleteTrivial )
 : TCrw( ), my_chunkName( ), my_tskip( ), my_mriHandler( mriHandler )
 {
  setChunk( chunkName, tDimName, deleteTrivial );
 }

TCrwPghMRI_W::TCrwPghMRI_W(       PghMRIhandler*              mriHandler,
                            const string&                     dimensions,
                            const uintVector&                 sizes,
                                  char                        tDimName,
                            const string&                     chunkName,
                            const LocationType&               chunkLower,
                                  PghMRIhandler::mriArrayType chunkType,
                            const string&                     chunkFile,
                                  bool                        littleEndian )
 : TCrw( ), my_chunkName( ), my_tskip( ), my_mriHandler( mriHandler )
 {
  setChunk( dimensions, sizes, tDimName,
            chunkName, chunkLower, chunkType, chunkFile, littleEndian );
 }

TCrwPghMRI_W::TCrwPghMRI_W(       PghMRIhandler*              mriHandler,
                            const TChandler*                  tch,
                            const string&                     chunkName,
                                  PghMRIhandler::mriArrayType chunkType,
                            const string&                     chunkFile,
                                  bool                        littleEndian )
 : TCrw( ), my_chunkName( ), my_tskip( ), my_mriHandler( mriHandler )
 {
  string dimensions = tch->nameStr( "" );
  setChunk( dimensions, tch->sizes( ), tch->names( )[tch->tdim( )][0],
            chunkName, tch->indexLower( ), chunkType, chunkFile, littleEndian );
 }

void
TCrwPghMRI_W::setChunk( const string& chunkName, char tDimName, bool deleteTrivial )
 {
  my_chunkName = chunkName;

  if ( my_mriHandler )
   {
    assert( my_mriHandler->hasKey( chunkName ) );
    assert( my_mriHandler->getString( chunkName ) == "[chunk]" );
   }

  openRM( tDimName, deleteTrivial );
 }

void
TCrwPghMRI_W::setMRIhandler( PghMRIhandler* mriHandler )
 {
  my_mriHandler = mriHandler;
 }

void
TCrwPghMRI_W::setChunk( const string&                     dimensions,
                        const uintVector&                 sizes,
                              char                        tDimName,
                        const string&                     chunkName,
                        const LocationType&               chunkLower,
                              PghMRIhandler::mriArrayType chunkType,
                        const string&                     chunkFile,
                              bool                        littleEndian )
 {
  my_chunkName = chunkName;

  assert( dimensions.size( ) == sizes.size( ) );

  vector<string> names( dimensions.size( ) );

  bool noneRepeated = true, nonzeroSizes = true;
  for ( size_t i = 0; i < sizes.size( ); i++ )
   {
    names[i] = dimensions[i];

    for ( size_t j = 0; j < i; j++ )
     noneRepeated = noneRepeated && ( dimensions[i] != dimensions[j] );

    nonzeroSizes = nonzeroSizes && ( sizes[i] > 0 );
   }

  assert( noneRepeated && nonzeroSizes );

  string::size_type tpos = dimensions.find( tDimName );

  TCrw::reinitialize( sizes, tpos, uintVector( ), chunkLower, names, false );

  my_tskip = 1;
  for ( size_t d = 0; d < tdim( ); d++ ) my_tskip *= TCrw::sizes( )[d];

  varray<int> sz( numDims( ) ), lw( numDims( ) );
  for ( size_t i = 0; i < numDims( ); i++ )
   {
    sz[i] = TCrw::sizes( )[i];
    lw[i] = TCrw::indexLower( )[i];
   }

  if ( my_mriHandler )
   {
    my_mriHandler->createChunk( chunkName, dimensions,
                                PghMRIhandler::type2name( chunkType, "float64" ),
                                chunkFile, sz( 0 ), lw( 0 ), littleEndian );
   }
 }

void
TCrwPghMRI_W::setChunk( const TChandler*                  tch,
                        const string&                     chunkName,
                              PghMRIhandler::mriArrayType chunkType,
                        const string&                     chunkFile,
                              bool                        littleEndian )
 {
  assert( tch );

  string dimensions = tch->nameStr( "" );
  setChunk( dimensions, tch->sizes( ), tch->names( )[tch->tdim( )][0],
            chunkName, tch->indexLower( ), chunkType, chunkFile, littleEndian );
 }

void
TCrwPghMRI_W::openRM( char tDimName, bool deleteTrivial )
 {
  if ( !my_mriHandler ) return;

  // cerr << "(debug) begin TCrwPghMRI_W::openRM( )" << endl;

  string dimensions = my_mriHandler->getString( my_chunkName + ".dimensions" );

  /*
  for ( size_t i = 0; i < dimensions.size( ); i++ )
   {
    cerr << "(debug) " << dimensions[i]
         << " size "
         << my_mriHandler->getInt( my_chunkName + ".extent." + dimensions[i] );
    if ( my_mriHandler->hasKey( my_chunkName + ".min." + dimensions[i] ) )
     cerr << " lower "
          << my_mriHandler->getInt( my_chunkName + ".min." + dimensions[i] );
    cerr << endl;
   }
  */

  string::size_type tpos = dimensions.find( tDimName );

  // cerr << "(debug) tpos " << tpos << endl;

  assert( tpos != string::npos );

  uintVector     sizes( dimensions.size( ) );
  vector<string> names( dimensions.size( ) );
  LocationType   lower( dimensions.size( ) );

  int l;
  string keyE = my_chunkName + ".extent.?";
  string keyM = my_chunkName + ".min.?";

  for ( size_t d = 0; d < sizes.size( ); d++ )
   {
    keyE[keyE.size( ) - 1] = dimensions[d];
    keyM[keyM.size( ) - 1] = dimensions[d];

    names[d] = dimensions[d];
    sizes[d] = my_mriHandler->getInt( keyE );

    l = 0;
    if ( my_mriHandler->hasKey( keyM ) ) l = my_mriHandler->getInt( keyM );

    lower[d] = ( l < 0 ) ? 0 : l;
   }

  // for ( size_t i = 0; i < sizes.size( ); i++ )
  //  cerr << "(debug) name " << names[i] << " size " << sizes[i] << " lower " << lower[i] << endl;

  TCrw::reinitialize( sizes, tpos, uintVector( ), lower, names, deleteTrivial );

  my_tskip = 1;
  for ( size_t d = 0; d < tdim( ); d++ ) my_tskip *= TCrw::sizes( )[d];

  // cerr << "(debug) my_tskip " << my_tskip << endl;

  // cerr << "(debug) end TCrwPghMRI_W::openRM( )" << endl;
 }
