/* ----------------------------------------------------------------------------
 *
 * class TCwriter
 *   base class for time-course data writers.
 *   see TChandler.h for more details.
 *
 * required headers
 *   see include header file TCwriter
 *
 * ----------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __TCwriter_H__
#define __TCwriter_H__

cpp_using_namespace( std );
cpp_using_namespace( BasicTypes );

class TCwriter : public TChandler
 {
  public:

   virtual
   ~TCwriter( ) = 0;

  // ------------------------------------- store time course at current location
  // ---------------------------------------------------------------------------

       // virtual functions are unavoidable in this design.  Short of an elegant
       // solution,  all  type conversions  to  the requested type  are  handled
       // through variants of store( ) for five basic numerical types  (the most
       // common case, double, is incorporated in store()). Templates would have
       // been ideal for this purpose  (virtual templates are illegal).

       // All store( ) variants  store time course at the  current-location into
       // timeCourse,  invoke seek( ),  invoke setMove( true ),  report  success
       // (assume that timeCourse is properly sized).

   virtual
   bool
   store( const varray<double>& timeCourse ) const = 0;

   virtual
   bool
   storeFloat( const varray<float>& timeCourse ) const = 0;

   virtual
   bool
   storeInt( const varray<int>& timeCourse ) const = 0;

   virtual
   bool
   storeShort( const varray<short>& timeCourse ) const = 0;

   virtual
   bool
   storeUchar( const varray<unsigned char>& timeCourse ) const = 0;

  protected:

  // ---------------------------------------------------------- construct/reinit
  // ---------------------------------------------------------------------------

   TCwriter( );

   TCwriter( const TCwriter& another );
   TCwriter( const TChandler& another );
   TCwriter( const TCrw& another );

   TCwriter& operator=( const TCwriter& rhs );
   TCwriter& operator=( const TChandler& rhs );
   TCwriter& operator=( const TCrw& rhs );

   TCwriter( const uintVector&     sizes,
                   size_t          tDimension    = 0,
             const uintVector&     indexOrder    = LocationType( ),
             const LocationType&   lowerBounds   = LocationType( ),
             const vector<string>& names         = vector<string>( ),
                   bool            deleteTrivial = true );

   void
   reinitialize( const uintVector&     sizes,
                       size_t          tDimension    = 0,
                 const uintVector&     indexOrder    = LocationType( ),
                 const LocationType&   lowerBounds   = LocationType( ),
                 const vector<string>& names         = vector<string>( ),
                       bool            deleteTrivial = true );

  private:
 };

#endif /* __TCwriter_H__ */
