/* --------------------------------------------------------------------------
 *
 * namespace cpplapack
 *   C++ prototypes for fortran LApack functions
 *
 * required headers
 *   see include header file cpplapack
 *
 * notes
 *   These prototypes are based on f2c-converted clapack 3.0 prototypes
 *   (http://www.netlib.org/lapack/). The original fortran definitions
 *   should be considered authentic should there be any ambiguity.
 *
 *   - input-only or non-array/matrix function arguments have been
 *     converted into const/non-const references for ease of use.
 *   - Arguments n, lda, ldb, lwork can usually be converted
 *     to const integer& from integer&.
 *   - prototypes involving complex arguments cannot be included for
 *     want of a c++ representation of complex numbers (single and arrays)
 *     that is completely compatible with the fortran representation.
 *   - explicit verification of the argument lists for other prototypes
 *     is highly recommended before use. All unverified prototypes have
 *     been commented out.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * ---------------------------------------------------------------------------- */

#ifndef __cpplapack_H__
#define __cpplapack_H__

cpp_gnamespace_begin( cpplapack )

  // -------------------------------------------------- BLAS character constants
  // ---------------------------------------------------------------------------

                     // character constants for arguments trans, transa, transb,
                     //           uplo, diag and side are the same as in cppblas

  // ------------------------------------------------ LApack character constants
  // ---------------------------------------------------------------------------

  static
  const character balanc_0 = 'N';
  static
  const character balanc_1 = 'P';
  static
  const character balanc_2 = 'S';
  static
  const character balanc_3 = 'B';

  static
  const character compq_0 = 'N';
  static
  const character compq_1 = 'I';
  static
  const character compq_2 = 'V';

  static
  const character compz_0 = 'N';
  static
  const character compz_1 = 'I';
  static
  const character compz_2 = 'V';

  static
  const character cmach_getEps     = 'E';
  static
  const character cmach_getSfmin   = 'S';
  static
  const character cmach_getBase    = 'B';
  static
  const character cmach_getEpsbase = 'P';
  static
  const character cmach_getT       = 'N';
  static
  const character cmach_getRnd     = 'R';
  static
  const character cmach_getEmin    = 'M';
  static
  const character cmach_getRmin    = 'U';
  static
  const character cmach_getEmax    = 'L';
  static
  const character cmach_getRmax    = 'O';

  static
  const character direct_forward  = 'F';
  static
  const character direct_backward = 'B';

  static
  const character eigsrc_0 = 'N';
  static
  const character eigsrc_1 = 'Q';

  static
  const character fact_0 = 'N';
  static
  const character fact_1 = 'F';
  static
  const character fact_2 = 'E';

  static
  const character howmny_0 = 'A';
  static
  const character howmny_1 = 'B';
  static
  const character howmny_2 = 'S';

  static
  const character id_incrOrder = 'I';
  static
  const character id_decrOrder = 'D';

  static
  const character initv_0 = 'N';
  static
  const character initv_1 = 'U';

  static
  const character job_0 = 'E';
  static
  const character job_1 = 'S';

  static
  const character jobq_0 = 'N';
  static
  const character jobq_1 = 'I';
  static
  const character jobq_2 = 'Q';

  static
  const character jobu_0 = 'N';
  static
  const character jobu_1 = 'I';
  static
  const character jobu_2 = 'U';

  static
  const character jobv_0 = 'N';
  static
  const character jobv_1 = 'I';
  static
  const character jobv_2 = 'V';

  static
  const character jobvl_0 = 'N';
  static
  const character jobvl_1 = 'V';

  static
  const character jobvr_0 = 'N';
  static
  const character jobvr_1 = 'V';

  static
  const character jobvs_0 = 'N';
  static
  const character jobvs_1 = 'V';

  static
  const character jobvsl_0 = 'N';
  static
  const character jobvsl_1 = 'V';

  static
  const character jobvsr_0 = 'N';
  static
  const character jobvsr_1 = 'V';

  static
  const character jobvt_0 = 'N';
  static
  const character jobvt_1 = 'A';
  static
  const character jobvt_2 = 'S';
  static
  const character jobvt_3 = 'O';

  static
  const character jobz_0 = 'N';
  static
  const character jobz_1 = 'V';

  static
  const character norm_M = 'M';
  static
  const character norm_F = 'F';
  static
  const character norm_1 = '1';
  static
  const character norm_I = 'I';

  static
  const character normin_0 = 'N';
  static
  const character normin_1 = 'Y';

  static
  const character order_block  = 'B';
  static
  const character order_entire = 'E';

  static
  const character pivot_variable = 'V';
  static
  const character pivot_top      = 'T';
  static
  const character pivot_bottom   = 'B';

  static
  const character range_0 = 'A';
  static
  const character range_1 = 'V';
  static
  const character range_2 = 'I';

  static
  const character sense_0 = 'N';
  static
  const character sense_1 = 'E';
  static
  const character sense_2 = 'V';
  static
  const character sense_3 = 'B';

  static
  const character sort_unsorted = 'N';
  static
  const character sort_sorted   = 'S';

  static
  const character sortev_byColumn = 'C';
  static
  const character sortev_byRow    = 'R';

  static
  const character type_full               = 'G';
  static
  const character type_lowerTriangular    = 'L';
  static
  const character type_upperTriangular    = 'U';
  static
  const character type_Hessenberg         = 'H';
  static
  const character type_symmetricBandLower = 'B';
  static
  const character type_symmetricBandUpper = 'Q';
  static
  const character type_band               = 'Z';

  static
  const character vect_0 = 'Q';
  static
  const character vect_1 = 'P';

  extern "C"
   {
    // ---------------------------------------------------- auxilliary functions
    // -------------------------------------------------------------------------

    // the following functions have been declared in cppblas.h
    //
    // logical lsame( const character* ca, const character* cb );
    // void xerbla( const character* srname, const integer& info );

    integer
    ilaenv( const integer& ispec, const character* name, const character& opts,
            const integer& n1, const integer& n2, const integer& n3, const integer& n4 );

    logical
    lsamen( const integer& n, const character* ca, const character* cb );

    // ----------------------------------------------------- verified prototypes
    // -------------------------------------------------------------------------

    void
    dggglm( integer& n, integer& m, integer& p,
            dReal* a, integer& lda, dReal* b, integer& ldb,
            dReal* d, dReal* x, dReal* y, dReal* work,
            integer& lwork, integer& info );

    void
    sggglm( integer& n, integer& m, integer& p,
            sReal* a, integer& lda, sReal* b, integer& ldb,
            sReal* d, sReal* x, sReal* y, sReal* work,
            integer& lwork, integer& info );

    void
    dposv( const character& uplo, const integer& n, const integer& nrhs,
           dReal* a, const integer& lda, dReal* b, const integer& ldb, integer& info );

    void
    sposv( const character& uplo, const integer& n, const integer& nrhs,
           sReal* a, const integer& lda, sReal* b, const integer& ldb, integer& info );

    void
    dpotrf( const character& uplo, const integer& n, dReal* a, const integer& lda, integer& info );

    void
    spotrf( const character& uplo, const integer& n, sReal* a, const integer& lda, integer& info );

    void
    dspev( const character& jobz, const character& uplo, const integer& n,
           dReal* ap, dReal* w, dReal* z, const integer& ldz, dReal* work, integer& info );

    void
    sspev( const character& jobz, const character& uplo, const integer& n,
           sReal* ap, sReal* w, sReal* z, const integer& ldz, sReal* work, integer& info );

    void
    dsptrf( const character& uplo, const integer& n, dReal* ap, integer* ipiv, integer& info );

    void
    ssptrf( const character& uplo, const integer& n, sReal* ap, integer* ipiv, integer& info );

    void
    dsptri( const character& uplo, const integer& n, dReal* ap, integer* ipiv, dReal* work, integer& info );

    void
    ssptri( const character& uplo, const integer& n, sReal* ap, integer* ipiv, sReal* work, integer& info );

    void
    dsyev( const character& jobz, const character& uplo, const integer& n,
           dReal* a, const integer& lda, dReal* w,
           dReal* work, const integer& lwork, integer& info );

    void
    ssyev( const character& jobz, const character& uplo, const integer& n,
           sReal* a, const integer& lda, sReal* w,
           sReal* work, const integer& lwork, integer& info );

    void
    dsygv( const integer& itype, const character& jobz, const character& uplo, const integer& n,
           dReal* a, const integer& lda, dReal* b, const integer& ldb, dReal* w,
           dReal* work, const integer& lwork, integer& info );

    void
    ssygv( const integer& itype, const character& jobz, const character& uplo, const integer& n,
           sReal* a, const integer& lda, sReal* b, const integer& ldb, sReal* w,
           sReal* work, const integer& lwork, integer& info );

    void
    dsytrf( const character& uplo, const integer& n,
            dReal* a, const integer& lda,
            integer* ipiv, dReal* work, const integer& lwork, integer& info );

    void
    ssytrf( const character& uplo, const integer& n,
            sReal* a, const integer& lda,
            integer* ipiv, sReal* work, const integer& lwork, integer& info );

    void
    dsytri( const character& uplo, const integer& n,
            dReal* a, const integer& lda,
            const integer* ipiv, dReal* work, integer& info );

    void
    ssytri( const character& uplo, const integer& n,
            sReal* a, const integer& lda,
            const integer* ipiv, sReal* work, integer& info );

    void
    dtrtri( const character& uplo, const character& diag, const integer& n,
            dReal* a, const integer& lda, integer& info );

    void
    strtri( const character& uplo, const character& diag, const integer& n,
            sReal* a, const integer& lda, integer& info );

    /*
    void
    cggglm
    ( integer& n, integer& m, integer& p, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& d, sComplex& x, sComplex& y, sComplex& work, integer& lwork, integer& info );

    void
    cposv
    ( const character& uplo, const integer& n, const integer& nrhs, sComplex& a, const integer& lda, sComplex& b, const integer& ldb, integer& info );

    void
    cbdsqr
    ( const character& uplo, integer& n, integer& ncvt, integer& nru, integer& ncc, sReal& d, sReal& e, sComplex& vt, integer& ldvt, sComplex& u, integer& ldu, sComplex& c, integer& ldc, sReal& rwork, integer& info );

    void
    cgbbrd
    ( const character& vect, integer& m, integer& n, integer& ncc, integer& kl, integer& ku, sComplex& ab, integer& ldab, sReal& d, sReal& e, sComplex& q, integer& ldq, sComplex& pt, integer& ldpt, sComplex& c, integer& ldc, sComplex& work, sReal& rwork, integer& info );

    void
    cgbcon
    ( const character& norm, integer& n, integer& kl, integer& ku, sComplex& ab, integer& ldab, integer& ipiv, sReal& anorm, sReal& rcond, sComplex& work, sReal& rwork, integer& info );

    void
    cgbequ
    ( integer& m, integer& n, integer& kl, integer& ku, sComplex& ab, integer& ldab, sReal& r, sReal& c, sReal& rowcnd, sReal& colcnd, sReal& amax, integer& info );

    void
    cgbrfs
    ( const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, sComplex& ab, integer& ldab, sComplex& afb, integer& ldafb, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cgbsv
    ( integer& n, integer& kl, integer& ku, integer& nrhs, sComplex& ab, integer& ldab, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    cgbsvx
    ( const character& fact, const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, sComplex& ab, integer& ldab, sComplex& afb, integer& ldafb, integer& ipiv, character& equed, sReal& r, sReal& c, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal &ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cgbtf2
    ( integer& m, integer& n, integer& kl, integer& ku, sComplex& ab, integer& ldab, integer& ipiv, integer& info );

    void
    cgbtrf
    ( integer& m, integer& n, integer& kl, integer& ku, sComplex& ab, integer& ldab, integer& ipiv, integer& info );

    void
    cgbtrs
    ( const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, sComplex& ab, integer& ldab, integer& ipiv, sComplex &b, integer& ldb, integer& info );

    void
    cgebak
    ( const character& job, const character& side, integer& n, integer& ilo, integer& ihi, sReal& scale, integer& m, sComplex& v, integer& ldv, integer& info );

    void
    cgebal
    ( const character& job, integer& n, sComplex& a, integer& lda, integer& ilo, integer& ihi, sReal& scale, integer& info );

    void
    cgebd2
    ( integer& m, integer& n, sComplex& a, integer& lda, sReal& d, sReal& e, sComplex& tauq, sComplex& taup, sComplex& work, integer& info );

    void
    cgebrd
    ( integer& m, integer& n, sComplex& a, integer& lda, sReal& d, sReal& e, sComplex& tauq, sComplex& taup, sComplex& work, integer& lwork, integer& info );

    void
    cgecon
    ( const character& norm, integer& n, sComplex& a, integer& lda, sReal& anorm, sReal& rcond, sComplex& work, sReal& rwork, integer& info );

    void
    cgeequ
    ( integer& m, integer& n, sComplex& a, integer& lda, sReal& r, sReal& c, sReal& rowcnd, sReal& colcnd, sReal& amax, integer& info );

    void
    cgees
    ( const character& jobvs, const character& sort, logical (*select)( sComplex& ), integer& n, sComplex& a, integer& lda, integer& sdim, sComplex& w, sComplex& vs, integer& ldvs, sComplex& work, integer& lwork, sReal& rwork, logical& bwork, integer& info );

    void
    cgeesx
    ( const character& jobvs, const character& sort, logical (*select)( sComplex& ), const character& sense, integer& n, sComplex& a, integer& lda, integer& sdim, sComplex& w, sComplex& vs, integer& ldvs, sReal& rconde, sReal& rcondv, sComplex& work, integer& lwork, sReal& rwork, logical& bwork, integer& info );

    void
    cgeev
    ( const character& jobvl, const character& jobvr, integer& n, sComplex& a, integer& lda, sComplex& w, sComplex& vl, integer& ldvl, sComplex& vr, integer& ldvr, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    cgeevx
    ( const character& balanc, const character& jobvl, const character& jobvr, const character& sense, integer& n, sComplex& a, integer& lda, sComplex& w, sComplex& vl, integer& ldvl, sComplex& vr, integer& ldvr, integer& ilo, integer& ihi, sReal& scale, sReal& abnrm, sReal& rconde, sReal& rcondv, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    cgegs
    ( const character& jobvsl, const character& jobvsr, integer& n, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& alpha, sComplex& beta, sComplex& vsl, integer& ldvsl, sComplex& vsr, integer& ldvsr, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    cgegv
    ( const character& jobvl, const character& jobvr, integer& n, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& alpha, sComplex& beta, sComplex& vl, integer& ldvl, sComplex& vr, integer& ldvr, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    cgehd2
    ( integer& n, integer& ilo, integer& ihi, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cgehrd
    ( integer& n, integer& ilo, integer& ihi, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer &info );

    void
    cgelq2
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cgelqf
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cgels
    ( const character& trans, integer& m, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& work, integer& lwork, integer& info );

    void
    cgelss
    ( integer& m, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& b, integer& ldb, sReal& s, sReal& rcond, integer& rank, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    cgelsx
    ( integer& m, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& b, integer& ldb, integer& jpvt, sReal& rcond, integer& rank, sComplex& work, sReal& rwork, integer& info );

    void
    cgeql2
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cgeqlf
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cgeqpf
    ( integer& m, integer& n, sComplex& a, integer& lda, integer& jpvt, sComplex& tau, sComplex& work, sReal& rwork, integer& info );

    void
    cgeqr2
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cgeqrf
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cgerfs
    ( const character& trans, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& af, integer& ldaf, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cgerq2
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cgerqf
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cgesv
    ( integer& n, integer& nrhs, sComplex& a, integer& lda, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    cgesvd
    ( const character& jobu, const character& jobvt, integer& m, integer& n, sComplex& a, integer& lda, sReal& s, sComplex& u, integer& ldu, sComplex& vt, integer& ldvt, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    cgesvx
    ( const character& fact, const character& trans, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& af, integer& ldaf, integer& ipiv, character& equed, sReal& r, sReal& c, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cgetf2
    ( integer& m, integer& n, sComplex& a, integer& lda, integer& ipiv, integer& info );

    void
    cgetrf
    ( integer& m, integer& n, sComplex& a, integer& lda, integer& ipiv, integer& info );

    void
    cgetri
    ( integer& n, sComplex& a, integer& lda, integer& ipiv, sComplex& work, integer& lwork, integer& info );

    void
    cgetrs
    ( const character& trans, integer& n, integer& nrhs, sComplex& a, integer& lda, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    cggbak
    ( const character& job, const character& side, integer& n, integer& ilo, integer& ihi, sReal& lscale, sReal& rscale, integer& m, sComplex& v, integer& ldv, integer& info );

    void
    cggbal
    ( const character& job, integer& n, sComplex& a, integer& lda, sComplex& b, integer& ldb, integer& ilo, integer& ihi, sReal& lscale, sReal& rscale, sReal& work, integer& info );
    */

    /*
    void
    cgghrd
    ( const character& compq, const character& compz, integer& n, integer& ilo, integer& ihi, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& q, integer& ldq, sComplex& z, integer& ldz, integer& info );

    void
    cgglse
    ( integer& m, integer& n, integer& p, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& c, sComplex& d, sComplex& x, sComplex& work, integer& lwork, integer& info );

    void
    cggqrf
    ( integer& n, integer& m, integer& p, sComplex& a, integer& lda, sComplex& taua, sComplex& b, integer& ldb, sComplex& taub, sComplex& work, integer& lwork, integer& info );

    void
    cggrqf
    ( integer& m, integer& p, integer& n, sComplex& a, integer& lda, sComplex& taua, sComplex& b, integer& ldb, sComplex& taub, sComplex& work, integer& lwork, integer& info );

    void
    cggsvd
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& n, integer& p, integer& k, integer& l, sComplex& a, integer& lda, sComplex& b, integer& ldb, sReal& alpha, sReal& beta, sComplex& u, integer& ldu, sComplex& v, integer& ldv, sComplex& q, integer& ldq, sComplex& work, sReal& rwork, integer& iwork, integer& info );

    void
    cggsvp
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& p, integer& n, sComplex& a, integer& lda, sComplex& b, integer &ldb, sReal& tola, sReal& tolb, integer& k, integer& l, sComplex& u, integer& ldu, sComplex& v, integer& ldv, sComplex& q, integer& ldq, integer& iwork, sReal& rwork, sComplex& tau, sComplex& work, integer& info );

    void
    cgtcon
    ( const character& norm, integer& n, sComplex& dl, sComplex& d, sComplex& du, sComplex& du2, integer& ipiv, sReal& anorm, sReal& rcond, sComplex& work, integer& info );

    void
    cgtrfs
    ( const character& trans, integer& n, integer& nrhs, sComplex& dl, sComplex& d, sComplex& du, sComplex& dlf, sComplex& df, sComplex& duf, sComplex& du2, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cgtsv
    ( integer& n, integer& nrhs, sComplex& dl, sComplex& d, sComplex& du, sComplex& b, integer& ldb, integer& info );

    void
    cgtsvx
    ( const character& fact, const character& trans, integer& n, integer& nrhs, sComplex& dl, sComplex& d, sComplex& du, sComplex& dlf, sComplex& df, sComplex& duf, sComplex& du2, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cgttrf
    ( integer& n, sComplex& dl, sComplex& d, sComplex& du, sComplex& du2, integer& ipiv, integer& info );

    void
    cgttrs
    ( const character& trans, integer& n, integer& nrhs, sComplex& dl, sComplex& d, sComplex& du, sComplex& du2, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    chbev
    ( const character& jobz, const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, sReal& w, sComplex& z, integer& ldz, sComplex& work, sReal& rwork, integer& info );

    void
    chbevd
    ( const character& jobz, const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, sReal& w, sComplex& z, integer& ldz, sComplex& work, integer& lwork, sReal& rwork, integer& lrwork, integer& iwork, integer& liwork, integer& info );

    void
    chbevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, sComplex& q, integer& ldq, sReal& vl, sReal& vu, integer& il, integer& iu, sReal& abstol, integer& m, sReal& w, sComplex& z, integer& ldz, sComplex& work, sReal& rwork, integer& iwork, integer& ifail, integer& info );

    void
    chbgst
    ( const character& vect, const character& uplo, integer& n, integer& ka, integer& kb, sComplex& ab, integer& ldab, sComplex& bb, integer& ldbb, sComplex& x, integer& ldx, sComplex& work, sReal& rwork, integer& info );

    void
    chbgv
    ( const character& jobz, const character& uplo, integer& n, integer& ka, integer& kb, sComplex& ab, integer& ldab, sComplex& bb, integer& ldbb, sReal& w, sComplex& z, integer& ldz, sComplex& work, sReal& rwork, integer& info );

    void
    chbtrd
    ( const character& vect, const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, sReal& d, sReal& e, sComplex& q, integer& ldq, sComplex& work, integer& info );

    void
    checon
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& ipiv, sReal& anorm, sReal& rcond, sComplex& work, integer& info );

    void
    cheev
    ( const character& jobz, const character& uplo, const integer& n, sComplex& a, const integer& lda, sReal& w, sComplex& work, const integer& lwork, sReal& rwork, integer& info );

    void
    cheevd
    ( const character& jobz, const character& uplo, integer& n, sComplex& a, integer& lda, sReal& w, sComplex& work, integer& lwork, sReal& rwork, integer& lrwork, integer& iwork, integer& liwork, integer& info );

    void
    cheevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, sComplex& a, integer& lda, sReal& vl, sReal& vu, integer& il, integer& iu, sReal& abstol, integer& m, sReal& w, sComplex& z, integer& ldz, sComplex& work, integer& lwork, sReal& rwork, integer& iwork, integer& ifail, integer& info );

    void
    chegs2
    ( integer& itype, const character& uplo, integer& n, sComplex& a, integer& lda, sComplex& b, integer& ldb, integer& info );

    void
    chegst
    ( integer& itype, const character& uplo, integer& n, sComplex& a, integer& lda, sComplex& b, integer& ldb, integer& info );

    void
    chegv
    ( const integer& itype, const character& jobz, const character& uplo, const integer& n, sComplex& a, const integer& lda, sComplex& b, const integer& ldb, sReal& w, sComplex& work, const integer& lwork, sReal& rwork, integer& info );

    void
    cherfs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& af, integer& ldaf, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    chesv
    ( const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, integer& ipiv, sComplex& b, integer& ldb, sComplex& work, integer& lwork, integer& info );

    void
    chesvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& af, integer& ldaf, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    chetd2
    ( const character& uplo, integer& n, sComplex& a, integer& lda, sReal& d, sReal& e, sComplex& tau, integer& info );

    void
    chetf2
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& ipiv, integer& info );

    void
    chetrd
    ( const character& uplo, integer& n, sComplex& a, integer& lda, sReal& d, sReal& e, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    chetrf
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& ipiv, sComplex& work, integer& lwork, integer& info );

    void
    chetri
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& ipiv, sComplex& work, integer& info );

    void
    chetrs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    chgeqz
    ( const character& job, const character& compq, const character& compz, integer& n, integer& ilo, integer& ihi, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& alpha, sComplex& beta, sComplex& q, integer& ldq, sComplex& z, integer& ldz, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    chpcon
    ( const character& uplo, integer& n, sComplex& ap, integer& ipiv, sReal& anorm, sReal& rcond, sComplex& work, integer& info );

    void
    chpev
    ( const character& jobz, const character& uplo, integer& n, sComplex& ap, sReal& w, sComplex& z, integer& ldz, sComplex& work, sReal& rwork, integer& info );

    void
    chpevd
    ( const character& jobz, const character& uplo, integer& n, sComplex& ap, sReal& w, sComplex& z, integer& ldz, sComplex& work, integer& lwork, sReal& rwork, integer& lrwork, integer& iwork, integer& liwork, integer& info );

    void
    chpevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, sComplex& ap, sReal& vl, sReal& vu, integer& il, integer& iu, sReal& abstol, integer& m, sReal& w, sComplex& z, integer& ldz, sComplex& work, sReal& rwork, integer& iwork, integer& ifail, integer& info );

    void
    chpgst
    ( integer& itype, const character& uplo, integer& n, sComplex& ap, sComplex& bp, integer& info );

    void
    chpgv
    ( integer& itype, const character& jobz, const character& uplo, integer& n, sComplex& ap, sComplex& bp, sReal& w, sComplex& z, integer& ldz, sComplex& work, sReal& rwork, integer& info );

    void
    chprfs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, sComplex& afp, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    chpsv
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    chpsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sComplex& ap, sComplex& afp, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    chptrd
    ( const character& uplo, integer& n, sComplex& ap, sReal& d, sReal& e, sComplex& tau, integer& info );

    void
    chptrf
    ( const character& uplo, integer& n, sComplex& ap, integer& ipiv, integer& info );

    void
    chptri
    ( const character& uplo, integer& n, sComplex& ap, integer& ipiv, sComplex& work, integer& info );

    void
    chptrs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    chsein
    ( const character& side, const character& eigsrc, const character& initv, logical& select, integer& n, sComplex& h, integer& ldh, sComplex& w, sComplex& vl, integer& ldvl, sComplex& vr, integer& ldvr, integer& mm, integer& m, sComplex& work, sReal& rwork, integer& ifaill, integer& ifailr, integer &info );

    void
    chseqr
    ( const character& job, const character& compz, integer& n, integer& ilo, integer& ihi, sComplex& h, integer& ldh, sComplex& w, sComplex& z, integer& ldz, sComplex& work, integer& lwork, integer& info );

    void
    clabrd
    ( integer& m, integer& n, integer& nb, sComplex& a, integer& lda, sReal& d, sReal& e, sComplex& tauq, sComplex& taup, sComplex &x, integer& ldx, sComplex& y, integer& ldy );

    void
    clacgv
    ( integer& n, sComplex& x, integer& incx );

    void
    clacon
    ( integer& n, sComplex& v, sComplex& x, sReal& est, integer& kase );

    void
    clacpy
    ( const character& uplo, integer& m, integer& n, sComplex& a, integer& lda, sComplex& b, integer& ldb );

    void
    clacrm
    ( integer& m, integer& n, sComplex& a, integer& lda, sReal& b, integer& ldb, sComplex& c, integer& ldc, sReal& rwork );

    void
    clacrt
    ( integer& n, sComplex& cx, integer& incx, sComplex& cy, integer& incy, sComplex& c, sComplex& s );

    sComplex
    cladiv
    (  sComplex& x, sComplex& y  );

    void
    claed0
    ( integer& qsiz, integer& n, sReal& d, sReal& e, sComplex& q, integer& ldq, sComplex& qstore, integer& ldqs, sReal& rwork, integer& iwork, integer& info );

    void
    claed7
    ( integer& n, integer& cutpnt, integer& qsiz, integer& tlvls, integer& curlvl, integer& curpbm, sReal& d, sComplex& q, integer& ldq, sReal& rho, integer& indxq, sReal& qstore, integer& qptr, integer& prmptr, integer& perm, integer& givptr, integer& givcol, sReal& givnum, sComplex& work, sReal& rwork, integer& iwork, integer& info );

    void
    claed8
    ( integer& k, integer& n, integer& qsiz, sComplex& q, integer& ldq, sReal& d, sReal& rho, integer& cutpnt, sReal& z, sReal& dlamda, sComplex& q2, integer& ldq2, sReal& w, integer& indxp, integer& indx, integer& indxq, integer& perm, integer& givptr, integer& givcol, sReal& givnum, integer& info );

    void
    claein
    ( logical& rightv, logical& noinit, integer& n, sComplex& h, integer& ldh, sComplex& w, sComplex& v, sComplex& b, integer &ldb, sReal& rwork, sReal& eps3, sReal& smlnum, integer& info );

    void
    claesy
    ( sComplex& a, sComplex& b, sComplex& c, sComplex& rt1, sComplex& rt2, sComplex& evscal, sComplex& cs1, sComplex& sn1 );

    void
    claev2
    ( sComplex& a, sComplex& b, sComplex& c, sReal& rt1, sReal& rt2, sReal& cs1, sComplex& sn1 );

    void
    clags2
    ( logical& upper, sReal& a1, sComplex& a2, sReal& a3, sReal& b1, sComplex& b2, sReal& b3, sReal& csu, sComplex& snu, sReal& csv, sComplex& snv, sReal& csq, sComplex& snq );

    void
    clagtm
    ( const character& trans, integer& n, integer& nrhs, sReal& alpha, sComplex& dl, sComplex& d, sComplex& du, sComplex& x, integer& ldx, sReal& beta, sComplex& b, integer& ldb );

    void
    clahef
    ( const character& uplo, integer& n, integer& nb, integer& kb, sComplex& a, integer& lda, integer& ipiv, sComplex& w, integer& ldw, integer& info );

    void
    clahqr
    ( logical& wantt, logical& wantz, integer& n, integer& ilo, integer& ihi, sComplex& h, integer& ldh, sComplex& w, integer& iloz, integer& ihiz, sComplex& z, integer& ldz, integer& info );

    void
    clahrd
    ( integer& n, integer& k, integer& nb, sComplex& a, integer& lda, sComplex& tau, sComplex& t, integer& ldt, sComplex& y, integer& ldy );

    void
    claic1
    ( integer& job, integer& j, sComplex& x, sReal& sest, sComplex& w, sComplex& gamma, sReal& sestpr, sComplex& s, sComplex& c );

    dReal
    clangb
    ( const character& norm, integer& n, integer& kl, integer& ku, sComplex& ab, integer& ldab, sReal& work );

    dReal
    clange
    ( const character& norm, integer& m, integer& n, sComplex& a, integer& lda, sReal& work );

    dReal
    clangt
    ( const character& norm, integer& n, sComplex& dl, sComplex& d, sComplex& du );

    dReal
    clanhb
    ( const character& norm, const character& uplo, integer& n, integer& k, sComplex& ab, integer& ldab, sReal& work );

    dReal
    clanhe
    ( const character& norm, const character& uplo, integer& n, sComplex& a, integer& lda, sReal& work );

    dReal
    clanhp
    ( const character& norm, const character& uplo, integer& n, sComplex& ap, sReal& work );

    dReal
    clanhs
    ( const character& norm, integer& n, sComplex& a, integer& lda, sReal& work );

    dReal
    clanht
    ( const character& norm, integer& n, sReal& d, sComplex& e );

    dReal
    clansb
    ( const character& norm, const character& uplo, integer& n, integer& k, sComplex& ab, integer& ldab, sReal& work );

    dReal
    clansp
    ( const character& norm, const character& uplo, integer& n, sComplex& ap, sReal& work );

    dReal
    clansy
    ( const character& norm, const character& uplo, integer& n, sComplex& a, integer& lda, sReal& work );

    dReal
    clantb
    ( const character& norm, const character& uplo, const character& diag, integer& n, integer& k, sComplex& ab, integer& ldab, sReal& work );

    dReal
    clantp
    ( const character& norm, const character& uplo, const character& diag, integer& n, sComplex& ap, sReal& work );

    dReal
    clantr
    ( const character& norm, const character& uplo, const character& diag, integer& m, integer& n, sComplex& a, integer& lda, sReal& work );

    void
    clapll
    ( integer& n, sComplex& x, integer& incx, sComplex& y, integer& incy, sReal& ssmin );

    void
    clapmt
    ( logical& forwrd, integer& m, integer& n, sComplex &x, integer& ldx, integer& k );

    void
    claqgb
    ( integer& m, integer& n, integer& kl, integer& ku, sComplex& ab, integer& ldab, sReal& r, sReal& c, sReal& rowcnd, sReal& colcnd, sReal& amax, character& equed );

    void
    claqge
    ( integer& m, integer& n, sComplex& a, integer& lda, sReal& r, sReal& c, sReal& rowcnd, sReal& colcnd, sReal& amax, character& equed );

    void
    claqhb
    ( const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    claqhe
    ( const character& uplo, integer& n, sComplex& a, integer& lda, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    claqhp
    ( const character& uplo, integer& n, sComplex& ap, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    claqsb
    ( const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    claqsp
    ( const character& uplo, integer& n, sComplex& ap, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    claqsy
    ( const character& uplo, integer& n, sComplex& a, integer& lda, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    clar2v
    ( integer& n, sComplex& x, sComplex& y, sComplex& z, integer& incx, sReal& c, sComplex& s, integer& incc );

    void
    clarf
    ( const character& side, integer& m, integer& n, sComplex& v, integer& incv, sComplex& tau, sComplex& c, integer& ldc, sComplex& work );

    void
    clarfb
    ( const character& side, const character& trans, const character& direct, character& storev, integer& m, integer& n, integer& k, sComplex& v, integer& ldv, sComplex& t, integer& ldt, sComplex& c, integer& ldc, sComplex& work, integer& ldwork );

    void
    clarfg
    ( integer& n, sComplex& alpha, sComplex& x, integer& incx, sComplex& tau );

    void
    clarft
    ( const character& direct, character& storev, integer& n, integer& k, sComplex& v, integer& ldv, sComplex& tau, sComplex& t, integer& ldt );

    void
    clarfx
    ( const character& side, integer& m, integer& n, sComplex& v, sComplex& tau, sComplex& c, integer& ldc, sComplex& work );

    void
    clargv
    ( integer& n, sComplex& x, integer& incx, sComplex& y, integer& incy, sReal& c, integer& incc );

    void
    clarnv
    ( integer& idist, integer& iseed, integer& n, sComplex& x );

    void
    clartg
    ( sComplex& f, sComplex& g, sReal& cs, sComplex& sn, sComplex& r );

    void
    clartv
    ( integer& n, sComplex& x, integer& incx, sComplex& y, integer& incy, sReal& c, sComplex& s, integer& incc );

    void
    clascl
    ( const character& type, integer& kl, integer& ku, sReal& cfrom, sReal& cto, integer& m, integer& n, sComplex& a, integer& lda, integer& info );

    void
    claset
    ( const character& uplo, integer& m, integer& n, sComplex& alpha, sComplex& beta, sComplex& a, integer& lda );

    void
    clasr
    ( const character& side, const character& pivot, const character& direct, integer& m, integer& n, sReal& c, sReal& s, sComplex& a, integer& lda );

    void
    classq
    ( integer& n, sComplex& x, integer& incx, sReal& scale, sReal& sumsq );

    void
    claswp
    ( integer& n, sComplex& a, integer& lda, integer& k1, integer& k2, integer& ipiv, integer& incx );

    void
    clasyf
    ( const character& uplo, integer& n, integer& nb, integer& kb, sComplex& a, integer& lda, integer& ipiv, sComplex& w, integer& ldw, integer& info );

    void
    clatbs
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, integer& kd, sComplex& ab, integer& ldab, sComplex& x, sReal& scale, sReal& cnorm, integer& info );

    void
    clatps
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, sComplex& ap, sComplex& x, sReal& scale, sReal& cnorm, integer& info );

    void
    clatrd
    ( const character& uplo, integer& n, integer& nb, sComplex& a, integer& lda, sReal& e, sComplex& tau, sComplex& w, integer& ldw );

    void
    clatrs
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, sComplex& a, integer& lda, sComplex& x, sReal& scale, sReal& cnorm, integer& info );

    void
    clatzm
    ( const character& side, integer& m, integer& n, sComplex& v, integer& incv, sComplex& tau, sComplex& c1, sComplex& c2, integer& ldc, sComplex& work );

    void
    clauu2
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& info );

    void
    clauum
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& info );

    void
    cpbcon
    ( const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, sReal& anorm, sReal& rcond, sComplex& work, sReal& rwork, integer& info );

    void
    cpbequ
    ( const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, sReal& s, sReal& scond, sReal& amax, integer& info );

    void
    cpbrfs
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, sComplex& ab, integer& ldab, sComplex& afb, integer& ldafb, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cpbstf
    ( const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, integer& info );

    void
    cpbsv
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, sComplex& ab, integer& ldab, sComplex& b, integer& ldb, integer& info );

    void
    cpbsvx
    ( const character& fact, const character& uplo, integer& n, integer& kd, integer& nrhs, sComplex& ab, integer& ldab, sComplex& afb, integer& ldafb, character& equed, sReal& s, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cpbtf2
    ( const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, integer& info );

    void
    cpbtrf
    ( const character& uplo, integer& n, integer& kd, sComplex& ab, integer& ldab, integer& info );

    void
    cpbtrs
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, sComplex& ab, integer& ldab, sComplex& b, integer& ldb, integer& info );

    void
    cpocon
    ( const character& uplo, integer& n, sComplex& a, integer& lda, sReal& anorm, sReal& rcond, sComplex& work, sReal& rwork, integer& info );

    void
    cpoequ
    ( integer& n, sComplex& a, integer& lda, sReal& s, sReal& scond, sReal& amax, integer& info );

    void
    cporfs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& af, integer& ldaf, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );
    */

    /*
    void
    cposvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& af, integer& ldaf, character& equed, sReal& s, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cpotf2
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& info );

    void
    cpotrf
    ( const character& uplo, const integer& n, sComplex& a, const integer& lda, integer& info );

    void
    cpotri
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& info );

    void
    cpotrs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& b, integer& ldb, integer& info );

    void
    cppcon
    ( const character& uplo, integer& n, sComplex& ap, sReal& anorm, sReal& rcond, sComplex& work, sReal& rwork, integer& info );

    void
    cppequ
    ( const character& uplo, integer& n, sComplex& ap, sReal& s, sReal& scond, sReal& amax, integer& info );

    void
    cpprfs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, sComplex& afp, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cppsv
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, sComplex& b, integer& ldb, integer& info );

    void
    cppsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sComplex& ap, sComplex& afp, character& equed, sReal& s, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal &berr, sComplex& work, sReal& rwork, integer& info );

    void
    cpptrf
    ( const character& uplo, integer& n, sComplex& ap, integer& info );

    void
    cpptri
    ( const character& uplo, integer& n, sComplex& ap, integer& info );

    void
    cpptrs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, sComplex& b, integer& ldb, integer& info );

    void
    cptcon
    ( integer& n, sReal& d, sComplex& e, sReal& anorm, sReal& rcond, sReal& rwork, integer& info );

    void
    cpteqr
    ( const character& compz, integer& n, sReal& d, sReal& e, sComplex& z, integer& ldz, sReal& work, integer& info );

    void
    cptrfs
    ( const character& uplo, integer& n, integer& nrhs, sReal& d, sComplex& e, sReal& df, sComplex& ef, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cptsv
    ( integer& n, integer& nrhs, sReal& d, sComplex& e, sComplex& b, integer& ldb, integer& info );

    void
    cptsvx
    ( const character& fact, integer& n, integer& nrhs, sReal& d, sComplex& e, sReal& df, sComplex& ef, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cpttrf
    ( integer& n, sReal& d, sComplex& e, integer& info );

    void
    cpttrs
    ( const character& uplo, integer& n, integer& nrhs, sReal& d, sComplex& e, sComplex& b, integer& ldb, integer& info );

    void
    crot
    ( integer& n, sComplex& cx, integer& incx, sComplex& cy, integer& incy, sReal& c, sComplex& s );

    void
    cspcon
    ( const character& uplo, integer& n, sComplex& ap, integer& ipiv, sReal& anorm, sReal& rcond, sComplex& work, integer& info );

    void
    cspmv
    ( const character& uplo, integer& n, sComplex& alpha, sComplex& ap, sComplex& x, integer& incx, sComplex& beta, sComplex& y, integer& incy );

    void
    cspr
    ( const character& uplo, integer& n, sComplex& alpha, sComplex& x, integer& incx, sComplex& ap );

    void
    csprfs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, sComplex& afp, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    cspsv
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    cspsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sComplex& ap, sComplex& afp, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    csptrf
    ( const character& uplo, integer& n, sComplex& ap, integer& ipiv, integer& info );

    void
    csptri
    ( const character& uplo, integer& n, sComplex& ap, integer& ipiv, sComplex& work, integer& info );

    void
    csptrs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& ap, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    csrot
    ( integer& n, sComplex& cx, integer& incx, sComplex& cy, integer& incy, sReal& c, sReal& s );

    void
    csrscl
    ( integer& n, sReal& sa, sComplex& sx, integer& incx );

    void
    cstedc
    ( const character& compz, integer& n, sReal& d, sReal& e, sComplex& z, integer& ldz, sComplex& work, integer& lwork, sReal& rwork, integer& lrwork, integer& iwork, integer& liwork, integer& info );

    void
    cstein
    ( integer& n, sReal& d, sReal& e, integer& m, sReal& w, integer& iblock, integer& isplit, sComplex& z, integer& ldz, sReal& work, integer& iwork, integer& ifail, integer& info );

    void
    csteqr
    ( const character& compz, integer& n, sReal& d, sReal& e, sComplex& z, integer& ldz, sReal& work, integer& info );

    void
    csycon
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& ipiv, sReal& anorm, sReal& rcond, sComplex& work, integer& info );

    void
    csymv
    ( const character& uplo, integer& n, sComplex& alpha, sComplex& a, integer& lda, sComplex& x, integer& incx, sComplex& beta, sComplex& y, integer& incy );

    void
    csyr
    ( const character& uplo, integer& n, sComplex& alpha, sComplex& x, integer& incx, sComplex& a, integer& lda );

    void
    csyrfs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& af, integer& ldaf, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    csysv
    ( const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, integer& ipiv, sComplex& b, integer& ldb, sComplex& work, integer& lwork, integer& info );

    void
    csysvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& af, integer& ldaf, integer& ipiv, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sComplex& work, integer& lwork, sReal& rwork, integer& info );

    void
    csytf2
    ( const character& uplo, integer& n, sComplex& a, integer& lda, integer& ipiv, integer& info );

    void
    csytrf
    ( const character& uplo, const integer& n, sComplex& a, const integer& lda, integer& ipiv, sComplex& work, const integer& lwork, integer& info );

    void
    csytri
    ( const character& uplo, const integer& n, sComplex& a, const integer& lda, const integer& ipiv, sComplex& work, integer& info );

    void
    csytrs
    ( const character& uplo, integer& n, integer& nrhs, sComplex& a, integer& lda, integer& ipiv, sComplex& b, integer& ldb, integer& info );

    void
    ctbcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, integer& kd, sComplex& ab, integer& ldab, sReal& rcond, sComplex& work, sReal& rwork, integer& info );

    void
    ctbrfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& kd, integer& nrhs, sComplex& ab, integer& ldab, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    ctbtrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& kd, integer& nrhs, sComplex& ab, integer& ldab, sComplex& b, integer& ldb, integer& info );

    void
    ctgevc
    ( const character& side, const character& howmny, logical& select, integer& n, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& vl, integer& ldvl, sComplex& vr, integer& ldvr, integer& mm, integer& m, sComplex& work, sReal& rwork, integer& info );

    void
    ctgsja
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& p, integer& n, integer& k, integer& l, sComplex& a, integer& lda, sComplex& b, integer& ldb, sReal& tola, sReal& tolb, sReal& alpha, sReal& beta, sComplex& u, integer& ldu, sComplex& v, integer& ldv, sComplex& q, integer& ldq, sComplex& work, integer& ncycle, integer& info );

    void
    ctpcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, sComplex& ap, sReal& rcond, sComplex& work, sReal& rwork, integer& info );

    void
    ctprfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, sComplex& ap, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal& rwork, integer& info );

    void
    ctptri
    ( const character& uplo, const character& diag, integer& n, sComplex& ap, integer& info );

    void
    ctptrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, sComplex& ap, sComplex& b, integer& ldb, integer& info );

    void
    ctrcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, sComplex& a, integer& lda, sReal& rcond, sComplex& work, sReal& rwork, integer& info );

    void
    ctrevc
    ( const character& side, const character& howmny, logical& select, integer& n, sComplex& t, integer& ldt, sComplex& vl, integer& ldvl, sComplex& vr, integer& ldvr, integer& mm, integer& m, sComplex& work, sReal& rwork, integer& info );

    void
    ctrexc
    ( const character& compq, integer& n, sComplex& t, integer& ldt, sComplex& q, integer& ldq, integer& ifst, integer& ilst, integer& info );

    void
    ctrrfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& x, integer& ldx, sReal& ferr, sReal& berr, sComplex& work, sReal &rwork, integer& info );

    void
    ctrsen
    ( const character& job, const character& compq, logical& select, integer &n, sComplex& t, integer& ldt, sComplex& q, integer& ldq, sComplex& w, integer& m, sReal& s, sReal& sep, sComplex& work, integer& lwork, integer& info );

    void
    ctrsna
    ( const character& job, const character& howmny, logical& select, integer& n, sComplex& t, integer& ldt, sComplex& vl, integer& ldvl, sComplex& vr, integer& ldvr, sReal& s, sReal& sep, integer& mm, integer& m, sComplex& work, integer& ldwork, sReal& rwork, integer& info );

    void
    ctrsyl
    ( const character& trana, const character& tranb, integer& isgn, integer &m, integer& n, sComplex& a, integer& lda, sComplex& b, integer& ldb, sComplex& c, integer& ldc, sReal& scale, integer& info );

    void
    ctrti2
    ( const character& uplo, const character& diag, integer& n, sComplex& a, integer& lda, integer& info );

    void
    ctrtri
    ( const character& uplo, const character& diag, const integer& n, sComplex& a, const integer& lda, integer& info );

    void
    ctrtrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, sComplex& a, integer& lda, sComplex& b, integer& ldb, integer& info );

    void
    ctzrqf
    ( integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, integer& info );

    void
    cung2l
    ( integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cung2r
    ( integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cungbr
    ( const character& vect, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cunghr
    ( integer& n, integer& ilo, integer& ihi, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer &info );

    void
    cungl2
    ( integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cunglq
    ( integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cungql
    ( integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cungqr
    ( integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cungr2
    ( integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& info );

    void
    cungrq
    ( integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cungtr
    ( const character& uplo, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& work, integer& lwork, integer& info );

    void
    cunm2l
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& info );

    void
    cunm2r
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& info );

    void
    cunmbr
    ( const character& vect, const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& lwork, integer& info );

    void
    cunmhr
    ( const character& side, const character& trans, integer& m, integer& n, integer& ilo, integer& ihi, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& lwork, integer& info );

    void
    cunml2
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& info );

    void
    cunmlq
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& lwork, integer& info );

    void
    cunmql
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& lwork, integer& info );

    void
    cunmqr
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& lwork, integer& info );

    void
    cunmr2
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& info );

    void
    cunmrq
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& lwork, integer& info );

    void
    cunmtr
    ( const character& side, const character& uplo, const character& trans, integer& m, integer& n, sComplex& a, integer& lda, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& lwork, integer& info );

    void
    cupgtr
    ( const character& uplo, integer& n, sComplex& ap, sComplex& tau, sComplex& q, integer& ldq, sComplex& work, integer& info );

    void
    cupmtr
    ( const character& side, const character& uplo, const character& trans, integer& m, integer& n, sComplex& ap, sComplex& tau, sComplex& c, integer& ldc, sComplex& work, integer& info );

    void
    dbdsqr
    ( const character& uplo, integer& n, integer& ncvt, integer& nru, integer& ncc, dReal& d, dReal& e, dReal& vt, integer& ldvt, dReal& u, integer& ldu, dReal& c, integer& ldc, dReal& work, integer& info );

    void
    ddisna
    ( const character& job, integer& m, integer& n, dReal& d, dReal& sep, integer& info );

    void
    dgbbrd
    ( const character& vect, integer& m, integer& n, integer& ncc, integer& kl, integer& ku, dReal& ab, integer& ldab, dReal& d, dReal& e, dReal& q, integer& ldq, dReal& pt, integer& ldpt, dReal& c, integer& ldc, dReal& work, integer &info );

    void
    dgbcon
    ( const character& norm, integer& n, integer& kl, integer& ku, dReal& ab, integer& ldab, integer& ipiv, dReal& anorm, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dgbequ
    ( integer& m, integer& n, integer& kl, integer& ku, dReal& ab, integer& ldab, dReal& r, dReal& c, dReal& rowcnd, dReal& colcnd, dReal& amax, integer& info );

    void
    dgbrfs
    ( const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, dReal& ab, integer& ldab, dReal& afb, integer& ldafb, integer& ipiv, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dgbsv
    ( integer& n, integer& kl, integer& ku, integer& nrhs, dReal& ab, integer& ldab, integer& ipiv, dReal& b, integer& ldb, integer& info );

    void
    dgbsvx
    ( const character& fact, const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, dReal& ab, integer& ldab, dReal& afb, integer& ldafb, integer& ipiv, character& equed, dReal& r, dReal& c, dReal& b, integer& ldb, dReal &x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dgbtf2
    ( integer& m, integer& n, integer& kl, integer& ku, dReal& ab, integer& ldab, integer& ipiv, integer& info );

    void
    dgbtrf
    ( integer& m, integer& n, integer& kl, integer& ku, dReal& ab, integer& ldab, integer& ipiv, integer& info );

    void
    dgbtrs
    ( const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, dReal& ab, integer& ldab, integer& ipiv, dReal& b, integer& ldb, integer& info );

    void
    dgebak
    ( const character& job, const character& side, integer& n, integer& ilo, integer& ihi, dReal& scale, integer& m, dReal& v, integer& ldv, integer& info );

    void
    dgebal
    ( const character& job, integer& n, dReal& a, integer& lda, integer& ilo, integer& ihi, dReal& scale, integer& info );

    void
    dgebd2
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& d, dReal& e, dReal& tauq, dReal& taup, dReal& work, integer& info );

    void
    dgebrd
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& d, dReal& e, dReal& tauq, dReal& taup, dReal& work, integer& lwork, integer& info );

    void
    dgecon
    ( const character& norm, integer& n, dReal& a, integer& lda, dReal& anorm, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dgeequ
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& r, dReal& c, dReal& rowcnd, dReal& colcnd, dReal& amax, integer& info );

    void
    dgees
    ( const character& jobvs, const character& sort, logical (*select)( dReal&, dReal& ), integer& n, dReal& a, integer& lda, integer& sdim, dReal& wr, dReal& wi, dReal& vs, integer& ldvs, dReal& work, integer& lwork, logical& bwork, integer& info );

    void
    dgeesx
    ( const character& jobvs, const character& sort, logical (*select)( dReal&, dReal& ), const character& sense, integer& n, dReal& a, integer& lda, integer& sdim, dReal& wr, dReal& wi, dReal& vs, integer& ldvs, dReal& rconde, dReal& rcondv, dReal& work, integer& lwork, integer& iwork, integer& liwork, logical& bwork, integer& info );

    void
    dgeev
    ( const character& jobvl, const character& jobvr, integer& n, dReal& a, integer& lda, dReal& wr, dReal& wi, dReal& vl, integer& ldvl, dReal& vr, integer& ldvr, dReal& work, integer& lwork, integer& info );

    void
    dgeevx
    ( const character& balanc, const character& jobvl, const character& jobvr, const character& sense, integer& n, dReal& a, integer& lda, dReal& wr, dReal& wi, dReal& vl, integer& ldvl, dReal& vr, integer& ldvr, integer& ilo, integer& ihi, dReal& scale, dReal& abnrm, dReal& rconde, dReal& rcondv, dReal &work, integer& lwork, integer& iwork, integer& info );

    void
    dgegs
    ( const character& jobvsl, const character& jobvsr, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& alphar, dReal& alphai, dReal& beta, dReal& vsl, integer& ldvsl, dReal& vsr, integer& ldvsr, dReal& work, integer& lwork, integer& info );

    void
    dgegv
    ( const character& jobvl, const character& jobvr, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& alphar, dReal& alphai, dReal& beta, dReal& vl, integer& ldvl, dReal& vr, integer& ldvr, dReal& work, integer& lwork, integer& info );

    void
    dgehd2
    ( integer& n, integer& ilo, integer& ihi, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dgehrd
    ( integer& n, integer& ilo, integer& ihi, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dgelq2
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dgelqf
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dgels
    ( const character& trans, integer& m, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& work, integer& lwork, integer& info );

    void
    dgelss
    ( integer& m, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& s, dReal& rcond, integer& rank, dReal& work, integer& lwork, integer& info );

    void
    dgelsx
    ( integer& m, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& b, integer& ldb, integer& jpvt, dReal& rcond, integer& rank, dReal& work, integer& info );

    void
    dgeql2
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dgeqlf
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dgeqpf
    ( integer& m, integer& n, dReal& a, integer& lda, integer& jpvt, dReal& tau, dReal& work, integer& info );

    void
    dgeqr2
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dgeqrf
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dgerfs
    ( const character& trans, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& af, integer& ldaf, integer& ipiv, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dgerq2
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dgerqf
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dgesv
    ( integer& n, integer& nrhs, dReal& a, integer &lda, integer& ipiv, dReal& b, integer& ldb, integer& info );

    void
    dgesvd
    ( const character& jobu, const character& jobvt, integer& m, integer& n, dReal& a, integer& lda, dReal& s, dReal& u, integer& ldu, dReal& vt, integer& ldvt, dReal& work, integer& lwork, integer& info );

    void
    dgesvx
    ( const character& fact, const character& trans, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& af, integer& ldaf, integer& ipiv, character& equed, dReal& r, dReal& c, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dgetf2
    ( integer& m, integer& n, dReal& a, integer& lda, integer& ipiv, integer& info );

    void
    dgetrf
    ( integer& m, integer& n, dReal& a, integer& lda, integer& ipiv, integer& info );

    void
    dgetri
    ( integer& n, dReal& a, integer& lda, integer &ipiv, dReal& work, integer& lwork, integer& info );

    void
    dgetrs
    ( const character& trans, integer& n, integer& nrhs, dReal& a, integer& lda, integer& ipiv, dReal& b, integer& ldb, integer& info );

    void
    dggbak
    ( const character& job, const character& side, integer& n, integer& ilo, integer& ihi, dReal& lscale, dReal& rscale, integer& m, dReal& v, integer& ldv, integer& info );

    void
    dggbal
    ( const character& job, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb, integer& ilo, integer& ihi, dReal& lscale, dReal& rscale, dReal& work, integer& info );
    */

    /*
    void
    dgghrd
    ( const character& compq, const character& compz, integer& n, integer& ilo, integer& ihi, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& q, integer& ldq, dReal& z, integer& ldz, integer& info );

    void
    dgglse
    ( integer& m, integer& n, integer& p, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& c, dReal& d, dReal& x, dReal& work, integer& lwork, integer& info );

    void
    dggqrf
    ( integer& n, integer& m, integer& p, dReal& a, integer& lda, dReal& taua, dReal& b, integer& ldb, dReal& taub, dReal& work, integer& lwork, integer& info );

    void
    dggrqf
    ( integer& m, integer& p, integer& n, dReal& a, integer& lda, dReal& taua, dReal& b, integer& ldb, dReal& taub, dReal& work, integer& lwork, integer& info );

    void
    dggsvd
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& n, integer& p, integer& k, integer& l, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& alpha, dReal& beta, dReal& u, integer& ldu, dReal& v, integer &ldv, dReal& q, integer& ldq, dReal& work, integer& iwork, integer& info );

    void
    dggsvp
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& p, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& tola, dReal& tolb, integer& k, integer &l, dReal& u, integer& ldu, dReal& v, integer& ldv, dReal& q, integer& ldq, integer& iwork, dReal& tau, dReal& work, integer& info );

    void
    dgtcon
    ( const character& norm, integer& n, dReal& dl, dReal& d, dReal& du, dReal& du2, integer& ipiv, dReal& anorm, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dgtrfs
    ( const character& trans, integer& n, integer& nrhs, dReal& dl, dReal& d, dReal& du, dReal& dlf, dReal& df, dReal& duf, dReal& du2, integer& ipiv, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dgtsv
    ( integer& n, integer& nrhs, dReal& dl, dReal& d, dReal& du, dReal& b, integer& ldb, integer& info );

    void
    dgtsvx
    ( const character& fact, const character& trans, integer& n, integer& nrhs, dReal& dl, dReal& d, dReal& du, dReal& dlf, dReal& df, dReal& duf, dReal& du2, integer& ipiv, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dgttrf
    ( integer& n, dReal& dl, dReal& d, dReal& du, dReal& du2, integer& ipiv, integer& info );

    void
    dgttrs
    ( const character& trans, integer& n, integer& nrhs, dReal& dl, dReal& d, dReal& du, dReal& du2, integer& ipiv, dReal& b, integer& ldb, integer& info );

    void
    dhgeqz
    ( const character& job, const character& compq, const character& compz, integer& n, integer& ilo, integer& ihi, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& alphar, dReal& alphai, dReal& beta, dReal& q, integer& ldq, dReal& z, integer& ldz, dReal& work, integer& lwork, integer& info );

    void
    dhsein
    ( const character& side, const character& eigsrc, const character& initv, logical& select, integer& n, dReal& h, integer& ldh, dReal& wr, dReal& wi, dReal& vl, integer& ldvl, dReal& vr, integer& ldvr, integer& mm, integer& m, dReal& work, integer& ifaill, integer& ifailr, integer& info );

    void
    dhseqr
    ( const character& job, const character& compz, integer& n, integer& ilo, integer& ihi, dReal& h, integer& ldh, dReal& wr, dReal& wi, dReal& z, integer& ldz, dReal& work, integer& lwork, integer& info );

    void
    dlabad
    ( dReal& small, dReal& large );

    void
    dlabrd
    ( integer& m, integer& n, integer& nb, dReal& a, integer& lda, dReal& d, dReal& e, dReal& tauq, dReal& taup, dReal& x, integer& ldx, dReal& y, integer &ldy );

    void
    dlacon
    ( integer& n, dReal& v, dReal& x, integer& isgn, dReal& est, integer& kase );

    void
    dlacpy
    ( const character& uplo, integer& m, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb );

    void
    dladiv
    ( dReal& a, dReal& b, dReal& c, dReal& d, dReal& p, dReal& q );

    void
    dlae2
    ( dReal& a, dReal& b, dReal& c, dReal& rt1, dReal& rt2 );

    void
    dlaebz
    ( integer& ijob, integer& nitmax, integer& n, integer& mmax, integer& minp, integer& nbmin, dReal& abstol, dReal& reltol, dReal& pivmin, dReal& d, dReal& e, dReal& e2, integer& nval, dReal& ab, dReal& c, integer &mout, integer& nab, dReal& work, integer& iwork, integer& info );

    void
    dlaed0
    ( integer& icompq, integer& qsiz, integer& n, dReal& d, dReal& e, dReal& q, integer& ldq, dReal &qstore, integer& ldqs, dReal& work, integer& iwork, integer& info );

    void
    dlaed1
    ( integer& n, dReal& d, dReal& q, integer& ldq, integer& indxq, dReal& rho, integer& cutpnt, dReal& work, integer& iwork, integer& info );

    void
    dlaed2
    ( integer& k, integer& n, dReal& d, dReal& q, integer& ldq, integer& indxq, dReal& rho, integer &cutpnt, dReal& z, dReal& dlamda, dReal& q2, integer& ldq2, integer& indxc, dReal& w, integer& indxp, integer& indx, integer& coltyp, integer& info );

    void
    dlaed3
    ( integer& k, integer& kstart, integer& kstop, integer& n, dReal& d, dReal& q, integer& ldq, dReal& rho, integer& cutpnt, dReal& dlamda, dReal& q2, integer& ldq2, integer& indxc, integer& ctot, dReal& w, dReal& s, integer& lds, integer& info );

    void
    dlaed4
    ( integer& n, integer& i, dReal& d, dReal& z, dReal& delta, dReal& rho, dReal& dlam, integer& info );

    void
    dlaed5
    ( integer& i, dReal& d, dReal& z, dReal& delta, dReal& rho, dReal& dlam );

    void
    dlaed6
    ( integer& kniter, logical& orgati, dReal& rho, dReal& d, dReal& z, dReal& finit, dReal& tau, integer& info );

    void
    dlaed7
    ( integer& icompq, integer& n, integer& qsiz, integer& tlvls, integer& curlvl, integer& curpbm, dReal& d, dReal& q, integer& ldq, integer& indxq, dReal& rho, integer &cutpnt, dReal& qstore, integer& qptr, integer& prmptr, integer& perm, integer& givptr, integer& givcol, dReal& givnum, dReal& work, integer& iwork, integer& info );

    void
    dlaed8
    ( integer& icompq, integer& k, integer& n, integer &qsiz, dReal& d, dReal& q, integer& ldq, integer& indxq, dReal& rho, integer& cutpnt, dReal& z, dReal& dlamda, dReal& q2, integer& ldq2, dReal& w, integer& perm, integer& givptr, integer& givcol, dReal& givnum, integer& indxp, integer& indx, integer& info );

    void
    dlaed9
    ( integer& k, integer& kstart, integer& kstop, integer& n, dReal& d, dReal& q, integer& ldq, dReal& rho, dReal& dlamda, dReal& w, dReal& s, integer& lds, integer& info );

    void
    dlaeda
    ( integer& n, integer& tlvls, integer& curlvl, integer& curpbm, integer& prmptr, integer& perm, integer& givptr, integer& givcol, dReal& givnum, dReal& q, integer& qptr, dReal& z, dReal& ztemp, integer& info );

    void
    dlaein
    ( logical& rightv, logical& noinit, integer& n, dReal& h, integer& ldh, dReal& wr, dReal& wi, dReal& vr, dReal& vi, dReal& b, integer& ldb, dReal& work, dReal& eps3, dReal& smlnum, dReal& bignum, integer& info );

    void
    dlaev2
    ( dReal& a, dReal& b, dReal& c, dReal& rt1, dReal& rt2, dReal& cs1, dReal& sn1 );

    void
    dlaexc
    ( logical& wantq, integer& n, dReal& t, integer& ldt, dReal& q, integer& ldq, integer& j1, integer& n1, integer& n2, dReal& work, integer& info );

    void
    dlag2
    ( dReal& a, integer& lda, dReal& b, integer& ldb, dReal& safmin, dReal& scale1, dReal& scale2, dReal& wr1, dReal& wr2, dReal& wi );

    void
    dlags2
    ( logical& upper, dReal& a1, dReal& a2, dReal& a3, dReal& b1, dReal& b2, dReal& b3, dReal& csu, dReal& snu, dReal& csv, dReal& snv, dReal& csq, dReal& snq );

    void
    dlagtf
    ( integer& n, dReal& a, dReal& lambda, dReal& b, dReal& c, dReal& tol, dReal& d, integer &in, integer& info );

    void
    dlagtm
    ( const character& trans, integer& n, integer& nrhs, dReal& alpha, dReal& dl, dReal& d, dReal& du, dReal& x, integer& ldx, dReal& beta, dReal& b, integer &ldb );

    void
    dlagts
    ( integer& job, integer& n, dReal& a, dReal& b, dReal& c, dReal& d, integer& in, dReal& y, dReal& tol, integer& info );

    void
    dlahqr
    ( logical& wantt, logical& wantz, integer& n, integer& ilo, integer& ihi, dReal& h, integer& ldh, dReal& wr, dReal& wi, integer& iloz, integer& ihiz, dReal& z, integer& ldz, integer& info );

    void
    dlahrd
    ( integer& n, integer& k, integer& nb, dReal& a, integer& lda, dReal& tau, dReal& t, integer& ldt, dReal& y, integer& ldy );

    void
    dlaic1
    ( integer& job, integer& j, dReal& x, dReal& sest, dReal& w, dReal& gamma, dReal& sestpr, dReal& s, dReal& c );

    void
    dlaln2
    ( logical& ltrans, integer& na, integer& nw, dReal& smin, dReal& ca, dReal& a, integer& lda, dReal& d1, dReal& d2, dReal& b, integer& ldb, dReal& wr, dReal& wi, dReal& x, integer& ldx, dReal& scale, dReal& xnorm, integer& info );

    dReal
    dlamch
    ( const character& cmach );

    void
    dlamrg
    ( integer& n1, integer& n2, dReal& a, integer &dtrd1, integer& dtrd2, integer& index );

    dReal
    dlangb
    ( const character& norm, integer& n, integer& kl, integer& ku, dReal& ab, integer& ldab, dReal& work );

    dReal
    dlange
    ( const character& norm, integer& m, integer& n, dReal& a, integer &lda, dReal& work );

    dReal
    dlangt
    ( const character& norm, integer& n, dReal& dl, dReal& d, dReal& du );

    dReal
    dlanhs
    ( const character& norm, integer& n, dReal& a, integer& lda, dReal& work );

    dReal
    dlansb
    ( const character& norm, const character& uplo, integer& n, integer& k, dReal &ab, integer& ldab, dReal& work );

    dReal
    dlansp
    ( const character& norm, const character& uplo, integer& n, dReal& ap, dReal& work );

    dReal
    dlanst
    ( const character& norm, integer& n, dReal& d, dReal& e );

    dReal
    dlansy
    ( const character& norm, const character& uplo, integer& n, dReal& a, integer &lda, dReal& work );

    dReal
    dlantb
    ( const character& norm, const character& uplo, const character& diag, integer& n, integer& k, dReal& ab, integer& ldab, dReal& work );

    dReal
    dlantp
    ( const character& norm, const character& uplo, const character& diag, integer& n, dReal &ap, dReal& work );

    dReal
    dlantr
    ( const character& norm, const character& uplo, const character& diag, integer& m, integer& n, dReal& a, integer& lda, dReal& work );

    void
    dlanv2
    ( dReal& a, dReal& b, dReal& c, dReal& d, dReal& rt1r, dReal& rt1i, dReal& rt2r, dReal& rt2i, dReal& cs, dReal& sn );

    void
    dlapll
    ( integer& n, dReal& x, integer& incx, dReal& y, integer& incy, dReal& ssmin );

    void
    dlapmt
    ( logical& forwrd, integer& m, integer& n, dReal& x, integer& ldx, integer& k );

    dReal
    dlapy2
    ( dReal& x, dReal& y );

    dReal
    dlapy3
    ( dReal& x, dReal& y, dReal& z );

    void
    dlaqgb
    ( integer& m, integer& n, integer& kl, integer& ku, dReal& ab, integer& ldab, dReal& r, dReal& c, dReal& rowcnd, dReal& colcnd, dReal& amax, character& equed );

    void
    dlaqge
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& r, dReal& c, dReal& rowcnd, dReal& colcnd, dReal& amax, character& equed );

    void
    dlaqsb
    ( const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    dlaqsp
    ( const character& uplo, integer& n, dReal& ap, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    dlaqsy
    ( const character& uplo, integer& n, dReal& a, integer& lda, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    dlaqtr
    ( logical& ltran, logical& lsReal, integer& n, dReal& t, integer& ldt, dReal& b, dReal& w, dReal &scale, dReal& x, dReal& work, integer& info );

    void
    dlar2v
    ( integer& n, dReal& x, dReal& y, dReal& z, integer& incx, dReal& c, dReal& s, integer& incc );

    void
    dlarf
    ( const character& side, integer& m, integer& n, dReal& v, integer& incv, dReal& tau, dReal& c, integer& ldc, dReal& work );

    void
    dlarfb
    ( const character& side, const character& trans, const character& direct, character& storev, integer& m, integer& n, integer& k, dReal& v, integer& ldv, dReal& t, integer& ldt, dReal& c, integer& ldc, dReal& work, integer& ldwork );

    void
    dlarfg
    ( integer& n, dReal& alpha, dReal& x, integer& incx, dReal& tau );

    void
    dlarft
    ( const character& direct, character& storev, integer& n, integer& k, dReal& v, integer& ldv, dReal& tau, dReal& t, integer& ldt );

    void
    dlarfx
    ( const character& side, integer& m, integer& n, dReal& v, dReal& tau, dReal& c, integer& ldc, dReal& work );

    void
    dlargv
    ( integer& n, dReal& x, integer& incx, dReal& y, integer& incy, dReal& c, integer& incc );

    void
    dlarnv
    ( integer& idist, integer& iseed, integer& n, dReal& x );

    void
    dlartg
    ( dReal& f, dReal& g, dReal& cs, dReal& sn, dReal& r );

    void
    dlartv
    ( integer& n, dReal& x, integer& incx, dReal& y, integer& incy, dReal& c, dReal& s, integer& incc );

    void
    dlaruv
    ( integer& iseed, integer& n, dReal& x );

    void
    dlas2
    ( dReal& f, dReal& g, dReal& h, dReal& ssmin, dReal& ssmax );

    void
    dlascl
    ( const character& type, integer& kl, integer& ku, dReal &cfrom, dReal& cto, integer& m, integer& n, dReal& a, integer& lda, integer& info );

    void
    dlaset
    ( const character& uplo, integer& m, integer& n, dReal& alpha, dReal& beta, dReal& a, integer& lda );

    void
    dlasq1
    ( integer& n, dReal& d, dReal& e, dReal& work, integer& info );

    void
    dlasq2
    ( integer& m, dReal& q, dReal& e, dReal& qq, dReal& ee, dReal& eps, dReal& tol2, dReal& small2, dReal& sup, integer& kend, integer& info );

    void
    dlasq3
    ( integer& n, dReal& q, dReal& e, dReal& qq, dReal& ee, dReal& sup, dReal& sigma, integer& kend, integer& off, integer& iphase, integer& iconv, dReal& eps, dReal& tol2, dReal& small2 );

    void
    dlasq4
    ( integer& n, dReal& q, dReal& e, dReal& tau, dReal& sup );

    void
    dlasr
    ( const character& side, const character& pivot, const character& direct, integer& m, integer& n, dReal& c, dReal& s, dReal& a, integer& lda );

    void
    dlasrt
    ( const character& id, integer& n, dReal& d, integer& info );

    void
    dlassq
    ( integer& n, dReal& x, integer& incx, dReal& scale, dReal& sumsq );

    void
    dlasv2
    ( dReal& f, dReal& g, dReal& h, dReal& ssmin, dReal& ssmax, dReal& snr, dReal& csr, dReal& snl, dReal& csl );

    void
    dlaswp
    ( integer& n, dReal& a, integer& lda, integer &k1, integer& k2, integer& ipiv, integer& incx );

    void
    dlasy2
    ( logical& ltranl, logical& ltranr, integer& isgn, integer& n1, integer& n2, dReal& tl, integer& ldtl, dReal& tr, integer& ldtr, dReal& b, integer& ldb, dReal& scale, dReal& x, integer& ldx, dReal& xnorm, integer& info );

    void
    dlasyf
    ( const character& uplo, integer& n, integer& nb, integer& kb, dReal& a, integer& lda, integer& ipiv, dReal& w, integer& ldw, integer& info );

    void
    dlatbs
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& x, dReal& scale, dReal& cnorm, integer& info );

    void
    dlatps
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, dReal& ap, dReal& x, dReal& scale, dReal& cnorm, integer& info );

    void
    dlatrd
    ( const character& uplo, integer& n, integer& nb, dReal& a, integer& lda, dReal& e, dReal& tau, dReal& w, integer& ldw );

    void
    dlatrs
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, dReal& a, integer& lda, dReal& x, dReal& scale, dReal& cnorm, integer& info );

    void
    dlatzm
    ( const character& side, integer& m, integer& n, dReal& v, integer& incv, dReal& tau, dReal& c1, dReal& c2, integer& ldc, dReal& work );

    void
    dlauu2
    ( const character& uplo, integer& n, dReal& a, integer& lda, integer& info );

    void
    dlauum
    ( const character& uplo, integer& n, dReal& a, integer& lda, integer& info );

    void
    dopgtr
    ( const character& uplo, integer& n, dReal& ap, dReal& tau, dReal& q, integer& ldq, dReal& work, integer& info );

    void
    dopmtr
    ( const character& side, const character& uplo, const character& trans, integer& m, integer& n, dReal& ap, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& info );

    void
    dorg2l
    ( integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dorg2r
    ( integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dorgbr
    ( const character& vect, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dorghr
    ( integer& n, integer& ilo, integer& ihi, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dorgl2
    ( integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dorglq
    ( integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dorgql
    ( integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dorgqr
    ( integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dorgr2
    ( integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& info );

    void
    dorgrq
    ( integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dorgtr
    ( const character& uplo, integer& n, dReal& a, integer& lda, dReal& tau, dReal& work, integer& lwork, integer& info );

    void
    dorm2l
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& info );

    void
    dorm2r
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& info );

    void
    dormbr
    ( const character& vect, const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& lwork, integer& info );

    void
    dormhr
    ( const character& side, const character& trans, integer& m, integer& n, integer& ilo, integer& ihi, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& lwork, integer& info );

    void
    dorml2
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& info );

    void
    dormlq
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& lwork, integer& info );

    void
    dormql
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& lwork, integer& info );

    void
    dormqr
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& lwork, integer& info );

    void
    dormr2
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& info );

    void
    dormrq
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& lwork, integer& info );

    void
    dormtr
    ( const character& side, const character& uplo, const character& trans, integer& m, integer& n, dReal& a, integer& lda, dReal& tau, dReal& c, integer& ldc, dReal& work, integer& lwork, integer& info );

    void
    dpbcon
    ( const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& anorm, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dpbequ
    ( const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& s, dReal& scond, dReal& amax, integer& info );

    void
    dpbrfs
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, dReal& ab, integer& ldab, dReal& afb, integer& ldafb, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dpbstf
    ( const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, integer& info );

    void
    dpbsv
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, dReal& ab, integer& ldab, dReal& b, integer& ldb, integer& info );

    void
    dpbsvx
    ( const character& fact, const character& uplo, integer& n, integer& kd, integer& nrhs, dReal& ab, integer& ldab, dReal& afb, integer& ldafb, character& equed, dReal& s, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dpbtf2
    ( const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, integer& info );

    void
    dpbtrf
    ( const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, integer& info );

    void
    dpbtrs
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, dReal& ab, integer& ldab, dReal& b, integer& ldb, integer& info );

    void
    dpocon
    ( const character& uplo, integer& n, dReal& a, integer& lda, dReal& anorm, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dpoequ
    ( integer& n, dReal& a, integer& lda, dReal& s, dReal& scond, dReal& amax, integer& info );

    void
    dporfs
    ( const character& uplo, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& af, integer& ldaf, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );
    */

    /*
    void
    dposvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& af, integer& ldaf, character& equed, dReal& s, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dpotf2
    ( const character& uplo, integer& n, dReal& a, integer& lda, integer& info );
    */

    /*
    void
    dpotri
    ( const character& uplo, integer& n, dReal& a, integer& lda, integer& info );

    void
    dpotrs
    ( const character& uplo, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& b, integer& ldb, integer& info );

    void
    dppcon
    ( const character& uplo, integer& n, dReal& ap, dReal& anorm, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dppequ
    ( const character& uplo, integer& n, dReal& ap, dReal& s, dReal& scond, dReal& amax, integer& info );

    void
    dpprfs
    ( const character& uplo, integer& n, integer& nrhs, dReal& ap, dReal& afp, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dppsv
    ( const character& uplo, integer& n, integer& nrhs, dReal &ap, dReal& b, integer& ldb, integer& info );

    void
    dppsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dReal& ap, dReal& afp, character& equed, dReal& s, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dpptrf
    ( const character& uplo, integer& n, dReal& ap, integer& info );

    void
    dpptri
    ( const character& uplo, integer& n, dReal& ap, integer& info );

    void
    dpptrs
    ( const character& uplo, integer& n, integer& nrhs, dReal& ap, dReal& b, integer& ldb, integer& info );

    void
    dptcon
    ( integer& n, dReal& d, dReal& e, dReal& anorm, dReal& rcond, dReal& work, integer& info );

    void
    dpteqr
    ( const character& compz, integer& n, dReal& d, dReal& e, dReal& z, integer& ldz, dReal& work, integer &info );

    void
    dptrfs
    ( integer& n, integer& nrhs, dReal& d, dReal& e, dReal& df, dReal& ef, dReal& b, integer &ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& info );

    void
    dptsv
    ( integer& n, integer& nrhs, dReal& d, dReal& e, dReal& b, integer& ldb, integer& info );

    void
    dptsvx
    ( const character& fact, integer& n, integer& nrhs, dReal& d, dReal& e, dReal& df, dReal& ef, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& info );

    void
    dpttrf
    ( integer& n, dReal& d, dReal& e, integer& info );

    void
    dpttrs
    ( integer& n, integer& nrhs, dReal& d, dReal& e, dReal& b, integer& ldb, integer& info );

    void
    drscl
    ( integer& n, dReal& sa, dReal& sx, integer& incx );

    void
    dsbev
    ( const character& jobz, const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& w, dReal& z, integer& ldz, dReal& work, integer& info );

    void
    dsbevd
    ( const character& jobz, const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& w, dReal& z, integer& ldz, dReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    dsbevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& q, integer& ldq, dReal& vl, dReal& vu, integer& il, integer& iu, dReal& abstol, integer& m, dReal& w, dReal& z, integer &ldz, dReal& work, integer& iwork, integer& ifail, integer& info );

    void
    dsbgst
    ( const character& vect, const character& uplo, integer& n, integer& ka, integer& kb, dReal& ab, integer& ldab, dReal& bb, integer& ldbb, dReal& x, integer& ldx, dReal& work, integer& info );

    void
    dsbgv
    ( const character& jobz, const character& uplo, integer& n, integer& ka, integer& kb, dReal& ab, integer& ldab, dReal& bb, integer& ldbb, dReal& w, dReal& z, integer& ldz, dReal& work, integer& info );

    void
    dsbtrd
    ( const character& vect, const character& uplo, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& d, dReal& e, dReal& q, integer& ldq, dReal& work, integer& info );

    void
    dspcon
    ( const character& uplo, integer& n, dReal& ap, integer& ipiv, dReal& anorm, dReal& rcond, dReal& work, integer &iwork, integer& info );
    */

    /*
    void
    dspevd
    ( const character& jobz, const character& uplo, integer& n, dReal& ap, dReal& w, dReal& z, integer& ldz, dReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    dspevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, dReal& ap, dReal& vl, dReal& vu, integer& il, integer& iu, dReal& abstol, integer& m, dReal& w, dReal& z, integer& ldz, dReal& work, integer& iwork, integer& ifail, integer& info );

    void
    dspgst
    ( integer& itype, const character& uplo, integer& n, dReal& ap, dReal& bp, integer& info );

    void
    dspgv
    ( integer& itype, const character& jobz, const character& uplo, integer& n, dReal& ap, dReal& bp, dReal& w, dReal& z, integer& ldz, dReal& work, integer& info );

    void
    dsprfs
    ( const character& uplo, integer& n, integer& nrhs, dReal& ap, dReal& afp, integer& ipiv, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dspsv
    ( const character& uplo, integer& n, integer& nrhs, dReal &ap, integer& ipiv, dReal& b, integer& ldb, integer& info );

    void
    dspsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dReal& ap, dReal& afp, integer& ipiv, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dsptrd
    ( const character& uplo, integer& n, dReal& ap, dReal& d, dReal& e, dReal& tau, integer& info );
    */

    /*
    void
    dsptrs
    ( const character& uplo, integer& n, integer& nrhs, dReal& ap, integer& ipiv, dReal& b, integer& ldb, integer& info );

    void
    dstebz
    ( const character& range, const character& order, integer& n, dReal &vl, dReal& vu, integer& il, integer& iu, dReal& abstol, dReal& d, dReal& e, integer& m, integer& nsplit, dReal &w, integer& iblock, integer& isplit, dReal& work, integer& iwork, integer& info );

    void
    dstedc
    ( const character& compz, integer& n, dReal& d, dReal& e, dReal& z, integer& ldz, dReal& work, integer &lwork, integer& iwork, integer& liwork, integer& info );

    void
    dstein
    ( integer& n, dReal& d, dReal& e, integer& m, dReal& w, integer& iblock, integer& isplit, dReal& z, integer& ldz, dReal& work, integer& iwork, integer& ifail, integer& info );

    void
    dsteqr
    ( const character& compz, integer& n, dReal& d, dReal& e, dReal& z, integer& ldz, dReal& work, integer &info );

    void
    dsterf
    ( integer& n, dReal& d, dReal& e, integer& info );

    void
    dstev
    ( const character& jobz, integer& n, dReal& d, dReal &e, dReal& z, integer& ldz, dReal& work, integer& info );

    void
    dstevd
    ( const character& jobz, integer& n, dReal& d, dReal& e, dReal& z, integer& ldz, dReal& work, integer &lwork, integer& iwork, integer& liwork, integer& info );

    void
    dstevx
    ( const character& jobz, const character& range, integer& n, dReal& d, dReal& e, dReal& vl, dReal& vu, integer& il, integer& iu, dReal& abstol, integer& m, dReal& w, dReal& z, integer& ldz, dReal& work, integer& iwork, integer& ifail, integer& info );

    void
    dsycon
    ( const character& uplo, integer& n, dReal& a, integer& lda, integer& ipiv, dReal& anorm, dReal& rcond, dReal& work, integer& iwork, integer& info );
    */

    /*
    void
    dsyevd
    ( const character& jobz, const character& uplo, integer& n, dReal& a, integer& lda, dReal& w, dReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    dsyevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, dReal& a, integer& lda, dReal& vl, dReal& vu, integer& il, integer& iu, dReal& abstol, integer& m, dReal& w, dReal& z, integer& ldz, dReal& work, integer& lwork, integer& iwork, integer& ifail, integer& info );

    void
    dsygs2
    ( integer& itype, const character& uplo, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb, integer& info );

    void
    dsygst
    ( integer& itype, const character& uplo, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb, integer& info );
    */

    /*
    void
    dsyrfs
    ( const character& uplo, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& af, integer& ldaf, integer& ipiv, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dsysv
    ( const character& uplo, integer& n, integer& nrhs, dReal &a, integer& lda, integer& ipiv, dReal& b, integer& ldb, dReal& work, integer& lwork, integer& info );

    void
    dsysvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& af, integer& ldaf, integer& ipiv, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dReal& work, integer& lwork, integer& iwork, integer& info );

    void
    dsytd2
    ( const character& uplo, integer& n, dReal& a, integer& lda, dReal& d, dReal& e, dReal& tau, integer& info );

    void
    dsytf2
    ( const character& uplo, integer& n, dReal& a, integer& lda, integer& ipiv, integer& info );

    void
    dsytrd
    ( const character& uplo, integer& n, dReal& a, integer& lda, dReal& d, dReal& e, dReal& tau, dReal& work, integer& lwork, integer& info );
    */

    /*
    void
    dsytrs
    ( const character& uplo, integer& n, integer& nrhs, dReal& a, integer& lda, integer& ipiv, dReal& b, integer& ldb, integer& info );

    void
    dtbcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, integer& kd, dReal& ab, integer& ldab, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dtbrfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& kd, integer& nrhs, dReal& ab, integer& ldab, dReal &b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dtbtrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& kd, integer& nrhs, dReal& ab, integer& ldab, dReal &b, integer& ldb, integer& info );

    void
    dtgevc
    ( const character& side, const character& howmny, logical& select, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& vl, integer& ldvl, dReal& vr, integer& ldvr, integer &mm, integer& m, dReal& work, integer& info );

    void
    dtgsja
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& p, integer& n, integer& k, integer& l, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& tola, dReal& tolb, dReal& alpha, dReal& beta, dReal& u, integer& ldu, dReal& v, integer& ldv, dReal& q, integer& ldq, dReal& work, integer& ncycle, integer& info );

    void
    dtpcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, dReal& ap, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dtprfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, dReal& ap, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dtptri
    ( const character& uplo, const character& diag, integer& n, dReal& ap, integer& info );

    void
    dtptrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, dReal& ap, dReal& b, integer& ldb, integer& info );

    void
    dtrcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, dReal& a, integer& lda, dReal& rcond, dReal& work, integer& iwork, integer& info );

    void
    dtrevc
    ( const character& side, const character& howmny, logical& select, integer& n, dReal& t, integer& ldt, dReal& vl, integer& ldvl, dReal& vr, integer& ldvr, integer& mm, integer& m, dReal& work, integer& info );

    void
    dtrexc
    ( const character& compq, integer& n, dReal& t, integer& ldt, dReal& q, integer& ldq, integer& ifst, integer& ilst, dReal& work, integer& info );

    void
    dtrrfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& x, integer& ldx, dReal& ferr, dReal& berr, dReal& work, integer& iwork, integer& info );

    void
    dtrsen
    ( const character& job, const character& compq, logical& select, integer &n, dReal& t, integer& ldt, dReal& q, integer& ldq, dReal& wr, dReal& wi, integer& m, dReal& s, dReal &sep, dReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    dtrsna
    ( const character& job, const character& howmny, logical& select, integer& n, dReal& t, integer& ldt, dReal& vl, integer& ldvl, dReal& vr, integer& ldvr, dReal& s, dReal& sep, integer& mm, integer& m, dReal& work, integer& ldwork, integer& iwork, integer& info );

    void
    dtrsyl
    ( const character& trana, const character& tranb, integer& isgn, integer &m, integer& n, dReal& a, integer& lda, dReal& b, integer& ldb, dReal& c, integer& ldc, dReal& scale, integer& info );

    void
    dtrti2
    ( const character& uplo, const character& diag, integer& n, dReal& a, integer& lda, integer& info );
    */

    /*
    void
    dtrtrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, dReal& a, integer& lda, dReal& b, integer& ldb, integer& info );

    void
    dtzrqf
    ( integer& m, integer& n, dReal& a, integer& lda, dReal& tau, integer& info );

    dReal
    dzsum1
    ( integer& n, dComplex& cx, integer& incx );

    integer
    icmax1
    ( integer& n, sComplex& cx, integer& incx );

    integer
    izmax1
    ( integer& n, dComplex& cx, integer& incx );

    void
    sbdsqr
    ( const character& uplo, integer& n, integer& ncvt, integer& nru, integer& ncc, sReal& d, sReal& e, sReal& vt, integer& ldvt, sReal& u, integer& ldu, sReal& c, integer& ldc, sReal& work, integer& info );

    dReal
    scsum1
    ( integer& n, sComplex& cx, integer& incx );

    void
    sdisna
    ( const character& job, integer& m, integer& n, sReal& d, sReal &sep, integer& info );

    void
    sgbbrd
    ( const character& vect, integer& m, integer& n, integer& ncc, integer& kl, integer& ku, sReal& ab, integer& ldab, sReal& d, sReal& e, sReal& q, integer& ldq, sReal& pt, integer& ldpt, sReal& c, integer& ldc, sReal& work, integer& info );

    void
    sgbcon
    ( const character& norm, integer& n, integer& kl, integer& ku, sReal& ab, integer& ldab, integer& ipiv, sReal& anorm, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    sgbequ
    ( integer& m, integer& n, integer& kl, integer& ku, sReal& ab, integer& ldab, sReal& r, sReal& c, sReal& rowcnd, sReal& colcnd, sReal& amax, integer& info );

    void
    sgbrfs
    ( const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, sReal& ab, integer& ldab, sReal& afb, integer& ldafb, integer& ipiv, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    sgbsv
    ( integer& n, integer& kl, integer& ku, integer& nrhs, sReal& ab, integer& ldab, integer& ipiv, sReal& b, integer& ldb, integer& info );

    void
    sgbsvx
    ( const character& fact, const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, sReal& ab, integer& ldab, sReal& afb, integer& ldafb, integer& ipiv, character& equed, sReal& r, sReal& c, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    sgbtf2
    ( integer& m, integer& n, integer& kl, integer& ku, sReal& ab, integer& ldab, integer& ipiv, integer& info );

    void
    sgbtrf
    ( integer& m, integer& n, integer& kl, integer& ku, sReal& ab, integer& ldab, integer& ipiv, integer& info );

    void
    sgbtrs
    ( const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, sReal& ab, integer& ldab, integer& ipiv, sReal& b, integer& ldb, integer& info );

    void
    sgebak
    ( const character& job, const character& side, integer& n, integer& ilo, integer& ihi, sReal& scale, integer& m, sReal& v, integer& ldv, integer &info );

    void
    sgebal
    ( const character& job, integer& n, sReal& a, integer& lda, integer& ilo, integer& ihi, sReal& scale, integer& info );

    void
    sgebd2
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& d, sReal& e, sReal& tauq, sReal& taup, sReal& work, integer& info );

    void
    sgebrd
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& d, sReal& e, sReal& tauq, sReal& taup, sReal& work, integer& lwork, integer& info );

    void
    sgecon
    ( const character& norm, integer& n, sReal& a, integer& lda, sReal& anorm, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    sgeequ
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& r, sReal& c, sReal& rowcnd, sReal& colcnd, sReal& amax, integer& info );

    void
    sgees
    ( const character& jobvs, const character& sort, logical (*select)( sReal&, sReal& ), integer& n, sReal& a, integer& lda, integer& sdim, sReal& wr, sReal& wi, sReal& vs, integer& ldvs, sReal& work, integer& lwork, logical& bwork, integer& info );

    void
    sgeesx
    ( const character& jobvs, const character& sort, logical (*select)( sReal&, sReal& ), const character& sense, integer& n, sReal& a, integer& lda, integer& sdim, sReal& wr, sReal& wi, sReal& vs, integer& ldvs, sReal& rconde, sReal& rcondv, sReal& work, integer& lwork, integer& iwork, integer& liwork, logical& bwork, integer& info );

    void
    sgeev
    ( const character& jobvl, const character& jobvr, integer& n, sReal& a, integer& lda, sReal& wr, sReal& wi, sReal& vl, integer& ldvl, sReal& vr, integer& ldvr, sReal& work, integer& lwork, integer& info );

    void
    sgeevx
    ( const character& balanc, const character& jobvl, const character& jobvr, const character& sense, integer& n, sReal& a, integer& lda, sReal& wr, sReal& wi, sReal& vl, integer& ldvl, sReal& vr, integer& ldvr, integer& ilo, integer& ihi, sReal& scale, sReal& abnrm, sReal& rconde, sReal& rcondv, sReal& work, integer& lwork, integer& iwork, integer& info );

    void
    sgegs
    ( const character& jobvsl, const character& jobvsr, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& alphar, sReal& alphai, sReal &beta, sReal& vsl, integer& ldvsl, sReal& vsr, integer& ldvsr, sReal& work, integer& lwork, integer& info );

    void
    sgegv
    ( const character& jobvl, const character& jobvr, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& alphar, sReal& alphai, sReal &beta, sReal& vl, integer& ldvl, sReal& vr, integer& ldvr, sReal& work, integer& lwork, integer& info );

    void
    sgehd2
    ( integer& n, integer& ilo, integer& ihi, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sgehrd
    ( integer& n, integer& ilo, integer& ihi, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sgelq2
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sgelqf
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sgels
    ( const character& trans, integer& m, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& work, integer& lwork, integer& info );

    void
    sgelss
    ( integer& m, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& s, sReal& rcond, integer& rank, sReal& work, integer& lwork, integer& info );

    void
    sgelsx
    ( integer& m, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& b, integer& ldb, integer& jpvt, sReal& rcond, integer& rank, sReal& work, integer& info );

    void
    sgeql2
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sgeqlf
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sgeqpf
    ( integer& m, integer& n, sReal& a, integer& lda, integer& jpvt, sReal& tau, sReal& work, integer& info );

    void
    sgeqr2
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sgeqrf
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sgerfs
    ( const character& trans, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& af, integer& ldaf, integer& ipiv, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    sgerq2
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sgerqf
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sgesv
    ( integer& n, integer& nrhs, sReal& a, integer& lda, integer& ipiv, sReal& b, integer& ldb, integer& info );

    void
    sgesvd
    ( const character& jobu, const character& jobvt, integer& m, integer& n, sReal& a, integer& lda, sReal& s, sReal& u, integer& ldu, sReal& vt, integer& ldvt, sReal& work, integer& lwork, integer& info );

    void
    sgesvx
    ( const character& fact, const character& trans, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& af, integer& ldaf, integer& ipiv, character& equed, sReal& r, sReal& c, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    sgetf2
    ( integer& m, integer& n, sReal& a, integer& lda, integer& ipiv, integer& info );

    void
    sgetrf
    ( integer& m, integer& n, sReal& a, integer& lda, integer& ipiv, integer& info );

    void
    sgetri
    ( integer& n, sReal& a, integer& lda, integer& ipiv, sReal& work, integer& lwork, integer& info );

    void
    sgetrs
    ( const character& trans, integer& n, integer& nrhs, sReal& a, integer& lda, integer& ipiv, sReal& b, integer& ldb, integer& info );

    void
    sggbak
    ( const character& job, const character& side, integer& n, integer& ilo, integer& ihi, sReal& lscale, sReal& rscale, integer& m, sReal& v, integer& ldv, integer& info );

    void
    sggbal
    ( const character& job, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb, integer& ilo, integer& ihi, sReal& lscale, sReal &rscale, sReal& work, integer& info );
    */

    /*
    void
    sgghrd
    ( const character& compq, const character& compz, integer& n, integer& ilo, integer& ihi, sReal& a, integer& lda, sReal& b, integer& ldb, sReal &q, integer& ldq, sReal& z, integer& ldz, integer& info );

    void
    sgglse
    ( integer& m, integer& n, integer& p, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& c, sReal& d, sReal& x, sReal& work, integer& lwork, integer& info );

    void
    sggqrf
    ( integer& n, integer& m, integer& p, sReal& a, integer& lda, sReal& taua, sReal& b, integer& ldb, sReal& taub, sReal& work, integer& lwork, integer& info );

    void
    sggrqf
    ( integer& m, integer& p, integer& n, sReal& a, integer& lda, sReal& taua, sReal& b, integer& ldb, sReal& taub, sReal& work, integer& lwork, integer& info );

    void
    sggsvd
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& n, integer& p, integer& k, integer& l, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& alpha, sReal& beta, sReal& u, integer& ldu, sReal& v, integer& ldv, sReal& q, integer& ldq, sReal& work, integer& iwork, integer& info );

    void
    sggsvp
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& p, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& tola, sReal& tolb, integer& k, integer& l, sReal& u, integer& ldu, sReal& v, integer& ldv, sReal& q, integer& ldq, integer& iwork, sReal& tau, sReal& work, integer& info );

    void
    sgtcon
    ( const character& norm, integer& n, sReal& dl, sReal& d, sReal& du, sReal& du2, integer& ipiv, sReal& anorm, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    sgtrfs
    ( const character& trans, integer& n, integer& nrhs, sReal& dl, sReal& d, sReal& du, sReal& dlf, sReal& df, sReal& duf, sReal& du2, integer& ipiv, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    sgtsv
    ( integer& n, integer& nrhs, sReal& dl, sReal& d, sReal& du, sReal& b, integer& ldb, integer& info );

    void
    sgtsvx
    ( const character& fact, const character& trans, integer& n, integer& nrhs, sReal& dl, sReal& d, sReal& du, sReal& dlf, sReal& df, sReal& duf, sReal& du2, integer& ipiv, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    sgttrf
    ( integer& n, sReal& dl, sReal& d, sReal& du, sReal& du2, integer& ipiv, integer& info );

    void
    sgttrs
    ( const character& trans, integer& n, integer& nrhs, sReal& dl, sReal& d, sReal& du, sReal& du2, integer& ipiv, sReal& b, integer& ldb, integer& info );

    void
    shgeqz
    ( const character& job, const character& compq, const character& compz, integer& n, integer& ilo, integer& ihi, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& alphar, sReal& alphai, sReal& beta, sReal& q, integer& ldq, sReal& z, integer& ldz, sReal& work, integer& lwork, integer& info );

    void
    shsein
    ( const character& side, const character& eigsrc, const character& initv, logical& select, integer& n, sReal& h, integer& ldh, sReal& wr, sReal& wi, sReal& vl, integer& ldvl, sReal& vr, integer& ldvr, integer& mm, integer& m, sReal& work, integer& ifaill, integer& ifailr, integer& info );

    void
    shseqr
    ( const character& job, const character& compz, integer& n, integer& ilo, integer& ihi, sReal& h, integer& ldh, sReal& wr, sReal& wi, sReal& z, integer& ldz, sReal& work, integer& lwork, integer& info );

    void
    slabad
    ( sReal& small, sReal& large );

    void
    slabrd
    ( integer& m, integer& n, integer& nb, sReal& a, integer& lda, sReal& d, sReal& e, sReal& tauq, sReal& taup, sReal& x, integer& ldx, sReal& y, integer& ldy );

    void
    slacon
    ( integer& n, sReal& v, sReal& x, integer& isgn, sReal& est, integer& kase );

    void
    slacpy
    ( const character& uplo, integer& m, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb );

    void
    sladiv
    ( sReal& a, sReal& b, sReal& c, sReal& d, sReal& p, sReal& q );

    void
    slae2
    ( sReal& a, sReal& b, sReal& c, sReal& rt1, sReal& rt2 );

    void
    slaebz
    ( integer& ijob, integer& nitmax, integer& n, integer& mmax, integer& minp, integer& nbmin, sReal& abstol, sReal& reltol, sReal& pivmin, sReal& d, sReal& e, sReal& e2, integer& nval, sReal &ab, sReal& c, integer& mout, integer& nab, sReal& work, integer& iwork, integer& info );

    void
    slaed0
    ( integer& icompq, integer& qsiz, integer& n, sReal &d, sReal& e, sReal& q, integer& ldq, sReal& qstore, integer& ldqs, sReal &work, integer& iwork, integer& info );

    void
    slaed1
    ( integer& n, sReal& d, sReal& q, integer& ldq, integer& indxq, sReal& rho, integer& cutpnt, sReal& work, integer& iwork, integer& info );

    void
    slaed2
    ( integer& k, integer& n, sReal& d, sReal& q, integer& ldq, integer& indxq, sReal& rho, integer& cutpnt, sReal& z, sReal& dlamda, sReal& q2, integer& ldq2, integer& indxc, sReal& w, integer& indxp, integer& indx, integer& coltyp, integer& info );

    void
    slaed3
    ( integer& k, integer& kstart, integer& kstop, integer& n, sReal& d, sReal& q, integer& ldq, sReal& rho, integer& cutpnt, sReal& dlamda, sReal& q2, integer& ldq2, integer& indxc, integer& ctot, sReal& w, sReal& s, integer& lds, integer& info );

    void
    slaed4
    ( integer& n, integer& i, sReal& d, sReal& z, sReal& delta, sReal& rho, sReal& dlam, integer& info );

    void
    slaed5
    ( integer& i, sReal& d, sReal& z, sReal& delta, sReal& rho, sReal& dlam );

    void
    slaed6
    ( integer& kniter, logical& orgati, sReal& rho, sReal& d, sReal& z, sReal& finit, sReal& tau, integer& info );

    void
    slaed7
    ( integer& icompq, integer& n, integer& qsiz, integer& tlvls, integer& curlvl, integer& curpbm, sReal& d, sReal& q, integer& ldq, integer& indxq, sReal& rho, integer& cutpnt, sReal& qstore, integer& qptr, integer& prmptr, integer& perm, integer& givptr, integer& givcol, sReal& givnum, sReal& work, integer& iwork, integer& info );

    void
    slaed8
    ( integer& icompq, integer& k, integer& n, integer &qsiz, sReal& d, sReal& q, integer& ldq, integer& indxq, sReal& rho, integer& cutpnt, sReal& z, sReal& dlamda, sReal& q2, integer& ldq2, sReal &w, integer& perm, integer& givptr, integer& givcol, sReal& givnum, integer& indxp, integer& indx, integer& info );

    void
    slaed9
    ( integer& k, integer& kstart, integer& kstop, integer& n, sReal& d, sReal& q, integer& ldq, sReal& rho, sReal& dlamda, sReal& w, sReal& s, integer& lds, integer& info );

    void
    slaeda
    ( integer& n, integer& tlvls, integer& curlvl, integer& curpbm, integer& prmptr, integer& perm, integer& givptr, integer& givcol, sReal& givnum, sReal& q, integer& qptr, sReal& z, sReal& ztemp, integer& info );

    void
    slaein
    ( logical& rightv, logical& noinit, integer& n, sReal& h, integer& ldh, sReal& wr, sReal& wi, sReal& vr, sReal& vi, sReal& b, integer& ldb, sReal& work, sReal& eps3, sReal& smlnum, sReal& bignum, integer& info );

    void
    slaev2
    ( sReal& a, sReal& b, sReal& c, sReal& rt1, sReal& rt2, sReal& cs1, sReal& sn1 );

    void
    slaexc
    ( logical& wantq, integer& n, sReal& t, integer& ldt, sReal& q, integer& ldq, integer& j1, integer& n1, integer& n2, sReal& work, integer& info );

    void
    slag2
    ( sReal& a, integer& lda, sReal& b, integer& ldb, sReal& safmin, sReal& scale1, sReal& scale2, sReal& wr1, sReal& wr2, sReal& wi );

    void
    slags2
    ( logical& upper, sReal& a1, sReal& a2, sReal& a3, sReal& b1, sReal& b2, sReal& b3, sReal& csu, sReal& snu, sReal& csv, sReal& snv, sReal& csq, sReal& snq );

    void
    slagtf
    ( integer& n, sReal& a, sReal& lambda, sReal& b, sReal &c, sReal& tol, sReal& d, integer& in, integer& info );

    void
    slagtm
    ( const character& trans, integer& n, integer& nrhs, sReal& alpha, sReal& dl, sReal& d, sReal& du, sReal& x, integer& ldx, sReal& beta, sReal& b, integer& ldb );

    void
    slagts
    ( integer& job, integer& n, sReal& a, sReal& b, sReal &c, sReal& d, integer& in, sReal& y, sReal& tol, integer& info );

    void
    slahqr
    ( logical& wantt, logical& wantz, integer& n, integer& ilo, integer& ihi, sReal& h, integer& ldh, sReal& wr, sReal& wi, integer& iloz, integer& ihiz, sReal& z, integer& ldz, integer& info );

    void
    slahrd
    ( integer& n, integer& k, integer& nb, sReal& a, integer& lda, sReal& tau, sReal& t, integer& ldt, sReal& y, integer& ldy );

    void
    slaic1
    ( integer& job, integer& j, sReal& x, sReal& sest, sReal& w, sReal& gamma, sReal& sestpr, sReal& s, sReal& c );

    void
    slaln2
    ( logical& ltrans, integer& na, integer& nw, sReal& smin, sReal& ca, sReal& a, integer& lda, sReal& d1, sReal& d2, sReal& b, integer& ldb, sReal& wr, sReal& wi, sReal& x, integer& ldx, sReal& scale, sReal& xnorm, integer& info );

    dReal
    slamch
    ( const character& cmach );

    void
    slamrg
    ( integer& n1, integer& n2, sReal& a, integer& strd1, integer& strd2, integer& index );

    dReal
    slangb
    ( const character& norm, integer& n, integer& kl, integer& ku, sReal& ab, integer& ldab, sReal& work );

    dReal
    slange
    ( const character& norm, integer& m, integer& n, sReal& a, integer& lda, sReal& work );

    dReal
    slangt
    ( const character& norm, integer& n, sReal& dl, sReal& d, sReal& du );

    dReal
    slanhs
    ( const character& norm, integer& n, sReal& a, integer& lda, sReal& work );

    dReal
    slansb
    ( const character& norm, const character& uplo, integer& n, integer& k, sReal& ab, integer& ldab, sReal& work );

    dReal
    slansp
    ( const character& norm, const character& uplo, integer& n, sReal& ap, sReal& work );

    dReal
    slanst
    ( const character& norm, integer& n, sReal& d, sReal& e );

    dReal
    slansy
    ( const character& norm, const character& uplo, integer& n, sReal& a, integer& lda, sReal& work );

    dReal
    slantb
    ( const character& norm, const character& uplo, const character& diag, integer& n, integer& k, sReal& ab, integer& ldab, sReal& work );

    dReal
    slantp
    ( const character& norm, const character& uplo, const character& diag, integer& n, sReal& ap, sReal& work );

    dReal
    slantr
    ( const character& norm, const character& uplo, const character& diag, integer& m, integer& n, sReal& a, integer& lda, sReal& work );

    void
    slanv2
    ( sReal& a, sReal& b, sReal& c, sReal& d, sReal& rt1r, sReal& rt1i, sReal& rt2r, sReal& rt2i, sReal& cs, sReal& sn );

    void
    slapll
    ( integer& n, sReal& x, integer& incx, sReal& y, integer& incy, sReal& ssmin );

    void
    slapmt
    ( logical& forwrd, integer& m, integer& n, sReal& x, integer& ldx, integer& k );

    dReal
    slapy2
    ( sReal& x, sReal& y );

    dReal
    slapy3
    ( sReal& x, sReal& y, sReal& z );

    void
    slaqgb
    ( integer& m, integer& n, integer& kl, integer& ku, sReal& ab, integer& ldab, sReal& r, sReal& c, sReal& rowcnd, sReal& colcnd, sReal& amax, character& equed );

    void
    slaqge
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& r, sReal& c, sReal& rowcnd, sReal& colcnd, sReal& amax, character& equed );

    void
    slaqsb
    ( const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    slaqsp
    ( const character& uplo, integer& n, sReal& ap, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    slaqsy
    ( const character& uplo, integer& n, sReal& a, integer& lda, sReal& s, sReal& scond, sReal& amax, character& equed );

    void
    slaqtr
    ( logical& ltran, logical& lsReal, integer& n, sReal &t, integer& ldt, sReal& b, sReal& w, sReal& scale, sReal& x, sReal& work, integer& info );

    void
    slar2v
    ( integer& n, sReal& x, sReal& y, sReal& z, integer& incx, sReal& c, sReal& s, integer& incc );

    void
    slarf
    ( const character& side, integer& m, integer& n, sReal& v, integer& incv, sReal& tau, sReal& c, integer& ldc, sReal& work );

    void
    slarfb
    ( const character& side, const character& trans, const character& direct, character& storev, integer& m, integer& n, integer& k, sReal& v, integer& ldv, sReal& t, integer& ldt, sReal& c, integer& ldc, sReal& work, integer& ldwork );

    void
    slarfg
    ( integer& n, sReal& alpha, sReal& x, integer& incx, sReal& tau );

    void
    slarft
    ( const character& direct, character& storev, integer& n, integer& k, sReal& v, integer& ldv, sReal& tau, sReal& t, integer& ldt );

    void
    slarfx
    ( const character& side, integer& m, integer& n, sReal& v, sReal& tau, sReal& c, integer& ldc, sReal& work );

    void
    slargv
    ( integer& n, sReal& x, integer& incx, sReal& y, integer& incy, sReal& c, integer& incc );

    void
    slarnv
    ( integer& idist, integer& iseed, integer& n, sReal &x );

    void
    slartg
    ( sReal& f, sReal& g, sReal& cs, sReal& sn, sReal& r );

    void
    slartv
    ( integer& n, sReal& x, integer& incx, sReal& y, integer& incy, sReal& c, sReal& s, integer& incc );

    void
    slaruv
    ( integer& iseed, integer& n, sReal& x );

    void
    slas2
    ( sReal& f, sReal& g, sReal& h, sReal& ssmin, sReal& ssmax );

    void
    slascl
    ( const character& type, integer& kl, integer& ku, sReal& cfrom, sReal& cto, integer& m, integer& n, sReal& a, integer& lda, integer& info );

    void
    slaset
    ( const character& uplo, integer& m, integer& n, sReal& alpha, sReal& beta, sReal& a, integer& lda );

    void
    slasq1
    ( integer& n, sReal& d, sReal& e, sReal& work, integer& info );

    void
    slasq2
    ( integer& m, sReal& q, sReal& e, sReal& qq, sReal& ee, sReal& eps, sReal& tol2, sReal& small2, sReal& sup, integer& kend, integer& info );

    void
    slasq3
    ( integer& n, sReal& q, sReal& e, sReal& qq, sReal& ee, sReal& sup, sReal& sigma, integer& kend, integer& off, integer& iphase, integer& iconv, sReal& eps, sReal& tol2, sReal& small2 );

    void
    slasq4
    ( integer& n, sReal& q, sReal& e, sReal& tau, sReal& sup );

    void
    slasr
    ( const character& side, const character& pivot, const character& direct, integer& m, integer& n, sReal& c, sReal& s, sReal& a, integer& lda );

    void
    slasrt
    ( const character& id, integer& n, sReal& d, integer& info );

    void
    slassq
    ( integer& n, sReal& x, integer& incx, sReal& scale, sReal& sumsq );

    void
    slasv2
    ( sReal& f, sReal& g, sReal& h, sReal& ssmin, sReal& ssmax, sReal& snr, sReal& csr, sReal& snl, sReal& csl );

    void
    slaswp
    ( integer& n, sReal& a, integer& lda, integer& k1, integer& k2, integer& ipiv, integer& incx );

    void
    slasy2
    ( logical& ltranl, logical& ltranr, integer& isgn, integer& n1, integer& n2, sReal& tl, integer& ldtl, sReal& tr, integer& ldtr, sReal& b, integer& ldb, sReal& scale, sReal& x, integer& ldx, sReal &xnorm, integer& info );

    void
    slasyf
    ( const character& uplo, integer& n, integer& nb, integer& kb, sReal& a, integer& lda, integer& ipiv, sReal& w, integer& ldw, integer &info );

    void
    slatbs
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& x, sReal& scale, sReal& cnorm, integer& info );

    void
    slatps
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, sReal& ap, sReal& x, sReal& scale, sReal& cnorm, integer& info );

    void
    slatrd
    ( const character& uplo, integer& n, integer& nb, sReal& a, integer& lda, sReal& e, sReal& tau, sReal& w, integer& ldw );

    void
    slatrs
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, sReal& a, integer& lda, sReal& x, sReal& scale, sReal &cnorm, integer& info );

    void
    slatzm
    ( const character& side, integer& m, integer& n, sReal& v, integer& incv, sReal& tau, sReal& c1, sReal& c2, integer& ldc, sReal& work );

    void
    slauu2
    ( const character& uplo, integer& n, sReal& a, integer& lda, integer& info );

    void
    slauum
    ( const character& uplo, integer& n, sReal& a, integer& lda, integer& info );

    void
    sopgtr
    ( const character& uplo, integer& n, sReal& ap, sReal& tau, sReal& q, integer& ldq, sReal& work, integer& info );

    void
    sopmtr
    ( const character& side, const character& uplo, const character& trans, integer& m, integer& n, sReal& ap, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& info );

    void
    sorg2l
    ( integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sorg2r
    ( integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sorgbr
    ( const character& vect, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer &info );

    void
    sorghr
    ( integer& n, integer& ilo, integer& ihi, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sorgl2
    ( integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sorglq
    ( integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sorgql
    ( integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sorgqr
    ( integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sorgr2
    ( integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& info );

    void
    sorgrq
    ( integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sorgtr
    ( const character& uplo, integer& n, sReal& a, integer& lda, sReal& tau, sReal& work, integer& lwork, integer& info );

    void
    sorm2l
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& info );

    void
    sorm2r
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& info );

    void
    sormbr
    ( const character& vect, const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& lwork, integer& info );

    void
    sormhr
    ( const character& side, const character& trans, integer& m, integer& n, integer& ilo, integer& ihi, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& lwork, integer& info );

    void
    sorml2
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& info );

    void
    sormlq
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& lwork, integer& info );

    void
    sormql
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& lwork, integer& info );

    void
    sormqr
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& lwork, integer& info );

    void
    sormr2
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& info );

    void
    sormrq
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& lwork, integer& info );

    void
    sormtr
    ( const character& side, const character& uplo, const character& trans, integer& m, integer& n, sReal& a, integer& lda, sReal& tau, sReal& c, integer& ldc, sReal& work, integer& lwork, integer& info );

    void
    spbcon
    ( const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& anorm, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    spbequ
    ( const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& s, sReal& scond, sReal& amax, integer& info );

    void
    spbrfs
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, sReal& ab, integer& ldab, sReal& afb, integer& ldafb, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    spbstf
    ( const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, integer& info );

    void
    spbsv
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, sReal& ab, integer& ldab, sReal& b, integer& ldb, integer& info );

    void
    spbsvx
    ( const character& fact, const character& uplo, integer& n, integer& kd, integer& nrhs, sReal& ab, integer& ldab, sReal& afb, integer& ldafb, character& equed, sReal& s, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    spbtf2
    ( const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, integer& info );

    void
    spbtrf
    ( const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, integer& info );

    void
    spbtrs
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, sReal& ab, integer& ldab, sReal& b, integer& ldb, integer& info );

    void
    spocon
    ( const character& uplo, integer& n, sReal& a, integer& lda, sReal& anorm, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    spoequ
    ( integer& n, sReal& a, integer& lda, sReal& s, sReal &scond, sReal& amax, integer& info );

    void
    sporfs
    ( const character& uplo, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& af, integer& ldaf, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );
    */

    /*
    void
    sposvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& af, integer& ldaf, character& equed, sReal& s, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    spotf2
    ( const character& uplo, integer& n, sReal& a, integer& lda, integer& info );
    */

    /*
    void
    spotri
    ( const character& uplo, integer& n, sReal& a, integer& lda, integer& info );

    void
    spotrs
    ( const character& uplo, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& b, integer& ldb, integer& info );

    void
    sppcon
    ( const character& uplo, integer& n, sReal& ap, sReal& anorm, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    sppequ
    ( const character& uplo, integer& n, sReal& ap, sReal& s, sReal& scond, sReal& amax, integer& info );

    void
    spprfs
    ( const character& uplo, integer& n, integer& nrhs, sReal& ap, sReal& afp, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    sppsv
    ( const character& uplo, integer& n, integer& nrhs, sReal& ap, sReal& b, integer& ldb, integer& info );

    void
    sppsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sReal& ap, sReal& afp, character& equed, sReal& s, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal &work, integer& iwork, integer& info );

    void
    spptrf
    ( const character& uplo, integer& n, sReal& ap, integer& info );

    void
    spptri
    ( const character& uplo, integer& n, sReal& ap, integer& info );

    void
    spptrs
    ( const character& uplo, integer& n, integer& nrhs, sReal& ap, sReal& b, integer& ldb, integer& info );

    void
    sptcon
    ( integer& n, sReal& d, sReal& e, sReal& anorm, sReal& rcond, sReal& work, integer& info );

    void
    spteqr
    ( const character& compz, integer& n, sReal& d, sReal& e, sReal& z, integer& ldz, sReal& work, integer& info );

    void
    sptrfs
    ( integer& n, integer& nrhs, sReal& d, sReal& e, sReal& df, sReal& ef, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& info );

    void
    sptsv
    ( integer& n, integer& nrhs, sReal& d, sReal& e, sReal &b, integer& ldb, integer& info );

    void
    sptsvx
    ( const character& fact, integer& n, integer& nrhs, sReal& d, sReal& e, sReal& df, sReal& ef, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal& work, integer& info );

    void
    spttrf
    ( integer& n, sReal& d, sReal& e, integer& info );

    void
    spttrs
    ( integer& n, integer& nrhs, sReal& d, sReal& e, sReal& b, integer& ldb, integer& info );

    void
    srscl
    ( integer& n, sReal& sa, sReal& sx, integer& incx );

    void
    ssbev
    ( const character& jobz, const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& w, sReal& z, integer& ldz, sReal& work, integer& info );

    void
    ssbevd
    ( const character& jobz, const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& w, sReal& z, integer& ldz, sReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    ssbevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& q, integer& ldq, sReal& vl, sReal& vu, integer& il, integer& iu, sReal& abstol, integer& m, sReal& w, sReal& z, integer& ldz, sReal& work, integer& iwork, integer& ifail, integer& info );

    void
    ssbgst
    ( const character& vect, const character& uplo, integer& n, integer& ka, integer& kb, sReal& ab, integer& ldab, sReal& bb, integer& ldbb, sReal& x, integer& ldx, sReal& work, integer& info );

    void
    ssbgv
    ( const character& jobz, const character& uplo, integer& n, integer& ka, integer& kb, sReal& ab, integer& ldab, sReal& bb, integer& ldbb, sReal& w, sReal& z, integer& ldz, sReal& work, integer& info );

    void
    ssbtrd
    ( const character& vect, const character& uplo, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& d, sReal& e, sReal& q, integer& ldq, sReal& work, integer& info );

    void
    sspcon
    ( const character& uplo, integer& n, sReal& ap, integer& ipiv, sReal& anorm, sReal& rcond, sReal& work, integer& iwork, integer& info );
    */

    /*
    void
    sspevd
    ( const character& jobz, const character& uplo, integer& n, sReal& ap, sReal& w, sReal& z, integer& ldz, sReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    sspevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, sReal& ap, sReal& vl, sReal& vu, integer& il, integer& iu, sReal& abstol, integer& m, sReal& w, sReal& z, integer& ldz, sReal& work, integer& iwork, integer& ifail, integer& info );

    void
    sspgst
    ( integer& itype, const character& uplo, integer& n, sReal& ap, sReal& bp, integer& info );

    void
    sspgv
    ( integer& itype, const character& jobz, const character& uplo, integer& n, sReal& ap, sReal& bp, sReal& w, sReal& z, integer& ldz, sReal& work, integer& info );

    void
    ssprfs
    ( const character& uplo, integer& n, integer& nrhs, sReal& ap, sReal& afp, integer& ipiv, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    sspsv
    ( const character& uplo, integer& n, integer& nrhs, sReal& ap, integer& ipiv, sReal& b, integer& ldb, integer& info );

    void
    sspsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sReal& ap, sReal& afp, integer& ipiv, sReal& b, integer& ldb, sReal &x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    ssptrd
    ( const character& uplo, integer& n, sReal& ap, sReal& d, sReal& e, sReal& tau, integer& info );
    */

    /*
    void
    ssptrs
    ( const character& uplo, integer& n, integer& nrhs, sReal& ap, integer& ipiv, sReal& b, integer& ldb, integer& info );

    void
    sstebz
    ( const character& range, const character& order, integer& n, sReal& vl, sReal& vu, integer& il, integer& iu, sReal& abstol, sReal& d, sReal& e, integer& m, integer& nsplit, sReal& w, integer& iblock, integer& isplit, sReal& work, integer& iwork, integer& info );

    void
    sstedc
    ( const character& compz, integer& n, sReal& d, sReal& e, sReal& z, integer& ldz, sReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    sstein
    ( integer& n, sReal& d, sReal& e, integer& m, sReal& w, integer& iblock, integer& isplit, sReal& z, integer& ldz, sReal& work, integer& iwork, integer& ifail, integer& info );

    void
    ssteqr
    ( const character& compz, integer& n, sReal& d, sReal& e, sReal& z, integer& ldz, sReal& work, integer& info );

    void
    ssterf
    ( integer& n, sReal& d, sReal& e, integer& info );

    void
    sstev
    ( const character& jobz, integer& n, sReal& d, sReal& e, sReal& z, integer& ldz, sReal& work, integer& info );

    void
    sstevd
    ( const character& jobz, integer& n, sReal& d, sReal& e, sReal& z, integer& ldz, sReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    sstevx
    ( const character& jobz, const character& range, integer& n, sReal& d, sReal& e, sReal& vl, sReal& vu, integer& il, integer& iu, sReal& abstol, integer& m, sReal& w, sReal& z, integer& ldz, sReal& work, integer& iwork, integer& ifail, integer& info );

    void
    ssycon
    ( const character& uplo, integer& n, sReal& a, integer& lda, integer& ipiv, sReal& anorm, sReal& rcond, sReal& work, integer& iwork, integer& info );
    */

    /*
    void
    ssyevd
    ( const character& jobz, const character& uplo, integer& n, sReal& a, integer& lda, sReal& w, sReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    ssyevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, sReal& a, integer& lda, sReal& vl, sReal& vu, integer& il, integer& iu, sReal& abstol, integer& m, sReal& w, sReal& z, integer& ldz, sReal& work, integer& lwork, integer& iwork, integer& ifail, integer& info );

    void
    ssygs2
    ( integer& itype, const character& uplo, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb, integer& info );

    void
    ssygst
    ( integer& itype, const character& uplo, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb, integer& info );
    */

    /*
    void
    ssyrfs
    ( const character& uplo, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& af, integer& ldaf, integer& ipiv, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    ssysv
    ( const character& uplo, integer& n, integer& nrhs, sReal& a, integer& lda, integer& ipiv, sReal& b, integer& ldb, sReal& work, integer& lwork, integer& info );

    void
    ssysvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& af, integer& ldaf, integer& ipiv, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& rcond, sReal& ferr, sReal& berr, sReal& work, integer& lwork, integer& iwork, integer& info );

    void
    ssytd2
    ( const character& uplo, integer& n, sReal& a, integer& lda, sReal& d, sReal& e, sReal& tau, integer& info );

    void
    ssytf2
    ( const character& uplo, integer& n, sReal& a, integer& lda, integer& ipiv, integer& info );

    void
    ssytrd
    ( const character& uplo, integer& n, sReal& a, integer& lda, sReal& d, sReal& e, sReal& tau, sReal& work, integer& lwork, integer& info );
    */

    /*
    void
    ssytrs
    ( const character& uplo, integer& n, integer& nrhs, sReal& a, integer& lda, integer& ipiv, sReal& b, integer& ldb, integer& info );

    void
    stbcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, integer& kd, sReal& ab, integer& ldab, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    stbrfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& kd, integer& nrhs, sReal& ab, integer& ldab, sReal& b, integer &ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    stbtrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& kd, integer& nrhs, sReal& ab, integer& ldab, sReal& b, integer &ldb, integer& info );

    void
    stgevc
    ( const character& side, const character& howmny, logical& select, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& vl, integer& ldvl, sReal& vr, integer& ldvr, integer& mm, integer& m, sReal &work, integer& info );

    void
    stgsja
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& p, integer& n, integer& k, integer& l, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& tola, sReal& tolb, sReal& alpha, sReal& beta, sReal& u, integer& ldu, sReal& v, integer& ldv, sReal& q, integer& ldq, sReal& work, integer& ncycle, integer& info );

    void
    stpcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, sReal& ap, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    stprfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, sReal& ap, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    stptri
    ( const character& uplo, const character& diag, integer& n, sReal& ap, integer& info );

    void
    stptrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, sReal& ap, sReal& b, integer& ldb, integer& info );

    void
    strcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, sReal& a, integer& lda, sReal& rcond, sReal& work, integer& iwork, integer& info );

    void
    strevc
    ( const character& side, const character& howmny, logical& select, integer& n, sReal& t, integer& ldt, sReal& vl, integer& ldvl, sReal& vr, integer& ldvr, integer& mm, integer& m, sReal& work, integer& info );

    void
    strexc
    ( const character& compq, integer& n, sReal& t, integer& ldt, sReal& q, integer& ldq, integer& ifst, integer& ilst, sReal& work, integer& info );

    void
    strrfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& x, integer& ldx, sReal& ferr, sReal& berr, sReal& work, integer& iwork, integer& info );

    void
    strsen
    ( const character& job, const character& compq, logical& select, integer &n, sReal& t, integer& ldt, sReal& q, integer& ldq, sReal& wr, sReal& wi, integer& m, sReal& s, sReal& sep, sReal& work, integer& lwork, integer& iwork, integer& liwork, integer& info );

    void
    strsna
    ( const character& job, const character& howmny, logical& select, integer& n, sReal& t, integer& ldt, sReal& vl, integer& ldvl, sReal& vr, integer& ldvr, sReal& s, sReal& sep, integer& mm, integer& m, sReal& work, integer& ldwork, integer& iwork, integer& info );

    void
    strsyl
    ( const character& trana, const character& tranb, integer& isgn, integer &m, integer& n, sReal& a, integer& lda, sReal& b, integer& ldb, sReal& c, integer& ldc, sReal& scale, integer& info );

    void
    strti2
    ( const character& uplo, const character& diag, integer& n, sReal& a, integer& lda, integer& info );
    */

    /*
    void
    strtrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, sReal& a, integer& lda, sReal& b, integer& ldb, integer& info );

    void
    stzrqf
    ( integer& m, integer& n, sReal& a, integer& lda, sReal& tau, integer& info );

    void
    zbdsqr
    ( const character& uplo, integer& n, integer& ncvt, integer& nru, integer& ncc, dReal& d, dReal& e, dComplex& vt, integer& ldvt, dComplex& u, integer& ldu, dComplex& c, integer& ldc, dReal& rwork, integer& info );

    void
    zdrot
    ( integer& n, dComplex& cx, integer& incx, dComplex& cy, integer& incy, dReal& c, dReal& s );

    void
    zdrscl
    ( integer& n, dReal& sa, dComplex& sx, integer& incx );

    void
    zgbbrd
    ( const character& vect, integer& m, integer& n, integer& ncc, integer& kl, integer& ku, dComplex& ab, integer& ldab, dReal& d, dReal& e, dComplex& q, integer& ldq, dComplex& pt, integer& ldpt, dComplex& c, integer& ldc, dComplex& work, dReal& rwork, integer& info );

    void
    zgbcon
    ( const character& norm, integer& n, integer& kl, integer& ku, dComplex& ab, integer& ldab, integer& ipiv, dReal& anorm, dReal& rcond, dComplex& work, dReal& rwork, integer& info );

    void
    zgbequ
    ( integer& m, integer& n, integer& kl, integer& ku, dComplex& ab, integer& ldab, dReal& r, dReal& c, dReal& rowcnd, dReal& colcnd, dReal& amax, integer& info );

    void
    zgbrfs
    ( const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, dComplex& ab, integer& ldab, dComplex& afb, integer& ldafb, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zgbsv
    ( integer& n, integer& kl, integer& ku, integer& nrhs, dComplex& ab, integer& ldab, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zgbsvx
    ( const character& fact, const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, dComplex& ab, integer& ldab, dComplex& afb, integer& ldafb, integer& ipiv, character& equed, dReal& r, dReal& c, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zgbtf2
    ( integer& m, integer& n, integer& kl, integer& ku, dComplex& ab, integer& ldab, integer& ipiv, integer& info );

    void
    zgbtrf
    ( integer& m, integer& n, integer& kl, integer& ku, dComplex& ab, integer& ldab, integer& ipiv, integer& info );

    void
    zgbtrs
    ( const character& trans, integer& n, integer& kl, integer& ku, integer& nrhs, dComplex& ab, integer& ldab, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zgebak
    ( const character& job, const character& side, integer& n, integer& ilo, integer& ihi, dReal& scale, integer& m, dComplex& v, integer& ldv, integer& info );

    void
    zgebal
    ( const character& job, integer& n, dComplex& a, integer &lda, integer& ilo, integer& ihi, dReal& scale, integer& info );

    void
    zgebd2
    ( integer& m, integer& n, dComplex& a, integer& lda, dReal& d, dReal& e, dComplex& tauq, dComplex& taup, dComplex& work, integer& info );

    void
    zgebrd
    ( integer& m, integer& n, dComplex& a, integer& lda, dReal& d, dReal& e, dComplex& tauq, dComplex& taup, dComplex& work, integer& lwork, integer& info );

    void
    zgecon
    ( const character& norm, integer& n, dComplex& a, integer& lda, dReal& anorm, dReal& rcond, dComplex& work, dReal& rwork, integer& info );

    void
    zgeequ
    ( integer& m, integer& n, dComplex& a, integer& lda, dReal& r, dReal& c, dReal& rowcnd, dReal& colcnd, dReal& amax, integer& info );

    void
    zgees
    ( const character& jobvs, const character& sort, logical (*select)( dComplex& ), integer& n, dComplex& a, integer& lda, integer& sdim, dComplex& w, dComplex& vs, integer& ldvs, dComplex& work, integer& lwork, dReal& rwork, logical& bwork, integer& info );

    void
    zgeesx
    ( const character& jobvs, const character& sort, logical (*select)( dComplex& ), const character& sense, integer& n, dComplex& a, integer& lda, integer& sdim, dComplex& w, dComplex& vs, integer& ldvs, dReal& rconde, dReal& rcondv, dComplex& work, integer& lwork, dReal& rwork, logical& bwork, integer& info );

    void
    zgeev
    ( const character& jobvl, const character& jobvr, integer& n, dComplex& a, integer& lda, dComplex& w, dComplex& vl, integer& ldvl, dComplex& vr, integer& ldvr, dComplex& work, integer& lwork, dReal& rwork, integer& info );

    void
    zgeevx
    ( const character& balanc, const character& jobvl, const character& jobvr, const character& sense, integer& n, dComplex& a, integer& lda, dComplex& w, dComplex& vl, integer& ldvl, dComplex& vr, integer& ldvr, integer& ilo, integer& ihi, dReal& scale, dReal& abnrm, dReal& rconde, dReal& rcondv, dComplex& work, integer& lwork, dReal& rwork, integer& info );

    void
    zgegs
    ( const character& jobvsl, const character& jobvsr, integer& n, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& alpha, dComplex& beta, dComplex& vsl, integer& ldvsl, dComplex& vsr, integer& ldvsr, dComplex& work, integer& lwork, dReal& rwork, integer& info );

    void
    zgegv
    ( const character& jobvl, const character& jobvr, integer& n, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& alpha, dComplex& beta, dComplex& vl, integer &ldvl, dComplex& vr, integer& ldvr, dComplex& work, integer &lwork, dReal& rwork, integer& info );

    void
    zgehd2
    ( integer& n, integer& ilo, integer& ihi, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zgehrd
    ( integer& n, integer& ilo, integer& ihi, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zgelq2
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zgelqf
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zgels
    ( const character& trans, integer& m, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& work, integer& lwork, integer& info );

    void
    zgelss
    ( integer& m, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& b, integer& ldb, dReal& s, dReal& rcond, integer& rank, dComplex& work, integer& lwork, dReal& rwork, integer& info );

    void
    zgelsx
    ( integer& m, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& b, integer& ldb, integer& jpvt, dReal& rcond, integer& rank, dComplex& work, dReal& rwork, integer& info );

    void
    zgeql2
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zgeqlf
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zgeqpf
    ( integer& m, integer& n, dComplex& a, integer& lda, integer& jpvt, dComplex& tau, dComplex& work, dReal& rwork, integer& info );

    void
    zgeqr2
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zgeqrf
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zgerfs
    ( const character& trans, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& af, integer& ldaf, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zgerq2
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zgerqf
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zgesv
    ( integer& n, integer& nrhs, dComplex& a, integer& lda, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zgesvd
    ( const character& jobu, const character& jobvt, integer& m, integer& n, dComplex& a, integer& lda, dReal& s, dComplex& u, integer& ldu, dComplex& vt, integer& ldvt, dComplex& work, integer& lwork, dReal& rwork, integer& info );

    void
    zgesvx
    ( const character& fact, const character& trans, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& af, integer& ldaf, integer& ipiv, character& equed, dReal& r, dReal& c, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zgetf2
    ( integer& m, integer& n, dComplex& a, integer& lda, integer& ipiv, integer& info );

    void
    zgetrf
    ( integer& m, integer& n, dComplex& a, integer& lda, integer& ipiv, integer& info );

    void
    zgetri
    ( integer& n, dComplex& a, integer& lda, integer& ipiv, dComplex& work, integer& lwork, integer& info );

    void
    zgetrs
    ( const character& trans, integer& n, integer& nrhs, dComplex& a, integer& lda, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zggbak
    ( const character& job, const character& side, integer& n, integer& ilo, integer& ihi, dReal& lscale, dReal& rscale, integer& m, dComplex& v, integer& ldv, integer& info );

    void
    zggbal
    ( const character& job, integer& n, dComplex& a, integer &lda, dComplex& b, integer& ldb, integer& ilo, integer& ihi, dReal& lscale, dReal& rscale, dReal& work, integer& info );

    void
    zggglm
    ( integer& n, integer& m, integer& p, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& d, dComplex& x, dComplex& y, dComplex& work, integer& lwork, integer& info );

    void
    zgghrd
    ( const character& compq, const character& compz, integer& n, integer& ilo, integer& ihi, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& q, integer& ldq, dComplex& z, integer& ldz, integer& info );

    void
    zgglse
    ( integer& m, integer& n, integer& p, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& c, dComplex& d, dComplex& x, dComplex& work, integer& lwork, integer& info );

    void
    zggqrf
    ( integer& n, integer& m, integer& p, dComplex& a, integer& lda, dComplex& taua, dComplex& b, integer& ldb, dComplex& taub, dComplex& work, integer& lwork, integer& info );

    void
    zggrqf
    ( integer& m, integer& p, integer& n, dComplex& a, integer& lda, dComplex& taua, dComplex& b, integer& ldb, dComplex& taub, dComplex& work, integer& lwork, integer& info );

    void
    zggsvd
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& n, integer& p, integer& k, integer& l, dComplex& a, integer& lda, dComplex& b, integer& ldb, dReal& alpha, dReal& beta, dComplex& u, integer& ldu, dComplex& v, integer& ldv, dComplex& q, integer& ldq, dComplex& work, dReal& rwork, integer& iwork, integer& info );

    void
    zggsvp
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& p, integer& n, dComplex& a, integer& lda, dComplex &b, integer& ldb, dReal& tola, dReal& tolb, integer& k, integer& l, dComplex& u, integer& ldu, dComplex& v, integer &ldv, dComplex& q, integer& ldq, integer& iwork, dReal& rwork, dComplex& tau, dComplex& work, integer& info );

    void
    zgtcon
    ( const character& norm, integer& n, dComplex& dl, dComplex& d, dComplex& du, dComplex& du2, integer& ipiv, dReal& anorm, dReal& rcond, dComplex& work, integer& info );

    void
    zgtrfs
    ( const character& trans, integer& n, integer& nrhs, dComplex& dl, dComplex& d, dComplex& du, dComplex &dlf, dComplex& df, dComplex& duf, dComplex& du2, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zgtsv
    ( integer& n, integer& nrhs, dComplex& dl, dComplex& d, dComplex& du, dComplex& b, integer& ldb, integer& info );

    void
    zgtsvx
    ( const character& fact, const character& trans, integer& n, integer& nrhs, dComplex& dl, dComplex& d, dComplex& du, dComplex& dlf, dComplex& df, dComplex& duf, dComplex& du2, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zgttrf
    ( integer& n, dComplex& dl, dComplex& d, dComplex& du, dComplex& du2, integer& ipiv, integer& info );

    void
    zgttrs
    ( const character& trans, integer& n, integer& nrhs, dComplex& dl, dComplex& d, dComplex& du, dComplex &du2, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zhbev
    ( const character& jobz, const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, dReal& w, dComplex& z, integer& ldz, dComplex& work, dReal& rwork, integer& info );

    void
    zhbevd
    ( const character& jobz, const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, dReal& w, dComplex& z, integer& ldz, dComplex& work, integer& lwork, dReal& rwork, integer& lrwork, integer& iwork, integer& liwork, integer& info );

    void
    zhbevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, dComplex& q, integer& ldq, dReal& vl, dReal& vu, integer& il, integer& iu, dReal& abstol, integer& m, dReal& w, dComplex& z, integer& ldz, dComplex& work, dReal& rwork, integer& iwork, integer& ifail, integer& info );

    void
    zhbgst
    ( const character& vect, const character& uplo, integer& n, integer& ka, integer& kb, dComplex& ab, integer& ldab, dComplex& bb, integer& ldbb, dComplex& x, integer& ldx, dComplex& work, dReal& rwork, integer& info );

    void
    zhbgv
    ( const character& jobz, const character& uplo, integer& n, integer& ka, integer& kb, dComplex& ab, integer& ldab, dComplex& bb, integer& ldbb, dReal& w, dComplex& z, integer& ldz, dComplex& work, dReal& rwork, integer& info );

    void
    zhbtrd
    ( const character& vect, const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, dReal& d, dReal& e, dComplex& q, integer& ldq, dComplex& work, integer& info );

    void
    zhecon
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& ipiv, dReal& anorm, dReal& rcond, dComplex& work, integer& info );

    void
    zheev
    ( const character& jobz, const character& uplo, const integer& n, dComplex &a, const integer& lda, dReal& w, dComplex& work, const integer& lwork, dReal& rwork, integer& info );

    void
    zheevd
    ( const character& jobz, const character& uplo, integer& n, dComplex& a, integer& lda, dReal& w, dComplex& work, integer& lwork, dReal& rwork, integer& lrwork, integer& iwork, integer& liwork, integer& info );

    void
    zheevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, dComplex& a, integer& lda, dReal& vl, dReal& vu, integer& il, integer& iu, dReal& abstol, integer& m, dReal& w, dComplex& z, integer& ldz, dComplex& work, integer& lwork, dReal& rwork, integer& iwork, integer& ifail, integer& info );

    void
    zhegs2
    ( integer& itype, const character& uplo, integer& n, dComplex& a, integer& lda, dComplex& b, integer& ldb, integer& info );

    void
    zhegst
    ( integer& itype, const character& uplo, integer& n, dComplex& a, integer& lda, dComplex& b, integer& ldb, integer& info );

    void
    zhegv
    ( const integer& itype, const character& jobz, const character& uplo, const integer& n, dComplex& a, const integer& lda, dComplex& b, const integer& ldb, dReal& w, dComplex& work, const integer& lwork, dReal& rwork, integer& info );

    void
    zherfs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& af, integer& ldaf, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zhesv
    ( const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, integer& ipiv, dComplex& b, integer& ldb, dComplex& work, integer& lwork, integer& info );

    void
    zhesvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& af, integer& ldaf, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, integer& lwork, dReal& rwork, integer& info );

    void
    zhetd2
    ( const character& uplo, integer& n, dComplex& a, integer& lda, dReal& d, dReal& e, dComplex& tau, integer& info );

    void
    zhetf2
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& ipiv, integer& info );

    void
    zhetrd
    ( const character& uplo, integer& n, dComplex& a, integer& lda, dReal& d, dReal& e, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zhetrf
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& ipiv, dComplex& work, integer& lwork, integer& info );

    void
    zhetri
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& ipiv, dComplex& work, integer& info );

    void
    zhetrs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zhgeqz
    ( const character& job, const character& compq, const character& compz, integer& n, integer& ilo, integer& ihi, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& alpha, dComplex& beta, dComplex& q, integer& ldq, dComplex& z, integer& ldz, dComplex& work, integer& lwork, dReal& rwork, integer& info );

    void
    zhpcon
    ( const character& uplo, integer& n, dComplex& ap, integer& ipiv, dReal& anorm, dReal& rcond, dComplex& work, integer& info );

    void
    zhpev
    ( const character& jobz, const character& uplo, integer& n, dComplex &ap, dReal& w, dComplex& z, integer& ldz, dComplex& work, dReal& rwork, integer& info );

    void
    zhpevd
    ( const character& jobz, const character& uplo, integer& n, dComplex& ap, dReal& w, dComplex& z, integer& ldz, dComplex& work, integer& lwork, dReal& rwork, integer& lrwork, integer& iwork, integer& liwork, integer& info );

    void
    zhpevx
    ( const character& jobz, const character& range, const character& uplo, integer& n, dComplex& ap, dReal& vl, dReal& vu, integer& il, integer& iu, dReal& abstol, integer& m, dReal& w, dComplex& z, integer& ldz, dComplex& work, dReal& rwork, integer& iwork, integer& ifail, integer& info );

    void
    zhpgst
    ( integer& itype, const character& uplo, integer& n, dComplex& ap, dComplex& bp, integer& info );

    void
    zhpgv
    ( integer& itype, const character& jobz, const character& uplo, integer& n, dComplex& ap, dComplex& bp, dReal& w, dComplex &z, integer& ldz, dComplex& work, dReal& rwork, integer& info );

    void
    zhprfs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, dComplex& afp, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zhpsv
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zhpsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dComplex& ap, dComplex& afp, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zhptrd
    ( const character& uplo, integer& n, dComplex& ap, dReal& d, dReal& e, dComplex& tau, integer& info );

    void
    zhptrf
    ( const character& uplo, integer& n, dComplex& ap, integer& ipiv, integer& info );

    void
    zhptri
    ( const character& uplo, integer& n, dComplex& ap, integer& ipiv, dComplex& work, integer& info );

    void
    zhptrs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zhsein
    ( const character& side, const character& eigsrc, const character& initv, logical& select, integer& n, dComplex& h, integer& ldh, dComplex& w, dComplex& vl, integer& ldvl, dComplex& vr, integer& ldvr, integer& mm, integer& m, dComplex& work, dReal& rwork, integer& ifaill, integer& ifailr, integer& info );

    void
    zhseqr
    ( const character& job, const character& compz, integer& n, integer& ilo, integer& ihi, dComplex& h, integer& ldh, dComplex& w, dComplex& z, integer& ldz, dComplex& work, integer& lwork, integer& info );

    void
    zlabrd
    ( integer& m, integer& n, integer& nb, dComplex& a, integer& lda, dReal& d, dReal& e, dComplex& tauq, dComplex& taup, dComplex& x, integer& ldx, dComplex& y, integer& ldy );

    void
    zlacgv
    ( integer& n, dComplex& x, integer& incx );

    void
    zlacon
    ( integer& n, dComplex& v, dComplex& x, dReal& est, integer& kase );

    void
    zlacpy
    ( const character& uplo, integer& m, integer& n, dComplex& a, integer& lda, dComplex& b, integer& ldb );

    void
    zlacrm
    ( integer& m, integer& n, dComplex& a, integer& lda, dReal& b, integer& ldb, dComplex& c, integer& ldc, dReal& rwork );

    void
    zlacrt
    ( integer& n, dComplex& cx, integer& incx, dComplex& cy, integer& incy, dComplex& c, dComplex& s );

    dComplex
    zladiv
    ( dComplex& x, dComplex& y );

    void
    zlaed0
    ( integer& qsiz, integer& n, dReal& d, dReal& e, dComplex& q, integer& ldq, dComplex& qstore, integer& ldqs, dReal& rwork, integer& iwork, integer& info );

    void
    zlaed7
    ( integer& n, integer& cutpnt, integer& qsiz, integer& tlvls, integer& curlvl, integer& curpbm, dReal& d, dComplex& q, integer& ldq, dReal& rho, integer& indxq, dReal& qstore, integer& qptr, integer& prmptr, integer& perm, integer& givptr, integer& givcol, dReal& givnum, dComplex& work, dReal& rwork, integer& iwork, integer& info );

    void
    zlaed8
    ( integer& k, integer& n, integer& qsiz, dComplex& q, integer& ldq, dReal& d, dReal& rho, integer& cutpnt, dReal& z, dReal& dlamda, dComplex& q2, integer& ldq2, dReal& w, integer& indxp, integer& indx, integer &indxq, integer& perm, integer& givptr, integer& givcol, dReal& givnum, integer& info );

    void
    zlaein
    ( logical& rightv, logical& noinit, integer& n, dComplex& h, integer& ldh, dComplex& w, dComplex& v, dComplex& b, integer& ldb, dReal& rwork, dReal& eps3, dReal& smlnum, integer& info );

    void
    zlaesy
    ( dComplex& a, dComplex& b, dComplex& c, dComplex& rt1, dComplex& rt2, dComplex& evscal, dComplex& cs1, dComplex& sn1 );

    void
    zlaev2
    ( dComplex& a, dComplex& b, dComplex& c, dReal& rt1, dReal& rt2, dReal& cs1, dComplex& sn1 );

    void
    zlags2
    ( logical& upper, dReal& a1, dComplex& a2, dReal& a3, dReal& b1, dComplex& b2, dReal& b3, dReal& csu, dComplex& snu, dReal& csv, dComplex& snv, dReal& csq, dComplex& snq );

    void
    zlagtm
    ( const character& trans, integer& n, integer& nrhs, dReal& alpha, dComplex& dl, dComplex& d, dComplex &du, dComplex& x, integer& ldx, dReal& beta, dComplex& b, integer& ldb );

    void
    zlahef
    ( const character& uplo, integer& n, integer& nb, integer& kb, dComplex& a, integer& lda, integer& ipiv, dComplex& w, integer& ldw, integer& info );

    void
    zlahqr
    ( logical& wantt, logical& wantz, integer& n, integer& ilo, integer& ihi, dComplex& h, integer& ldh, dComplex& w, integer& iloz, integer& ihiz, dComplex& z, integer& ldz, integer& info );

    void
    zlahrd
    ( integer& n, integer& k, integer& nb, dComplex& a, integer& lda, dComplex& tau, dComplex& t, integer& ldt, dComplex& y, integer& ldy );

    void
    zlaic1
    ( integer& job, integer& j, dComplex& x, dReal& sest, dComplex& w, dComplex& gamma, dReal& sestpr, dComplex& s, dComplex& c );

    dReal
    zlangb
    ( const character& norm, integer& n, integer& kl, integer& ku, dComplex& ab, integer& ldab, dReal& work );

    dReal
    zlange
    ( const character& norm, integer& m, integer& n, dComplex& a, integer& lda, dReal& work );

    dReal
    zlangt
    ( const character& norm, integer& n, dComplex& dl, dComplex& d, dComplex& du );

    dReal
    zlanhb
    ( const character& norm, const character& uplo, integer& n, integer& k, dComplex& ab, integer& ldab, dReal& work );

    dReal
    zlanhe
    ( const character& norm, const character& uplo, integer& n, dComplex& a, integer& lda, dReal& work );

    dReal
    zlanhp
    ( const character& norm, const character& uplo, integer& n, dComplex& ap, dReal& work );

    dReal
    zlanhs
    ( const character& norm, integer& n, dComplex& a, integer& lda, dReal& work );

    dReal
    zlanht
    ( const character& norm, integer& n, dReal& d, dComplex& e );

    dReal
    zlansb
    ( const character& norm, const character& uplo, integer& n, integer& k, dComplex& ab, integer& ldab, dReal& work );

    dReal
    zlansp
    ( const character& norm, const character& uplo, integer& n, dComplex& ap, dReal& work );

    dReal
    zlansy
    ( const character& norm, const character& uplo, integer& n, dComplex& a, integer& lda, dReal& work );

    dReal
    zlantb
    ( const character& norm, const character& uplo, const character& diag, integer& n, integer& k, dComplex& ab, integer& ldab, dReal& work );

    dReal
    zlantp
    ( const character& norm, const character& uplo, const character& diag, integer& n, dComplex& ap, dReal& work );

    dReal
    zlantr
    ( const character& norm, const character& uplo, const character& diag, integer& m, integer& n, dComplex& a, integer& lda, dReal& work );

    void
    zlapll
    ( integer& n, dComplex& x, integer& incx, dComplex& y, integer& incy, dReal& ssmin );

    void
    zlapmt
    ( logical& forwrd, integer& m, integer& n, dComplex& x, integer& ldx, integer& k );

    void
    zlaqgb
    ( integer& m, integer& n, integer& kl, integer& ku, dComplex& ab, integer& ldab, dReal& r, dReal& c, dReal& rowcnd, dReal& colcnd, dReal& amax, character& equed );

    void
    zlaqge
    ( integer& m, integer& n, dComplex& a, integer& lda, dReal& r, dReal& c, dReal& rowcnd, dReal& colcnd, dReal& amax, character& equed );

    void
    zlaqhb
    ( const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    zlaqhe
    ( const character& uplo, integer& n, dComplex& a, integer& lda, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    zlaqhp
    ( const character& uplo, integer& n, dComplex& ap, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    zlaqsb
    ( const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    zlaqsp
    ( const character& uplo, integer& n, dComplex& ap, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    zlaqsy
    ( const character& uplo, integer& n, dComplex& a, integer& lda, dReal& s, dReal& scond, dReal& amax, character& equed );

    void
    zlar2v
    ( integer& n, dComplex& x, dComplex& y, dComplex& z, integer& incx, dReal& c, dComplex& s, integer& incc );

    void
    zlarf
    ( const character& side, integer& m, integer& n, dComplex &v, integer& incv, dComplex& tau, dComplex& c, integer& ldc, dComplex& work );

    void
    zlarfb
    ( const character& side, const character& trans, const character& direct, character& storev, integer& m, integer& n, integer& k, dComplex& v, integer &ldv, dComplex& t, integer& ldt, dComplex& c, integer& ldc, dComplex& work, integer& ldwork );

    void
    zlarfg
    ( integer& n, dComplex& alpha, dComplex& x, integer& incx, dComplex& tau );

    void
    zlarft
    ( const character& direct, character& storev, integer& n, integer& k, dComplex& v, integer& ldv, dComplex& tau, dComplex& t, integer& ldt );

    void
    zlarfx
    ( const character& side, integer& m, integer& n, dComplex& v, dComplex& tau, dComplex& c, integer& ldc, dComplex& work );

    void
    zlargv
    ( integer& n, dComplex& x, integer& incx, dComplex& y, integer& incy, dReal& c, integer& incc );

    void
    zlarnv
    ( integer& idist, integer& iseed, integer& n, dComplex& x );

    void
    zlartg
    ( dComplex& f, dComplex& g, dReal& cs, dComplex& sn, dComplex& r );

    void
    zlartv
    ( integer& n, dComplex& x, integer& incx, dComplex& y, integer& incy, dReal& c, dComplex& s, integer& incc );

    void
    zlascl
    ( const character& type, integer& kl, integer& ku, dReal &cfrom, dReal& cto, integer& m, integer& n, dComplex& a, integer& lda, integer& info );

    void
    zlaset
    ( const character& uplo, integer& m, integer& n, dComplex& alpha, dComplex& beta, dComplex& a, integer& lda );

    void
    zlasr
    ( const character& side, const character& pivot, const character& direct, integer& m, integer& n, dReal& c, dReal& s, dComplex& a, integer& lda );

    void
    zlassq
    ( integer& n, dComplex& x, integer& incx, dReal& scale, dReal& sumsq );

    void
    zlaswp
    ( integer& n, dComplex& a, integer& lda, integer& k1, integer& k2, integer& ipiv, integer& incx );

    void
    zlasyf
    ( const character& uplo, integer& n, integer& nb, integer& kb, dComplex& a, integer& lda, integer& ipiv, dComplex& w, integer& ldw, integer& info );

    void
    zlatbs
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, integer& kd, dComplex& ab, integer& ldab, dComplex& x, dReal& scale, dReal& cnorm, integer& info );

    void
    zlatps
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, dComplex& ap, dComplex& x, dReal& scale, dReal& cnorm, integer& info );

    void
    zlatrd
    ( const character& uplo, integer& n, integer& nb, dComplex& a, integer& lda, dReal& e, dComplex& tau, dComplex& w, integer& ldw );

    void
    zlatrs
    ( const character& uplo, const character& trans, const character& diag, const character& normin, integer& n, dComplex& a, integer& lda, dComplex& x, dReal& scale, dReal& cnorm, integer& info );

    void
    zlatzm
    ( const character& side, integer& m, integer& n, dComplex& v, integer& incv, dComplex& tau, dComplex& c1, dComplex& c2, integer& ldc, dComplex& work );

    void
    zlauu2
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& info );

    void
    zlauum
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& info );

    void
    zpbcon
    ( const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, dReal& anorm, dReal& rcond, dComplex& work, dReal& rwork, integer& info );

    void
    zpbequ
    ( const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, dReal& s, dReal& scond, dReal& amax, integer& info );

    void
    zpbrfs
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, dComplex& ab, integer& ldab, dComplex& afb, integer& ldafb, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zpbstf
    ( const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, integer& info );

    void
    zpbsv
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, dComplex& ab, integer& ldab, dComplex& b, integer& ldb, integer& info );

    void
    zpbsvx
    ( const character& fact, const character& uplo, integer& n, integer& kd, integer& nrhs, dComplex& ab, integer& ldab, dComplex& afb, integer& ldafb, character& equed, dReal& s, dComplex& b, integer &ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zpbtf2
    ( const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, integer& info );

    void
    zpbtrf
    ( const character& uplo, integer& n, integer& kd, dComplex& ab, integer& ldab, integer& info );

    void
    zpbtrs
    ( const character& uplo, integer& n, integer& kd, integer& nrhs, dComplex& ab, integer& ldab, dComplex& b, integer& ldb, integer& info );

    void
    zpocon
    ( const character& uplo, integer& n, dComplex& a, integer& lda, dReal& anorm, dReal& rcond, dComplex& work, dReal& rwork, integer& info );

    void
    zpoequ
    ( integer& n, dComplex& a, integer& lda, dReal& s, dReal& scond, dReal& amax, integer& info );

    void
    zporfs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& af, integer& ldaf, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zposv
    ( const character& uplo, const integer& n, const integer& nrhs, dComplex& a, const integer& lda, dComplex& b, const integer& ldb, integer& info );

    void
    zposvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& af, integer& ldaf, character& equed, dReal& s, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zpotf2
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& info );

    void
    zpotrf
    ( const character& uplo, const integer& n, dComplex& a, const integer& lda, integer& info );

    void
    zpotri
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& info );

    void
    zpotrs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& b, integer& ldb, integer& info );

    void
    zppcon
    ( const character& uplo, integer& n, dComplex& ap, dReal& anorm, dReal& rcond, dComplex& work, dReal &rwork, integer& info );

    void
    zppequ
    ( const character& uplo, integer& n, dComplex& ap, dReal& s, dReal& scond, dReal& amax, integer& info );

    void
    zpprfs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, dComplex& afp, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zppsv
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, dComplex& b, integer& ldb, integer& info );

    void
    zppsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dComplex& ap, dComplex& afp, character& equed, dReal& s, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zpptrf
    ( const character& uplo, integer& n, dComplex& ap, integer& info );

    void
    zpptri
    ( const character& uplo, integer& n, dComplex& ap, integer& info );

    void
    zpptrs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, dComplex& b, integer& ldb, integer& info );

    void
    zptcon
    ( integer& n, dReal& d, dComplex& e, dReal& anorm, dReal& rcond, dReal& rwork, integer& info );

    void
    zpteqr
    ( const character& compz, integer& n, dReal& d, dReal& e, dComplex& z, integer& ldz, dReal& work, integer& info );

    void
    zptrfs
    ( const character& uplo, integer& n, integer& nrhs, dReal& d, dComplex& e, dReal& df, dComplex& ef, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zptsv
    ( integer& n, integer& nrhs, dReal& d, dComplex& e, dComplex& b, integer& ldb, integer& info );

    void
    zptsvx
    ( const character& fact, integer& n, integer& nrhs, dReal& d, dComplex& e, dReal& df, dComplex& ef, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zpttrf
    ( integer& n, dReal& d, dComplex& e, integer& info );

    void
    zpttrs
    ( const character& uplo, integer& n, integer& nrhs, dReal& d, dComplex& e, dComplex& b, integer& ldb, integer& info );

    void
    zrot
    ( integer& n, dComplex& cx, integer& incx, dComplex& cy, integer& incy, dReal& c, dComplex& s );

    void
    zspcon
    ( const character& uplo, integer& n, dComplex& ap, integer& ipiv, dReal& anorm, dReal& rcond, dComplex& work, integer& info );

    void
    zspmv
    ( const character& uplo, integer& n, dComplex& alpha, dComplex& ap, dComplex& x, integer& incx, dComplex& beta, dComplex& y, integer& incy );

    void
    zspr
    ( const character& uplo, integer& n, dComplex& alpha, dComplex& x, integer& incx, dComplex& ap );

    void
    zsprfs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, dComplex& afp, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zspsv
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zspsvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dComplex& ap, dComplex& afp, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zsptrf
    ( const character& uplo, integer& n, dComplex& ap, integer& ipiv, integer& info );

    void
    zsptri
    ( const character& uplo, integer& n, dComplex& ap, integer& ipiv, dComplex& work, integer& info );

    void
    zsptrs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& ap, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    zstedc
    ( const character& compz, integer& n, dReal& d, dReal& e, dComplex& z, integer& ldz, dComplex& work, integer& lwork, dReal& rwork, integer& lrwork, integer& iwork, integer& liwork, integer& info );

    void
    zstein
    ( integer& n, dReal& d, dReal& e, integer& m, dReal& w, integer& iblock, integer& isplit, dComplex& z, integer& ldz, dReal& work, integer& iwork, integer& ifail, integer& info );

    void
    zsteqr
    ( const character& compz, integer& n, dReal& d, dReal& e, dComplex& z, integer& ldz, dReal& work, integer& info );

    void
    zsycon
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& ipiv, dReal& anorm, dReal& rcond, dComplex& work, integer& info );

    void
    zsymv
    ( const character& uplo, integer& n, dComplex& alpha, dComplex& a, integer& lda, dComplex& x, integer& incx, dComplex& beta, dComplex& y, integer& incy );

    void
    zsyr
    ( const character& uplo, integer& n, dComplex& alpha, dComplex& x, integer& incx, dComplex& a, integer& lda );

    void
    zsyrfs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& af, integer& ldaf, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    zsysv
    ( const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, integer& ipiv, dComplex& b, integer& ldb, dComplex& work, integer& lwork, integer& info );

    void
    zsysvx
    ( const character& fact, const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& af, integer& ldaf, integer& ipiv, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& rcond, dReal& ferr, dReal& berr, dComplex& work, integer& lwork, dReal& rwork, integer& info );

    void
    zsytf2
    ( const character& uplo, integer& n, dComplex& a, integer& lda, integer& ipiv, integer& info );

    void
    zsytrf
    ( const character& uplo, const integer& n, dComplex& a, const integer& lda, integer& ipiv, dComplex& work, const integer& lwork, integer& info );

    void
    zsytri
    ( const character& uplo, const integer& n, dComplex& a, const integer& lda, const integer& ipiv, dComplex& work, integer& info );

    void
    zsytrs
    ( const character& uplo, integer& n, integer& nrhs, dComplex& a, integer& lda, integer& ipiv, dComplex& b, integer& ldb, integer& info );

    void
    ztbcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, integer& kd, dComplex& ab, integer& ldab, dReal& rcond, dComplex& work, dReal& rwork, integer& info );

    void
    ztbrfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& kd, integer& nrhs, dComplex& ab, integer& ldab, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    ztbtrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& kd, integer& nrhs, dComplex& ab, integer& ldab, dComplex& b, integer& ldb, integer& info );

    void
    ztgevc
    ( const character& side, const character& howmny, logical& select, integer& n, dComplex& a, integer& lda, dComplex& b, integer &ldb, dComplex& vl, integer& ldvl, dComplex& vr, integer& ldvr, integer& mm, integer& m, dComplex& work, dReal& rwork, integer& info );

    void
    ztgsja
    ( const character& jobu, const character& jobv, const character& jobq, integer& m, integer& p, integer& n, integer& k, integer& l, dComplex& a, integer& lda, dComplex& b, integer& ldb, dReal& tola, dReal& tolb, dReal& alpha, dReal& beta, dComplex& u, integer& ldu, dComplex& v, integer& ldv, dComplex& q, integer& ldq, dComplex& work, integer& ncycle, integer& info );

    void
    ztpcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, dComplex& ap, dReal& rcond, dComplex& work, dReal &rwork, integer& info );

    void
    ztprfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, dComplex& ap, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    ztptri
    ( const character& uplo, const character& diag, integer& n, dComplex& ap, integer& info );

    void
    ztptrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, dComplex& ap, dComplex& b, integer& ldb, integer& info );

    void
    ztrcon
    ( const character& norm, const character& uplo, const character& diag, integer& n, dComplex& a, integer& lda, dReal& rcond, dComplex& work, dReal& rwork, integer& info );

    void
    ztrevc
    ( const character& side, const character& howmny, logical& select, integer& n, dComplex& t, integer& ldt, dComplex& vl, integer& ldvl, dComplex& vr, integer& ldvr, integer& mm, integer &m, dComplex& work, dReal& rwork, integer& info );

    void
    ztrexc
    ( const character& compq, integer& n, dComplex& t, integer& ldt, dComplex& q, integer& ldq, integer& ifst, integer& ilst, integer& info );

    void
    ztrrfs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& x, integer& ldx, dReal& ferr, dReal& berr, dComplex& work, dReal& rwork, integer& info );

    void
    ztrsen
    ( const character& job, const character& compq, logical& select, integer &n, dComplex& t, integer& ldt, dComplex& q, integer& ldq, dComplex& w, integer& m, dReal& s, dReal& sep, dComplex& work, integer& lwork, integer& info );

    void
    ztrsna
    ( const character& job, const character& howmny, logical& select, integer& n, dComplex& t, integer& ldt, dComplex& vl, integer& ldvl, dComplex& vr, integer& ldvr, dReal& s, dReal& sep, integer& mm, integer& m, dComplex& work, integer& ldwork, dReal& rwork, integer& info );

    void
    ztrsyl
    ( const character& trana, const character& tranb, integer& isgn, integer &m, integer& n, dComplex& a, integer& lda, dComplex& b, integer& ldb, dComplex& c, integer& ldc, dReal& scale, integer& info );

    void
    ztrti2
    ( const character& uplo, const character& diag, integer& n, dComplex& a, integer& lda, integer& info );

    void
    ztrtri
    ( const character& uplo, const character& diag, const integer& n, dComplex& a, const integer& lda, integer& info );

    void
    ztrtrs
    ( const character& uplo, const character& trans, const character& diag, integer& n, integer& nrhs, dComplex& a, integer& lda, dComplex& b, integer& ldb, integer& info );

    void
    ztzrqf
    ( integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, integer& info );

    void
    zung2l
    ( integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zung2r
    ( integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zungbr
    ( const character& vect, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zunghr
    ( integer& n, integer& ilo, integer& ihi, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zungl2
    ( integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zunglq
    ( integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zungql
    ( integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zungqr
    ( integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zungr2
    ( integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& info );

    void
    zungrq
    ( integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zungtr
    ( const character& uplo, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& work, integer& lwork, integer& info );

    void
    zunm2l
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& info );

    void
    zunm2r
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& info );

    void
    zunmbr
    ( const character& vect, const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex &tau, dComplex& c, integer& ldc, dComplex& work, integer& lwork, integer& info );

    void
    zunmhr
    ( const character& side, const character& trans, integer& m, integer& n, integer& ilo, integer& ihi, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& lwork, integer& info );

    void
    zunml2
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& info );

    void
    zunmlq
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& lwork, integer& info );

    void
    zunmql
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& lwork, integer& info );

    void
    zunmqr
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& lwork, integer& info );

    void
    zunmr2
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& info );

    void
    zunmrq
    ( const character& side, const character& trans, integer& m, integer& n, integer& k, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& lwork, integer& info );

    void
    zunmtr
    ( const character& side, const character& uplo, const character& trans, integer& m, integer& n, dComplex& a, integer& lda, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& lwork, integer& info );

    void
    zupgtr
    ( const character& uplo, integer& n, dComplex& ap, dComplex& tau, dComplex& q, integer& ldq, dComplex& work, integer& info );

    void
    zupmtr
    ( const character& side, const character& uplo, const character& trans, integer& m, integer& n, dComplex& ap, dComplex& tau, dComplex& c, integer& ldc, dComplex& work, integer& info );
    */
   } /* extern "C" {} */

cpp_gnamespace_end

#endif /* __cpplapack_H__ */
